/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */

package com.moneyhouse.psvn.util.timestamp;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;

import com.moneyhouse.psvn.exceptions.BadDataInputRuntimeException;

/**
 * UNIQUE WAY TO CREATE TIMES STMAPS - ACCORINDG TO tHE CORRECT TIME ZONE
 * @since 20160618
 * @author Frederic
 *
 */
public class TimeStampProducer {

	
	/**
	 * Provide the time for your Timestamp object in format yyyy-MM-dd hh:mm:ss.SSS or yyyy-MM-dd hh:mm:ss  
	 * 
	 * @param timeForStamp
	 * @return Timestamp obj
	 */
	public static Timestamp createTimestampWithTime(String timeForStamp) {
		if(timeForStamp.length() == "yyyy-MM-dd hh:mm:ss".length()) {
			timeForStamp = timeForStamp + ".000";
		}
		
		Timestamp timestamp = null;
		try {
		    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss.SSS");
		    Date parsedDate = dateFormat.parse(timeForStamp);
		    timestamp = new java.sql.Timestamp(parsedDate.getTime());
		} catch(Exception e) { //this generic but you can control another types of exception
		    // look the origin of excption
			throw new BadDataInputRuntimeException("ERROR: TIMESTAMP PROVIDED AS STRING ["+timeForStamp+"] IS NOT SUPPORTED - It MUST BE IN THE FORMAT yyyy-MM-dd hh:mm:ss.SSS or yyyy-MM-dd hh:mm:ss");
		}
		return timestamp;
		
	}
	

	
	/**
	 * Provide the time for your Date string
	 * 
	 * @param timeForStamp
	 * @param format like yyyy.MM.dd or yyyy-MM-dd hh:mm:ss.SSS
	 * @return Timestamp obj
	 */
	public static Timestamp createTimestampFromDate(String timeForDate, String format) {
	   Timestamp retVal = null;
		try {
		      DateFormat formatter = new SimpleDateFormat(format);
		       // you can change format of date
		      Date date = formatter.parse(timeForDate);
		      retVal = new Timestamp(date.getTime());

		} catch (ParseException e) {
		      System.out.println("Exception :" + e);

		    }
		return retVal;
	}	
	
	/**
	 * USE THE UTC TIME AS WE WANt TO HAVE UTC IN THE DB - AND NOT THE BASESTATION LOCAL TIMEZONE
	 * @return String with format "yyyy-MM-dd HH:mm:ss" for UTC time
	 */
	public static String createTimestampWithTimeZoneUTC(){
		return createTimestampWithTimeZone("UTC");
	}


	
	/**
	 * CREATES UTC TIME STAMP AS WE WANt TO HAVE UTC IN THE DB - AND NOT THE BASESTATION LOCAL TIMEZONE
	 * FOR DRI & FILE NAMES 
	 * @return String with format "yyyy-MM-dd HH:mm:ss" for UTC time
	 */
	public static String createTimestampWithTimeZoneUTCForFileName(){
		String preRes = createTimestampWithTimeZone("UTC");
		String res = preRes.replace(" ", "_").replace(":", "").replace(".", "").replace("-", "");
		return res;
	}


	/**
	 * CREATES UTC TIME STAMP WITH TIME timeZone - AND RETurN FILENAME CONFORM STRING
	 * FOR DRI & FILE NAMES 
	 * @return String with format "yyyy-MM-dd HH:mm:ss" for UTC time
	 */
	public static String createTimestampWithTimeZoneForFileName(String timeZone){
		String preRes = createTimestampWithTimeZone(timeZone);
		String res = preRes.replace(" ", "_").replace(":", "").replace(".", "").replace("-", "");
		return res;
	}
	
	
	
	
	/**
	 * USE THE UTC TIME AS WE WANt TO HAVE UTC IN THE DB - AND NOT THE BASESTATION LOCAL TIMEZONE
	 * @param String timezone code like UTC
	 * @return String
	 */
	public static String createTimestampWithTimeZone(String timeZone){
		if(timeZone == null || timeZone.isEmpty()){
			throw new BadDataInputRuntimeException("ERROR: THE TIMEZONE CODE IS NULL OR EMPTY - WE EXPECT SOMETHING LIKE UTC");
		}
		TimeZone utc = TimeZone.getTimeZone(timeZone);
		Calendar calendar = Calendar.getInstance(utc);
		Timestamp tsnow = new Timestamp(calendar.getTimeInMillis());
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		sdf.setTimeZone(TimeZone.getTimeZone(timeZone));
		String retval = sdf.format(tsnow);

		return retval;
	}

	/**
	 * KAES THE LONG AS MILLISECS ANd FROMATS IT TO yyyy-MM-dd HH:mm:ss
	 * @param long ms
	 * @return String
	 */
	public static String createTimestampFromLong(long ms){
		Timestamp tsnow = new Timestamp(ms);
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		String retval = sdf.format(tsnow);

		return retval;
	}
	
	
	
	
	/**
	 * USE THE UTC TIME AS WE WANt TO HAVE UTC IN THE DB - AND NOT THE BASESTATION LOCAL TIMEZONE
	 * @param String timezone code like UTC
	 * @param String resulting timestamo format  like yyyy-MM-dd HH:mm:ss
	 * @return String
	 */
	public static String createTimestampWithTimeZone(String timeZone, String tsformat){
		if(timeZone == null || timeZone.isEmpty()){
			throw new BadDataInputRuntimeException("ERROR: THE TIMEZONE CODE IS NULL OR EMPTY - WE EXPECT SOMETHING LIKE UTC");
		}
		if(tsformat == null || tsformat.isEmpty()){
			throw new BadDataInputRuntimeException("ERROR: THE timestamp format CODE IS NULL OR EMPTY - WE EXPECT SOMETHING LIKE yyyy-MM-dd HH:mm:ss");
		}
		TimeZone utc = TimeZone.getTimeZone(timeZone);
		Calendar calendar = Calendar.getInstance(utc);
		Timestamp tsnow = new Timestamp(calendar.getTimeInMillis());
		SimpleDateFormat sdf = new SimpleDateFormat(tsformat);
		sdf.setTimeZone(TimeZone.getTimeZone(timeZone));
		String retval = sdf.format(tsnow);
		return retval;
	}
	
	
	
	/**
	 * USE THE UTC TIME AS WE WANt TO HAVE UTC IN THE DB - AND NOT THE BASESTATION LOCAL TIMEZONE
	 * @param String timezone code like UTC
	 * @return Timestamp
	 */
	public static Timestamp createTimestampForTimeZone(String timeZone){
		if(timeZone == null || timeZone.isEmpty()){
			throw new BadDataInputRuntimeException("ERROR: THE TIMEZONE CODE IS NULL OR EMPTY - WE EXPECT SOMETHING LIKE UTC");
		}
		String  nowStr = TimeStampProducer.createTimestampWithTimeZone(timeZone,"yyyy-MM-dd HH:mm:ss.SSS");
		Timestamp now = null;
		try{
		    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		    Date parsedDate = dateFormat.parse(nowStr);
		    now = new java.sql.Timestamp(parsedDate.getTime());
		}catch(Exception e){
			throw new BadDataInputRuntimeException("ERROR: CANT PARSE TIMESTAMP ["+nowStr+"]", e);
		}
		return now;
	}


	/**
	 * @param timeZone String such as "UTC", "Europe/Zurich"
	 * @param timestampstrformat String such as  "dd-M-yyyy HH:mm:ss.sss" 
	 * @param now java.sql.Timestamp
	 * @return
	 */
	public static String createTimestampStringForTimeZone(String timeZone, String timestampstrformat, Timestamp now){
		if(timeZone == null || timeZone.isEmpty()){
			throw new BadDataInputRuntimeException("ERROR: THE TIMEZONE CODE IS NULL OR EMPTY - WE EXPECT SOMETHING LIKE UTC");
		}
		if(timestampstrformat == null || timestampstrformat.isEmpty()){
			throw new BadDataInputRuntimeException("ERROR: THE timestamp format CODE IS NULL OR EMPTY - WE EXPECT SOMETHING LIKE yyyy-MM-dd HH:mm:ss");
		}
		SimpleDateFormat timeFormat = new SimpleDateFormat(timestampstrformat);
		timeFormat.setTimeZone(TimeZone.getTimeZone(timeZone));
		return timeFormat.format(now);
	}
	
	
	/**
	 * @param now java.sql.Timestamp
	 * @param timestampstrformat String such as  "dd-MM-yyyy HH:mm:ss.sss" 
	 * @return String
	 */
	public static String timeStampAsString(Timestamp now, String timestampstrformat) {
		SimpleDateFormat timeFormat = new SimpleDateFormat(timestampstrformat);
		return timeFormat.format(now);

	}

	/** STRING DOES NOT CONTAIN : NOR SPACE
	 * @param now java.sql.Timestamp
	 * @param timestampstrformat String such as  "dd-MM-yyyy HH:mm:ss.sss" 
	 * @return String
	 */
	public static String timeStampAsStringForFileName(Timestamp now, String timestampstrformat) {
		String t = timeStampAsString(now, timestampstrformat);
		String retVal = t.replace(" ", "_").replace(":", "").replace(".", "_").trim();
		return retVal;
		
	}

	/** STRING DOES NOT CONTAIN : NOR SPACE
	 * @param String timestampAsString  like "dd-MM-yyyy HH:mm:ss.sss"
	 * @return String
	 */
	public static String timeStampAsStringForFileName(String timestampAsString) {
		String retVal = timestampAsString.replace("-", "").replace(" ", "_").replace(":", "").replace(".", "_").trim();
		return retVal;
		
	}
	
	
	/**
	 * FORMATS THE TIMESTAMP STRING - SO THAT IS CAN BE usED FOR 
	 * Timestamp.valueOf(xxxx)
	 * @param ts
	 * @return
	 */
	public static synchronized String checkAndModifyTimeStamp(String ts){
		String retVal = ts;
		if(ts.contains(".")){
			// YES - WE HAVE ms - GET RID OF THEM
			if(ts.charAt(ts.length()-4) == '.' && ts.length() == 23){
				retVal = ts.substring(0, ts.length()-4);
			} 
			else if(ts.charAt(ts.length()-3) == '.' && ts.length() == 22){
				retVal = ts.substring(0, ts.length()-3);
			} 
			else if(ts.charAt(ts.length()-2) == '.' && ts.length() == 21){
				retVal = ts.substring(0, ts.length()-2);
			} 

			
			else{
				// UNEXPECTED timestamp format THROW EXPECTION 
				throw new RuntimeException("ERROR: TIMESTAMP ["+ts+"] EXPECTED TO BE IN FORMAT [2014-10-31 08:29:25.814] BUT IS NOT");
			}
			
		} else {
			// NO - WE DO NOT HAVE ms
			retVal = ts;
		}
		
		return retVal;
	}
	
	/**
	 * ASSUMES INPUT timestamp in form yyyy-MM-dd hh:mm:ss.SSS is UTC AND
	 * TRNASFORMS THIS UTC TIME TO SYSTEM (WALL CLOCK) TIME 
	 * @param timestamp
	 * @return
	 */
	public static String transfromUTCTimeStampToWallClockTimestamp(String timestamp){
		if(timestamp == null || timestamp.isEmpty()){
			throw new BadDataInputRuntimeException("ERROR: THE timestamp IS NULL OR EMPTY - WE EXPECT A TIMESTAMP IM FORMAT yyyy-MM-dd hh:mm:ss.SSS");
		}

		
		String retval = "";
		SimpleDateFormat sdf = null;
		if(timestamp.length() == "yyyy-MM-dd hh:mm:ss.SSS".length()){
			sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss.SSS");
		} 
		else if(timestamp.length() == "yyyy-MM-dd hh:mm:ss".length()){
			sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
		} 
		else {
			timestamp = checkAndModifyTimeStamp(timestamp);
		}
		try {
			sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
			Date parsedDate = sdf.parse(timestamp);
			Timestamp timestampObj = new java.sql.Timestamp(parsedDate.getTime());
			retval = timestampObj.toString();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return retval;
	}
	
	
	/**
	 * ASSUMES INPUT timestamp in form yyyy-MM-dd hh:mm:ss.SSS is UTC AND
	 * TRNASFORMS THIS UTC TIME TO SYSTEM (WALL CLOCK) TIME 
	 * @param timestamp
	 * @return
	 */
	public static String transfromUTCTimeStampToBaseStationTimestamp(String timestamp){
		if(timestamp == null || timestamp.isEmpty()){
			throw new BadDataInputRuntimeException("ERROR: THE timestamp IS NULL OR EMPTY - WE EXPECT A TIMESTAMP IM FORMAT yyyy-MM-dd hh:mm:ss.SSS");
		}

		
		String retval = "";
		SimpleDateFormat sdf = null;
		if(timestamp.length() == "yyyy-MM-dd hh:mm:ss.SSS".length()){
			sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss.SSS");
		} 
		else if(timestamp.length() == "yyyy-MM-dd hh:mm:ss".length()){
			sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
		} 
		else {
			timestamp = checkAndModifyTimeStamp(timestamp);
			sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
		}
		try {
			sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
			Date parsedDate = sdf.parse(timestamp);
			
			GregorianCalendar greg = new GregorianCalendar();
			greg.setTimeZone(TimeZone.getTimeZone("UTC"));
			greg.setTimeInMillis(parsedDate.getTime());
			
			greg.setTimeZone(TimeZone.getTimeZone("Asia/Tokyo"));
			
			
			Timestamp timestampObj = new java.sql.Timestamp(greg.getTimeInMillis());
			retval = timestampObj.toString();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return retval;
	}
}
