package com.moneyhouse.sensors.filetrigger;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.sql.Timestamp;
import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;

import com.moneyhouse.sensors.xbee.dto.XBeeComandObject;
import com.moneyhouse.sensors.xbee.dto.XBeeDataTransportObject;

public class FileMonitorToBrickSingleton implements Serializable{
	private static final long serialVersionUID = -5333838836482531831L;
	private static FileMonitorToBrickSingleton instance = null;
	private static Logger logger = Logger.getLogger(FileMonitorToBrickSingleton.class);	
	private int filecounter = 0;
	
	
	protected FileMonitorToBrickSingleton(){}
	
	public static FileMonitorToBrickSingleton getInstance() {
		logger.debug("----> getInstance()");
		if(instance == null) {
			instance = new FileMonitorToBrickSingleton();
		}
		logger.debug("<---- getInstance() instance = "+instance);
		return instance;
	}	
	
	

	

	
	
	
   

	public synchronized XBeeDataTransportObject readFileAndReturnDataObject(String filename) {
		logger.info("----> readFileAndReturnDataObject("+filename+")");
		BufferedReader br = null;
		XBeeDataTransportObject retVal = null; 
		try {
 
			String sCurrentLine;
 
			br = new BufferedReader(new FileReader(filename));
 
			while ((sCurrentLine = br.readLine()) != null) {
				retVal = processLine(sCurrentLine);
			}
 
		} catch (IOException e) {
			e.printStackTrace(); logger.error("ERROR OCCURED: ",e);
		} finally {
			try {
				if (br != null)br.close();
			} catch (IOException ex) {
				ex.printStackTrace(); logger.error("ERROR OCCURED: ", ex);
			}
		}
		logger.info("<---- readFileAndReturnDataObject("+filename+")");
		
		return retVal;
	}
	
	public synchronized XBeeComandObject readCommandFileAndReturnDataObject(String filename) {
		logger.info("----> readCommandFileAndReturnDataObject("+filename+")");
		BufferedReader br = null;
		XBeeComandObject retVal = null; 
		try {
 
			String sCurrentLine;
 
			br = new BufferedReader(new FileReader(filename));
 
			while ((sCurrentLine = br.readLine()) != null) {
				retVal = processCommandLine(sCurrentLine);
			}
 
		} catch (IOException e) {
			e.printStackTrace(); logger.error("ERROR OCCURED: ",e);
		} finally {
			try {
				if (br != null)br.close();
			} catch (IOException ex) {
				ex.printStackTrace(); logger.error("ERROR OCCURED: ", ex);
			}
		}
		logger.info("<---- readCommandFileAndReturnDataObject("+filename+")");
		
		return retVal;
	}	
	/**
	 * Read file content of data request file)
	 * 
	 * @param currentLine
	 * @return
	 */
	private static XBeeDataTransportObject processLine(String currentLine) {
		logger.info("----> processLine("+currentLine+")");
		List<String> items = Arrays.asList(currentLine.split("\\s*\t\\s*"));
		XBeeDataTransportObject retVal = null;
		if (items.size() == 7) {
			// data request file
			retVal = new XBeeDataTransportObject(items.get(1).trim(), 
												items.get(3).trim(), 
												items.get(4).trim(), 
												items.get(5).trim());
			@SuppressWarnings("unused")
			String entrymode = items.get(6).trim();
			// input type = 0 brickdata ; 1 manual; -1 simulated

			for (int i = 0; i < items.size(); i++) {
				String token = items.get(i);
				System.out.println(token);
			}
		}
		else if (items.size() == 5) {
			// 31.10.2014 - NOW IT WORKS WITH 5 ELEMENTS - LIKE sendPING
			retVal = new XBeeDataTransportObject(items.get(0).trim(), 
												items.get(1).trim(), 
												items.get(3).trim(), 
												items.get(4).trim());

		}

		
		else 
		{
			throw new RuntimeException("ERROR: DATA REQUEST FILE IS SUPPOSED TO HAVE 7 ELEMENTS - BUT THIS ONE INLY HAS ["+items.size()+"] - CHECK CONTENT: "+currentLine);
		}
		logger.info("<---- processLine("+currentLine+")");
		return retVal;
	}

	/**
	 * Read file content of data request file)
	 * 
	 * @param currentLine
	 * @return
	 */
	private static XBeeComandObject processCommandLine(String currentLine) {
		logger.info("----> processCommandLine("+currentLine+")");
		List<String> items = Arrays.asList(currentLine.split("\\s*\t\\s*"));
		logger.debug("     items count= " + items.size());

		XBeeComandObject retVal = null;
		if (items.size() == 4) {
			// data request file
			retVal = new XBeeComandObject();
			retVal.setBrickid(items.get(0).trim());
			retVal.setAtcommand(items.get(1).trim());
			retVal.setAtcommandparameter(items.get(2).trim());
			retVal.setCorrelID(items.get(3).trim());
		}
		else if (items.size() == 5) {
			// data request file
			retVal = new XBeeComandObject();
			retVal.setBrickid(items.get(0).trim());
			retVal.setAtcommand(items.get(1).trim());
			retVal.setAtcommandparameter(items.get(2).trim());
			retVal.setCorrelID(items.get(3).trim());
			String timestampstr = items.get(4).trim();
			Timestamp timestp = Timestamp.valueOf(timestampstr);
			retVal.setTimestamp(timestp);
		}
		else 
		{
			throw new RuntimeException("ERROR: COMMAND FILE IS SUPPOSED TO HAVE 4 ELEMENTS - BUT THIS ONE INLY HAS ["+items.size()+"] - CHECK CONTENT: "+currentLine);
		}
		logger.info("<---- processCommandLine("+currentLine+")");
		
		return retVal;
	}	
	
	public boolean isCommandFile(String filename) {
		logger.debug("----> isCommandFile("+filename+")");
		boolean retval = false;	
		BufferedReader br = null;
		
		try {
			 
			String sCurrentLine;
 
			br = new BufferedReader(new FileReader(filename));
 
			while ((sCurrentLine = br.readLine()) != null) {
				List<String> items = Arrays.asList(sCurrentLine.split("\\s*\t\\s*"));
				if (items.size() == 4 || items.size() == 5 ) {
					retval = true;
				}		
			}
 
		} catch (IOException e) {
			e.printStackTrace(); logger.error("ERROR OCCURED: ",e);
		} finally {
			try {
				if (br != null)br.close();
			} catch (IOException ex) {
				ex.printStackTrace(); logger.error("ERROR OCCURED: ", ex);
			}
		}
		logger.debug("<---- isCommandFile("+filename+") RETVAL= "+retval);
		
		return retval;
	}
	

	
	
	/**
	 * @see #isCommandFile(String)
	 * THIS ONE CHECKS WHETHER tHE typeofFile parameter can be found in the file
	 * @param filename
	 * @param typeofFile
	 * @return
	 */
	public boolean isFile(String filename, String typeofFile) {
		logger.debug("----> isFile("+filename+")");
		boolean retval = false;	
		BufferedReader br = null;
		
		try {
			String sCurrentLine;
 
			br = new BufferedReader(new FileReader(filename));
 
			while ((sCurrentLine = br.readLine()) != null) {
				List<String> items = Arrays.asList(sCurrentLine.split("\\s*\t\\s*"));
				for(int g= 0 ; g < items.size();g++){
					String ele =  items.get(g);
					if (typeofFile.equalsIgnoreCase(ele)) {
						retval = true;
						break;
					}		
				}
			}
 
		} catch (IOException e) {
			e.printStackTrace(); logger.error("ERROR OCCURED: ",e);
		} finally {
			try {
				if (br != null)br.close();
			} catch (IOException ex) {
				ex.printStackTrace(); logger.error("ERROR OCCURED: ", ex);
			}
		}
		logger.debug("<---- isFile("+filename+") RETVAL= "+ retval);
		
		return retval;
	}
	
	
	
	public boolean isFileExistant(String filename) {
		logger.info("----> isFileExistant(" + filename + ")");
		boolean retval = false;
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(filename));
			retval = true;
		} catch (java.io.FileNotFoundException fnfe) {
			retval = false;
		} catch (Exception e) {
			logger.error("SOME OTHER ERROR OCCURED WHILE TESTING FOR EXISTANCE OF FILE ", 	e);
		} finally {
			if(retval == true){
				try {
					if (br != null)
						br.close();
				} catch (IOException ex) {
					logger.error("ERROR OCCURED: ", ex);
				}
			}
		}
		logger.info("<---- isFileExistant(" + generateShortFileName(filename) + ") retval="+retval);
		return retval;
	}
	
	
	/** TO GET A NICER LOOGER OUTPUT - FILENAMES ARE REALLy LONG
	 * @param longfilename
	 * @return
	 */
	private String generateShortFileName(String longfilename){
		String retval = "";
		String cleandup = longfilename.replace(",0x", "");
		if(cleandup.contains("__CORR_")){
			int pos = cleandup.indexOf("__CORR_");
			retval = cleandup.substring(pos-4, pos + 16).trim();
		} else {
			// no correlation ID retrun the long name 
			retval = longfilename;
		}
		
		return retval;
	}
	
	
	public int decrementFileCounter(){
		filecounter--;
		return filecounter;
	}

	public int incrementFileCounter(){
		filecounter++;
		return filecounter;
	}

	public int getFilecounter() {
		return filecounter;
	}

	public void setFilecounter(int filecounter) {
		this.filecounter = filecounter;
	}
	
}
	

