package com.moneyhouse.sensors.json;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;

import org.apache.log4j.Logger;

import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.config.ServerEnvironmentSingleton;
import com.moneyhouse.sensors.xbee.dto.XBeeDataTransportObject;

import flexjson.JSONSerializer;
import flexjson.transformer.DateTransformer;

public class CreateJsonFile {
	private static Logger logger = Logger.getLogger(CreateJsonFile.class);	

	public CreateJsonFile(){}
	
	/**
	 * Entry Point when new data is available
	 * 1. Collected data XBee serial interface
	 * 2. Transform data to json format
	 * 3. Save json format file
	 * @param XBeeDataTransportObject
	 * @param jsonfilename
	 */
	public void createXBeeResponseFile(XBeeDataTransportObject bdo, String jsonfilename) {
		logger.debug("---> CreateJsonFile.createXBeeResponseFile()");
		// 1. Get Data with Time window 

		// 2. transfrom
		ArrayList<JSONAbstract> jsonformwewant = TransformDataToJasonFileFormat.transform(bdo);
		
		// 3. prep file + write
		BufferedWriter  bw = prepareFile(jsonfilename);
		
		JSONSerializer serializer = new JSONSerializer().transform(new DateTransformer("MM-dd-yyyy"), Date.class); 
		String val =  serializer.include("data").include("lines").exclude("*.class").serialize( jsonformwewant );
		logger.debug(val);
		logger.debug(val);
		
		try {
			bw.write(val);
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger.debug("<--- CreateJsonFile.createXBeeResponseFile()");
	}
	
	
	
	/**
	 * Opens the filestream - with docroot and file name
	 * @param filename
	 * @return
	 */
	static private BufferedWriter  prepareFile(String filename){
		BufferedWriter bw = null;
		
		try {
	    	
			String docroot = ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION;
			logger.debug("prepareFile("+filename+") to: "+ docroot+filename);
			
	    	File file = new File(docroot+filename);
	
			// if file doesnt exists, then create it
			if (!file.exists()) {
					file.createNewFile();
				
			}
	
			FileWriter fw = new FileWriter(file.getAbsoluteFile());
			bw = new BufferedWriter(fw);
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		return bw;
	}	
}
