package com.moneyhouse.sensors.xbee;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import com.rapplogic.xbee.api.ApiId;
import com.rapplogic.xbee.api.AtCommand;
import com.rapplogic.xbee.api.AtCommandResponse;
import com.rapplogic.xbee.api.PacketListener;
import com.rapplogic.xbee.api.RemoteAtRequest;
import com.rapplogic.xbee.api.RemoteAtResponse;
import com.rapplogic.xbee.api.XBee;
import com.rapplogic.xbee.api.XBeeAddress16;
import com.rapplogic.xbee.api.XBeeAddress64;
import com.rapplogic.xbee.api.XBeeException;
import com.rapplogic.xbee.api.XBeeRequest;
import com.rapplogic.xbee.api.XBeeResponse;
import com.rapplogic.xbee.api.XBeeTimeoutException;
import com.rapplogic.xbee.api.zigbee.NodeDiscover;
import com.rapplogic.xbee.api.zigbee.ZBForceSampleRequest;
import com.rapplogic.xbee.api.zigbee.ZNetExplicitTxRequest;
import com.rapplogic.xbee.api.zigbee.ZNetRxIoSampleResponse;
import com.rapplogic.xbee.api.zigbee.ZNetTxRequest;
import com.rapplogic.xbee.util.ByteUtils;
import com.rapplogic.xbee.util.DoubleByte;

public class DiscoverAndSendExplicit {
	private final static Logger log = Logger.getLogger(DiscoverAndSendExplicit.class);
	private XBee xbee = new XBee();
	private ArrayList<NodeDiscover> myNodes = new ArrayList<NodeDiscover>();

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		PropertyConfigurator.configure("log4j.properties");
		DiscoverAndSendExplicit d = new DiscoverAndSendExplicit() ;
		try 
		{
			d.findNodes();
			log.info("NUMBER OF NODE FOUND: "+ d.myNodes.size());
			Iterator<NodeDiscover> doce_it = d.myNodes.iterator();
			while(doce_it.hasNext()){
				NodeDiscover nod = doce_it.next();
				// try to contact explicit
				d.sendExcpicit(nod);
				
				// toggle led
				d.blinkPortDIO0_PIN20(nod.getNodeAddress64());
				
				// read analog port 
				d.readPortAD1_PIN19(nod.getNodeAddress64());
			}

		} 
		catch (XBeeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	
	
	
	
	
	
	
	
	private void sendExcpicit(NodeDiscover node){
		try {
			// replace with your com port and baud rate. this is the com port of my coordinator
			Thread.sleep(2000);
			xbee.open("COM8", 9600);
			// xbee.open("/dev/tty.usbserial-A6005v5M", 9600);
			
			// replace with end device's 64-bit address (SH + SL)
//			XBeeAddress64 addr64 = new XBeeAddress64(0, 0x13, 0xa2, 0, 0x40, 0x0a, 0x3e, 0x02);
			
			
//			XBeeAddress64 addr64 = new XBeeAddress64(0, 0x13, 0xa2, 0, 0x40, 0x0a, 0x42, 0xc5);	// WITH NO ANTENNA 
//			XBeeAddress64 addr64 = new XBeeAddress64(0, 0x13, 0xa2, 0, 0x40, 0x3E, 0x07, 0x98);	// WITH BIG ANTENNA 

			XBeeAddress64 addr64 = node.getNodeAddress64();
			
			
			// create an array of arbitrary data to send
			int[] payload = new int[] { 0, 0x66, 0xee };
			
			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA.getValue();
			
			DoubleByte clusterId = new DoubleByte(0x0, ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			//DoubleByte clusterId = new DoubleByte(0x0, ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());
			
			// first request we just send 64-bit address.  we get 16-bit network address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff, addr64, XBeeAddress16.ZNET_BROADCAST, 
						ZNetTxRequest.DEFAULT_BROADCAST_RADIUS, ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint, destinationEndpoint, clusterId, ZNetExplicitTxRequest.znetProfileId);
			
			log.info("sending explicit " + request.toString());
			
			for (int i = 0 ; i < 2 ; i++) {
				xbee.sendAsynchronous(request);
				
				XBeeResponse response = xbee.getResponse();
				
				log.info(i + "\treceived response " + response.toString());
					
				try {
					// wait a bit then send another packet
					Thread.sleep(2000);
				} catch(InterruptedException  ex){
					ex.printStackTrace();
				}
			}
		} catch(InterruptedException  ex){
			ex.printStackTrace();
		}catch (XBeeException e) {
			e.printStackTrace();
		} finally {
			xbee.close();
		}		
		
		
	}
	
	private void blinkPortDIO0_PIN20(XBeeAddress64 address) {
		log.info("-----> blinkPort20(" + address + ")");
		try {
			Thread.sleep(2000);
			xbee.open("COM8", 9600);
			for (int i = 0; i < 10; i++) {
				// pin 20 corresponds to D0, and 5 activates the output (Digital
				// output high)
				RemoteAtRequest request = new RemoteAtRequest(address, "D0",  new int[] { 5 });

				// turn on LED
				log.info(i + "\t\tturn on LED");

				RemoteAtResponse response = (RemoteAtResponse) xbee.sendSynchronous(request, 10000);

				if (response.isOk()) {
					// success
				}

				// pause for 4 seconds
				log.info(i + "\t\tpause 1 seconds");
				Thread.sleep(1000);

				// command to turn off pin 20
				log.info(i + "\t\tturn off LED");
				request = new RemoteAtRequest(address, "D0", new int[] { 4 });

				// send the command
				response = (RemoteAtResponse) xbee.sendSynchronous(request,
						10000);

				if (response.isOk()) {
					// success
					log.info(i + "\t\tLED is off");
				}
				log.info(i + "\t\tpause 1 seconds");
				Thread.sleep(1000);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		} finally {
			xbee.close();
		}

		log.info("<----- blinkPort20(" + address + ")");

	}	
	
	private void readPortAD1_PIN19(XBeeAddress64 address) {
		log.info("-----> readPortAD1_PIN19(" + address + ")");
		try {
			Thread.sleep(2000);
			xbee.open("COM8", 9600);
			for (int i = 0; i < 80; i++) {
				// pin 20 corresponds to D0, and 5 activates the output (Digital
				// output high)

				
				log.debug(i + "\t---- Configure Port AD1_PIN19 ----");
				RemoteAtRequest request = new RemoteAtRequest(address, "D1",  new int[] { 2 });
				RemoteAtResponse response = (RemoteAtResponse) xbee.sendSynchronous(request, 10000);
				if (response.isOk()) {
					// success
					log.debug(i + "\t--- CALL SAMPLE ---");
					XBeeRequest samplerequest = new ZBForceSampleRequest(address);

					readDataFromPin(samplerequest);					
					
					
					
				} else if(response.isError()){
					log.info(i + "\t\tResponse is ERROR");
					log.info(i + "\t\tResponse = "+ response);
					
				} else {
					log.info(i + "\t\tResponse NOT NOT NOT ok");
					log.info(i + "\t\tResponse = "+ response);
					
				}

				// pause for 4 seconds
				log.debug(i + "\t\tpause 1 second");
				Thread.sleep(500);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		} finally {
			xbee.close();
		}

		log.info("<----- readPortAD1_PIN19(" + address + ")");

	}








	private void readDataFromPin(XBeeRequest sample)
			throws XBeeException, IOException {
		try {
			XBeeResponse sampleresponse = xbee.sendSynchronous(sample, 6000);
			RemoteAtResponse remoteAt = (RemoteAtResponse) sampleresponse;
			
			if (remoteAt.isOk())  {
				// extract the i/o sample
				ZNetRxIoSampleResponse ioSample = ZNetRxIoSampleResponse.parseIsSample(remoteAt);
				// make sure you configured the remote XBee to D1=2 (analog input) or you will get an error
				log.info("10 bit analog1 sample is " + ioSample.getAnalog1());
			} else {
				log.info("Remote AT request failed: " + sampleresponse);
			}		
		} catch (XBeeTimeoutException e) {
			log.info("request timed out");	
		}
	}		
	
	private void findNodes() throws XBeeException, InterruptedException {

		try {
			// replace with your serial port
			// xbee.open("/dev/tty.usbserial-A6005v5M", 9600);
			xbee.open("COM8", 9600);

			// get the Node discovery timeout
			xbee.sendAsynchronous(new AtCommand("NT"));
			AtCommandResponse nodeTimeout = (AtCommandResponse) xbee
					.getResponse();

			// default is 6 seconds
			long nodeDiscoveryTimeout = ByteUtils
					.convertMultiByteToInt(nodeTimeout.getValue()) * 100;

			log.info("Node discovery timeout is " + nodeDiscoveryTimeout + " milliseconds");

			xbee.addPacketListener(new PacketListener() {

				public void processResponse(XBeeResponse response) {
					if (response.getApiId() == ApiId.AT_RESPONSE) 
					{
						NodeDiscover nd = NodeDiscover
								.parse((AtCommandResponse) response);
						log.info("Node discover response is: " + nd);
						myNodes.add(nd);
					} 
					else 
					{
						log.info("Ignoring unexpected response: " + response);
					}
				}

			});

			log.info("Sending node discover command");
			xbee.sendAsynchronous(new AtCommand("ND"));

			// wait for nodeDiscoveryTimeout milliseconds
			Thread.sleep(nodeDiscoveryTimeout);

			log.info("Time is up!  You should have heard back from all nodes by now.  If not make sure all nodes are associated and/or try increasing the node timeout (NT)");
		} finally {
			xbee.close();
		}
	}

}
