package com.moneyhouse.sensors.xbee;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.sql.Timestamp;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;

import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.filetrigger.FileMonitorFromBrickSingleton;
import com.moneyhouse.sensors.filetrigger.FileMonitorToBrickSingleton;
import com.moneyhouse.sensors.internal.idgenerator.IDGenerator;
import com.moneyhouse.sensors.json.CreateJsonFile;
import com.moneyhouse.sensors.util.TimeStampProducer;
import com.moneyhouse.sensors.xbee.dto.XBeeDataTransportObject;
import com.rapplogic.xbee.api.XBeeAddress64;
import com.rapplogic.xbee.api.XBeeResponse;
import com.rapplogic.xbee.api.zigbee.ZNetRxResponse;
import com.rapplogic.xbee.util.ByteUtils;

public class SerialReceiver6ComposeAnswer {
	private final static Logger logger = Logger.getLogger(SerialReceiver6ComposeAnswer.class);
	private final static Logger loggerfile = Logger.getLogger("requestLogger");
	
	
	
	
	/**
	 * Data Received from XBee Radio are processed here and written to File 
	 * @param answer --- serial input from the xbee radio
	 * @param list of files to brick - to get the match between send and receive correlId
	 */
	public synchronized static XBeeDataTransportObject prepareFileFROM_BRICK(XBeeResponse answer) {
		logger.info("----> prepareFileFROM_BRICK(XBeeResponse answer)");
		XBeeDataTransportObject retVal = null;
		ZNetRxResponse answerRxRes = (ZNetRxResponse)answer; 
		// its a return from the user
		// - process it
		FileMonitorFromBrickSingleton fm = FileMonitorFromBrickSingleton.getInstance();
		String filename = "FROM_BRICK__";			// FILE PREFIX
		String brickidfrom = "";					// 64 BIT ADDRESS OF RADIO MODULE
		String command = "";						// COMMAND RECEIVED PING, 
		String parameter = "";
		String correlId = "";						// CORRELATION ID RECEIVED FROM tHE BRICK
		Timestamp tsnow = currentTimestamp();		// THE TIMe NOW OF THE PROCESSING
		String execTime = "";						// TIME IN MS FOR THE EXECUTION OF A COMMAND
		
		String value = "";			// SENDING PORT OF THE EVENT
		String eventValue = "";						// SENDING VALUE OF THE EVENT
		
		XBeeAddress64 address = answerRxRes.getRemoteAddress64();
		brickidfrom = address.toString();
		
		String commandAndCorrelIDAndVAL = ByteUtils.toString(answerRxRes.getData());		// Brick returns COMMAND[CORRELID] [VAL]
		logger.debug("RECEIVED DATA: " + commandAndCorrelIDAndVAL);
		
		
		loggerfile.info("RECEIVED DATA: " + commandAndCorrelIDAndVAL);
		
		StringTokenizer strTokens = new StringTokenizer(commandAndCorrelIDAndVAL, " ");
		
		int tokencounter = 0;
		while (strTokens.hasMoreElements()) {
			tokencounter++;
			String tok = strTokens.nextToken();
			if(tokencounter == 1){
				// get the command + correlId out
				StringTokenizer cmdTokens = new StringTokenizer(tok, "[");
				command = cmdTokens.nextToken();
				if(command != null && command.equals("EVENT")){
					// ITS AN EVENT
					String correlIdPre = cmdTokens.nextToken();
					if(correlIdPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						correlId = correlIdPre.substring(0, (correlIdPre.length()-1));
					} else {
						// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
					String eventPortNameNumberPre = cmdTokens.nextToken();
					if(eventPortNameNumberPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						value = eventPortNameNumberPre.substring(0, (eventPortNameNumberPre.length()-1));
					} else {
						// THE PORT IDENTIFIER TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
					String eventValuePre = cmdTokens.nextToken();
					if(eventValuePre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						eventValue = eventValuePre.substring(0, (eventValuePre.length()-1));
					} else {
						// THE PORT IDENTIFIER TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
				}
				
				else if(command != null && command.equals("RDOW")){
					// ITS AN READ ONE WIRE RETURN ANSWER
					String correlIdPre = cmdTokens.nextToken();
					if(correlIdPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						correlId = correlIdPre.substring(0, (correlIdPre.length()-1));
					} else {
						// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
					String valuePre = cmdTokens.nextToken();
					if(valuePre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						value = valuePre.substring(0, (valuePre.length()-1)).trim();
					} else {
						// THE PORT IDENTIFIER TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
				}
				else if(command != null && command.equals("RDOP")){
					// ITS AN READ ONE WIRE RETURN ANSWER
					String correlIdPre = cmdTokens.nextToken();
					if(correlIdPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						correlId = correlIdPre.substring(0, (correlIdPre.length()-1));
					} else {
						// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
					String valuePre = cmdTokens.nextToken();
					if(valuePre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						value = valuePre.substring(0, (valuePre.length()-1)).trim();
					} else {
						// THE PORT IDENTIFIER TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
				}
				else if(command != null && command.equals("SLIST")){
					// ITS IN THIS FORMAT
					// SLIST[B6uwm27A][A][01234567][O]
					retVal = parseSLIST(retVal, commandAndCorrelIDAndVAL, cmdTokens);
					
					// just read the rest of the tokes
					while(cmdTokens.hasMoreElements()){
						cmdTokens.nextToken();
					}
				}
				else if(command != null && command.equals("PLIST")){
					// ITS IN THIS FORMAT
					// SLIST[B6uwm27A][A][01234567][O]
					retVal = parsePLIST(retVal, commandAndCorrelIDAndVAL, cmdTokens);
					
					// just read the rest of the tokes
					while(cmdTokens.hasMoreElements()){
						cmdTokens.nextToken();
					}
				}
				
				//else if(command != null && command.equals("PING") || command.equals("PLIST"))
				else if(command != null && command.equals("PING"))	// 28.01.2015
				{
					// ITS A PING OR A PLIST, SLIST
					String correlIdPre = cmdTokens.nextToken();
					if(correlIdPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						correlId = correlIdPre.substring(0, (correlIdPre.length()-1));
						
						// lets look whether we have the originating trigger file here - like for ping
						int filecounterWithCorrelid = 0;
						
						String tobrickdir = PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick");
						File folder = new File(tobrickdir);
						File[] listOfFilesToBrick2 = folder.listFiles();
						
						for(int jd = 0 ; jd < listOfFilesToBrick2.length; jd++){
							File ff =  listOfFilesToBrick2[jd];
							String fname = ff.getName();
							if((ff.isFile() && fname.contains(correlId))|| ff.getPath().contains(correlId)){
								filecounterWithCorrelid++;
							}
						}
						if(filecounterWithCorrelid == 0){
							// no trigger file found - so its a push event comming from the brick
						} else if(filecounterWithCorrelid == 1){
							// one trigger file found - perfect
							File theonefile =  null;
							for(int jd = 0 ; jd < listOfFilesToBrick2.length; jd++){
								theonefile =  listOfFilesToBrick2[jd];
								if(theonefile.isFile() && theonefile.getName().contains(correlId)){
									break;
								}
							}
							String fn = theonefile.getAbsolutePath();
							FileMonitorToBrickSingleton fileaccess = FileMonitorToBrickSingleton.getInstance();
					        XBeeDataTransportObject content =  fileaccess.readFileAndReturnDataObject(fn);
					        
					        // return value
					        retVal = content;
					        long noww = tsnow.getTime();
					        long bfore = content.getCreatedtimestamp().getTime();
	
					        long difference = noww - bfore;		// get the roundtrip time in millis
					        
					        execTime = "" + difference; 		// store it into the execTime
					        content.setExecutiontime(difference);
					        content.setCommand(command);		// the Command like ping
					        // NOW DELETE THE TOBRICK FILE
					        theonefile.delete();
					        
					        
					        // create JSON FILE FOR DISPLAY
					        CreateJsonFile json = new CreateJsonFile();
					        json.createXBeeResponseFile(content, "msg_command.json");
					        
						} else if(filecounterWithCorrelid > 1){
							// more than one trigger file found - get the newest file - ignore the others
							File[] listOfFilesToBrick3 = folder.listFiles();
							
							// index to find the newest / last file 
							Timestamp last = null; 
							File theonefile =  null;
							
							for(int jd = 0 ; jd < listOfFilesToBrick3.length; jd++){
								File ff =  listOfFilesToBrick3[jd];
								String fname = ff.getName();
								if((ff.isFile() && fname.contains(correlId))|| ff.getPath().contains(correlId)){
									filecounterWithCorrelid++;
									
									String sCurrentLine = "";
									try 
									{
										BufferedReader br = new BufferedReader(new FileReader(ff.getAbsolutePath()));
										
										List<String> items = null;
										while ((sCurrentLine = br.readLine()) != null) {
											items = Arrays.asList(sCurrentLine.split("\\s*\t\\s*"));
										}
	
										String tsStr = items.get(4).trim();
										Timestamp ts = Timestamp.valueOf(tsStr);
										
										if(last == null || last.before(ts)){
											theonefile =  ff;
											last = ts;
										}
									}
									catch(Exception ex){
										throw new RuntimeException("ERROR: PROBLEMS READING toBrick FILES WITH Correlation ID ["+correlId+"]");
									}
								}
							}
							
							String fn = theonefile.getAbsolutePath();
							FileMonitorToBrickSingleton fileaccess = FileMonitorToBrickSingleton.getInstance();
					        XBeeDataTransportObject content =  fileaccess.readFileAndReturnDataObject(fn);
					        
					        // return value
					        retVal = content;
					        long noww = tsnow.getTime();
					        long bfore = content.getCreatedtimestamp().getTime();
	
					        long difference = noww - bfore;		// get the roundtrip time in millis
					        
					        execTime = "" + difference; 		// store it into the execTime
					        content.setExecutiontime(difference);
					        content.setCommand(command);		// the Command like ping
					        // NOW DELETE THE TOBRICK FILE
					        theonefile.delete();
					        
					        
					        // create JSON FILE FOR DISPLAY
					        CreateJsonFile json = new CreateJsonFile();
					        json.createXBeeResponseFile(content, "msg_command.json");

						}
					}
					

				}
				while (cmdTokens.hasMoreElements()) {
					// HERE COMES THE VALUE TREATMENT IN
					// disabled on the 20141209
					// command = cmdTokens.nextToken();
					cmdTokens.nextToken();
				}
			} else if(tokencounter >2){
				// there shouldn't be more than 2 Token... 
				throw new RuntimeException("ERROR: CHECK THE FORMAT OF XBEE RECEIVED DATA - IT HAS THE FORMAT COMMAND[CORRELID] [VALUE] --- BUT WE HAVE RECEIVED: "+commandAndCorrelIDAndVAL);
			}
		}
		if(command.equals("EVENT")){
			retVal = parseEVENT(fm, filename, brickidfrom, command, parameter, correlId, tsnow, value, eventValue);

			
		}else if(command.equals("RDOW")){
			retVal = parseRDOW(fm, filename, brickidfrom, command, correlId, tsnow, execTime, value);

	        
			
		}else if(command.equals("RDOP")){
			String prevalue = commandAndCorrelIDAndVAL.substring(14, 26);
			if(!prevalue.startsWith("[")){
				throw new RuntimeException("ERROR: THIS PART SHOULD START WITH [");
			}
			if(!prevalue.endsWith("]")){
				throw new RuntimeException("ERROR: THIS PART SHOULD END WITH ]");
			}
			value = prevalue.replace("[", "").replace("]", "").replace(" ", "").trim();
			
			filename = filename + brickidfrom +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+".txt";
			String pathfilename = PropertiesFileReader.getInstance().getProperty("filetrigger.dir.frombrick");
			
			pathfilename = pathfilename + filename;
			logger.debug("send command by writing file into  "+pathfilename);
			
			String tsnowstr = tsnow.toString();
			tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);
			logger.debug("tsnowstr "+tsnowstr);
			String content =brickidfrom+"\t"+command+ "\t" + correlId+"\t"+value+"\t"+"\t"+tsnowstr+ "\tEXECUTIONTIME="+execTime;
			
			fm.createFileAsEvent(pathfilename, content);
			
	        XBeeDataTransportObject contentObj = new XBeeDataTransportObject(brickidfrom, "RDOP", "B", "",value, correlId,  tsnow); 
	        
	        // return value
	        retVal = contentObj;

			
		} 
		else if(command.equals("SLIST") || command.equals("PLIST") ){
			// WRITE THE FINDINGS INTO THE CONFIG FILE FOLDER	
			String correlIdd = retVal.getCorrelid();
			filename = filename + brickidfrom +"___CORR_"+correlIdd+"___" + new IDGenerator().getNewID()+".txt";
			String pathfilename = PropertiesFileReader.getInstance().getProperty("filetrigger.dir.brickconfig");
			
			pathfilename = pathfilename + filename;
			logger.debug("send command by writing file into  "+pathfilename);
			parameter = retVal.getReadingport() +"\t"+retVal.getReadingvalue() + "\t"+retVal.getReadingtype();
			String tsnowstr = tsnow.toString();
			tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);
			logger.debug("tsnowstr "+tsnowstr);
			String content =brickidfrom+"\t"+command+ "\t" + correlIdd+"\t"+parameter+"\t"+tsnowstr+ "\tEXECUTIONTIME="+execTime;
			fm.createFileAsEvent(pathfilename, content);
			
		}
		else {
			for(int j = 0 ; j < 10; j++){
				logger.info("THE COMMAND ["+command+"] IS NOT SUPPORTED - CODE IT - 20150914");
			}
			filename = filename + brickidfrom +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+".txt";
			String pathfilename = PropertiesFileReader.getInstance().getProperty("filetrigger.dir.frombrick");
			
			pathfilename = pathfilename + filename;
			logger.debug("send command by writing file into  "+pathfilename);
			
			String tsnowstr = tsnow.toString();
			tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);

			logger.debug("tsnowstr "+tsnowstr);
			String content =brickidfrom+"\t"+command+ "\t" + correlId+"\t"+parameter+"\t"+"\t"+tsnowstr+ "\tEXECUTIONTIME="+execTime;
			
			fm.createFileAsEvent(pathfilename, content);
		}
		logger.info("<---- prepareFileFROM_BRICK(XBeeResponse answer) retVal="+retVal);

		return retVal;
	}


	private static XBeeDataTransportObject parseEVENT(
			FileMonitorFromBrickSingleton fm, String filename,
			String brickidfrom, String command, String parameter,
			String correlId, Timestamp tsnow, String value, String eventValue) {
		XBeeDataTransportObject retVal;
		filename = filename + brickidfrom +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+".txt";
		String pathfilename = PropertiesFileReader.getInstance().getProperty("filetrigger.dir.frombrick");
		
		pathfilename = pathfilename + filename;
		logger.debug("send event by writing file into  "+pathfilename);
		
		String tsnowstr = tsnow.toString();
		tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);
		
		logger.info("COMMAND EVENT ["+correlId+"]\tPARAMETER ["+parameter+"]\tVALUE ["+value+"]");

		String content =brickidfrom+"\t"+command+ "\t" + correlId+"\t"+parameter+"\t"+value+"\t"+eventValue+"\t"+tsnowstr;
		
		fm.createFileAsEvent(pathfilename, content);
		
		
		// create JSON FILE FOR DISPLAY
		CreateJsonFile json = new CreateJsonFile();
		XBeeDataTransportObject contentObj = new XBeeDataTransportObject(brickidfrom, "EVENT", value, "",eventValue, correlId,  tsnow); 
		
		// return value
		retVal = contentObj;
		json.createXBeeResponseFile(contentObj, "msg_event.json");
		return retVal;
	}


	/**
	 * Extracted on the 14.09.2015
	 * @param fm
	 * @param filename
	 * @param brickidfrom
	 * @param command
	 * @param correlId
	 * @param tsnow
	 * @param execTime
	 * @param value
	 * @return
	 */
	private static XBeeDataTransportObject parseRDOW(FileMonitorFromBrickSingleton fm, String filename, String brickidfrom, String command, String correlId, Timestamp tsnow, String execTime, String value) {
		XBeeDataTransportObject retVal;
		logger.info("parseRDOW filename "+ filename);		
		logger.info("parseRDOW brickidfrom "+ brickidfrom + "\tbeautified "+ beautifyXBeeAddress(brickidfrom));		
		logger.info("parseRDOW command "+ command);		
		
		logger.info("parseRDOW correlId "+ correlId);		
		logger.info("parseRDOWTimestamp "+ tsnow);		
		logger.info("parseRDOW execTime "+ execTime);		
		logger.info("parseRDOW Value "+ value);		
		
		
		filename = filename + brickidfrom +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+".txt";
		String pathfilename = PropertiesFileReader.getInstance().getProperty("filetrigger.dir.frombrick");
		
		pathfilename = pathfilename + filename;
		logger.debug("send command by writing file into  "+pathfilename);
		
		String tsnowstr = tsnow.toString();
		tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);
		logger.info("COMMAND RDOW ["+correlId+"]\tVALUE ["+value+"]");
		String content =brickidfrom+"\t"+command+ "\t" + correlId+"\t"+value+"\t"+"\t"+tsnowstr+ "\tEXECUTIONTIME="+execTime;
		
		fm.createFileAsEvent(pathfilename, content);
		
		XBeeDataTransportObject contentObj = new XBeeDataTransportObject(brickidfrom, "RDOW", "OW", "",value, correlId,  tsnow); 
		
		// return value
		retVal = contentObj;
		
		// DELETE toBRICK file - check the correl ID
		// added on the 16.07.2015
//	        deleteFilesWithCorrelIDFromDir(correlId, PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick"));

		// REMOVE FROM tHE toBRickQueue
		XBeeDataQueueSingleton toBrick = XBeeDataQueueSingleton.getInstance();
		toBrick.deleteContainingCharSequenceToBrick(correlId);
		loggerfile.info("DELETE FROM toBRICK QUEUE: " + correlId);
		return retVal;
	}
	
	
	private static XBeeDataTransportObject parsePLIST(
			XBeeDataTransportObject retVal, String commandAndCorrelIDAndVAL,
			StringTokenizer cmdTokens) {
			String correlIdPre = cmdTokens.nextToken();
			String correlIdstr = "";
			if(correlIdPre.endsWith("]")){
				// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
				correlIdstr = correlIdPre.substring(0, (correlIdPre.length()-1));
			} else {
				// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
				// todo 
			}
			String st = Character.toString(commandAndCorrelIDAndVAL.charAt(16));
			
			if(st.equals("A") || st.equals("B") || st.equals("C")){
				// ITS A IO PORT 
				// read out the pins
				retVal = new XBeeDataTransportObject(st);
				retVal.setReadingport(st); 
				retVal.setCorrelid(correlIdstr);
				retVal.setCommand("PLIST");
				String ports = "";
				for(int i = 19 ; i < 19+8; i++){
					st = Character.toString(commandAndCorrelIDAndVAL.charAt(i));
					if(st.equals("]")){
						// end of port number parameter
						break;
					} else {
						ports = ports + st;
					}
				}
				retVal.setReadingvalue(ports);
				retVal.setCreatedtimestamp(new Timestamp(System.currentTimeMillis()));
				String outOrIn =  st = Character.toString(commandAndCorrelIDAndVAL.charAt(29));
				if(outOrIn.equals("O") || outOrIn.equals("I") ){
					retVal.setReadingtype(outOrIn);
				} else {
					throw new RuntimeException("ERROR: EXPECTED WAS A O or I BUT RECEIVED ["+outOrIn+"] - CHEKC DATA GRAM SLIST FOR PORTS A, B, C ");
				}
			} else{
				throw new RuntimeException("ERROR: EXPECTED WAS A, B, C BUT RECEIVED ["+st+"] - CHEKC DATA GRAM SLIST FOR PORTS A, B, C ");
				
			}
			return retVal;
		}

	
	/**SLOW OR FAST VERSION 
	 * @param cmd
	 * @param result
	 * @param waittime in ms
	 * @return
	 */
	private static XBeeDataTransportObject parseSLIST(
			XBeeDataTransportObject retVal, String commandAndCorrelIDAndVAL,
			StringTokenizer cmdTokens) {
		String correlIdPre = cmdTokens.nextToken();
		String correlIdstr = "";
		if(correlIdPre.endsWith("]")){
			// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
			correlIdstr = correlIdPre.substring(0, (correlIdPre.length()-1));
		} else {
			// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
			// todo 
		}
		String st = Character.toString(commandAndCorrelIDAndVAL.charAt(16));
		
		if(st.equals("A") || st.equals("B") || st.equals("C")){
			// ITS A IO PORT 
			// read out the pins
			retVal = new XBeeDataTransportObject(st);
			retVal.setReadingport(st); 
			retVal.setCorrelid(correlIdstr);
			retVal.setCommand("SLIST");
			String ports = "";
			for(int i = 19 ; i < 19+8; i++){
				st = Character.toString(commandAndCorrelIDAndVAL.charAt(i));
				if(st.equals("]")){
					// end of port number parameter
					break;
				} else {
					ports = ports + st;
				}
			}
			retVal.setReadingvalue(ports);
			retVal.setCreatedtimestamp(new Timestamp(System.currentTimeMillis()));
			String outOrIn =  st = Character.toString(commandAndCorrelIDAndVAL.charAt(29));
			if(outOrIn.equals("O") || outOrIn.equals("I") ){
				retVal.setReadingtype(outOrIn);
			} else {
				throw new RuntimeException("ERROR: EXPECTED WAS A O or I BUT RECEIVED ["+st+"] - CHEKC DATA GRAM SLIST FOR PORTS A, B, C ");
			}
		} else if(st.equals("O")){
			// ITS A ONE WIRE PORT - GET THE ADDRESS
			String port = commandAndCorrelIDAndVAL.substring(16, 18);
			if(port.equals("OW")){
				// YES ITS THE ONE WIRE PORT - GET THE OW ADDRESS
				// THE FORMAT RECEIVED: SLIST[fCX-_ptM][OW][28040F0105000017]
				if(commandAndCorrelIDAndVAL.length() != "SLIST[fCX-_ptM][OW][28040F0105000017]".length()){
					throw new RuntimeException("ERROR: DATAGRAM LENGTH EXPEXTED IS 37 - BUT WE RECEIVED A DATAGRAM WITH LENGTH ["+commandAndCorrelIDAndVAL.length()+"] - CHECK FROMAT OF DATAGRAM ["+commandAndCorrelIDAndVAL+"]");
				} else {
					// right length
					// check the brackets on the right position 
					String brack1 = Character.toString(commandAndCorrelIDAndVAL.charAt(5));
					String brack2 = Character.toString(commandAndCorrelIDAndVAL.charAt(14));
					String brack3 = Character.toString(commandAndCorrelIDAndVAL.charAt(15));
					String brack4 = Character.toString(commandAndCorrelIDAndVAL.charAt(18));
					String brack5 = Character.toString(commandAndCorrelIDAndVAL.charAt(19));
					String brack6 = Character.toString(commandAndCorrelIDAndVAL.charAt(36));
					if(brack1.equals("[") && brack2.equals("]") && brack3.equals("[")&& brack4.equals("]")&& brack5.equals("[")&& brack6.equals("]")){
						// brackes ok la
						// pack the obj with data
						retVal = new XBeeDataTransportObject(st);
						retVal.setReadingport(port); 
						String correlIDfromOW = commandAndCorrelIDAndVAL.substring(6, 14);
						retVal.setCorrelid(correlIDfromOW);
						retVal.setCommand("SLIST");
						retVal.setCreatedtimestamp(new Timestamp(System.currentTimeMillis()));
						retVal.setReadingtype("I");
						String sensoraddress = commandAndCorrelIDAndVAL.substring(20, 36);
						retVal.setReadingvalue(sensoraddress);
						
						
					} else {
						// brackets wrong
						throw new RuntimeException("ERROR: DATAGRAM ["+commandAndCorrelIDAndVAL.length()+"] - HAS WRONG FORMATTING - CHECK THE BRACKETS AND THEIR POSITION");
						
					}
				}
			}
		
		
		}
		return retVal;
	}
	private static Timestamp currentTimestamp() {
		Date d = new java.util.Date();
		Timestamp now = new Timestamp(d.getTime());
		return now;
	}	
	
	
	private static String beautifyXBeeAddress(String xbeeAddress){
		String retval = xbeeAddress.replace("0,0x", "").replace("0x0", "").trim();
		return retval;
	}

}
