package com.moneyhouse.sensors.xbee;

import static java.nio.file.StandardWatchEventKinds.ENTRY_CREATE;
import static java.nio.file.StandardWatchEventKinds.ENTRY_DELETE;
import static java.nio.file.StandardWatchEventKinds.OVERFLOW;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.nio.file.WatchEvent;
import java.nio.file.WatchKey;
import java.nio.file.WatchService;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Queue;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import com.moneyhouse.sensors.config.CONTSTANTINTERFACE;
import com.moneyhouse.sensors.config.LogHelper;
import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.config.ServerEnvironmentSingleton;
import com.moneyhouse.sensors.filetrigger.FileMonitorFromBrickSingleton;
import com.moneyhouse.sensors.filetrigger.FileMonitorToBrickSingleton;
import com.moneyhouse.sensors.internal.idgenerator.IDGenerator;
import com.moneyhouse.sensors.pi.DSSensor;
import com.moneyhouse.sensors.pi.dto.DSSensorResultDO;
import com.moneyhouse.sensors.xbee.dto.FileDescDataObject;
import com.moneyhouse.sensors.xbee.dto.JSONQueueContainer;
import com.moneyhouse.sensors.xbee.dto.ToBrickErrorFileQueueElementDataObject;
import com.moneyhouse.sensors.xbee.dto.ToBrickFileQueueElementDataObject;
import com.moneyhouse.sensors.xbee.dto.XBeeComandObject;
import com.moneyhouse.sensors.xbee.dto.XBeeComandResultObject;
import com.moneyhouse.sensors.xbee.dto.JSONXBeeCommandHeartBeatObject;
import com.moneyhouse.sensors.xbee.dto.XBeeConstants;
import com.moneyhouse.sensors.xbee.dto.XBeeDataTransportObject;
import com.moneyhouse.sensors.xbee.thread.QueueMaintenanceThread;
import com.moneyhouse.sensors.xbee.thread.ToBrickFileReceiverThread;
import com.rapplogic.xbee.api.AtCommand;
import com.rapplogic.xbee.api.AtCommandResponse;
import com.rapplogic.xbee.api.RemoteAtRequest;
import com.rapplogic.xbee.api.RemoteAtResponse;
import com.rapplogic.xbee.api.XBee;
import com.rapplogic.xbee.api.XBeeAddress16;
import com.rapplogic.xbee.api.XBeeAddress64;
import com.rapplogic.xbee.api.XBeeException;
import com.rapplogic.xbee.api.XBeeResponse;
import com.rapplogic.xbee.api.XBeeTimeoutException;
import com.rapplogic.xbee.api.zigbee.NodeDiscover;
import com.rapplogic.xbee.api.zigbee.ZNetExplicitTxRequest;
import com.rapplogic.xbee.api.zigbee.ZNetRxResponse;
import com.rapplogic.xbee.api.zigbee.ZNetTxRequest;
import com.rapplogic.xbee.api.zigbee.ZNetTxStatusResponse;
import com.rapplogic.xbee.util.ByteUtils;
import com.rapplogic.xbee.util.DoubleByte;

import flexjson.JSONSerializer;
import flexjson.transformer.DateTransformer;
/**
 * Creates a new thread for receiving data from Xbee via UART
 * 
 * @author Frederic
 *
 */

public class SerialReceiver8 implements Runnable{
	private final static Logger logger = Logger.getLogger(SerialReceiver8.class);
	private final static Logger loggerfile = Logger.getLogger("requestLogger");
//	private final static Logger loggerqueue = Logger.getLogger("requestToBrickQueueLogger");
	
	
//	private File[] filelist = null;
	private static Queue<String> send_fifo_buffer = new LinkedList<String>();
//	private static long printoutcounter = 0; // to reduce the prtinout to console when idle
	private static long printoutcounterDispl = 0; // to reduce the prtinout to console when idle
	private static XBee xbeeSingleton = null;
	private static ArrayList<XBeeDataTransportObject> latestDataInQueue = new ArrayList<XBeeDataTransportObject>();
	
	private long startTimestamp = 0; 							// its the time when the SerialReciever Was Started
	private boolean isXBEERadioMounted= false; 					// is the Radio module mounted or not? By default not
	static Thread  toBrickFileListener;    						// looks at directory toBrick	
	
	public SerialReceiver8(){
		logger.info("-----> SerialReceiver8.SerialReceiver8()");
		logger.info("\t Initializing SerialReceiver V8.2 for XBEE communication - SERIAL");
		logger.info("\t TIME STAMP: 20160606 1720");

		if(PropertiesFileReader.isRunEnvLinux()){
			logger.info("\t We run on Linux");
			logger.info("\t 0a. Setting gnu.io.rxtx.SerialPorts in porperties");
			String serialPort = PropertiesFileReader.getInstance().getProperty("comport");
			System.setProperty("gnu.io.rxtx.SerialPorts", serialPort); 
			// com ports between wheezy and jessie changed - wheezy is /dev/ttyAMA0 whereas jessie is /dev/ttyS0
		} else{
			logger.info("\t We run on Win - nothing to do");
		}
		logger.info("\t 1a. Getting XBee Singleton");
		
		XBee xbee = XBeeSingleton.getInstance().getXbee();
		logger.info("\t 1b. XBee Singleton = "+ xbee);
		logger.info("\t 2a. Get COM port for XBee communication");
		String port = PropertiesFileReader.getInstance().getProperty("comport");
		logger.info("\t 2b. COM Port used: "+port);

		try 
		{
			logger.info("\t 3a. Prepare to open XBee communication");
			xbee.open(port, 9600);
			logger.info("\t 3b. XBee open");
			
			isXBEERadioMounted = true;
			logger.info("\t 3c. Prepare to check XBeE coordinator");
			String sh = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SH"))).getValue());
			String sl = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SL"))).getValue());
			logger.info("\t 3d. XBeE coordinator addess: " + sh +","+ sl);
			logger.info("\t 3e. Register XBeE coordinator exists");

			
			
			
			logger.info("\t 4a. Prepare to close XBee communication");
			xbee.close();
			logger.info("\t 4b. XBee closed");

			
		} 
		catch (XBeeException e) {
			// TODO Auto-generated catch block
			logger.info("\t 3b. XBee can not be opened or cant be closed");
			
			String errormsg = e.toString();
			logger.error("THE FUCKING ERROR MESSAGE: ", e);
			e.printStackTrace();
			logger.error(errormsg);
			
		}

		logger.info("SOME LOG4J LOGGING LEVEL TEST - WE TEST FATAL, ERROR, WARN, INFO, DEBUG");
		logger.fatal("LOG4J *FATAL* ERROR WARN INFO DEBUG LEVEL");
		logger.error("LOG4J FATAL *ERROR* WARN INFO DEBUG LEVEL");
		logger.warn("LOG4J FATAL ERROR *WARN* INFO DEBUG LEVEL");
		logger.info("LOG4J FATAL ERROR WARN *INFO* DEBUG LEVEL");
		logger.debug("LOG4J FATAL ERROR WARN INFO *DEBUG* LEVEL");
		
		
		logger.info("ALSO LOGGING TO - requestLogger						xbee_requests.log");
		logger.info("ALSO LOGGING TO - requestToBrickQueueLogger			xbee_requests_queue.log");
		loggerfile.info	("PROGRAM START - CHECK requestLoggerlogfile		xbee_requests.log");
//		loggerqueue.info("PROGRAM START - CHECK requestToBrickQueueLogger	xbee_requests_queue.log");
		logger.info("WROTE INTO THESE TWO LOGGER FILES - CHECK log4jxbee.properties");

		
		logger.info("STATUS JSON: data_result_manage_brick_network_queues.json");
		logger.info("BASE STATION OW READING RESULTS CAN BE FOUND VIA KEY WORDS: grep \"READ SENSOR RESULT:\"");
		logger.info("BASE STATION OW READING RESULTS CAN BE FOUND VIA KEY WORDS: grep \") retVal=\"");

		setStartTimestamp(System.currentTimeMillis());
		

		// CLEAN FILE 	
		try{
			cleanFileFromDocRoot("error_queue_content.txt");
			cleanFileFromDocRoot("tobrick_queue_content.txt");
		}catch(Exception r){
			logger.fatal("FATAL ERROR: CAN NOT CLEAN FILES error_queue_content.txt AND tobrick_queue_content.txt IN DOC ROOT DIR!!");
			loggerfile.fatal("FATAL ERROR: CAN NOT CLEAN FILES error_queue_content.txt AND tobrick_queue_content.txt IN DOC ROOT DIR!!");
			throw new RuntimeException("FATAL ERROR: CAN NOT CLEAN FILES error_queue_content.txt AND tobrick_queue_content.txt IN DOC ROOT DIR!!", r);
		}
		
		logger.info("<----- SerialReceiver8.SerialReceiver8()");
	}


	
	
	private static void cleanFileFromDocRoot(String filename) throws FileNotFoundException {
		logger.debug("-----> cleanFileFromDocRoot("+filename+")");
		if(isPresentationWarDeployedStatic()){
			String docroot = ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION;
			File file =new File(docroot+filename);
			if(!file.exists()){
				try {
					file.createNewFile();
					PrintWriter writer = new PrintWriter(file);
					writer.print("");
					writer.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					logger.error("ERROR: WHILE CLEANING FILE FORM DOC ROOT ["+filename+"]");
					logger.error(e);
					
				}
			} else {
				PrintWriter writer = new PrintWriter(file);
				writer.print("");
				writer.close();
			}			
		}
		else{
			logger.debug("WAR IS NOT DEPLOYED - NO NEED TO CLEAN UP ["+filename+"]");
			
		}
		

		logger.debug("<----- cleanFileFromDocRoot("+filename+")");

	}
	
	

	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try{
			init();
	
			SerialReceiver8 mb = new SerialReceiver8();
	
			ToBrickFileReceiverThread tbfrt = new ToBrickFileReceiverThread();
			tbfrt.start();
	
			QueueMaintenanceThread qmt = new QueueMaintenanceThread();
			qmt.start();
			
			
			// TO AVOID NULL POINTER EXCEPTION IN STARTUP PHASE
			goWasteTime(2000);
			
			// nodeAddress64=0x00,0x13,0xa2,0x00,0x40,0x0a,0x4a,0x0a,
			// nodeAddress64=0x00,0x13,0xa2,0x00,0x40,0xa0,0x98,0xf3
			// nodeAddress64=0x00,0x13,0xa2,0x00,0x40,0x0a,0x42,0xc5
			
			// XBeeAddress64 address = new XBeeAddress64("00 13 A2 00 40 A0 98 F3");
			File folder = new File(PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick"));
			File[] listOfFilesToBrick = folder.listFiles();
			
			
	//		mb.filelist = listOfFilesToBrick;
			
	//		boolean execute = false;
			long tsping = System.currentTimeMillis();				// FOR THE POLLING MECHANISM
			long tsQueueReadOut = System.currentTimeMillis();		// FOR THE POLLING MECHANISM
			long timeout = System.currentTimeMillis();				// FOR THE TIMEOUT EXCEPTION FROM READING THE XBEE SERIAL WIRE INCOME DATA
			long deleteFilesDuration = System.currentTimeMillis();	// FOR THE LOGGING THE THE DELETE FILES In TOBRICK FOLDER
			
			
			boolean firstRun = true;
			
			
	//		XBeeReceiverThread t1 = mb.new XBeeReceiverThread( "Thread-1");
	//	    t1.start();
			
			try {
				openCOMPort(xbeeSingleton);
			} catch (XBeeException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
				logger.error("ERROR: PROBLEMS OPENING XBEE PORT", e1);
			} catch (InterruptedException ie) {
				// TODO Auto-generated catch block
				ie.printStackTrace();
				logger.error("ERROR: PROBLEMS OPENING XBEE PORT", ie);
			}
			for(;;){
				long tsping2 = System.currentTimeMillis();
				
				if(tsping2 - tsping > 1200000 || firstRun == true){
					tsping = printOutInRegularInterval(tsping2);
					// deleteFilesOlderThanFromDir("3h", PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick"));
					firstRun = false;
				}
	
				
				
				if(tsping2 - deleteFilesDuration > 300000 ){
					logger.info("AFTER ["+((tsping2 - deleteFilesDuration)/1000 )+"] s WE CLEAN THE filetrigger.dir.tobrick DIRECTORY");
	
					deleteFilesOlderThanFromDir("2h", PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick"));
					
					mb.writeQueueEntriesToFile();
					deleteFilesDuration = tsping2;
					
				}
				
				
				
				
				if(tsping2 - tsQueueReadOut > 2400000 || firstRun == true){
	//				tsQueueReadOut = printOutInRegularIntervalQueue(tsping2);
					// deleteFilesOlderThanFromDir("3h", PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick"));
				}
				
				try{
					timeout = mainLoopBody(mb, getXbeeSingleton(), timeout);
					
				} catch(Exception rd){
					logger.fatal("FATAL: LOGGING NEEDS TO BE IMPROVED WE CATCHED AN EXCPETION IN THE MAIN LOOP BODY - FOR NOW THIS EXPECTION WAS CATCHED: ", rd);
					logger.fatal("FATAL: MESSAGE OF THE EXCEPTION ["+rd.getMessage()+"] AND ALL THE OTHER INFORMATION: "+ rd.toString());
					
				}
			}
		}catch (Exception ex){
			logger.fatal("FATAL: SOMETHING WENT WRONG - LOGGING NEEDS TO BE IMPROVED - FOR NOW THIS EXPECTION WAS CATCHED: ", ex);
			logger.fatal("FATAL: MESSAGE OF THE EXCEPTION ["+ex.getMessage()+"] AND ALL THE OTHER INFORMATION: "+ ex.toString());
			
		}
	}

	/**
	 * WRITES THE CONTENT OF THE QUEUES TO THE FOLLOWING TEXT FILES 
	 * 
	 * tobrick_queue_content.txt
	 * error_queue_content.txt
	 * 
	 * IN THE DOC ROOT DIRECTORY 
	 */
	private synchronized void writeQueueEntriesToFile(){
		boolean isWARThere = isPresentationWarDeployed();
		if(isWARThere){
			XBeeDataQueueSingleton dataqueue = XBeeDataQueueSingleton.getInstance();
			
	    	String docroot = ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION;
	    	
	    	writeErrorQueueToTextFile(dataqueue, docroot);
	    	writeToBrickQueueToTextFile(dataqueue, docroot);
		}
	}

	private void writeToBrickQueueToTextFile(XBeeDataQueueSingleton dataqueue, String docroot) {
		String sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(System.currentTimeMillis()));
		String toBrickQueueContent = "TO BRICK QUEUE LIST CREATED TIMESTAMP: " + sdf + "\n";
		for(int i = 0; i < dataqueue.obtainToBrickDataList().size(); i++){
			ToBrickFileQueueElementDataObject tbefqedo = dataqueue.obtainToBrickDataList().get(i);
			toBrickQueueContent = toBrickQueueContent + "["+i+"]\t\t" + tbefqedo.toString() + "\n";
		}
		if(dataqueue.obtainToBrickErrorDataList().size() == 0){
			toBrickQueueContent = "NO ELEMENT IN TO BRICK QUEUE AT ["+sdf+"]";
		}
		
		
		try {
			cleanFileFromDocRoot("tobrick_queue_content.txt");
		    Files.write(Paths.get(docroot+"tobrick_queue_content.txt"), toBrickQueueContent.getBytes(), StandardOpenOption.WRITE);
		}catch (IOException e) {
		    //exception handling left as an exercise for the reader
			logger.fatal("FATAL ERROR: WhILE WRITING DOC ROOT tobrick_queue_content.txt",e);

			e.printStackTrace();
		}
	}
	
	

	private void writeErrorQueueToTextFile(XBeeDataQueueSingleton dataqueue, String docroot) {
		String sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(System.currentTimeMillis()));
		String errorqueueContent = "ERROR QUEUE LIST CREATED TIMESTAMP: " + sdf + "\n";
		for(int i = 0; i < dataqueue.obtainToBrickErrorDataList().size(); i++){
			ToBrickErrorFileQueueElementDataObject tbefqedo = dataqueue.obtainToBrickErrorDataList().get(i);
			errorqueueContent = errorqueueContent + "["+i+"]\t\t" + tbefqedo.toString() + "\n";
		}
		if(dataqueue.obtainToBrickErrorDataList().size() == 0){
			sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(System.currentTimeMillis()));
			errorqueueContent = "NO ELEMENT IN ERROR QUEUE AT ["+sdf+"]";
		}
		
		
		try {
			cleanFileFromDocRoot("error_queue_content.txt");

		    Files.write(Paths.get(docroot+"error_queue_content.txt"), errorqueueContent.getBytes(), StandardOpenOption.WRITE);
		}catch (IOException e) {
		    //exception handling left as an exercise for the reader
			logger.fatal("FATAL ERROR: WhILE WRITING DOC ROOT error_queue_content.txt",e);
			e.printStackTrace();
		}
	}

	private static long mainLoopBody(SerialReceiver8 mb, XBee xbee, long timeoutts) {
		// CLEANUP FILEs FROM TO BRICK DIR WHICH ARE OLDER THAN 2 HOURS
		
		XBeeDataQueueSingleton toBrickQueue = XBeeDataQueueSingleton.getInstance();
		ToBrickFileQueueElementDataObject element = toBrickQueue.readOldestToBrick(9);
/*		if(element != null){
			loggerqueue.info("=====================================================================================================");
			loggerqueue.info(toBrickQueue);
			loggerqueue.info("READ ELEMENT: "+ element);
			loggerqueue.info("=====================================================================================================");
		}
*/		
		XBeeComandResultObject directResultFromXBeeReuqest = null;
		if(element != null){
			if(element.getFilename().toString().contains(CONTSTANTINTERFACE.FILENAME_PART_FOR_ND_FINDALL)){
				// ND COMMAND TAKES VERY LONG - SO GIVEM TIME
				goWasteTime(500 * element.getReadingCounter() + 1000);
			} else {
				// EVERY THING ELSE CAN GO FASTER
				goWasteTime(200 * element.getReadingCounter());
				
			}
			
			File theNewOne = element.getFilename();
			logger.info("-------------------------------------------------------------------------------------------------------");
			logger.info("theNewOne Abso: "+ theNewOne.getAbsolutePath());
			logger.info("theNewOne Pare: "+ theNewOne.getParent());
			logger.info("theNewOne Path: "+ theNewOne.getPath());
			logger.info("theNewOne File: "+ theNewOne.getName() +"\t CREATED: "+element.getReceivedTimestampFormated());
			logger.info("Element Counter: "+ element.getReadingCounter());
			logger.info("-------------------------------------------------------------------------------------------------------");
			loggerfile.info("NEW RQT: "+ LogHelper.writeShortFileName(theNewOne.getName()) +"\t CREATED: "+element.getReceivedTimestampFormated()+"\t COUNTER: "+element.getReadingCounter());
			boolean exists = FileMonitorToBrickSingleton.getInstance().isFileExistant(theNewOne.getAbsolutePath());
			if(exists){
				directResultFromXBeeReuqest = mb.parseFile(theNewOne, xbee);
				
				// check whether we received a direct result from the xbee - LIKE  sendSETOUTPORTB0 command
				if(directResultFromXBeeReuqest != null && directResultFromXBeeReuqest.getXbeeResultAsString() != null && directResultFromXBeeReuqest.getXbeeResultAsString().length() !=0  ){
					if(directResultFromXBeeReuqest.getXbeeResultAsString().contains("apiId=ZNET_TX_STATUS_RESPONSE") &&
							directResultFromXBeeReuqest.getXbeeResultAsString().contains("deliveryStatus=SUCCESS") &&
							directResultFromXBeeReuqest.getXbeeResultAsString().contains("error=false")){
						// PERFECT - THE SEND PACKAGE WAS WELL PROCESSED
						logger.info("WE HAVE AN IMMEDIATE RESPONSE FROM THE XBEE RADIO MODULE WHICH IS VALID - LETS PROCESS IT");
						logger.info("XBEE RESULT STRING: "+directResultFromXBeeReuqest.getXbeeResultAsString());
						logger.info("CORRELID : "+directResultFromXBeeReuqest.getCorrelID());
					
						// DELETE FROM TO BRICK QUEUE
						XBeeDataQueueSingleton toBrick = XBeeDataQueueSingleton.getInstance();
						toBrick.deleteContainingCharSequenceToBrick(directResultFromXBeeReuqest.getCorrelID());
					} else if(directResultFromXBeeReuqest instanceof  JSONXBeeCommandHeartBeatObject &&
							directResultFromXBeeReuqest.getXbeeResultAsString() != null && 
							directResultFromXBeeReuqest.getXbeeResultAsString().equalsIgnoreCase("HEARTBEAT: SOME STRING TO SAY THAT WE WORKED DILLIGENTLY") ){
						// PERFECT - THE SEND PACKAGE WAS WELL PROCESSED
						logger.info("HEARTBEAT WITH CORRELID ["+directResultFromXBeeReuqest.getCorrelID()+"] WORKED");
					
						// DELETE FROM TO BRICK QUEUE
						XBeeDataQueueSingleton toBrick = XBeeDataQueueSingleton.getInstance();
						toBrick.deleteContainingCharSequenceToBrickAndFile(directResultFromXBeeReuqest.getCorrelID());
					} else {
						// THIS SHOULD COME WHEN THE SEND PACKAGE HAS PROBLEMS 
						logger.warn("%% A TROUBLESOME IMMEDIATE RESPONSE FROM THE XBEE RADIO MODULE WAS RECEIVED - HERE IT IS");
						logger.warn("%% XBEE RESULT STRING: "+directResultFromXBeeReuqest.getXbeeResultAsString());
						logger.warn("%% CORRELID : "+directResultFromXBeeReuqest.getCorrelID());
						logger.warn("%% SHORT FILENAME: " +  LogHelper.writeShortFileName(theNewOne.getName()));
					}
				} else if(directResultFromXBeeReuqest != null && (directResultFromXBeeReuqest.getXbeeResultAsString()==null || directResultFromXBeeReuqest.getXbeeResultAsString().length() ==0 ) ){
					// IT MIGHT BE A SHSL Coordinaotr command
					if(directResultFromXBeeReuqest.getCommand() != null && directResultFromXBeeReuqest.getCommand().getAtcommand().equals("SHSL")){
						// ITS A GET COORDINATOR ADDRESS COMMAND
						logger.info("SHSL WITH CORRELID ["+directResultFromXBeeReuqest.getCorrelID()+"] WORKED");
						
						// DELETE FROM TO BRICK QUEUE
						XBeeDataQueueSingleton toBrick = XBeeDataQueueSingleton.getInstance();
						toBrick.deleteContainingCharSequenceToBrickAndFile(directResultFromXBeeReuqest.getCorrelID());
					} else if(directResultFromXBeeReuqest.getCommand() != null && directResultFromXBeeReuqest.getResponseValue().contains("THE XBEE RADIO MODULE IS NOT PLUGGED IN") && directResultFromXBeeReuqest.isResponseOK() == false){
						// IT SEEMS THE XBEE RADIO MODULE DOES NOT EXIST
						logger.warn("%% XBEE RADIO MODULE DOES NOT EXIST: ");
						logger.warn("%% WE MOVE REQUEST WITH CORREL ID ["+directResultFromXBeeReuqest.getCommand().getCorrelID()+"] TO ERROR QUEUE ");
						XBeeDataQueueSingleton toBrick = XBeeDataQueueSingleton.getInstance();
						toBrick.moveFromToBrickToErrorListContainingCharSequence(directResultFromXBeeReuqest.getCommand().getCorrelID(), "NO_XBEE_RADIO_MODULE");
					}
				}
			} 
			else 
			{
				logger.error("ERROR: THE FILE ["+LogHelper.writeShortFileName(theNewOne.getName())+"] DOES NOT EXIST");
			}
		}

		
		for(int i = 0 ;i < 3; i++ ){
			// logger.info("BODY LOOP ["+i+"] DIRECT_RESULT= "+directResultFromXBeeReuqest );

			try
			{
				long tsbeforeResponse = System.currentTimeMillis();
				XBeeResponse answer =  xbee.getResponse(1000);
				long tsafterResponse = System.currentTimeMillis();
	
				logger.info("XBEE RADIO RESPONSE AFTER ["+(tsafterResponse-tsbeforeResponse)+"] MS");
				if(answer instanceof ZNetRxResponse){
					logger.info(answer + " IS ZNetRxResponse");
					String correlId = getCorrelIDFromDatagram(answer);
					if(isAlreadyReceived(correlId)){
						// already received - do nothing
						logger.info("["+correlId+"] WAS ALREADY RECEIVED - WE DO NOTHING");
					} else {
						// no double with this correlation ID found ... to move on with processing the stuff
						logger.info("["+correlId+"] FIRST TIME RECEIVED - WE PROCESS IT!");

						XBeeDataTransportObject xbeeobj = SerialReceiver6ComposeAnswer.prepareFileFROM_BRICK(answer);
						latestDataInQueue.add(xbeeobj);
						break;
					}
				} 
				else if(answer instanceof com.rapplogic.xbee.api.AtCommandResponse) {
					logger.info(answer + " IS AtCommandResponse");
					AtCommandResponse answered = (com.rapplogic.xbee.api.AtCommandResponse)answer;
					
					String comanddo = getATCommand(answered);
					if(comanddo != null && comanddo.equalsIgnoreCase("ND")){
						// get all the nodes
						String nomde = getDiscoveredNDNode(answered);
						FileMonitorFromBrickSingleton fm = FileMonitorFromBrickSingleton.getInstance();
						
						String correlId = directResultFromXBeeReuqest.getCorrelID();
						
						String filename = "FROM_BRICK__"+nomde+"__AT_CMD_"+comanddo + CONTSTANTINTERFACE.FILENAME_PART_CORR +correlId +"___" +new IDGenerator().getNewID()+".txt";
						String pathfilename = PropertiesFileReader.getInstance().getProperty("filetrigger.dir.frombrick");
						
						pathfilename = pathfilename + filename;
						logger.debug("send command by writing file into  "+pathfilename);
						Timestamp tsnow = new Timestamp(System.currentTimeMillis());
						String tsnowstr = tsnow.toString();
						logger.debug("tsnowstr "+tsnowstr);
						String content =nomde+"\t"+comanddo+ "\t" + "none"+"\t"+ correlId +"\t"+tsnowstr;
						fm.createFileAsEvent(pathfilename, content);
					}
				}
				else if(answer instanceof com.rapplogic.xbee.api.zigbee.ZNetTxStatusResponse){
					logger.info("XBee MODULE SENT OBJECT OF TYPE: "+ answer.getClass());
					ZNetTxStatusResponse anserZNetStatusResponse = (ZNetTxStatusResponse)answer;
					if(anserZNetStatusResponse.isSuccess()){
						logger.info("XBee MODULE SENT OBJECT IS SUCCESSFUL");
						
					} else {
						logger.info("XBee MODULE SENT OBJECT IS NOT SUCCESSFUL");
					}
				}
				else 
				{
					// some other type received - we throw an exception
					logger.error("ERROR: WE RECEIVED AN ANSWER WHICH iS NOT OF TYPE ZNetRxResponse BUT ["+answer.getClass()+"]");
					throw new RuntimeException("ERROR: WE RECEIVED AN ANSWER WHICH iS NOT OF TYPE ZNetRxResponse BUT ["+answer.getClass()+"]");
				}
			} 
				
			
			catch(XBeeTimeoutException exc){
				long tsping3 = System.currentTimeMillis();
	
				if(tsping3 - timeoutts > 60000 ){
					printoutcounterDispl++;
					logger.info("TIMEOUT - NO DATA FROM REMOTE XBEE MODULE RECEVEIVED DON'T WORRY - LOOP ON ["+printoutcounterDispl+"]");
					// logger.info("NOW: "+tsping2+  "\ttimeoutts: "+timeoutts);
					timeoutts = tsping3;
				} 
			} catch(Exception exc){
				logger.error(exc);
				exc.printStackTrace();
			} finally{
				// xbee.close();
			}
		}
		return timeoutts;
	}


	private static long printOutInRegularInterval(long tsping2) {
		long tsping;
		// logger.debug(".");
		tsping = tsping2;
		for(int o = 0; o< 1; o++){
			logger.info("===========================================================================");
		}

		logger.info("READ SENSORSLIST START");

		DSSensor ds = new DSSensor();
		ArrayList<String> piSensorList = ds.readSensorList();
		for(int p = 0; p < piSensorList.size(); p++){
			logger.info("["+p+"] SENSOR ID = ["+piSensorList.get(p)+"] ");
		}
		logger.info("READ SENSORSLIST END");
		

		logger.info("READ SENSORS START");
		for(int p = 0; p < piSensorList.size(); p++){
			DSSensorResultDO res = ds.readSensorValueWithID(piSensorList.get(p));
			logger.info("READ SENSOR RESULT: " + res);

		}
		
		logger.info("READ SENSORS END");
		
		
		for(int o = 0; o< 1; o++){
			logger.info("===========================================================================");
		}
		return tsping;
	}

	
	private static long printOutInRegularIntervalQueue(long tsqueue) {
		long tsping;
		// logger.debug(".");
		tsping = tsqueue;
		
		XBeeDataQueueSingleton toBrickQueue = XBeeDataQueueSingleton.getInstance();

		
		for(int o = 0; o< 1; o++){
			loggerfile.info("________________________________________________________________________________________________________");
		}

		loggerfile.info("READ QUEUE ENTRIES - START");
		loggerfile.info(toBrickQueue);
		loggerfile.info("READ QUEUE ENTRIES - END");
		for(int o = 0; o< 1; o++){
			loggerfile.info("________________________________________________________________________________________________________");
		}
		return tsping;
	}

	
	private static String getATCommand(com.rapplogic.xbee.api.AtCommandResponse answer){
		String command = answer.getCommand();
		return command;
	}
	
	
	private static String getDiscoveredNDNode(com.rapplogic.xbee.api.AtCommandResponse answer){
		String retval = "";
		
		String command = answer.getCommand();
		if(command != null && command.equals("ND")){
			// its a node discovery ---> get the address
			String data = ByteUtils.toBase16(answer.getProcessedPacketBytes()) ;
			String vlue = ByteUtils.toBase16(answer.getValue());
			System.out.println(data);
			System.out.println(vlue);
			StringTokenizer toker = new StringTokenizer(vlue,",");
			int counter = 0;
			String nodeaddressRaw = "";
			while(toker.hasMoreElements()){
				counter++;
				String toe = toker.nextToken();
				if(counter > 2 && counter < 11){
					nodeaddressRaw = nodeaddressRaw +","+ toe;
				}
			}
			retval = nodeaddressRaw.substring(1); 
		}
		return retval;
	}
	
	private static boolean isAlreadyReceived(String correlId){
		boolean retval = false;
		long now = System.currentTimeMillis();
		for(int i = 0 ; i < latestDataInQueue.size(); i++){
			XBeeDataTransportObject data = latestDataInQueue.get(i);
			if(data.getCorrelid().equals(correlId)){
				// ups we have one record with the same correlation Id - check for time
				long timeiff = now - data.getCreatedtimestamp().getTime();
				if(timeiff > 120000){
					// more than 120 seconds ago
				} else {
					// less than 120 seconds ago - its a double
					retval = true;
					break;
				}
			}
		}
		// clean up latestDataInQueue
		for(int i = 0 ; i < latestDataInQueue.size(); i++){
			XBeeDataTransportObject data = latestDataInQueue.get(i);
			long timediff = now - data.getCreatedtimestamp().getTime();
			if(timediff > 120000){
				// delete every thing that is older than 120 secs
				latestDataInQueue.remove(i);
			}
		}		
		return retval;
	}
	
	private static String getCorrelIDFromDatagram(XBeeResponse answer){
		ZNetRxResponse answerRxRes = (ZNetRxResponse)answer;
		String retVal = "";
		String commandAndCorrelIDAndVAL = ByteUtils.toString(answerRxRes.getData());
		logger.debug("commandAndCorrelIDAndVAL = "+commandAndCorrelIDAndVAL);
		StringTokenizer strTokens = new StringTokenizer(commandAndCorrelIDAndVAL, " ");
		
		int tokencounter = 0;
		while (strTokens.hasMoreElements()) {
			tokencounter++;
			String tok = strTokens.nextToken();
			if(tokencounter == 1){
				// get the command + correlId out
				StringTokenizer cmdTokens = new StringTokenizer(tok, "[");
				@SuppressWarnings("unused")
				String cmd = cmdTokens.nextToken();
				retVal = cmdTokens.nextToken();
			}
		}
		if(retVal.length() != 9){
			// ERROR - WE EXPECT LEGTH OF 9 = correlId length = 8 and ] 
			logger.error("ERROR - WE EXPECT LEGTH OF 9 = correlId length = 8 and ] --- THE DATA RECEIVED IS: "+commandAndCorrelIDAndVAL);

			throw new RuntimeException("ERROR: CORRELATION ID MUST BE 8 char long plus ']' END TOKEN - BUt WE RECEIVED ["+retVal+"]");
		} else {
			retVal = retVal.substring(0, 8);
		}
		return retVal;
	}



	
	
	
	/**
	 * DELETE THE FILES FROM BRICK TO BRICK OR BRICKCONFIG WHICH ARE OLDER THAN PARAMETER olderThank
	 * @param olderThan 1d, 2d, 1w, 10w etc
	 * @param directory
	 */
	private static int deleteFilesOlderThanFromDir(String olderThan, String directory) {
		logger.info("---> deleteFilesOlderThanFromDir("+olderThan+", "+directory+")");  
		int retvalCountDel =0;
		try{
			
			int daysBack = 1;
			int multiplier = 1;
			int number = -1;
			long basehours = 24 * 60 * 60 * 1000;
			if(olderThan.contains("w") || olderThan.contains("W")){
				multiplier = 7;
			} else if(olderThan.contains("d") ||olderThan.contains("D")){
				multiplier = 1;
			} else if(olderThan.contains("h") ||olderThan.contains("H")){
				// change the base from days to hours
				basehours = 1 * 60 * 60 * 1000;		
			}  else {
				logger.error("ERROR: PARAMETER ["+olderThan+"] NOT SUPPORTED - EXPECTED IS 1d, 2D, 3w, 4W");
				throw new RuntimeException("PARAMETER ["+olderThan+"] h or H NOT SUPPORTED");
			}
			try{
				number = new Integer(olderThan.substring(0,  olderThan.length()-1)).intValue();
			} catch (NumberFormatException nfe){
				logger.error("ERROR: PARAMETER ["+olderThan+"] NOT SUPPORTED - EXPECTED IS 1h, 1d, 2D, 3w, 4W", nfe);
				throw new RuntimeException("ERROR: PARAMETER ["+olderThan+"] NOT SUPPORTED - EXPECTED IS 1d, 2D, 3w, 4W", nfe);
			}
	
			if(number < 0){
				logger.error("ERROR: PARAMETER ["+olderThan+"] CAN NOT BE NEGATIVE - EXPECTED IS 0h, 1h, 1d, 2D, 3w, 4W");
				throw new RuntimeException("ERROR: PARAMETER ["+olderThan+"] CAN NOT BE NEGATIVE - EXPECTED IS 1d, 2D, 3w, 4W");
			}
			daysBack = multiplier * number;
			
			long filecounter= 0, purgeTime= 0, delcounter = 0;
			
			File dir = new File(directory);
			if(dir.exists()){
	
			    File[] listFiles = dir.listFiles();    
			    filecounter = listFiles.length;
			    purgeTime = System.currentTimeMillis() - (daysBack * basehours);
			    for(File listFile : listFiles) {
			        if(listFile.lastModified() < purgeTime) {
			        	boolean isDelSuccess = listFile.delete();
			        	delcounter++;
			        	retvalCountDel++;
			            if(!isDelSuccess) {
			            	delcounter--;
			            	logger.error("Unable to delete file: " + listFile);
			            }
			         }
			      }
			   }
			// GETR THE TIMESTAMP
			DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			Date deldate = new Date();
			deldate.setTime(purgeTime);
			logger.info("DELETED ["+delcounter+"] FILES FROM TOTAL ["+filecounter+"] THAT WERE OLDER THAN ["+dateFormat.format(deldate)+"] ACCORDING TO PARAMETER ["+olderThan+"]");  
		}
		catch (Exception ex){
			logger.error("ERROR: ERROR WHILE DELETING FILES" , ex);
		}
		logger.info("<--- deleteFilesOlderThanFromDir("+olderThan+", "+directory+") COUNT DELETED FILES= "+ retvalCountDel);
		return retvalCountDel;
	}

	
	/**
	 * DELETE THE FILES FROM BRICK TO BRICK OR BRICKCONFIG WHICH HAVE CORREL ID
	 * @since 17.07.2015
	 * @param correlID 
	 * @param directory
	 * @deprecated not used - successful files are deleted anyhow
	 */
	private static void deleteFilesWithCorrelIDFromDir(String correlID, String directory) {
		logger.info("---> SerialReceiver5.deleteFilesWithCorrelIDFromDir("+correlID+", "+directory+")");  
		try{

			if(correlID == null || correlID.isEmpty()){
				logger.error("ERROR: PARAMETER correlID CAN NOT BE NULL OR EMPTY ");
				throw new RuntimeException("ERROR: PARAMETER correlID CAN NOT BE NULL OR EMPTY ");
			}
			long filecounter= 0;
			
			File dir = new File(directory);
			if(dir.exists())
			{
			    File[] listFiles = dir.listFiles();    
			    for(File listFile : listFiles) {
			    	if(listFile.isDirectory()){
			    		// ITS A DIRECTORY - THIS SHOULD NOT BE THE CASE
						logger.error("ERROR: FILE ["+listFile.getName()+"] IS A DIRECTORY - EXPECTED IS A FILE ");  
						throw new RuntimeException("ERROR: FILE ["+listFile.getName()+"] IS A DIRECTORY - EXPECTED IS A FILE ");
			    	} else {
			    		// ITS A FILE - READ IT AND COMPARE WITH THE CORREL-ID
			    		String content = readFile(listFile.getPath());
			    		if(content.contains(correlID)){
			    			// CORRELID IS IN THE CONTENT
			    			filecounter++;
			    		}
			    	}
			    }
			    
				logger.debug("["+filecounter+"] FILECOUNTER OF FILES CONTAINING CORRELID ["+correlID+"] FROM DIR ["+directory+"] ");  
			    
			    if(filecounter == 1){
			    	// YES THIS IS THe HAPPY CASE - ONLY ON FILE WITH THE SAME CORRELID

				    File[] listFiles2 = dir.listFiles();    
				    filecounter = listFiles2.length;
				    for(File listFile2 : listFiles2) {
				    	if(listFile2.isDirectory()){
				    		// ITS A DIRECTORY - THIS SHOULD NOT BE THE CASE
							logger.error("ERROR: FILE ["+listFile2.getName()+"] IS A DIRECTORY - EXPECTED IS A FILE ");  
							throw new RuntimeException("ERROR: FILE ["+listFile2.getName()+"] IS A DIRECTORY - EXPECTED IS A FILE ");
				    	} else {
				    		// ITS A FILE - READ IT AND COMPARE WITH THE CORREL-ID
							logger.debug("DELETE ["+listFile2.getName()+"] FROM DIR ["+directory+"] WITH CORRELID ["+correlID+"]");  
				    		listFile2.delete();
				    	}
				    }
			    } 
			    else 
			    {
		    	// BAD CASE - NOT FOUND - OF FOUND SEVERAL TIMES - CORRELID SHOULD BE UNIQUE
			    	logger.error("ERROR: CORRELID ["+correlID+"] IS FOUND ["+filecounter+"] IN DIRECTORY ["+directory+"] -- WE ECPECTED TO FIND 1 OCCURENCE - SO THAT WE CAN DELETE IT");  
					throw new RuntimeException("ERROR: CORRELID ["+correlID+"] IS FOUND ["+filecounter+"] IN DIRECTORY ["+directory+"] -- WE ECPECTED TO FIND 1 OCCURENCE - SO THAT WE CAN DELETE IT");
			    }
			}
		}
		catch (Exception ex){
			logger.error("ERROR: ERROR WHILE DELETING FILE" , ex);
		}
		logger.info("<--- SerialReceiver5.deleteFilesWithCorrelIDFromDir("+correlID+", "+directory+")");  
	}
	
	/** READ THE FILE - TO CHECK OF THE CORREL ID
	 * @since 16.07.2015
	 * @param path
	 * @return
	 */
	private static String readFile(String path) 
	{
		String retVal = "";
		byte[] encoded;
		try {
			encoded = Files.readAllBytes(Paths.get(path));
			retVal = new String(encoded);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			logger.error("ERROR: READING FILE ["+Paths.get(path)+"]", e);
		}
		return retVal;
	}
	
	
	
	private static void init() {
		String log4jxbeeconfig = PropertiesFileReader.getInstance().getProperty("log4jxbeeconfig");
		System.out.println("log4jxbeeconfig property key to file log4jxbee.properties is located: "+ log4jxbeeconfig);
		PropertyConfigurator.configure(log4jxbeeconfig);
	
		deleteFilesOlderThanFromDir("0h", PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick"));
		deleteFilesOlderThanFromDir("0h", PropertiesFileReader.getInstance().getProperty("filetrigger.dir.frombrick"));

		
		// ADDED ON tHE 7.09.2015
		deleteFilesOlderThanFromDir("0h", PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrickprocessed"));
		
		// ADDED ON tHE 10.11.2015
		deleteFilesOlderThanFromDir("0h", PropertiesFileReader.getInstance().getProperty("filetrigger.dir.brickconfig"));
		
		
		logger.info("LISTENING TO DIRECTORY: "+PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick"));
		
		logger.info("INSTANZIATING XBEE SINGLETON ");
		xbeeSingleton = getXbeeSingleton();
		logger.info("XBEE SINGLETON INSTANZIATED "+ getXbeeSingleton());
		
		logger.info("Reading out this directory - to check permissions");
		ArrayList<FileDescDataObject> ls = createFileListOfFolder(PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick"));
		if(ls.size() > 0 )
		{
			for(int i = 0 ; i < ls.size(); i++){
				logger.info("|" + i+ "\t" + ls.get(i));
			}
		} 
		else 
		{
			logger.info("0 files found in directory");
		}
	}
	
	protected void cmdBlinkPortDIO0_PIN20(String addr, XBeeComandResultObject result) {
		// transform normal string to xbee64bit address string - e.g. with gaps
		
		
		String finaladdr = transformStringToValid64BitXBeeAddress(addr);
		
		XBeeAddress64 address = new XBeeAddress64(finaladdr);
		cmdBlinkPortDIO0_PIN20(address, result);
	}


	
	
	protected void cmdBlinkPortDIO0_PIN20(XBeeAddress64 address, XBeeComandResultObject result) {
		logger.info("-----> cmdBlinkPortDIO0_PIN20(" + address + ")");
		XBee xbee = XBeeSingleton.getInstance().getXbee();
		try {
			
			openCOMPort(xbee);
			for (int i = 0; i < 10; i++) {
				// pin 20 corresponds to D0, and 5 activates the output (Digital
				// output high)
				RemoteAtRequest request = new RemoteAtRequest(address, "D0",  new int[] { 5 });

				// turn on LED
				logger.info(i + "\t\tturn on LED for 1 seconds");

				RemoteAtResponse response = (RemoteAtResponse) xbee.sendSynchronous(request, 10000);

				if (response.isOk()) {
					// success
				}

				Thread.sleep(1000);

				// command to turn off pin 20
				logger.info(i + "\t\tturn off LED for 1 seconds");
				request = new RemoteAtRequest(address, "D0", new int[] { 4 });

				// send the command
				response = (RemoteAtResponse) xbee.sendSynchronous(request,
						10000);

				if (response.isOk()) {
					// success
					result.setResponseOK(true);
				} 
				if(response.isError()){
					result.setResponseError(true);
				}

				Thread.sleep(1000);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			logger.error("POS Ax1: ", ex);
		} finally {
			xbee.close();
			logger.info("<----- cmdBlinkPortDIO0_PIN20(" + address + ")");
			
		}
	}

	private static void openCOMPort(XBee xbee) throws XBeeException, InterruptedException {
		Thread.sleep(200);
		String port = PropertiesFileReader.getInstance().getProperty("comport");
		xbee.open(port, 9600);
	}
	
	private XBeeComandResultObject parseFile(File filename, XBee xbees) {
		logger.debug("----> parseFile("+filename+", "+xbees+") ");
		XBeeComandResultObject retval = null;
		
        FileMonitorToBrickSingleton fileaccess = FileMonitorToBrickSingleton.getInstance();
//	            String fn = transformFileNameToString(filename.getName());
        String fn = filename.getAbsolutePath(); //  + System.getProperty("file.separator")+  filename.getName();
        boolean isCommand = fileaccess.isCommandFile(fn);
        boolean isHeartBeat = fileaccess.isFile(fn, XBeeConstants.COMMAND_heartBEAT);
        boolean isWARThere = isPresentationWarDeployed();

        if(isCommand && !isHeartBeat)
        {
        	// parse command
        	XBeeComandResultObject result = parseCommand(fileaccess, fn, xbees);
        	retval = result;
        	if(isWARThere){
        		writeResultJSONFile(result);
        	} else {
        		logger.warn("PRESENTATION WAR FILE MIGHT NOT BE DEPLOYED - DOC ROOT ["+ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION+"] IS NOT PRESENT ON FILE SYSTEM");
        	}
        }
        else if(isCommand && isHeartBeat){
    		logger.info("ITS A HEARTBEAT");
        	// parse command
        	Object result = parseCommand(fileaccess, fn, xbees);
        	if(result instanceof JSONXBeeCommandHeartBeatObject){
            	JSONXBeeCommandHeartBeatObject xrhbo = (JSONXBeeCommandHeartBeatObject) result;
            	retval = xrhbo;
        		if(isWARThere){
            		writeResultJSONFile(xrhbo);
            	} else {
            		logger.warn("PRESENTATION WAR FILE MIGHT NOT BE DEPLOYED - DOC ROOT ["+ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION+"] IS NOT PRESENT ON FILE SYSTEM");
            	}
       		
        	}else {
        		logger.error("ERROR: EXPECTED IS A XBeeResponseHeartBeatObject OBJ AS RESULT OF HEARTBEAT REQUEST - BUT RECEIVED ["+result+"] ["+result.getClass()+"]");
        		
        	}
        }
        else
        {
            // 1. read file content
            XBeeDataTransportObject xbee = fileaccess.readFileAndReturnDataObject(fn);
            
            // 2. create datagram sting
            String datagram = xbee.generateDataGram();
            getSend_fifo_buffer().add(datagram);
        }
		logger.debug("<---- parseFile("+filename+", "+xbees+") retval= "+retval);
		return retval;
	}

	// LOOKS INTO THE FILE AND PARSES IT
	private XBeeComandResultObject parseCommand(FileMonitorToBrickSingleton fileaccess, String fn, XBee xbee) {
		logger.debug("-----> parseCommand("+fn+", "+xbee+") ");
		XBeeComandObject cmd = fileaccess.readCommandFileAndReturnDataObject(fn);
		XBeeComandResultObject result = new XBeeComandResultObject(cmd);
		
		String higher = "0013A200";
		String lower = cmd.getBrickid().substring(0, 8);
		String addr = higher+lower;
		
		
		
		// CALL THE RIGHT SERVICE
		if(!isXBEERadioMounted){
			// THE RADIO MODULE IS NOT MOUNTED - SO INDICATE THIS ON tHE RESPONSE
			result.setCommand(cmd);
			result.setResponseValue("PAYLOAD: NULL\tRESPONSE: THE XBEE RADIO MODULE IS NOT PLUGGED IN - cHECK YOUR PHYSICAL BASE STATION SETUP" );
			result.setResponseOK(false);
		}
		if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("findall")){
			// find all nodes
			ArrayList<NodeDiscover> myNodes = null;;
			try {
				myNodes = SerialReceiverCommands.cmdFindNodes(xbee);
				String res = "NUMBER OF NODES FOUND: "+ myNodes.size() +"\t";
				String list = "<br>";
				for(int i = 0 ; i < myNodes.size(); i++){
					list = list + "   NODE: "+i+ " " + myNodes.get(i) + "<br>";
					if(i < myNodes.size()-1){
						list = list + "<br>";
					}
				}
				result.setResponseValue(res + list);
			} catch (XBeeException e) {
				// TODO Auto-generated catch block
				e.printStackTrace(); logger.error("SOME XBEE ERROR: ",e);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace(); logger.error("SOME ERROR: ",e);
			}
			for(int i = 0 ; i < myNodes.size(); i++){
				logger.info(i+"\t NODE: "+ myNodes.get(i));
			}
		}
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("blink10")){
			cmdBlinkPortDIO0_PIN20(addr, result);
		} 
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("ATID")){
			SerialReceiverCommandsNotNeeded.cmdAT_GENERIC(addr , "ID",result, xbee);
		} 
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("ATVR")){
			SerialReceiverCommandsNotNeeded.cmdAT_GENERIC(addr , "VR",result, xbee);
		} 
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("ATNI")){
			SerialReceiverCommandsNotNeeded.cmdAT_GENERIC(addr , "NI",result, xbee);
		} 
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("AT%V")){
			SerialReceiverCommandsNotNeeded.cmdAT_GENERIC(addr , "%V",result, xbee);
		} 
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("ATCH")){
			SerialReceiverCommandsNotNeeded.cmdAT_GENERIC(addr , "CH",result, xbee);
		} 
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("ADNR")){
			String portnumber = cmd.getAtcommandparameter();
			ArrayList<String> res =  SerialReceiverCommandsNotNeeded.cmdReadPortAD(addr, portnumber, xbee) ;
			String resAsStr = res.size() +" VALUES READ FROM THIS A/D CONVERTER <br>";
			for(int i = 0; i < res.size(); i++){
				resAsStr =resAsStr + res.get(i) + ", ";
				if(i> 0 && i % 20 == 0){
					resAsStr =resAsStr + "<br>";
				}
			}
			result.setResponseValue(resAsStr);
		} 
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("SHSL")){
			SerialReceiverCommands.cmdReadCoordinatorSHSL(xbee, cmd, result);
		} 
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("sendConstantFFFE")){	// test case to send a datagram
			SerialReceiverCommandsNotNeeded.sendConstantFFFE(cmd, result, xbee);
		} 
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("sendDataIncrement")){	// test case to send a datagram
			SerialReceiverCommandsNotNeeded.sendIncrementData(xbee, cmd, result, 500);
		} 
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("sendDataIncrementSlow")){	// test case to send a datagram
			SerialReceiverCommandsNotNeeded.sendIncrementData(xbee, cmd, result, 10000);
		} 
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase("sendPING")){	// test case to send a datagram
			SerialReceiverCommands.sendPING(cmd, result, 10000, xbee);
		} 	            	
		else if(cmd.getAtcommand().equalsIgnoreCase("sendSLIST")){	// test case to receive SENSORLIST
			if(cmd.getBrickid().equals(XBeeConstants.MODULE_ADDRESS_BASESTATION)){
				// ON THE RASPBERRY PI
				SerialReceiverCommands.sendSENSORLIST_BASIS(cmd);
			} 
			else if(isXBEERadioMounted && !cmd.getBrickid().equals(XBeeConstants.MODULE_ADDRESS_BASESTATION))
			{
				// VIA XBEE TO BRICK	
				SerialReceiverCommands.sendSENSORLIST(cmd, result, 10000, xbee);
			}
		} 	            	
		else if(cmd.getAtcommand().equalsIgnoreCase("sendPLIST")){	// test case to receive PORTLIST
			if(cmd.getBrickid().equals(XBeeConstants.MODULE_ADDRESS_BASESTATION)){
				SerialReceiverCommands.sendSENSORLIST_BASIS(cmd);
			} 
			else if(isXBEERadioMounted && !cmd.getBrickid().equals(XBeeConstants.MODULE_ADDRESS_BASESTATION))
			{
				// VIA XBEE TO BRICK	
				SerialReceiverCommands.sendPORTLIST(cmd, result, 10000, xbee);
			}
		} 	            	
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase(XBeeConstants.COMMAND_sendSPTB0)){	// test case to receive PORTLIST
			XBeeResponse response = SerialReceiverCommands.sendSETOUTPORTB0(cmd, result, 10000, xbee);
			if(response != null){
				result.setXbeeResultAsString(response.toString());
			} else {
				result.setXbeeResultAsString(XBeeConstants.RETURN_VALUE_RADIO_MODULE);
			}
		} 	            	
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase(XBeeConstants.COMMAND_sendSPTB1)){	// test case to receive PORTLIST
			XBeeResponse response = SerialReceiverCommands.sendSETOUTPORTB1(cmd, result, 10000, xbee);
			if(response != null){
				result.setXbeeResultAsString(response.toString());
			} else {
				result.setXbeeResultAsString(XBeeConstants.RETURN_VALUE_RADIO_MODULE);
			}			
		} 	            	
		
		else if(cmd.getAtcommand().equalsIgnoreCase(XBeeConstants.COMMAND_readOW)){	// test case to read OW
			if(cmd.getBrickid().equals(XBeeConstants.MODULE_ADDRESS_BASESTATION)){
				result = SerialReceiverCommands.sendREADOW_BASIS(cmd);
			} 
			else if(isXBEERadioMounted && !cmd.getBrickid().equals(XBeeConstants.MODULE_ADDRESS_BASESTATION))
			{
				// ITS FOR THE XBEE NETWORK
				SerialReceiverCommands.sendREADOW(cmd, result, 10000, xbee);
			}
		} 	            	
		
		else if(isXBEERadioMounted && cmd.getAtcommand().equalsIgnoreCase(XBeeConstants.COMMAND_readOP)){	// test case to read OUTPUT PORT
			XBeeResponse response = SerialReceiverCommands.sendREADOP(cmd, result, 10000, xbee);
			if(response != null){
				result.setXbeeResultAsString(response.toString());
			} else {
				result.setXbeeResultAsString(XBeeConstants.RETURN_VALUE_RADIO_MODULE);
			}	
		} 	
		
		else if(cmd.getAtcommand().equalsIgnoreCase(XBeeConstants.COMMAND_heartBEAT)){	// test case to heartbeat
			// IMPLEMENT THE QUEUE STATUS CHECK AND WRITE A JSON FILE WITH THE WHOLE INTO IT
			XBeeDataQueueSingleton toBrickQueue = XBeeDataQueueSingleton.getInstance();
			
			JSONQueueContainer heartbeatJSON = new JSONQueueContainer(toBrickQueue);
			
			heartbeatJSON.setTimeStampOfSending(cmd.getTimestamp());
			result.setXbeeResultAsString("HEARTBEAT: SOME STRING TO SAY THAT WE WORKED DILLIGENTLY");
			
			// RETURN SPECIAL RESULT OBJECT FOR HEARTBEAT (DOWNCASTED)
			JSONXBeeCommandHeartBeatObject result2 = new JSONXBeeCommandHeartBeatObject(result, heartbeatJSON);
			result = result2;
		} 	            	
		
		else
		{
			// should not go in here
			result.setResponseValue("ERROR: THIS COMMAND ["+cmd.getAtcommand()+"] IS NOT KNOWN!!!");
		}
		logger.debug("<----- parseCommand("+fn+", "+xbee+") result= "+result);
		return result;
	}

	

	
	/**
	 * WRITES AS OUTPUT THE QUEUES OF THE SERIAL MODULE:  "data_result_manage_brick_network_queues.json"
	 * @param result XBeeResponseHeartBeatObject
	 */
	private void writeResultJSONFile(JSONXBeeCommandHeartBeatObject result) {
		logger.debug("---> writeResultJSONFile()");
		result.setTimestampNow();
		
		result.setSerialUptimeString(this.getUptimeAsString());
		result.setSerialUptimeLongString(this.getUptimeAsLong() + "");
		result.setSerialStartTimeLongString(this.getStartTimestamp() + "");
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.mmm");
		String strstarime = sdf.format(new Timestamp(this.getStartTimestamp()));
		logger.info("SERIAL UPTIME ["+result.getSerialUptimeString()+"] OR IN ms ["+result.getSerialUptimeLongString()+"] - START TIME WAS: ["+strstarime+"]");
		
		// 2. bring result ds into right form
		ArrayList<XBeeComandResultObject> res = new ArrayList<XBeeComandResultObject>();
		res.add(result);
		
		// 3. Prep file
		BufferedWriter  bw = prepareFile("data_result_manage_brick_network_queues.json");
		
		// 4. serialize
		
		
		JSONSerializer serializer = new JSONSerializer().transform(new DateTransformer("yyyy-MM-dd HH:mm:ss"), Date.class); 
		String val =  serializer.include("jsonQueueContainer").
								include("command").
								exclude("jsonQueueContainer.toBrickDataList.filename.path").
								exclude("jsonQueueContainer.toBrickDataList.filename.path").
								exclude("jsonQueueContainer.toBrickDataList.filename.readable").
								exclude("jsonQueueContainer.toBrickDataList.filename.parentFile").
								exclude("jsonQueueContainer.toBrickDataList.filename.prefixLength").
								exclude("jsonQueueContainer.toBrickDataList.filename.totalSpace").
								exclude("jsonQueueContainer.toBrickDataList.filename.hidden").
								exclude("jsonQueueContainer.toBrickDataList.filename.lastModified").
								exclude("jsonQueueContainer.toBrickDataList.filename.invalid").
								exclude("jsonQueueContainer.toBrickDataList.filename.usableSpace").
								exclude("jsonQueueContainer.toBrickDataList.filename.directory").
								exclude("jsonQueueContainer.toBrickDataList.filename.executable").
								exclude("jsonQueueContainer.toBrickDataList.filename.canonicalPath").
								exclude("jsonQueueContainer.toBrickDataList.filename.writable").
								
								
								
								exclude("jsonQueueContainer.toBrickDataList.filename.freeSpace").
								exclude("jsonQueueContainer.toBrickDataList.filename.absolute").
								exclude("jsonQueueContainer.toBrickDataList.filename.file").
								
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.path").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.readable").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.parentFile").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.prefixLength").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.totalSpace").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.hidden").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.lastModified").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.invalid").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.usableSpace").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.directory").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.executable").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.canonicalPath").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.writable").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.freeSpace").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.absolute").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.file").
								
								exclude("*.class").deepSerialize( res );
		
		
		try {
			bw.write(val);
			bw.close();
			logger.info("WE WROTE JSON FILE data_result_manage_brick_network_queues.json SUCCESSFULLY");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			logger.error("ERROR WRITING JSON FILE POS 1: ", e);

		} catch (Exception ex) {
			// TODO Auto-generated catch block
			ex.printStackTrace();
			logger.error("ERROR WRITING JSON FILE POS 2: ", ex);

		} 
		finally {
			logger.debug("<--- writeResultJSONFile()");
			
		}
	}

	
	
	/**
	 * WRITES AS OUTPUT:  "data_result_manage_brick_network.json"
	 * @param result
	 */
	private void writeResultJSONFile(XBeeComandResultObject result) {
		logger.debug("---> writeResultJSONFile()");
		result.setTimestampNow();
		// 2. bring result ds into right form
		ArrayList<XBeeComandResultObject> res = new ArrayList<XBeeComandResultObject>();
		res.add(result);
		
		// 3. Prep file
		BufferedWriter  bw = prepareFile("data_result_manage_brick_network.json");
		
		// 4. serialize
		JSONSerializer serializer = new JSONSerializer().transform(new DateTransformer("yyyy-MM-dd HH:mm:ss"), Date.class); 
		String val =  serializer.exclude("*.class").serialize( res );
		
		try {
			bw.write(val);
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			logger.error("ERROR WRITING JSON FILE POS 1: ", e);

		} catch (Exception ex) {
			// TODO Auto-generated catch block
			ex.printStackTrace();
			logger.error("ERROR WRITING JSON FILE POS 2: ", ex);

		} 
		finally {
			logger.debug("<--- writeResultJSONFile()");
			
		}
	}
	
	
	

	
	
	
	
	
	
	
	
	
	
	
	

	/** get CoordinatorAddress
	 * @throws XBeeException
	 */
	protected synchronized String cmdSHSL(XBeeComandResultObject result)  {
		logger.info("-----> cmdSHSL()");
		XBee xbee = XBeeSingleton.getInstance().getXbee();
		String retval = "";
		try {
			
			openCOMPort(xbee);	
			String sh = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SH"))).getValue());
			String sl = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SL"))).getValue());
			
			retval = sh +","+ sl;
			
			// get the 8 byte SH/SL address
			logger.info("SH is " + sh);
			logger.info("SL is " + sl);
			
			logger.info("RETURN VALUE IS: " + retval);
			result.setResponseValue(retval);
			result.setResponseOK(true);
		
		} catch(Exception e){
			result.setResponseError(true);
			e.printStackTrace(); logger.error("SOME ERROR: ",e);
		} 
		finally {
			xbee.close();
			logger.info("<----- cmdSHSL()");
		}
		return retval;
	}

	

	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	

	

	
	


	public Queue<String> getSend_fifo_buffer() {
		return send_fifo_buffer;
	}

	/**
	 * Opens the file stream - with docroot and file name
	 * @param filename
	 * @return
	 */
	protected BufferedWriter  prepareFile(String filename){
		BufferedWriter bw = null;
		if(isPresentationWarDeployed()){
			
			try {
		    	String docroot = ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION;
	
		    	logger.debug("prepareFile("+filename+") to: "+ docroot+filename);
		    	File file = new File(docroot+filename);
		
				// if file doesnt exists, then create it
				if (!file.exists()) {
						file.createNewFile();
				}
		
				FileWriter fw = new FileWriter(file.getAbsoluteFile());
				bw = new BufferedWriter(fw);
			
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace(); logger.error("ERROR WRITING FILE ["+filename+"]: ",e);
			}		
		}
		return bw;
	}	

	
	
	
	protected boolean isPresentationWarDeployed(){
		boolean retval = false;
    	String docroot = null;
    	docroot = ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION;
		if(docroot == null ){
			logger.error("ERROR: docroot PROPERTY IS NOT CONFIGURED - IT IS NULL - CHECK FILE tmor.properties");
			throw new RuntimeException("ERROR: docroot PROPERTY IS NOT CONFIGURED - IT IS NULL - CHECK FILE tmor.properties");
		} else {
			try{
				File file = new File(docroot);
				if(file.isDirectory()){
					retval = true;
				}
			} catch (Exception ioe){
				logger.error("ERROR: APPLICATION NOT PROPERLY CONFIGURED - CHECK property docroot in FILE tmor.properties", ioe);
				throw new RuntimeException("ERROR: docroot PROPERTY IS NOT CONFIGURED - IT IS NULL - CHECK FILE tmor.properties");
			}
		}
		return retval;
	}
	
	
	
	protected static boolean isPresentationWarDeployedStatic(){
		boolean retval = false;
    	String docroot = null;
    	docroot = ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION;
		if(docroot == null ){
			logger.error("ERROR: docroot PROPERTY IS NOT CONFIGURED - IT IS NULL - CHECK FILE tmor.properties");
			throw new RuntimeException("ERROR: docroot PROPERTY IS NOT CONFIGURED - IT IS NULL - CHECK FILE tmor.properties");
		} else {
			try{
				File file = new File(docroot);
				if(file.isDirectory()){
					retval = true;
				}
			} catch (Exception ioe){
				logger.error("ERROR: APPLICATION NOT PROPERLY CONFIGURED - CHECK property docroot in FILE tmor.properties", ioe);
				throw new RuntimeException("ERROR: docroot PROPERTY IS NOT CONFIGURED - IT IS NULL - CHECK FILE tmor.properties");
			}
		}
		return retval;
	}
	
	
	
	
	
	
	
	
	
	
	
	public void setSend_fifo_buffer(Queue<String> fifo_buffer) {
		send_fifo_buffer = fifo_buffer;
	}  
	
	
	
	
	/**SEND THE PINg COMMAND TO BRICK 
	 * @param cmd
	 * @param result
	 * @param waittime in ms
	 * @return
	 */
	protected synchronized String xxxsendPING(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee)  {
			logger.info("-----> sendPING()");
//			XBee xbee = XBeeSingleton.getInstance().getXbee();
			String retval = "";
			result.setCommand(cmd);
			try {
//				openCOMPort(xbee);	
				String destbrickID = cmd.getBrickid();
				logger.info("DEST BRICK ID: "+ destbrickID);

				int[] payload = null;
				if(cmd.getCorrelID() == null|| cmd.getCorrelID().isEmpty()){
					payload = new int[] { 0x50, 0x49 , 0x4E, 0x47, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20};
				} 
				else
				{
					int correlIdLength = cmd.getCorrelID().length();
					
					payload = new int[] { 0x50, 0x49 , 0x4E, 0x47, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20};
					for(int i = 0; i < correlIdLength; i++){
						char digitstr = cmd.getCorrelID().charAt(i);
						
						int digitnr = (int)digitstr;
						payload[5+i] = digitnr;		// char as int value into the payload
					}
					payload[5+correlIdLength] = 0x5D;	// close the parameter bracket ]  
				}
				
				
				XBeeAddress64 dest64 = new XBeeAddress64(XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

				ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
				logger.info("PAYLOAD: "+ ByteUtils.toString(payload));
					
				// loopback test
				int sourceEndpoint = 0;
				int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA.getValue();
				
				
				logger.debug("       DESTINATION: " + cmd.getBrickid());
				logger.debug("       START SENDING");

				DoubleByte clusterId = new DoubleByte(0x0, ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
				//DoubleByte clusterId = new DoubleByte(0x0, ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());
				
				// first request we just send 64-bit address.  we get 16-bit network address with status response
				ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff, dest64, XBeeAddress16.ZNET_BROADCAST, 
							ZNetTxRequest.DEFAULT_BROADCAST_RADIUS, ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint, destinationEndpoint, clusterId, ZNetExplicitTxRequest.znetProfileId);
				
				logger.info("sending explicit " + request.toString());
				
				xbee.sendAsynchronous(req);
//				xbee.sendAsynchronous(request);
				XBeeResponse response = xbee.getResponse();
				// XBeeResponse response = xbee.sendSynchronous(request);
				result.setResponseValue("PAYLOAD: "+ ByteUtils.toString(payload)+ "   RESPONSE:" + response.toString());
				result.setResponseOK(true);
				
				logger.info("received response " + response.toString());
			} catch (Exception e) {
				e.printStackTrace(); logger.error("SOME ERROR: ",e);
			} 
			finally {
				// xbee.close();
				logger.info("<----- sendPING()");
			}
			return retval;
		}
	
	
	
	
	
	


	private static ArrayList<FileDescDataObject> createFileListOfFolder(String directory) {
		// PrepareDataDownload pdd = new PrepareDataDownload();
		File f = new File(directory);
		ArrayList<FileDescDataObject> fileinfolist = new ArrayList<FileDescDataObject>();

		ArrayList<File> files = new ArrayList<File>(Arrays.asList(f.listFiles()));
		Iterator<File> files_it = files.iterator();
		while(files_it.hasNext()){
			File fi = files_it.next();
			if(fi.isFile()){
				FileDescDataObject ne = new FileDescDataObject();
				ne.setName(fi.getName());
				ne.setSize(fi.length());
				ne.setCreatedtimestamp(fi.lastModified());
				fileinfolist.add(ne);
			}
		}
		return fileinfolist;
	}

	
		/**
		 * Receives the data from XBee /UART
		 * @author Frederic
		 *
		 */
		class XBeeReceiverThread extends Thread {
			private Thread t;
			private String threadName;
				   
			// 1. STEP
			XBeeReceiverThread( String name){
				threadName = name;
				System.out.println("Creating " +  threadName );
			}
			
			// 2. STARTS
			public void start ()
			{
				System.out.println("Starting " +  threadName );
				if (t == null)
				{
					t = new Thread (this, threadName);
					t.start ();
				}
			}
			   
			   
			// 2. RUNS
			public void run() {
				System.out.println("Running " +  threadName );

				while(true){
				// POLLING FOR RECEIVING DATA
					XBee xbee = null;
					try{
						xbee = XBeeSingleton.getInstance().getXbee();

						openCOMPort(xbee);
						XBeeResponse response = xbee.getResponse(2000);
						int []i = response.getRawPacketBytes();
						System.out.println("DATA RECEIVED START -------------------------------------------------------------------------------------------------");

						for(int j = 0; j < i.length; j++){
							System.out.print(" "+i[j]);
						}
						System.out.println("\n------------------------------------------------------------------------------------------------- DATA RECEIVED END");
					} catch(XBeeTimeoutException exc){
						// System.out.print(" TIMEOUT - NOT TO WORRY ");
					} catch(Exception exc){
						exc.printStackTrace();
					} finally{
						xbee.close();
					}
				}
			}
			private void nioFileListenerToBrick() {
				// TODO Auto-generated method stub
				Path dir = FileSystems.getDefault().getPath("C:\\TEMP\\sensors\\filetrigger\\tobrick");
				WatchKey key = null;
				
				try {
					WatchService watcher = FileSystems.getDefault().newWatchService();
				    key = dir.register(watcher, ENTRY_CREATE, ENTRY_DELETE /*, ENTRY_MODIFY*/);
				    
				    
				    for(;;){
				    	try {
				            key = watcher.take();
				        } catch (InterruptedException x) {
				            return;
				        }
				    	
				    	
				    	for (WatchEvent<?> event: key.pollEvents()) {
				            WatchEvent.Kind<?> kind = event.kind();
				            System.out.format("kind  %s%n", kind);

				            // This key is registered only
				            // for ENTRY_CREATE events,
				            // but an OVERFLOW event can
				            // occur regardless if events
				            // are lost or discarded.
				            if (kind == OVERFLOW) {
				                continue;
				            }
				            
				            if (kind == ENTRY_CREATE) {
					            WatchEvent<Path> ev = (WatchEvent<Path>)event;
					            Path filename = ev.context();

				            	System.out.format("ITS AN ENTRY_CREATE EVENT %s%n ", filename);
				            	System.out.println("********************************************************************************");
				            	System.out.println("********************************************************************************");
				            	System.out.println("********************************************************************************");
				            	System.out.println("********************************************************************************");
				            	
				            	// SerialReceiver6 mb = new SerialReceiver6();
				            	// mb.parseFile(filename.toFile(), getXbeeSingleton());
				            	
				            	System.out.format("THIS IS THE SINGLETON %s%n ", getXbeeSingleton());
				            	
				            	
				            	// goWasteTime("FILE");
				            	continue;
				            }
				            
				            else if(kind == ENTRY_DELETE){
					            WatchEvent<Path> ev = (WatchEvent<Path>)event;
					            Path filename = ev.context();

				            	System.out.format("ITS AN ENTRY_DELETE EVENT %s%n ", filename);
				            	continue;
				            	
				            }
				            else {
				            	System.out.println("THIS EVENT TYPE IS NOT SUPPORTED");
				            	
				            }

				            // The filename is the
				            // context of the event.
				            WatchEvent<Path> ev = (WatchEvent<Path>)event;
				            Path filename = ev.context();

				            // Verify that the new
				            //  file is a text file.
				            try {
				                // Resolve the filename against the directory.
				                // If the filename is "test" and the directory is "foo",
				                // the resolved name is "test/foo".
				                Path child = dir.resolve(filename);
				                if (!Files.probeContentType(child).equals("text/plain")) {
				                    System.err.format("New file '%s'" + " is not a plain text file.%n", filename);
				                    continue;
				                }
				            } catch (IOException x) {
				                System.err.println(x);
				                continue;
				            }

				            // Email the file to the
				            //  specified email alias.
				            System.out.format("Emailing file %s%n", filename);
				            //Details left to reader....
				        }

				        // Reset the key -- this step is critical if you want to
				        // receive further watch events.  If the key is no longer valid,
				        // the directory is inaccessible so exit the loop.
				        boolean valid = key.reset();
				        if (!valid) {
				            break;
				        }
				    }		    	
				} catch (IOException x) {
				    System.err.println(x);
				}
			}
		}
		
		
		/**
		 * Receives the data from XBee /UART
		 * @author Frederic
		 *
		 */
		


		private static Timestamp currentTimestamp() {
			Date d = new java.util.Date();
			Timestamp now = new Timestamp(d.getTime());
			return now;
		}		
		
		
		
		
		public void run() {
			long startime = System.currentTimeMillis();
			System.out.println("=========================> start Thread()");
			// Thread.
			// nioFileListenerToBrick();

			
			
			long endtime = System.currentTimeMillis();
			BigDecimal timeDiffMS = new BigDecimal(endtime - startime);
			BigDecimal timeDiffS = timeDiffMS.divide(new BigDecimal(1000), MathContext.DECIMAL32);
			
			System.out.println("DURATION ="+ timeDiffS+ " s");

			System.out.println("<========================= end Thread() DURATION ="+ timeDiffS+ " s");
		}
		
		
		

		
		



		public static XBee getXbeeSingleton() {
			if(xbeeSingleton == null){
				xbeeSingleton = XBeeSingleton.getInstance().getXbee();
				try{
					Thread.sleep(10);
				}catch(Exception d){
					logger.error(d);
				}
			}
			return xbeeSingleton;
		}


		public static void setXbeeSingleton(XBee xbeeSingleton) {
			SerialReceiver8.xbeeSingleton = xbeeSingleton;
		}	
		
		
		private String transformStringToValid64BitXBeeAddress(String addr) {
			String retval = "";
			
			// ha hb hc hd he hf hg hh la lb lc ld le lf lg lh 
			if(addr.contains(" ") && addr.length() == 47){
				// do nothing - its already in the right format
				 retval = addr;
			} else {
				// format it to 
				// ha hb hc hd he hf hg hh la lb lc ld le lf lg lh
				if(addr.length() == 16){
					// yes every thing is ok la
					for(int i = 0; i < 16; i++){
						retval = retval + addr.charAt(i);
						if((i+1) % 2 == 0 ){
							retval = retval + " ";
						}
					}
				}
			}
			return retval;
		}		
		
		
		
		private static void goWasteTime(long ms) {
			logger.debug("-----> goWasteTime("+ms+")");
			long start = System.currentTimeMillis();
			try{
				Thread.sleep(ms);
			} catch(Exception e){
				logger.error("ERROR WHILE SLEEPING!!!", e);
				e.printStackTrace();
			}
			long end = System.currentTimeMillis();

			BigDecimal res = new BigDecimal((end-start)).divide(new BigDecimal(1000), RoundingMode.FLOOR); 
			logger.debug("<----- goWasteTime("+ms+") \tWASTED = ["+res+"] s");
		}


		public long getStartTimestamp() {
			return startTimestamp;
		}


		public void setStartTimestamp(long startTimestamp) {
			this.startTimestamp = startTimestamp;
		}
		
		
		
		public long getUptimeAsLong() {
			long now = System.currentTimeMillis();
			
			long durationL = now - this.getStartTimestamp();
			return durationL;
		}
		
		/** 
		 * @return UPTIME: 151 days 21:11:49
		 */
		public String getUptimeAsString() {
			
			long durationL = getUptimeAsLong();

			System.out.println("durationL "+ durationL);
			

			
			// 1 day = 1000 ms * 60 * 60 * 24
			long days = durationL / (1000 * 60 * 60 * 24);

			long restofdayMS =  durationL - (days * (1000 * 60 * 60 * 24) );
			
			long hours = restofdayMS /(1000 * 60 * 60);
			
			long restofHourMS = restofdayMS -  hours  * (1000 * 60 * 60);
			
			long min = restofHourMS /(1000 * 60 );

			long restofMinuteMS = restofHourMS -  min  * (1000 * 60);

			long sec = restofMinuteMS / (1000 );
			
			String uptime = days +" days "; 
			
			if (hours < 10){
				uptime += "0"+hours+":";
			} else {
				uptime += +hours+":";
			}
			
			if(min < 10){
				uptime += "0"+min+":";
			} else {
				uptime += +min+":";
			}
			if(sec < 10){
				uptime += "0"+sec;
			} else {
				uptime += +sec;
			}
			
			System.out.println("days "+ days +"\thours "+hours+"\tmin "+min+"\tsec "+sec);
			
			System.out.println("UPTIME: "+uptime);
	
			
			return uptime;
		}

}
