package com.moneyhouse.sensors.xbee;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Iterator;

import org.apache.log4j.Logger;

import com.moneyhouse.sensors.config.CONTSTANTINTERFACE;
import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.filetrigger.FileMonitorFromBrickSingleton;
import com.moneyhouse.sensors.internal.idgenerator.IDGenerator;
import com.moneyhouse.sensors.pi.DSSensor;
import com.moneyhouse.sensors.pi.dto.DSSensorResultDO;
import com.moneyhouse.sensors.util.TimeStampProducer;
import com.moneyhouse.sensors.xbee.dto.XBeeComandObject;
import com.moneyhouse.sensors.xbee.dto.XBeeComandResultObject;
import com.rapplogic.xbee.api.ApiId;
import com.rapplogic.xbee.api.AtCommand;
import com.rapplogic.xbee.api.AtCommandResponse;
import com.rapplogic.xbee.api.PacketListener;
import com.rapplogic.xbee.api.XBee;
import com.rapplogic.xbee.api.XBeeAddress16;
import com.rapplogic.xbee.api.XBeeAddress64;
import com.rapplogic.xbee.api.XBeeException;
import com.rapplogic.xbee.api.XBeeFrameIdResponse;
import com.rapplogic.xbee.api.XBeeResponse;
import com.rapplogic.xbee.api.zigbee.NodeDiscover;
import com.rapplogic.xbee.api.zigbee.ZNetExplicitTxRequest;
import com.rapplogic.xbee.api.zigbee.ZNetTxRequest;
import com.rapplogic.xbee.api.zigbee.ZNetTxStatusResponse;
import com.rapplogic.xbee.util.ByteUtils;
import com.rapplogic.xbee.util.DoubleByte;

/**
 * CONTAINS ALL THE COMMANDS THAT ARE SENT TO THE XBEE RADIO IT WORKS IN
 * CONJUNCTION WITH SerialReceiver
 * 
 * @since 05.11.2014
 * @author Frederic
 * 
 */
public class SerialReceiverCommands {
	private final static Logger logger = Logger
			.getLogger(SerialReceiverCommands.class);

	/**
	 * SEND THE PORTLIST REQUEST COMMAND TO BRICK
	 * 
	 * @param cmd
	 * @param result
	 * @param waittime
	 *            in ms
	 * @return
	 */
	public static synchronized String sendPORTLIST(XBeeComandObject cmd,
			XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("-----> sendPORTLIST()");
		// XBee xbee = XBeeSingleton.getInstance().getXbee();
		String retval = "";
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);

			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x50, 0x4C, 0x49, 0x53, 0x54, 0x20, 0x20,	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {
				int correlIdLength = cmd.getCorrelID().length();

				payload = new int[] { 0x50, 0x4C, 0x49, 0x53, 0x54, 0x5B, 0x20,	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[6 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[6 + correlIdLength] = 0x5D; // close the parameter
													// bracket ]
			}

			XBeeAddress64 dest64 = new XBeeAddress64(
					XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			XBeeResponse response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);

			logger.info("RECEIVED RESPONSE " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<----- sendPORTLIST()");
		}
		return retval;
	}

	
	
	/**
	 * SEND THE SENSORLIST REQ COMMAND TO BASIS STATION - NOT THE BRICK
	 * WRITES THE RESULT IMMEDIATLY INTO FILE
	 * @param cmd
	 * @since 13.07.2015
	 */
	public static synchronized void sendSENSORLIST_BASIS(XBeeComandObject cmd) {
		// ON THE BASESTATION RASPBERRY PI
		DSSensor dss = new DSSensor();
		ArrayList<String> sensorlist = dss.readSensorList();
		
		String correlID = cmd.getCorrelID().substring(0, cmd.getCorrelID().length()-1);
		
		for(int i = 0; i < sensorlist.size(); i++){
			// write result
			String correl = correlID + i;
			String nomde = CONTSTANTINTERFACE.MODULE_ADDRESS_BASESTATION;
			String comanddo = "SLIST";
			String filename = "FROM_BRICK__"+nomde+"___CORR_"+correl+"__CMD_"+comanddo+"___" + new IDGenerator().getNewID()+".txt";
			String pathfilename = PropertiesFileReader.getInstance().getProperty("filetrigger.dir.brickconfig");
			
			pathfilename = pathfilename + filename;
			logger.debug("send command by writing file into  "+pathfilename);
			Timestamp tsnow = TimeStampProducer.createTimestampForTimeZone("UTC");
			String tsnowstr = tsnow.toString();
			logger.debug("UTC tsnowstr "+tsnowstr);
			String content =nomde+"\t"+comanddo+ "\t" +correl+"\t"+"OW"+"\t"+sensorlist.get(i)+"\t"+tsnowstr+"\tEXECUTIONTIME=";
			
			FileMonitorFromBrickSingleton fm = FileMonitorFromBrickSingleton.getInstance();
			fm.createFileAsEvent(pathfilename, content);
		}
	}	
	
	
	
	
	/**
	 * SEND THE PORTLIST REQ COMMAND TO BRICK
	 * 
	 * @param cmd
	 * @param result
	 * @param waittime
	 *            in ms
	 * @return
	 */
	public static synchronized String sendSENSORLIST(XBeeComandObject cmd,
			XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("-----> sendSENSORLIST()");
		String retval = "";
		result.setCommand(cmd);
		try {
			logger.info("DEST BRICK ID: " + cmd.getBrickid());

			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x53, 0x4C, 0x49, 0x53, 0x54, 0x20, 0x20,	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
						0x20, 0x20, 0x20, 0x20 };
			} else {
				int correlIdLength = cmd.getCorrelID().length();

				payload = new int[] { 0x53, 0x4C, 0x49, 0x53, 0x54, 0x5B, 0x20,	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
						0x20, 0x20, 0x20, 0x20 };
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[6 + i] = digitnr; // char as int value into the payload
				}
				payload[6 + correlIdLength] = 0x5D; // close the parameter bracket ]
			}

			XBeeAddress64 dest64 = new XBeeAddress64(
					XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			XBeeResponse response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload) + "   RESPONSE:" + response.toString());
			result.setResponseOK(true);

			logger.info("RECEIVED RESPONSE " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			logger.info("<----- sendSENSORLIST()");
		}
		return retval;
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	/**SEND THE PINg COMMAND TO BRICK 
	 * @param cmd
	 * @param result
	 * @param waittime in ms
	 * @return
	 */
	public static synchronized String sendPING(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("-----> sendPING()");
		// XBee xbee = XBeeSingleton.getInstance().getXbee();
		String retval = "";
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);

			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x50, 0x49, 0x4E, 0x47, 0x20, 0x20, 0x20,
						0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
						0x20, 0x20, 0x20, 0x20 };
			} else {
				int correlIdLength = cmd.getCorrelID().length();

				payload = new int[] { 0x50, 0x49, 0x4E, 0x47, 0x5B, 0x20, 0x20,
						0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
						0x20, 0x20, 0x20, 0x20 };
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
													// bracket ]
			}

			XBeeAddress64 dest64 = new XBeeAddress64(
					XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("sending explicit " + request.toString());

			xbee.sendAsynchronous(req);
			// xbee.sendAsynchronous(request);
			XBeeResponse response = xbee.getResponse();
			// XBeeResponse response = xbee.sendSynchronous(request);
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)
					+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);

			logger.info("received response " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<----- sendPING()");
		}
		return retval;
	}
	
	
	
	
	
	
	/** 
	 * READ THE COORDINATOR ADDRESS
	 * @since 20160129
	 * @author Frederic

	 */
	public static synchronized void cmdReadCoordinatorSHSL(XBee xbee,XBeeComandObject cmd,  XBeeComandResultObject result)	 {
		logger.info("-----> cmdReadCoordinatorSHSL()");
		String retval = "";
		
		result.setCommand(cmd);
		try {
			String sh = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SH"))).getValue());
			String sl = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SL"))).getValue());
			
			retval = sh +","+ sl;
			
			// get the 8 byte SH/SL address
			logger.info("SH is " + sh);
			logger.info("SL is " + sl);
			
			logger.info("RETURN VALUE IS: " + retval);
			result.setResponseValue("PAYLOAD: " + retval + "   RESPONSE:" +sh +sl);
			result.setResponseOK(true);
		} catch(Exception e){

			e.printStackTrace(); 
			logger.error("ERROR: WHILE READING COORDINATOR ADDRESS WE RECEIVED THE FOLLOWING ERROR",e);
		} 
		finally {
			logger.info("<----- cmdReadCoordinatorSHSL()");
		}
		//return retval;
	}	
	
	
	
	
	
	
	
	
	

	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	/**
	 * SEND THE VALUE TO PUT ON PORT B0
	 * 
	 * @since 20141127
	 * @param cmd
	 * @param result
	 * @param waittime
	 *            in ms
	 * @return
	 */
	public static synchronized XBeeResponse sendSETOUTPORTB0(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("------> sendSETOUTPORTB0()");
		// XBee xbee = XBeeSingleton.getInstance().getXbee();
		XBeeResponse retval = null;
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);
			int correlIdLength = cmd.getCorrelID().length();

			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x53, 0x50, 0x52, 0x54, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {				// S     P     R     T	

				payload = new int[] { 0x53, 0x50, 0x52, 0x54, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
			}
			// add the port B0 
			payload[6 + correlIdLength] = 0x5B; // open the parameter [
			payload[7 + correlIdLength] = 0x30; // port address 0
			payload[8 + correlIdLength] = 0x5D; // close the parameter ]
			payload[9 + correlIdLength] = 0x5B; // open the parameter [
			if(cmd.getAtcommandparameter().equals("0")){
				payload[10 + correlIdLength] = 0x30; // port value 0
			} else {
				payload[10 + correlIdLength] = 0x31; // port value 1
			}
			payload[11 + correlIdLength] = 0x5D; // close the parameter ]
			

			XBeeAddress64 dest64 = new XBeeAddress64(XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			XBeeResponse response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);
			retval = response;

			logger.info("RECEIVED RESPONSE " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<------ sendSETOUTPORTB0() retval= "+retval);
		}
		return retval;
	}
	
	
	
	
	/**
	 * SEND THE VALUE TO PUT ON PORT B1
	 * 
	 * @since 20141127
	 * @param cmd
	 * @param result
	 * @param waittime
	 *            in ms
	 * @return
	 */
	public static synchronized XBeeResponse sendSETOUTPORTB1(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("------> sendSETOUTPORTB1()");
		// XBee xbee = XBeeSingleton.getInstance().getXbee();
		XBeeResponse retval = null;
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);
			int correlIdLength = cmd.getCorrelID().length();

			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x53, 0x50, 0x52, 0x54, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {				// S     P     R     T	

				payload = new int[] { 0x53, 0x50, 0x52, 0x54, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
			}
			// add the port B0 
			payload[6 + correlIdLength] = 0x5B; // open the parameter [
			payload[7 + correlIdLength] = 0x31; // port address 0
			payload[8 + correlIdLength] = 0x5D; // close the parameter ]
			payload[9 + correlIdLength] = 0x5B; // open the parameter [
			if(cmd.getAtcommandparameter().equals("0")){
				payload[10 + correlIdLength] = 0x30; // port value 0
			} else {
				payload[10 + correlIdLength] = 0x31; // port value 1
			}
			payload[11 + correlIdLength] = 0x5D; // close the parameter ]
			

			XBeeAddress64 dest64 = new XBeeAddress64(
					XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			XBeeResponse response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);
			retval = response;

			logger.info("RECEIVED RESPONSE " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<------ sendSETOUTPORTB1()");
		}
		return retval;
	}

	
	/**
	 * ASSEBLE QUERY THE BASIS FILe DIRECTORY
	 * @param cmd
	 * @since 13.07.2015
	 */
	public static synchronized  XBeeComandResultObject  sendREADOW_BASIS(XBeeComandObject cmd) {
		logger.info("---> sendREADOW_BASIS("+cmd+")");
		// ITS ON THE PI BASIS-STATION
		XBeeComandResultObject retVal = new XBeeComandResultObject(cmd);
		DSSensor dss = new DSSensor();
		ArrayList<String> sensorlist = dss.readSensorList();
		logger.info("OW SENSORLIST BASESTATION HAS A SIZE Of ["+sensorlist.size()+"]");
		for(int i = 0 ; i < sensorlist.size(); i++){
			logger.info("["+i+"] SENSOR WITH ID ["+sensorlist.get(i)+"]");
		}
		
		//		sensorlist.add("28-000004475b0d");
		//		sensorlist.add("28-00000460d5b1");
		//	280D5B4704000049 SENSOR ID FORMAT ON XBEE MODULE
		
		String sensorID= cmd.getAtcommandparameter();
		if(sensorID.length() != "28-000004475b0d".length()){
			logger.error("ERROR: IT SEEMS THE OW SENSOR ADDRESS ["+sensorID+"] FOR BASESTATION IS NOT IN THE SAME LENGTH AS 28-000004475b0d");
			throw new RuntimeException("ERROR: IT SEEMS THE OW SENSOR ADDRESS ["+sensorID+"] FOR BASESTATION IS NOT IN THE SAME LENGTH AS 28-000004475b0d");
		}
		
		DSSensorResultDO result = dss.readSensorValueWithID(sensorID);
		retVal.setResponseOK(true);
		retVal.setResponseValue("PAYLOAD: BASE STATION SENSOR "+result.getSensorID()+"\tRESPONSE: BASE STATION SENSOR READING: "+ result.getReadingValueStr());
		// write result
		String nomde = CONTSTANTINTERFACE.MODULE_ADDRESS_BASESTATION;
		String comanddo = "RDOW";
		String filename = "FROM_BRICK__"+nomde+"___CORR_"+cmd.getCorrelID()+"__CMD_"+comanddo+"___" + new IDGenerator().getNewID()+".txt";
		String pathfilename = PropertiesFileReader.getInstance().getProperty("filetrigger.dir.frombrick");
		
		pathfilename = pathfilename + filename;
		logger.debug("send command by writing file into  "+pathfilename);
//		Timestamp tsnow = new Timestamp(System.currentTimeMillis());
//		String tsnowstr = tsnow.toString();

		String tsnowstr = TimeStampProducer.createTimestampWithTimeZone("UTC");
		logger.debug("UTC tsnowstr "+tsnowstr);
		String content =nomde+"\t"+comanddo+ "\t" +cmd.getCorrelID()+"\t"+result.getReadingValueStr()+"\t"+"\t"+tsnowstr+"\tEXECUTIONTIME=";
		
		FileMonitorFromBrickSingleton fm = FileMonitorFromBrickSingleton.getInstance();
		fm.createFileAsEvent(pathfilename, content);	
		logger.info("<--- sendREADOW_BASIS("+cmd+")");
		return retVal;

	}	
	
	
	
	
	
	/**
	 * ASSEBLE THE XBEE COMMAND AND SEND IT OFF VIA THE COORDINATOR 
	 * @param cmd
	 * @param result
	 * @param waittime
	 * @param xbee
	 * @return
	 */
	public static synchronized String sendREADOW(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("-----> sendREADOW()");
		// XBee xbee = XBeeSingleton.getInstance().getXbee();
		String retval = "";
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);
			int correlIdLength = cmd.getCorrelID().length();

			String owaddress = cmd.getAtcommandparameter();
			if(owaddress ==null || owaddress.isEmpty() || owaddress.length() < 10){
				// TAKE THE STANDARD OW ADDRESS
				owaddress = "28040F0105000017";
				logger.warn("OW ADDRESS FROM tHE COMMAND PARAMETERS IS NULL OR EMPTY OR LENGTH < 10 - WE USE DEFAULT OW ADDRESS 28040F0105000017");

				
			}
			
			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x52, 0x44, 0x4F, 0x57, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {				// R     D     O     W	

				payload = new int[] { 0x52, 0x44, 0x4F, 0x57, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 , 0x20};
									// S     D     O     W     [     C     O     R      R     E    L	 I      D     ]     [   OW1   OW2   OW3   OW4   OW5   OW6   OW7   OW8   OW9   OW10  OW11  OW12  OW13  OW14  OW15  OW16   ]     CHCKS
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
			}
			// add the port B0 
			payload[6 + correlIdLength] = 0x5B; // open the parameter [
			
			int pos = 6 + correlIdLength;
			char[] oqaddressChar = owaddress.toCharArray();
			for(char b: oqaddressChar){
				pos++;
				payload[pos] = b; 				// 16 character OW address code
			}
			
			payload[pos + 1] = 0x5D; // close the parameter ]
			
			
			XBeeAddress64 dest64 = new XBeeAddress64(XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));
			logger.info("PAYLOAD LENGTH: " + payload.length);

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			Object response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);

			if(response instanceof ZNetTxStatusResponse){
				System.out.println("ZNetTxStatusResponse");
				ZNetTxStatusResponse zResponse = (ZNetTxStatusResponse)response;
				if(zResponse.getDeliveryStatus().equals(ZNetTxStatusResponse.DeliveryStatus.ADDRESS_NOT_FOUND)){
					// 
					XBeeDataQueueSingleton sing = XBeeDataQueueSingleton.getInstance();
					sing.moveFromToBrickToErrorListContainingCharSequence(cmd.getCorrelID(),CONTSTANTINTERFACE.ADDRESS_NOT_FOUND);
					logger.warn("XBEE DELIVERY STATUS: ADDRESS NOT FOUND");
					
				} else if(zResponse.getDeliveryStatus().equals(ZNetTxStatusResponse.DeliveryStatus.SUCCESS)){
					logger.info("XBEE DELIVERY STATUS: DELIVERY SUCCESSFUL");
				}
			} else if(response instanceof XBeeFrameIdResponse){
				logger.info("XBeeFrameIdResponse");
				
			} else if(response instanceof XBeeResponse){
				logger.info("XBeeResponse");
			
			} else {
				
				logger.info("???? ["+response.toString()+"]");
			}
			
			logger.info("OW RECEIVED RESPONSE " + response.toString());
			// response.getDeliveryStatus();
//			  ApiId ff = response.getApiId();
			  // response.
//			  System.out.println("FF = "+ff);
			  
			  
		} catch (Exception e) {
			logger.error("ERROR: SOME ERROR WIHLE RECEIVING",e);
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<----- sendREADOW()");
		}
		return retval;
	}

	
	
	
	
	
	
	
	
	
	
	
	
	/** SENDS A READ OUTPUT PORT VALUE - TO SEE WHETHER THE OUTPUT PORT IS O OR ONE
	 * @param cmd
	 * @param result
	 * @param waittime
	 * @param xbee
	 * @return
	 */
	public static synchronized XBeeResponse sendREADOP(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("------> sendREADOP()");
		XBeeResponse retval = null;
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);
			int correlIdLength = cmd.getCorrelID().length();

			String owaddress = cmd.getAtcommandparameter();
			if(owaddress ==null || owaddress.isEmpty() || owaddress.length() < 10){
				// TAKE THE STANDARD OW ADDRESS
				owaddress = "28040F0105000017";
			}
			
			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x52, 0x44, 0x4F, 0x50, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {				// R     D     O     P	

				payload = new int[] { 0x52, 0x44, 0x4F, 0x50, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 , 0x20};
									// S     D     O     P     [     C     O     R      R     E    L	 I      D     ]     [   OW1   OW2   OW3   OW4   OW5   OW6   OW7   OW8   OW9   OW10  OW11  OW12  OW13  OW14  OW15  OW16   ]     CHCKS
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
			}
			// add the port B0 
//			payload[6 + correlIdLength] = 0x5B; // open the parameter [
			
//			int pos = 6 + correlIdLength;
//			char[] oqaddressChar = owaddress.toCharArray();
//			for(char b: oqaddressChar){
//				pos++;
//				payload[pos] = b; 				// 16 character OW address code
//			}
			
//			payload[pos + 1] = 0x5D; // close the parameter ]
			
			
			XBeeAddress64 dest64 = new XBeeAddress64(
					XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));
			logger.info("PAYLOAD LENGTH: " + payload.length);

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			XBeeResponse response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);
			retval = response;

			logger.info("RECEIVED RESPONSE " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<------ sendREADOP()");
		}
		return retval;
	}



	/** 
	 * FIND ALL XBEE RADIOS
	 * @param xbee
	 * @return ArrayList<NodeDiscover>
	 * @throws XBeeException
	 * @throws InterruptedException
	 */
	public static synchronized ArrayList<NodeDiscover> cmdFindNodes(XBee xbee)	throws XBeeException, InterruptedException {
		logger.info("-----> cmdFindNodes()");
		final ArrayList<NodeDiscover> retval = new ArrayList<NodeDiscover>();
		try {
			xbee.sendAsynchronous(new AtCommand("NT"));
			AtCommandResponse nodeTimeout = (AtCommandResponse) xbee.getResponse();
	
			// default is 6 seconds
			long nodeDiscoveryTimeout = ByteUtils.convertMultiByteToInt(nodeTimeout.getValue()) * 100;
	
			logger.info("Node discovery timeout is " + nodeDiscoveryTimeout + " milliseconds");
	
			xbee.addPacketListener(new PacketListener() {
				public void processResponse(XBeeResponse response) {
					if (response.getApiId() == ApiId.AT_RESPONSE) {
						NodeDiscover nd = NodeDiscover.parse((AtCommandResponse) response);
						logger.info("Node discover response is: " + nd);
						retval.add(nd);
					} else if(response.getApiId() == ApiId.ZNET_TX_STATUS_RESPONSE){
						logger.info("ITS OK I QUESS - ZNET TX STATUS RESPONSE: " + response);
					} 
					else {
						logger.info("Ignoring unexpected response: " + response);
					}
				}
			});
	
			logger.info("Sending node discover command");
			xbee.sendAsynchronous(new AtCommand("ND"));
	
			// wait for nodeDiscoveryTimeout milliseconds
			Thread.sleep(nodeDiscoveryTimeout);
	
			Iterator<NodeDiscover> doce_it = retval.iterator();
			while (doce_it.hasNext()) {
				NodeDiscover nod = doce_it.next();
				int count = 0;
				int marker = 0;
				for (int i = 0; i < retval.size(); i++) {
					NodeDiscover elem = retval.get(i);
					if (nod.equals(elem)) {
						count++;
						marker = i;
						if (count > 1) {
							// ups we have this element two times
							// remove one - marker points to it
							retval.remove(marker);
							i = 0;
							count--;
						}
					}
				}
				if (count == 1) {
					// this is expected
	
				} else {
					throw new RuntimeException("ERROR: THE NODE [" + nod + "] IS FOUND [" + count + "] TIME(S) - EXPECTED IS 1");
				}
	
			}
			logger.info("Time is up!  You should have heard back from all nodes by now.  If not make sure all nodes are associated and/or try increasing the node timeout (NT)");
		} finally {
			// xbee.close();
			logger.info("<----- cmdFindNodes()");
		}
		return retval;
	}	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
}
