package com.moneyhouse.sensors.xbee;

import static java.nio.file.StandardWatchEventKinds.ENTRY_CREATE;
import static java.nio.file.StandardWatchEventKinds.ENTRY_DELETE;
import static java.nio.file.StandardWatchEventKinds.OVERFLOW;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.MathContext;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.nio.file.WatchEvent;
import java.nio.file.WatchKey;
import java.nio.file.WatchService;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.Queue;

import org.apache.log4j.Logger;

import com.moneyhouse.sensors.config.CONTSTANTINTERFACE;
import com.moneyhouse.sensors.config.LogHelper;
import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.config.ServerEnvironmentSingleton;
import com.moneyhouse.sensors.filetrigger.FileMonitorToBrickSingleton;
import com.moneyhouse.sensors.pi.DSSensor;
import com.moneyhouse.sensors.pi.dto.DSSensorResultDO;
import com.moneyhouse.sensors.xbee.dto.JSONQueueContainer;
import com.moneyhouse.sensors.xbee.dto.JSONXBeeCommandHeartBeatObject;
import com.moneyhouse.sensors.xbee.dto.ToBrickErrorFileQueueElementDataObject;
import com.moneyhouse.sensors.xbee.dto.ToBrickFileQueueElementDataObject;
import com.moneyhouse.sensors.xbee.dto.XBeeComandObject;
import com.moneyhouse.sensors.xbee.dto.XBeeComandResultObject;
import com.moneyhouse.sensors.xbee.dto.XBeeConstants;
import com.moneyhouse.sensors.xbee.dto.XBeeDataTransportObject;
import com.moneyhouse.sensors.xbee.thread.QueueMaintenanceThread;
import com.moneyhouse.sensors.xbee.thread.ToBrickFileReceiverThread;
import com.rapplogic.xbee.api.AtCommand;
import com.rapplogic.xbee.api.AtCommandResponse;
import com.rapplogic.xbee.api.XBee;
import com.rapplogic.xbee.api.XBeeException;
import com.rapplogic.xbee.api.XBeeResponse;
import com.rapplogic.xbee.api.XBeeTimeoutException;
import com.rapplogic.xbee.util.ByteUtils;

import flexjson.JSONSerializer;
import flexjson.transformer.DateTransformer;
/**
 * Creates a new thread for receiving data from Xbee via UART
 * 
 * @author Frederic
 *
 */

public class SerialReceiverNoSerialPort extends SerialReceiverAbstract implements Runnable{
	private final static Logger logger = Logger.getLogger(SerialReceiverNoSerialPort.class);
	private final static Logger loggerfile = Logger.getLogger("requestLogger");
//	private final static Logger loggerqueue = Logger.getLogger("requestToBrickQueueLogger");
	
	
//	private File[] filelist = null;
	private static Queue<String> send_fifo_buffer = new LinkedList<String>();
//	private static long printoutcounter = 0; // to reduce the prtinout to console when idle
	private static long printoutcounterDispl = 0; // to reduce the prtinout to console when idle
	private static XBee xbeeSingleton = null;
	private static ArrayList<XBeeDataTransportObject> latestDataInQueue = new ArrayList<XBeeDataTransportObject>();
	

	private boolean isXBEERadioMounted= false; 					// is the Radio module mounted or not? By default not
	static Thread  toBrickFileListener;    						// looks at directory toBrick	
	
	public SerialReceiverNoSerialPort(){
		logger.info("-----> SerialReceiverNoSerialPort.SerialReceiverNoSerialPort()");
		logger.info("\t Initializing SerialReceiverNoSerialPort V9.0 for WRAPPING MISSING SERIAL PORT");
		logger.info("\t WHEN THE SERAIL PORT CAN NOT BE OPENED, THEN THE THIS DUMMY CLASS IS RUNNING");
		logger.info("\t TIME STAMP: 20180313 1200");

		if(PropertiesFileReader.isRunEnvLinux()){
			logger.info("\t We run on Linux");
			logger.info("\t 0a. Setting gnu.io.rxtx.SerialPorts in porperties");
			String serialPort = PropertiesFileReader.getInstance().getProperty("comport");
			System.setProperty("gnu.io.rxtx.SerialPorts", serialPort); 
			// com ports between wheezy and jessie changed - wheezy is /dev/ttyAMA0 whereas jessie is /dev/ttyS0
		} else{
			logger.info("\t We run on Win - nothing to do");
		}
		logger.info("\t 1a. Getting XBee Singleton");
		
		XBee xbee = XBeeSingleton.getInstance().getXbee();
		logger.info("\t 1b. XBee Singleton = "+ xbee);
		logger.info("\t 2a. Get COM port for XBee communication");
		String port = PropertiesFileReader.getInstance().getProperty("comport");
		logger.info("\t 2b. COM Port used: "+port);

		try 
		{
			logger.info("\t 3a. Prepare to open XBee communication");
			xbee.open(port, 9600);
			logger.info("\t 3b. XBee open");
			
			isXBEERadioMounted = true;
			logger.info("\t 3c. Prepare to check XBeE coordinator");
			String sh = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SH"))).getValue());
			String sl = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SL"))).getValue());
			logger.info("\t 3d. XBeE coordinator addess: " + sh +","+ sl);
			logger.info("\t 3e. Register XBeE coordinator exists");

			
			
			
			logger.info("\t 4a. Prepare to close XBee communication");
			xbee.close();
			logger.info("\t 4b. XBee closed");

			
		} 
		catch (XBeeException e) {
			// TODO Auto-generated catch block
			logger.info("\t 3b. XBee can not be opened or cant be closed");
			
			String errormsg = e.toString();
			logger.error("\t 4  THIS IS THE CORRECT BEHAVIOUR, AS WE USE THe NO SERIAL PORT VERSION : ");
			logger.error("\t    WOULD A SERIAL PORT EXIST/BE AVAILABLE THE SerialReceiverWithSerialPort VERSION WOULD BE CALLED");
//			e.printStackTrace();
//			logger.error(errormsg);
			
		}

		logger.info("SOME LOG4J LOGGING LEVEL TEST - WE TEST FATAL, ERROR, WARN, INFO, DEBUG");
		logger.fatal("LOG4J *FATAL* ERROR WARN INFO DEBUG LEVEL");
		logger.error("LOG4J FATAL *ERROR* WARN INFO DEBUG LEVEL");
		logger.warn("LOG4J FATAL ERROR *WARN* INFO DEBUG LEVEL");
		logger.info("LOG4J FATAL ERROR WARN *INFO* DEBUG LEVEL");
		logger.debug("LOG4J FATAL ERROR WARN INFO *DEBUG* LEVEL");
		
		
		logger.info("ALSO LOGGING TO - requestLogger						xbee_requests.log");
		logger.info("ALSO LOGGING TO - requestToBrickQueueLogger			xbee_requests_queue.log");
		loggerfile.info	("PROGRAM START - CHECK requestLoggerlogfile		xbee_requests.log");
//		loggerqueue.info("PROGRAM START - CHECK requestToBrickQueueLogger	xbee_requests_queue.log");
		logger.info("WROTE INTO THESE TWO LOGGER FILES - CHECK log4jxbee.properties");

		
		logger.info("STATUS JSON: data_result_manage_brick_network_queues.json");
		logger.info("BASE STATION OW READING RESULTS CAN BE FOUND VIA KEY WORDS: grep \"READ SENSOR RESULT:\"");
		logger.info("BASE STATION OW READING RESULTS CAN BE FOUND VIA KEY WORDS: grep \") retVal=\"");

		setStartTimestamp(System.currentTimeMillis());
		

		// CLEAN FILE 	
		try{
			cleanFileFromDocRoot("error_queue_content.txt");
			cleanFileFromDocRoot("tobrick_queue_content.txt");
		}catch(Exception r){
			logger.fatal("FATAL ERROR: CAN NOT CLEAN FILES error_queue_content.txt AND tobrick_queue_content.txt IN DOC ROOT DIR!!");
			loggerfile.fatal("FATAL ERROR: CAN NOT CLEAN FILES error_queue_content.txt AND tobrick_queue_content.txt IN DOC ROOT DIR!!");
			throw new RuntimeException("FATAL ERROR: CAN NOT CLEAN FILES error_queue_content.txt AND tobrick_queue_content.txt IN DOC ROOT DIR!!", r);
		}
		
		logger.info("<----- SerialReceiver8.SerialReceiver8()");
	}


	
	
	
	

	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try{
//			init();
	
			SerialReceiverNoSerialPort mb = new SerialReceiverNoSerialPort();
	
			ToBrickFileReceiverThread tbfrt = new ToBrickFileReceiverThread();
			tbfrt.start();
	
			QueueMaintenanceThread qmt = new QueueMaintenanceThread();
			qmt.start();
			
			
			// TO AVOID NULL POINTER EXCEPTION IN STARTUP PHASE
			goWasteTime(2000);
			
			// nodeAddress64=0x00,0x13,0xa2,0x00,0x40,0x0a,0x4a,0x0a,
			// nodeAddress64=0x00,0x13,0xa2,0x00,0x40,0xa0,0x98,0xf3
			// nodeAddress64=0x00,0x13,0xa2,0x00,0x40,0x0a,0x42,0xc5
			
			// XBeeAddress64 address = new XBeeAddress64("00 13 A2 00 40 A0 98 F3");
			File folder = new File(PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick"));
			File[] listOfFilesToBrick = folder.listFiles();
			
			
	//		mb.filelist = listOfFilesToBrick;
			
	//		boolean execute = false;
			long tsping = System.currentTimeMillis();				// FOR THE POLLING MECHANISM
			long tsQueueReadOut = System.currentTimeMillis();		// FOR THE POLLING MECHANISM
			long timeout = System.currentTimeMillis();				// FOR THE TIMEOUT EXCEPTION FROM READING THE XBEE SERIAL WIRE INCOME DATA
			long deleteFilesDuration = System.currentTimeMillis();	// FOR THE LOGGING THE THE DELETE FILES In TOBRICK FOLDER
			
			
			boolean firstRun = true;
			
			
	//		XBeeReceiverThread t1 = mb.new XBeeReceiverThread( "Thread-1");
	//	    t1.start();
			
//			try {
//				openCOMPort(xbeeSingleton);
//			} catch (XBeeException e1) {
				// TODO Auto-generated catch block
//				e1.printStackTrace();
//				logger.error("ERROR: PROBLEMS OPENING XBEE PORT", e1);
//			} catch (InterruptedException ie) {
				// TODO Auto-generated catch block
//				ie.printStackTrace();
//				logger.error("ERROR: PROBLEMS OPENING XBEE PORT", ie);
//			}
			for(;;){
				long tsping2 = System.currentTimeMillis();
				
				if(tsping2 - tsping > 1200000 || firstRun == true){
					tsping = printOutInRegularInterval(tsping2);
					// deleteFilesOlderThanFromDir("3h", PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick"));
					firstRun = false;
				}
	
				
				
				if(tsping2 - deleteFilesDuration > 300000 ){
					logger.info("AFTER ["+((tsping2 - deleteFilesDuration)/1000 )+"] s WE CLEAN THE filetrigger.dir.tobrick DIRECTORY");
	
					deleteFilesOlderThanFromDir("2h", PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick"));
					deleteFilesOlderThanFromDir("2h", PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrickprocessed"));
					
					mb.writeQueueEntriesToFile();
					deleteFilesDuration = tsping2;
					
				}
				
				
				
				
				if(tsping2 - tsQueueReadOut > 2400000 || firstRun == true){
	//				tsQueueReadOut = printOutInRegularIntervalQueue(tsping2);
					// deleteFilesOlderThanFromDir("3h", PropertiesFileReader.getInstance().getProperty("filetrigger.dir.tobrick"));
				}
				
				try{
					timeout = mainLoopBody(mb, getXbeeSingleton(), timeout);
					
				} catch(Exception rd){
					logger.fatal("FATAL: LOGGING NEEDS TO BE IMPROVED WE CATCHED AN EXCPETION IN THE MAIN LOOP BODY - FOR NOW THIS EXPECTION WAS CATCHED: ", rd);
					logger.fatal("FATAL: MESSAGE OF THE EXCEPTION ["+rd.getMessage()+"] AND ALL THE OTHER INFORMATION: "+ rd.toString());
					
				}
			}
		}catch (Exception ex){
			logger.fatal("FATAL: SOMETHING WENT WRONG - LOGGING NEEDS TO BE IMPROVED - FOR NOW THIS EXPECTION WAS CATCHED: ", ex);
			logger.fatal("FATAL: MESSAGE OF THE EXCEPTION ["+ex.getMessage()+"] AND ALL THE OTHER INFORMATION: "+ ex.toString());
			
		}
	}

	/**
	 * WRITES THE CONTENT OF THE QUEUES TO THE FOLLOWING TEXT FILES 
	 * 
	 * tobrick_queue_content.txt
	 * error_queue_content.txt
	 * 
	 * IN THE DOC ROOT DIRECTORY 
	 */
	private synchronized void writeQueueEntriesToFile(){
		boolean isWARThere = isPresentationWarDeployed();
		if(isWARThere){
			XBeeDataQueueSingleton dataqueue = XBeeDataQueueSingleton.getInstance();
			
	    	String docroot = ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION;
	    	
	    	writeErrorQueueToTextFile(dataqueue, docroot);
	    	writeToBrickQueueToTextFile(dataqueue, docroot);
		}
	}

	private void writeToBrickQueueToTextFile(XBeeDataQueueSingleton dataqueue, String docroot) {
		String sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(System.currentTimeMillis()));
		String toBrickQueueContent = "TO BRICK QUEUE LIST CREATED TIMESTAMP: " + sdf + "\n";
		for(int i = 0; i < dataqueue.obtainToBrickDataList().size(); i++){
			ToBrickFileQueueElementDataObject tbefqedo = dataqueue.obtainToBrickDataList().get(i);
			toBrickQueueContent = toBrickQueueContent + "["+i+"]\t\t" + tbefqedo.toString() + "\n";
		}
		if(dataqueue.obtainToBrickErrorDataList().size() == 0){
			toBrickQueueContent = "NO ELEMENT IN TO BRICK QUEUE AT ["+sdf+"]";
		}
		
		
		try {
			cleanFileFromDocRoot("tobrick_queue_content.txt");
		    Files.write(Paths.get(docroot+"tobrick_queue_content.txt"), toBrickQueueContent.getBytes(), StandardOpenOption.WRITE);
		}catch (IOException e) {
		    //exception handling left as an exercise for the reader
			logger.fatal("FATAL ERROR: WhILE WRITING DOC ROOT tobrick_queue_content.txt",e);

			e.printStackTrace();
		}
	}
	
	

	private void writeErrorQueueToTextFile(XBeeDataQueueSingleton dataqueue, String docroot) {
		String sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(System.currentTimeMillis()));
		String errorqueueContent = "ERROR QUEUE LIST CREATED TIMESTAMP: " + sdf + "\n";
		for(int i = 0; i < dataqueue.obtainToBrickErrorDataList().size(); i++){
			ToBrickErrorFileQueueElementDataObject tbefqedo = dataqueue.obtainToBrickErrorDataList().get(i);
			errorqueueContent = errorqueueContent + "["+i+"]\t\t" + tbefqedo.toString() + "\n";
		}
		if(dataqueue.obtainToBrickErrorDataList().size() == 0){
			sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(System.currentTimeMillis()));
			errorqueueContent = "NO ELEMENT IN ERROR QUEUE AT ["+sdf+"]";
		}
		
		
		try {
			cleanFileFromDocRoot("error_queue_content.txt");

		    Files.write(Paths.get(docroot+"error_queue_content.txt"), errorqueueContent.getBytes(), StandardOpenOption.WRITE);
		}catch (IOException e) {
		    //exception handling left as an exercise for the reader
			logger.fatal("FATAL ERROR: WhILE WRITING DOC ROOT error_queue_content.txt",e);
			e.printStackTrace();
		}
	}

	private static long mainLoopBody(SerialReceiverNoSerialPort mb, XBee xbee, long timeoutts) {
		// CLEANUP FILEs FROM TO BRICK DIR WHICH ARE OLDER THAN 2 HOURS
		
		XBeeDataQueueSingleton toBrickQueue = XBeeDataQueueSingleton.getInstance();
		ToBrickFileQueueElementDataObject element = toBrickQueue.readOldestToBrick(9);
/*		if(element != null){
			loggerqueue.info("=====================================================================================================");
			loggerqueue.info(toBrickQueue);
			loggerqueue.info("READ ELEMENT: "+ element);
			loggerqueue.info("=====================================================================================================");
		}
*/		
		XBeeComandResultObject directResultFromXBeeReuqest = null;
		if(element != null){
			if(element.getFilename().toString().contains(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_HEARTBEAT)){
				// EVERY THING ELSE CAN GO FASTER
				goWasteTime(200 * element.getReadingCounter());
			} else if(element.getFilename().toString().contains(CONTSTANTINTERFACE.FILENAME_PART_FOR_ND_FINDALL)){
				// ND COMMAND NOT VALID AS NO SERIAL PORT CONNECTED 
				// MOVE ELEMENT DIRECTLY TO ERROR QUEUE
				toBrickQueue.moveFromToBrickToErrorListContainingCharSequence(element.getFilename().toString(), "NO_SERIAL_PORT");
			} else if(!element.getFilename().toString().contains(XBeeConstants.MODULE_ADDRESS_BASESTATION)){
				// XBEE SRIAL COMMAND NOT VALID AS NO SERIAL PORT CONNECTED 
				// MOVE ELEMENT DIRECTLY TO ERROR QUEUE
				toBrickQueue.moveFromToBrickToErrorListContainingCharSequence(element.getFilename().toString(), "NO_SERIAL_PORT");
			}else {
				// EVERY THING ELSE CAN GO FASTER
				goWasteTime(200 * element.getReadingCounter());
			}
			
			File theNewOne = element.getFilename();
			logger.info("-------------------------------------------------------------------------------------------------------");
			logger.info("theNewOne Abso: "+ theNewOne.getAbsolutePath());
			logger.info("theNewOne Pare: "+ theNewOne.getParent());
			logger.info("theNewOne Path: "+ theNewOne.getPath());
			logger.info("theNewOne File: "+ theNewOne.getName() +"\t CREATED: "+element.getReceivedTimestampFormated());
			logger.info("Element Counter: "+ element.getReadingCounter());
			logger.info("-------------------------------------------------------------------------------------------------------");
			loggerfile.info("NEW RQT: "+ LogHelper.writeShortFileName(theNewOne.getName()) +"\t CREATED: "+element.getReceivedTimestampFormated()+"\t COUNTER: "+element.getReadingCounter());
			boolean exists = FileMonitorToBrickSingleton.getInstance().isFileExistant(theNewOne.getAbsolutePath());
			if(exists){
				directResultFromXBeeReuqest = mb.parseFile(theNewOne, xbee);
				
				// check whether we received a direct result from the xbee - LIKE  sendSETOUTPORTB0 command
				if(directResultFromXBeeReuqest != null && directResultFromXBeeReuqest.getXbeeResultAsString() != null && directResultFromXBeeReuqest.getXbeeResultAsString().length() !=0  ){
					if(directResultFromXBeeReuqest.getXbeeResultAsString().contains("apiId=ZNET_TX_STATUS_RESPONSE") &&
							directResultFromXBeeReuqest.getXbeeResultAsString().contains("deliveryStatus=SUCCESS") &&
							directResultFromXBeeReuqest.getXbeeResultAsString().contains("error=false")){
						// PERFECT - THE SEND PACKAGE WAS WELL PROCESSED
						logger.info("WE HAVE AN IMMEDIATE RESPONSE FROM THE XBEE RADIO MODULE WHICH IS VALID - LETS PROCESS IT");
						logger.info("XBEE RESULT STRING: "+directResultFromXBeeReuqest.getXbeeResultAsString());
						logger.info("CORRELID : "+directResultFromXBeeReuqest.getCorrelID());
					
						// DELETE FROM TO BRICK QUEUE
						XBeeDataQueueSingleton toBrick = XBeeDataQueueSingleton.getInstance();
						toBrick.deleteContainingCharSequenceToBrick(directResultFromXBeeReuqest.getCorrelID());
					} else if(directResultFromXBeeReuqest instanceof  JSONXBeeCommandHeartBeatObject &&
							directResultFromXBeeReuqest.getXbeeResultAsString() != null && 
							directResultFromXBeeReuqest.getXbeeResultAsString().equalsIgnoreCase("HEARTBEAT: SOME STRING TO SAY THAT WE WORKED DILLIGENTLY") ){
						// PERFECT - THE SEND PACKAGE WAS WELL PROCESSED
						logger.info("HEARTBEAT WITH CORRELID ["+directResultFromXBeeReuqest.getCorrelID()+"] WORKED");
					
						// DELETE FROM TO BRICK QUEUE
						XBeeDataQueueSingleton toBrick = XBeeDataQueueSingleton.getInstance();
						toBrick.deleteContainingCharSequenceToBrickAndFile(directResultFromXBeeReuqest.getCorrelID());
					} else {
						// THIS SHOULD COME WHEN THE SEND PACKAGE HAS PROBLEMS 
						logger.warn("%% A TROUBLESOME IMMEDIATE RESPONSE FROM THE XBEE RADIO MODULE WAS RECEIVED - HERE IT IS");
						logger.warn("%% XBEE RESULT STRING: "+directResultFromXBeeReuqest.getXbeeResultAsString());
						logger.warn("%% CORRELID : "+directResultFromXBeeReuqest.getCorrelID());
						logger.warn("%% SHORT FILENAME: " +  LogHelper.writeShortFileName(theNewOne.getName()));
					}
				} else if(directResultFromXBeeReuqest != null && (directResultFromXBeeReuqest.getXbeeResultAsString()==null || directResultFromXBeeReuqest.getXbeeResultAsString().length() ==0 ) ){
					// IT MIGHT BE A SHSL Coordinaotr command
					if(directResultFromXBeeReuqest.getCommand() != null && directResultFromXBeeReuqest.getCommand().getAtcommand().equals("SHSL")){
						// ITS A GET COORDINATOR ADDRESS COMMAND
						logger.info("SHSL WITH CORRELID ["+directResultFromXBeeReuqest.getCorrelID()+"] WORKED");
						
						// DELETE FROM TO BRICK QUEUE
						XBeeDataQueueSingleton toBrick = XBeeDataQueueSingleton.getInstance();
						toBrick.deleteContainingCharSequenceToBrickAndFile(directResultFromXBeeReuqest.getCorrelID());
					} else if(directResultFromXBeeReuqest.getCommand() != null && directResultFromXBeeReuqest.getResponseValue().contains("THE XBEE RADIO MODULE IS NOT PLUGGED IN") && directResultFromXBeeReuqest.isResponseOK() == false){
						// IT SEEMS THE XBEE RADIO MODULE DOES NOT EXIST
						logger.warn("%% XBEE RADIO MODULE DOES NOT EXIST: ");
						logger.warn("%% WE MOVE REQUEST WITH CORREL ID ["+directResultFromXBeeReuqest.getCommand().getCorrelID()+"] TO ERROR QUEUE ");
						XBeeDataQueueSingleton toBrick = XBeeDataQueueSingleton.getInstance();
						toBrick.moveFromToBrickToErrorListContainingCharSequence(directResultFromXBeeReuqest.getCommand().getCorrelID(), "NO_XBEE_RADIO_MODULE");
					}
				}
			} 
			else 
			{
				logger.error("ERROR: THE FILE ["+LogHelper.writeShortFileName(theNewOne.getName())+"] DOES NOT EXIST");
			}
		}

		// NO SERIAL CONNECTED
		// timeoutts = checkForAnswer(xbee, timeoutts, directResultFromXBeeReuqest);
		return timeoutts;
	}




	private static long printOutInRegularInterval(long tsping2) {
		long tsping;
		// logger.debug(".");
		tsping = tsping2;
		for(int o = 0; o< 1; o++){
			logger.info("===========================================================================");
		}

		logger.info("READ SENSORSLIST START");

		DSSensor ds = new DSSensor();
		ArrayList<String> piSensorList = ds.readSensorList();
		for(int p = 0; p < piSensorList.size(); p++){
			logger.info("["+p+"] SENSOR ID = ["+piSensorList.get(p)+"] ");
		}
		logger.info("READ SENSORSLIST END");
		

		logger.info("READ SENSORS START");
		for(int p = 0; p < piSensorList.size(); p++){
			DSSensorResultDO res = ds.readSensorValueWithID(piSensorList.get(p));
			logger.info("READ SENSOR RESULT: " + res);

		}
		
		logger.info("READ SENSORS END");
		
		
		for(int o = 0; o< 1; o++){
			logger.info("===========================================================================");
		}
		return tsping;
	}

	
	private static long printOutInRegularIntervalQueue(long tsqueue) {
		long tsping;
		// logger.debug(".");
		tsping = tsqueue;
		
		XBeeDataQueueSingleton toBrickQueue = XBeeDataQueueSingleton.getInstance();

		
		for(int o = 0; o< 1; o++){
			loggerfile.info("________________________________________________________________________________________________________");
		}

		loggerfile.info("READ QUEUE ENTRIES - START");
		loggerfile.info(toBrickQueue);
		loggerfile.info("READ QUEUE ENTRIES - END");
		for(int o = 0; o< 1; o++){
			loggerfile.info("________________________________________________________________________________________________________");
		}
		return tsping;
	}


	
	

	



	
	
	
	/**
	 * DELETE THE FILES FROM BRICK TO BRICK OR BRICKCONFIG WHICH HAVE CORREL ID
	 * @since 17.07.2015
	 * @param correlID 
	 * @param directory
	 * @deprecated not used - successful files are deleted anyhow
	 */
	private static void deleteFilesWithCorrelIDFromDir(String correlID, String directory) {
		logger.info("---> SerialReceiver5.deleteFilesWithCorrelIDFromDir("+correlID+", "+directory+")");  
		try{

			if(correlID == null || correlID.isEmpty()){
				logger.error("ERROR: PARAMETER correlID CAN NOT BE NULL OR EMPTY ");
				throw new RuntimeException("ERROR: PARAMETER correlID CAN NOT BE NULL OR EMPTY ");
			}
			long filecounter= 0;
			
			File dir = new File(directory);
			if(dir.exists())
			{
			    File[] listFiles = dir.listFiles();    
			    for(File listFile : listFiles) {
			    	if(listFile.isDirectory()){
			    		// ITS A DIRECTORY - THIS SHOULD NOT BE THE CASE
						logger.error("ERROR: FILE ["+listFile.getName()+"] IS A DIRECTORY - EXPECTED IS A FILE ");  
						throw new RuntimeException("ERROR: FILE ["+listFile.getName()+"] IS A DIRECTORY - EXPECTED IS A FILE ");
			    	} else {
			    		// ITS A FILE - READ IT AND COMPARE WITH THE CORREL-ID
			    		String content = readFile(listFile.getPath());
			    		if(content.contains(correlID)){
			    			// CORRELID IS IN THE CONTENT
			    			filecounter++;
			    		}
			    	}
			    }
			    
				logger.debug("["+filecounter+"] FILECOUNTER OF FILES CONTAINING CORRELID ["+correlID+"] FROM DIR ["+directory+"] ");  
			    
			    if(filecounter == 1){
			    	// YES THIS IS THe HAPPY CASE - ONLY ON FILE WITH THE SAME CORRELID

				    File[] listFiles2 = dir.listFiles();    
				    filecounter = listFiles2.length;
				    for(File listFile2 : listFiles2) {
				    	if(listFile2.isDirectory()){
				    		// ITS A DIRECTORY - THIS SHOULD NOT BE THE CASE
							logger.error("ERROR: FILE ["+listFile2.getName()+"] IS A DIRECTORY - EXPECTED IS A FILE ");  
							throw new RuntimeException("ERROR: FILE ["+listFile2.getName()+"] IS A DIRECTORY - EXPECTED IS A FILE ");
				    	} else {
				    		// ITS A FILE - READ IT AND COMPARE WITH THE CORREL-ID
							logger.debug("DELETE ["+listFile2.getName()+"] FROM DIR ["+directory+"] WITH CORRELID ["+correlID+"]");  
				    		listFile2.delete();
				    	}
				    }
			    } 
			    else 
			    {
		    	// BAD CASE - NOT FOUND - OF FOUND SEVERAL TIMES - CORRELID SHOULD BE UNIQUE
			    	logger.error("ERROR: CORRELID ["+correlID+"] IS FOUND ["+filecounter+"] IN DIRECTORY ["+directory+"] -- WE ECPECTED TO FIND 1 OCCURENCE - SO THAT WE CAN DELETE IT");  
					throw new RuntimeException("ERROR: CORRELID ["+correlID+"] IS FOUND ["+filecounter+"] IN DIRECTORY ["+directory+"] -- WE ECPECTED TO FIND 1 OCCURENCE - SO THAT WE CAN DELETE IT");
			    }
			}
		}
		catch (Exception ex){
			logger.error("ERROR: ERROR WHILE DELETING FILE" , ex);
		}
		logger.info("<--- SerialReceiver5.deleteFilesWithCorrelIDFromDir("+correlID+", "+directory+")");  
	}
	
	/** READ THE FILE - TO CHECK OF THE CORREL ID
	 * @since 16.07.2015
	 * @param path
	 * @return
	 */
	private static String readFile(String path) 
	{
		String retVal = "";
		byte[] encoded;
		try {
			encoded = Files.readAllBytes(Paths.get(path));
			retVal = new String(encoded);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			logger.error("ERROR: READING FILE ["+Paths.get(path)+"]", e);
		}
		return retVal;
	}
	
	
	

	


	
	
	

	private static void openCOMPort(XBee xbee) throws XBeeException, InterruptedException {
		Thread.sleep(200);
		String port = PropertiesFileReader.getInstance().getProperty("comport");
		xbee.open(port, 9600);
	}
	
	private XBeeComandResultObject parseFile(File filename, XBee xbees) {
		logger.debug("----> parseFile("+filename+", "+xbees+") ");
		XBeeComandResultObject retval = null;
		
        FileMonitorToBrickSingleton fileaccess = FileMonitorToBrickSingleton.getInstance();
//	            String fn = transformFileNameToString(filename.getName());
        String fn = filename.getAbsolutePath(); //  + System.getProperty("file.separator")+  filename.getName();
        boolean isCommand = fileaccess.isCommandFile(fn);
        boolean isHeartBeat = fileaccess.isFile(fn, XBeeConstants.COMMAND_heartBEAT);
        boolean isWARThere = isPresentationWarDeployed();

        if(isCommand && !isHeartBeat)
        {
        	// parse command
        	XBeeComandResultObject result = parseCommand(fileaccess, fn, xbees);
        	retval = result;
        	if(isWARThere){
        		writeResultJSONFile(result);
        	} else {
        		logger.warn("PRESENTATION WAR FILE MIGHT NOT BE DEPLOYED - DOC ROOT ["+ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION+"] IS NOT PRESENT ON FILE SYSTEM");
        	}
        }
        else if(isCommand && isHeartBeat){
    		logger.info("ITS A HEARTBEAT");
        	// parse command
        	Object result = parseCommand(fileaccess, fn, xbees);
        	if(result instanceof JSONXBeeCommandHeartBeatObject){
            	JSONXBeeCommandHeartBeatObject xrhbo = (JSONXBeeCommandHeartBeatObject) result;
            	retval = xrhbo;
        		if(isWARThere){
            		writeResultJSONFile(xrhbo);
            	} else {
            		logger.warn("PRESENTATION WAR FILE MIGHT NOT BE DEPLOYED - DOC ROOT ["+ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION+"] IS NOT PRESENT ON FILE SYSTEM");
            	}
       		
        	}else {
        		logger.error("ERROR: EXPECTED IS A XBeeResponseHeartBeatObject OBJ AS RESULT OF HEARTBEAT REQUEST - BUT RECEIVED ["+result+"] ["+result.getClass()+"]");
        		
        	}
        }
        else
        {
            // 1. read file content
            XBeeDataTransportObject xbee = fileaccess.readFileAndReturnDataObject(fn);
            
            // 2. create datagram sting
            String datagram = xbee.generateDataGram();
            getSend_fifo_buffer().add(datagram);
        }
		logger.debug("<---- parseFile("+filename+", "+xbees+") retval= "+retval);
		return retval;
	}

	// LOOKS INTO THE FILE AND PARSES IT
	private XBeeComandResultObject parseCommand(FileMonitorToBrickSingleton fileaccess, String fn, XBee xbee) {
		logger.debug("-----> parseCommand("+fn+", "+xbee+") ");
		XBeeComandObject cmd = fileaccess.readCommandFileAndReturnDataObject(fn);
		XBeeComandResultObject result = new XBeeComandResultObject(cmd);
		
		
		
		if(cmd.getBrickid().equals(XBeeConstants.MODULE_ADDRESS_BASESTATION)){
			goWasteTime(200);
			if(cmd.getAtcommand().equalsIgnoreCase(XBeeConstants.COMMAND_readOW)){	// test case to read OW
				result = SerialReceiverCommands.sendREADOW_BASIS(cmd);
			} else if(cmd.getAtcommand().equalsIgnoreCase("sendPLIST")){
				SerialReceiverCommands.sendSENSORLIST_BASIS(cmd);
			} else if(cmd.getAtcommand().equalsIgnoreCase("sendSLIST")){
				SerialReceiverCommands.sendSENSORLIST_BASIS(cmd);
			} else if(cmd.getAtcommand().equalsIgnoreCase(XBeeConstants.COMMAND_heartBEAT)){	// test case to heartbeat
				// IMPLEMENT THE QUEUE STATUS CHECK AND WRITE A JSON FILE WITH THE WHOLE INTO IT
				XBeeDataQueueSingleton toBrickQueue = XBeeDataQueueSingleton.getInstance();
				JSONQueueContainer heartbeatJSON = new JSONQueueContainer(toBrickQueue);
				heartbeatJSON.setTimeStampOfSending(cmd.getTimestamp());
				result.setXbeeResultAsString("HEARTBEAT: SOME STRING TO SAY THAT WE WORKED DILLIGENTLY");
				
				// RETURN SPECIAL RESULT OBJECT FOR HEARTBEAT (DOWNCASTED)
				JSONXBeeCommandHeartBeatObject result2 = new JSONXBeeCommandHeartBeatObject(result, heartbeatJSON);
				result = result2;
			}	 			
		} 
		
		 else 
		{
			result.setCommand(cmd);
			result.setResponseValue("ERROR: THIS COMMAND ["+cmd.getAtcommand()+"] IS NOT KNOWN!!! PAYLOAD: NULL\tRESPONSE: SERIAL PORT NOT PRESENT - CHECK THE SERiAL PORT CONFIGURATION" );
			result.setResponseOK(false);
		}
		
		logger.debug("<----- parseCommand("+fn+", "+xbee+") result= "+result);
		return result;
	}

	

	
	/**
	 * WRITES AS OUTPUT THE QUEUES OF THE SERIAL MODULE:  "data_result_manage_brick_network_queues.json"
	 * @param result XBeeResponseHeartBeatObject
	 */
	private void writeResultJSONFile(JSONXBeeCommandHeartBeatObject result) {
		logger.debug("---> writeResultJSONFile()");
		result.setTimestampNow();
		
		result.setSerialUptimeString(this.getUptimeAsString());
		result.setSerialUptimeLongString(this.getUptimeAsLong() + "");
		result.setSerialStartTimeLongString(this.getStartTimestamp() + "");
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.mmm");
		String strstarime = sdf.format(new Timestamp(this.getStartTimestamp()));
		logger.info("SERIAL UPTIME ["+result.getSerialUptimeString()+"] OR IN ms ["+result.getSerialUptimeLongString()+"] - START TIME WAS: ["+strstarime+"]");
		
		// 2. bring result ds into right form
		ArrayList<XBeeComandResultObject> res = new ArrayList<XBeeComandResultObject>();
		res.add(result);
		
		// 3. Prep file
		BufferedWriter  bw = prepareFile("data_result_manage_brick_network_queues.json");
		
		// 4. serialize
		
		
		JSONSerializer serializer = new JSONSerializer().transform(new DateTransformer("yyyy-MM-dd HH:mm:ss"), Date.class); 
		String val =  serializer.include("jsonQueueContainer").
								include("command").
								exclude("jsonQueueContainer.toBrickDataList.filename.path").
								exclude("jsonQueueContainer.toBrickDataList.filename.path").
								exclude("jsonQueueContainer.toBrickDataList.filename.readable").
								exclude("jsonQueueContainer.toBrickDataList.filename.parentFile").
								exclude("jsonQueueContainer.toBrickDataList.filename.prefixLength").
								exclude("jsonQueueContainer.toBrickDataList.filename.totalSpace").
								exclude("jsonQueueContainer.toBrickDataList.filename.hidden").
								exclude("jsonQueueContainer.toBrickDataList.filename.lastModified").
								exclude("jsonQueueContainer.toBrickDataList.filename.invalid").
								exclude("jsonQueueContainer.toBrickDataList.filename.usableSpace").
								exclude("jsonQueueContainer.toBrickDataList.filename.directory").
								exclude("jsonQueueContainer.toBrickDataList.filename.executable").
								exclude("jsonQueueContainer.toBrickDataList.filename.canonicalPath").
								exclude("jsonQueueContainer.toBrickDataList.filename.writable").
								
								
								
								exclude("jsonQueueContainer.toBrickDataList.filename.freeSpace").
								exclude("jsonQueueContainer.toBrickDataList.filename.absolute").
								exclude("jsonQueueContainer.toBrickDataList.filename.file").
								
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.path").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.readable").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.parentFile").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.prefixLength").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.totalSpace").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.hidden").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.lastModified").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.invalid").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.usableSpace").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.directory").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.executable").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.canonicalPath").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.writable").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.freeSpace").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.absolute").
								exclude("jsonQueueContainer.toBrickErrorDataList.filename.file").
								
								exclude("*.class").deepSerialize( res );
		
		
		try {
			bw.write(val);
			bw.close();
			logger.info("WE WROTE JSON FILE data_result_manage_brick_network_queues.json SUCCESSFULLY");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			logger.error("ERROR WRITING JSON FILE POS 1: ", e);

		} catch (Exception ex) {
			// TODO Auto-generated catch block
			ex.printStackTrace();
			logger.error("ERROR WRITING JSON FILE POS 2: ", ex);

		} 
		finally {
			logger.debug("<--- writeResultJSONFile()");
			
		}
	}

	
	
	/**
	 * WRITES AS OUTPUT:  "data_result_manage_brick_network.json"
	 * @param result
	 */
	private void writeResultJSONFile(XBeeComandResultObject result) {
		logger.debug("---> writeResultJSONFile()");
		result.setTimestampNow();
		// 2. bring result ds into right form
		ArrayList<XBeeComandResultObject> res = new ArrayList<XBeeComandResultObject>();
		res.add(result);
		
		// 3. Prep file
		BufferedWriter  bw = prepareFile("data_result_manage_brick_network.json");
		
		// 4. serialize
		JSONSerializer serializer = new JSONSerializer().transform(new DateTransformer("yyyy-MM-dd HH:mm:ss"), Date.class); 
		String val =  serializer.exclude("*.class").serialize( res );
		
		try {
			bw.write(val);
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			logger.error("ERROR WRITING JSON FILE POS 1: ", e);

		} catch (Exception ex) {
			// TODO Auto-generated catch block
			ex.printStackTrace();
			logger.error("ERROR WRITING JSON FILE POS 2: ", ex);

		} 
		finally {
			logger.debug("<--- writeResultJSONFile()");
			
		}
	}
	

	
	


	public Queue<String> getSend_fifo_buffer() {
		return send_fifo_buffer;
	}

	/**
	 * Opens the file stream - with docroot and file name
	 * @param filename
	 * @return
	 */
	protected BufferedWriter  prepareFile(String filename){
		BufferedWriter bw = null;
		if(isPresentationWarDeployed()){
			
			try {
		    	String docroot = ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION;
	
		    	logger.debug("prepareFile("+filename+") to: "+ docroot+filename);
		    	File file = new File(docroot+filename);
		
				// if file doesnt exists, then create it
				if (!file.exists()) {
						file.createNewFile();
				}
		
				FileWriter fw = new FileWriter(file.getAbsoluteFile());
				bw = new BufferedWriter(fw);
			
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace(); logger.error("ERROR WRITING FILE ["+filename+"]: ",e);
			}		
		}
		return bw;
	}	

	
	
	
	protected boolean isPresentationWarDeployed(){
		boolean retval = false;
    	String docroot = null;
    	docroot = ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION;
		if(docroot == null ){
			logger.error("ERROR: docroot PROPERTY IS NOT CONFIGURED - IT IS NULL - CHECK FILE tmor.properties");
			throw new RuntimeException("ERROR: docroot PROPERTY IS NOT CONFIGURED - IT IS NULL - CHECK FILE tmor.properties");
		} else {
			try{
				File file = new File(docroot);
				if(file.isDirectory()){
					retval = true;
				}
			} catch (Exception ioe){
				logger.error("ERROR: APPLICATION NOT PROPERLY CONFIGURED - CHECK property docroot in FILE tmor.properties", ioe);
				throw new RuntimeException("ERROR: docroot PROPERTY IS NOT CONFIGURED - IT IS NULL - CHECK FILE tmor.properties");
			}
		}
		return retval;
	}
	
	
	

	
	
	
	
	
	
	
	
	
	
	public void setSend_fifo_buffer(Queue<String> fifo_buffer) {
		send_fifo_buffer = fifo_buffer;
	}  
	
	
	
	

	
	
	
	
	
	




	
		/**
		 * Receives the data from XBee /UART
		 * @author Frederic
		 *
		 */
		class XBeeReceiverThread extends Thread {
			private Thread t;
			private String threadName;
				   
			// 1. STEP
			XBeeReceiverThread( String name){
				threadName = name;
				System.out.println("Creating " +  threadName );
			}
			
			// 2. STARTS
			public void start ()
			{
				System.out.println("Starting " +  threadName );
				if (t == null)
				{
					t = new Thread (this, threadName);
					t.start ();
				}
			}
			   
			   
			// 2. RUNS
			public void run() {
				System.out.println("Running " +  threadName );

				while(true){
				// POLLING FOR RECEIVING DATA
					XBee xbee = null;
					try{
						xbee = XBeeSingleton.getInstance().getXbee();

						openCOMPort(xbee);
						XBeeResponse response = xbee.getResponse(2000);
						int []i = response.getRawPacketBytes();
						System.out.println("DATA RECEIVED START -------------------------------------------------------------------------------------------------");

						for(int j = 0; j < i.length; j++){
							System.out.print(" "+i[j]);
						}
						System.out.println("\n------------------------------------------------------------------------------------------------- DATA RECEIVED END");
					} catch(XBeeTimeoutException exc){
						// System.out.print(" TIMEOUT - NOT TO WORRY ");
					} catch(Exception exc){
						exc.printStackTrace();
					} finally{
						xbee.close();
					}
				}
			}
			private void nioFileListenerToBrick() {
				// TODO Auto-generated method stub
				Path dir = FileSystems.getDefault().getPath("C:\\TEMP\\sensors\\filetrigger\\tobrick");
				WatchKey key = null;
				
				try {
					WatchService watcher = FileSystems.getDefault().newWatchService();
				    key = dir.register(watcher, ENTRY_CREATE, ENTRY_DELETE /*, ENTRY_MODIFY*/);
				    
				    
				    for(;;){
				    	try {
				            key = watcher.take();
				        } catch (InterruptedException x) {
				            return;
				        }
				    	
				    	
				    	for (WatchEvent<?> event: key.pollEvents()) {
				            WatchEvent.Kind<?> kind = event.kind();
				            System.out.format("kind  %s%n", kind);

				            // This key is registered only
				            // for ENTRY_CREATE events,
				            // but an OVERFLOW event can
				            // occur regardless if events
				            // are lost or discarded.
				            if (kind == OVERFLOW) {
				                continue;
				            }
				            
				            if (kind == ENTRY_CREATE) {
					            WatchEvent<Path> ev = (WatchEvent<Path>)event;
					            Path filename = ev.context();

				            	System.out.format("ITS AN ENTRY_CREATE EVENT %s%n ", filename);
				            	System.out.println("********************************************************************************");
				            	System.out.println("********************************************************************************");
				            	System.out.println("********************************************************************************");
				            	System.out.println("********************************************************************************");
				            	
				            	// SerialReceiver6 mb = new SerialReceiver6();
				            	// mb.parseFile(filename.toFile(), getXbeeSingleton());
				            	
				            	System.out.format("THIS IS THE SINGLETON %s%n ", getXbeeSingleton());
				            	
				            	
				            	// goWasteTime("FILE");
				            	continue;
				            }
				            
				            else if(kind == ENTRY_DELETE){
					            WatchEvent<Path> ev = (WatchEvent<Path>)event;
					            Path filename = ev.context();

				            	System.out.format("ITS AN ENTRY_DELETE EVENT %s%n ", filename);
				            	continue;
				            	
				            }
				            else {
				            	System.out.println("THIS EVENT TYPE IS NOT SUPPORTED");
				            	
				            }

				            // The filename is the
				            // context of the event.
				            WatchEvent<Path> ev = (WatchEvent<Path>)event;
				            Path filename = ev.context();

				            // Verify that the new
				            //  file is a text file.
				            try {
				                // Resolve the filename against the directory.
				                // If the filename is "test" and the directory is "foo",
				                // the resolved name is "test/foo".
				                Path child = dir.resolve(filename);
				                if (!Files.probeContentType(child).equals("text/plain")) {
				                    System.err.format("New file '%s'" + " is not a plain text file.%n", filename);
				                    continue;
				                }
				            } catch (IOException x) {
				                System.err.println(x);
				                continue;
				            }

				            // Email the file to the
				            //  specified email alias.
				            System.out.format("Emailing file %s%n", filename);
				            //Details left to reader....
				        }

				        // Reset the key -- this step is critical if you want to
				        // receive further watch events.  If the key is no longer valid,
				        // the directory is inaccessible so exit the loop.
				        boolean valid = key.reset();
				        if (!valid) {
				            break;
				        }
				    }		    	
				} catch (IOException x) {
				    System.err.println(x);
				}
			}
		}
		


		public void run() {
			long startime = System.currentTimeMillis();
			System.out.println("=========================> start Thread()");
			// Thread.
			// nioFileListenerToBrick();

			
			
			long endtime = System.currentTimeMillis();
			BigDecimal timeDiffMS = new BigDecimal(endtime - startime);
			BigDecimal timeDiffS = timeDiffMS.divide(new BigDecimal(1000), MathContext.DECIMAL32);
			
			System.out.println("DURATION ="+ timeDiffS+ " s");

			System.out.println("<========================= end Thread() DURATION ="+ timeDiffS+ " s");
		}
		
		
		public static XBee getXbeeSingleton() {
			if(xbeeSingleton == null){
				xbeeSingleton = XBeeSingleton.getInstance().getXbee();
				try{
					Thread.sleep(10);
				}catch(Exception d){
					logger.error(d);
				}
			}
			return xbeeSingleton;
		}


		public static void setXbeeSingleton(XBee xbeeSingleton) {
			SerialReceiverNoSerialPort.xbeeSingleton = xbeeSingleton;
		}	
		
		
	
}
