package com.moneyhouse.sensors.xbee.dto;

import java.sql.Timestamp;

/**
 * HOLDS A FILE CONTENT FORM THE FOLDERS TOBRICK - FROMBRICK AS AN OBJECT
 * IT MIGHT BE, THAT tHE CHECKSUM IS NOT NEEDED AS XBEE PROTOCOLL CARES FOR IT
 * @since 31.10.2014
 * @author Frederic
 *
 */
public class XBeeDataTransportObject {
	private String uniqueId;
	private String brickstuniqueId;
	private String command;					// like sendPING
	private String readingtype;				// TEMP, SWITCH
	private String readingvalue;			// 23,-10, ON, OFF etc.
	private String readingport;				// A1, A2... , O-WIRE ADDRESS, OR WHATEVER

	private String correlid;				// CORRELATION ID - TO MATCH REQ WITH RESPONSE
	private Timestamp createdtimestamp;
	private String checksum; 			// 0 brickdata ; 1 manual; -1 simulated
	private long executiontime = 0 ;

	public XBeeDataTransportObject(String readingport){
		this.setReadingport(readingport);
	}
	
	/**
	 * FOR COMPLEX FILE CONTENT - SUCH AS requestDATA - or EVENT
	 * @param brickstuniqueId
	 * @param command
	 * @param readingport
	 * @param readingtype
	 * @param readingtvalue
	 * @param correlid
	 * @param createdtimestamp
	 */
	public XBeeDataTransportObject(String brickstuniqueId, String command, String readingport, String readingtype , String readingtval, String correlid, Timestamp createdtimestamp) {
		this.setBrickstuniqueId(brickstuniqueId);
		this.setCreatedtimestamp(createdtimestamp);
		this.setReadingtype(readingtype);
		this.setReadingvalue(readingtval);
		this.setCorrelid(correlid);
		this.setCommand(command);
		this.setReadingport(readingport);
	}	
	
	
	
	
	
	
	
	
	
	
	
	
	/**
	 * FOR COMPLEX FILE CONTENT - SUCH AS requestDATA
	 * @param brickstuniqueId
	 * @param readingtype
	 * @param readingtvalue
	 * @param correlid
	 * @param createdtimestamp
	 */
	public XBeeDataTransportObject(String brickstuniqueId, String readingtype , String readingtvalue, String correlid, String createdtimestamp) {
		Timestamp ts = Timestamp.valueOf(checkAndModifyTimeStamp(createdtimestamp));
		long fractions = getFractionalSeconds(createdtimestamp);
		long tstime = ts.getTime();
		tstime += fractions;
		this.setBrickstuniqueId(brickstuniqueId);
		this.setCreatedtimestamp(new Timestamp(tstime));
		this.setReadingtype(readingtype);
		this.setReadingvalue(readingvalue);
		this.setCorrelid(correlid);
	}

	/**
	 * FOR SIMPLE FILE CONTENT - SUCH AS sendPING
	 * @param brickstuniqueId
	 * @param command
	 * @param correlid
	 * @param createdtimestamp
	 */
	public XBeeDataTransportObject(String brickstuniqueId, String command, String correlid, String createdtimestamp) {
		Timestamp ts = Timestamp.valueOf(checkAndModifyTimeStamp(createdtimestamp));
		long fractions = getFractionalSeconds(createdtimestamp);
		long tstime = ts.getTime();
		tstime += fractions;

		this.setBrickstuniqueId(brickstuniqueId);
		this.setCreatedtimestamp(new Timestamp(tstime));
		this.setCommand(command);
		this.setCorrelid(correlid);
	}
	
	private String checkAndModifyTimeStamp(String ts){
		String retVal = ts;
		if(ts.contains(".")){
			// YES - WE HAVE ms - GET RID OF THEM
			if(ts.charAt(ts.length()-4) == '.' && ts.length() == 23){
				retVal = ts.substring(0, ts.length()-4);
			} 
			else if(ts.charAt(ts.length()-3) == '.' && ts.length() == 22){
				retVal = ts.substring(0, ts.length()-3);
			} 
			else if(ts.charAt(ts.length()-2) == '.' && ts.length() == 21){
				retVal = ts.substring(0, ts.length()-2);
			} 

			
			else{
				// UNEXPECTED timestamp format THROW EXPECTION 
				throw new RuntimeException("ERROR: TIMESTAMP ["+ts+"] EXPECTED TO BE IN FORMAT [2014-10-31 08:29:25.814] BUT IS NOT");
			}
			
		} else {
			// NO - WE DO NOT HAVE ms
			retVal = ts;
		}
		
		return retVal;
	}
	
	
	/** Gets the ms from the String timestamp
	 * @param ts timestamp
	 * @return
	 */
	private long getFractionalSeconds(String ts){
		long retVal = 0;
		if(ts.contains(".")){
			// YES - WE HAVE ms - GET RID OF THEM
			if(ts.charAt(ts.length()-4) == '.' && ts.length() == 23){
				String ms = ts.substring(ts.length()-3, ts.length());
				retVal = new Long(ms).longValue();
			} 
			else if(ts.charAt(ts.length()-3) == '.' && ts.length() == 22){
				String ms = ts.substring(ts.length()-2, ts.length());
				retVal = new Long(ms).longValue();
			} 
			else if(ts.charAt(ts.length()-2) == '.' && ts.length() == 21){
				String ms = ts.substring(ts.length()-1, ts.length());
				retVal = new Long(ms).longValue();
			} 
			
			else{
				// UNEXPECTED timestamp format THROW EXPECTION 
				throw new RuntimeException("ERROR: TIMESTAMP ["+ts+"] EXPECTED TO BE IN FORMAT [2014-10-31 08:29:25.814] BUT IS NOT");
			}
			
		} else {
			// NO - WE DO NOT HAVE ms
			retVal = 0;
		}
		
		return retVal;
	}
	
	
	
	/**
	 * FROMAT OF DATAGRAM:
	 * # START
	 * ^ END
	 * @ DELEMETERS
	 * CORREL 	= 8 charaters
	 * BRICKID	= 17 charaters
	 * TYPE		
	 * CHECKSUM
	 * 
	 * #@CORREL@BRICKID@CHECKSUM@^
	 * #@
	 * 
	 * @return String to pass to brick via Xbee
	 */
	public String generateDataGram(){
		String retval = "#";
		retval = retval + "@" + this.getCorrelid();
		retval = retval + "@" + this.getBrickstuniqueId();
		retval = retval + "@" + this.getReadingtype();
		

		long checksum = 10000019;
		for(int index = 0; index < retval.length(); index++){
//			checksum += 100003 * (int)retval.charAt(index);
			checksum += multiply(retval.charAt(index));
		}			 // 7457
		checksum += (int) '@';
		checksum += (int) '^';
		retval = retval + "@" + checksum + "@^";
		
		this.setChecksum(""+checksum);
		
		return retval;
	}
	
	private long multiply(char a){
		long retval = 0;
		if((int)a < 70){
			retval = (int)a * 139;
		} else if((int)a < 75){
			retval = (int)a * 4517;
			
		} else if((int)a < 85){
			retval = (int)a * 7001;
			
		} else {
			retval = 100003 * a;
		}
		
		return retval;
	}
	
	public String getUniqueId() {
		return uniqueId;
	}

	public void setUniqueId(String uniqueId) {
		this.uniqueId = uniqueId;
	}

	public String getBrickstuniqueId() {
		return brickstuniqueId;
	}

	public void setBrickstuniqueId(String brickstuniqueId) {
		this.brickstuniqueId = brickstuniqueId;
	}

	public String getReadingtype() {
		return readingtype;
	}

	public void setReadingtype(String readingtype) {
		this.readingtype = readingtype;
	}


	public Timestamp getCreatedtimestamp() {
		return createdtimestamp;
	}

	public void setCreatedtimestamp(Timestamp createdtimestamp) {
		this.createdtimestamp = createdtimestamp;
	}


	public String getCorrelid() {
		return correlid;
	}

	public void setCorrelid(String correlid) {
		this.correlid = correlid;
	}

	public String getChecksum() {
		return checksum;
	}

	public void setChecksum(String checksum) {
		this.checksum = checksum;
	}
	
	public String getReadingvalue() {
		return readingvalue;
	}

	public void setReadingvalue(String readingvalue) {
		this.readingvalue = readingvalue;
	}

	public String getCommand() {
		return command;
	}

	public void setCommand(String command) {
		this.command = command;
	}

	public long getExecutiontime() {
		return executiontime;
	}

	public void setExecutiontime(long executiontime) {
		this.executiontime = executiontime;
	}












	public String getReadingport() {
		return readingport;
	}












	public void setReadingport(String readingport) {
		this.readingport = readingport;
	}

	
	
}
