package com.moneyhouse.sensors.xbee.thread;

import static java.nio.file.StandardWatchEventKinds.ENTRY_CREATE;
import static java.nio.file.StandardWatchEventKinds.ENTRY_DELETE;
import static java.nio.file.StandardWatchEventKinds.OVERFLOW;

import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.WatchEvent;
import java.nio.file.WatchKey;
import java.nio.file.WatchService;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import com.moneyhouse.exceptions.DataInconsistentRuntimeException;
import com.moneyhouse.sensors.config.CONTSTANTINTERFACE;
import com.moneyhouse.sensors.config.LogHelper;
import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.util.FileNameHelper;
import com.moneyhouse.sensors.xbee.XBeeDataQueueSingleton;
import com.moneyhouse.sensors.xbee.dto.ToBrickErrorFileQueueElementDataObject;
import com.moneyhouse.sensors.xbee.dto.ToBrickFileQueueElementDataObject;

public class ToBrickFileReceiverThread extends Thread {
	private final static Logger logger = Logger.getLogger(ToBrickFileReceiverThread.class);
	private final static Logger loggerfile = Logger.getLogger("requestLogger");
	private Thread t;
	private String threadName = "ToBrickFileReceiverV8" ;
	
	
	
	
	
	
	
	// 1. STEP
	public ToBrickFileReceiverThread(){
		logger.info("--> ToBrickFileReceiverThread()");
		logger.info("-- ITS THE START OF THIS THREAD");
		logger.info("-- IT HAS TWO LOGGERS - logger and loggerfile (\"requestLogger\")");
		loggerfile.info("-- ITS THE START OF THIS THREAD");
		loggerfile.info("-- IT HAS TWO LOGGERS - logger and loggerfile (\"requestLogger\")");
		logger.info("STATUS JSON: data_result_manage_brick_network_queues.json");
		loggerfile.info("STATUS JSON: data_result_manage_brick_network_queues.json");
		logger.info("<-- ToBrickFileReceiverThread()");
	}
	
	// 2. STARTS
	public void start ()
	{
		logger.info("--> ToBrickFileReceiverThread.start()");
		System.out.println("Starting " +  threadName );
		if (t == null)
		{
			t = new Thread (this, threadName);
			t.setPriority(MIN_PRIORITY);
			t.start ();
		}
		logger.info("<-- ToBrickFileReceiverThread.start()");

	}
	   
	   
	// 2. RUNS
	public void run() {
		logger.info("--> ToBrickFileReceiverThread.run()");
		nioFileListenerToBrick();
		logger.info("<-- ToBrickFileReceiverThread.run()");

	}

	@SuppressWarnings("unchecked")
	protected void nioFileListenerToBrick() {
		logger.debug("---> nioFileListenerToBrick()");
		// TODO Auto-generated method stub

		String toBrickDir = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_DIR);
		logger.debug("READING OUT toBrickDir ["+ toBrickDir+"]");
		Path dir = FileSystems.getDefault().getPath(toBrickDir);
		
		WatchKey key = null;
		
		try {
			WatchService watcher = FileSystems.getDefault().newWatchService();
		    key = dir.register(watcher, ENTRY_CREATE, ENTRY_DELETE /*, ENTRY_MODIFY*/);
		    
		    
		    for(;;){
		    	try {
		            key = watcher.take();
		        } catch (InterruptedException x) {
		            return;
		        }
		    	
		    	
		    	for (WatchEvent<?> event: key.pollEvents()) {
		            WatchEvent.Kind<?> kind = event.kind();
		            logger.info("EVENT TYPE ["+kind+"]");
	
		            // This key is registered only
		            // for ENTRY_CREATE events,
		            // but an OVERFLOW event can
		            // occur regardless if events
		            // are lost or discarded.
		            if (kind == OVERFLOW) {
		                continue;
		            }
		            
		            if (kind == ENTRY_CREATE) {
			            createFile(dir, event);
		            	continue;
		            }
		            
		            else if(kind == ENTRY_DELETE){
			            deleteFile(dir, event);
		            	
		            	
		            	continue;
		            	
		            }
		            else {
		            	logger.warn("THIS EVENT TYPE IS NOT SUPPORTED");
		            	
		            }
	
		            // The filename is the
		            // context of the event.
		            WatchEvent<Path> ev = (WatchEvent<Path>)event;
		            Path filename = ev.context();
	
		            // Verify that the new
		            //  file is a text file.
		            try {
		                // Resolve the filename against the directory.
		                // If the filename is "test" and the directory is "foo",
		                // the resolved name is "test/foo".
		                Path child = dir.resolve(filename);
		                if (!Files.probeContentType(child).equals("text/plain")) {
		                    System.err.format("New file '%s'" + " is not a plain text file.%n", filename);
		                    logger.error("New file ["+filename + "] is not a plain text file" );
		                    continue;
		                }
		            } catch (IOException x) {
		            	logger.error(x);
		                System.err.println(x);
		                continue;
		            }
	
		            // Email the file to the
		            //  specified email alias.
		            // System.out.format("Emailing file %s%n", filename);
		            //Details left to reader....
		        }
	
		        // Reset the key -- this step is critical if you want to
		        // receive further watch events.  If the key is no longer valid,
		        // the directory is inaccessible so exit the loop.
		        boolean valid = key.reset();
		        if (!valid) {
		        	logger.error("ERROR: KEY RESET NOT VALID - STOIP LOOP");
		            break;
		        }
		    }		    	
		} catch (IOException x) {
		    System.err.println(x);
			logger.error("ERROR: OCCURED"+ x);

		}
		logger.debug("<--- nioFileListenerToBrick()");

	}

	@SuppressWarnings("unchecked")
	private void deleteFile(Path dir, WatchEvent<?> event) {
		try{
			WatchEvent<Path> ev = (WatchEvent<Path>)event;
			Path filename = ev.context();
	
			loggerfile.info("*X* ITS AN ENTRY_DELETE EVENT ["+ LogHelper.writeShortFileName(filename.toString())+"]");
			logger.info("*X* ITS AN ENTRY_DELETE EVENT ["+ LogHelper.writeShortFileName(filename.toString())+"]");
			
			XBeeDataQueueSingleton queue = XBeeDataQueueSingleton.getInstance();
			String startsWith = FileNameHelper.returnFileNameStart(filename.toString());
			String correlID = FileNameHelper.returnCorrelationId(filename.toString());
			loggerfile.info("*X*        ENTRY_DELETE ELEMENT STARTING WITH ["+startsWith+"] AND CORRELID ["+correlID+"] WILL BE REMOVED FROM THE QUEUE");
			logger.info("*X*        ENTRY_DELETE ELEMENT STARTING WITH ["+startsWith+"] AND CORRELID ["+correlID+"] WILL BE REMOVED FROM THE QUEUE");
			
			ToBrickFileQueueElementDataObject elementInToBRICKQueue = queue.readContainingCharSequenceToBrick(startsWith, correlID);
			if(elementInToBRICKQueue != null){
			int retries = elementInToBRICKQueue.getReadingCounter();
				queue.deleteContainingCharSequenceToBrick(startsWith, correlID);
				loggerfile.info("AFTER ["+retries+"] ATTEMPTS TO SUCCESSFULLY CONTACT XBEE RADIO WE NOW DELETE THE FILE: " + LogHelper.writeShortFileName(dir.resolve(filename).toString()) +"\tWE HAVE THOUGH ["+queue.errorCount()+"] ELEMENTS IN THE ERROR QUEUE AND ANOTHER ["+queue.toBrickCount()+"] IN THE toBrick QUEUE");
			} else if(elementInToBRICKQueue == null) {
				// KOMMENTIERUNG ENTFERNT AM 4.7. DA SCHEINBAR DIE QUEUE UEBERLAUFT... WENN CALIB EINGESCHALTET - toBrick List HAT 174   Eintraege
				//                     																			  tobrickProcessed 29927 Eintraege
				//																								                   7. Juni 13:00 letzter eintrag BATCH_PULL__BASESTATION__CORR_6eFMM5su__BwwTyXDpvcovf2GXvjcXTA.txt	
				loggerfile.warn("TOBRICK QUEUE ENTRY DOES NOT EXIST FOR THE DELETED FILE: " + LogHelper.writeShortFileName(dir.resolve(filename).toString()) +"\tWE HAVE THOUGH ["+queue.errorCount()+"] ELEMENTS IN THE ERROR QUEUE AND ANOTHER ["+queue.toBrickCount()+"] IN THE toBrick QUEUE");
				ArrayList<ToBrickErrorFileQueueElementDataObject> elementsInErrorQueue = queue.findInToBrickErrorListQueueFilename(startsWith, correlID);
				if(elementsInErrorQueue != null ){
					loggerfile.info("TOBRICK ERROR QUEUE ENTRY HOLDS ["+elementsInErrorQueue.size()+"] ELEMENTS oF FILENAME START ["+startsWith+"] CORRELID ["+correlID+"] IN THIS QUEUE");
				} else {
					loggerfile.warn("TOBRICK ERROR QUEUE ENTRY RETURNS NULL WHEN SEARCHING FOR  ELEMENTS WITH oF FILENAME START ["+startsWith+"] CORRELID ["+correlID+"] ");
				}
				if(elementsInErrorQueue == null || elementsInErrorQueue.isEmpty()){
					// NO ELLEMENTS FOUNDS WITH THIS FILENAME START AND CORREL ID
					if(startsWith.equals(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_HEARTBEAT)){
						// HEARTBEAT GETS DELETED AS SOON AS JSON IS WRITTEN - SO NO ENTRY IN QUEUE
					} else {
						loggerfile.warn("TOBRICK AND ERROR QUEUE ENTRY DOES NOT EXIST FOR THE DELETED FILE: " + LogHelper.writeShortFileName(dir.resolve(filename).toString()) +"\tWE HAVE THOUGH ["+queue.errorCount()+"] ELEMENTS IN THE ERROR QUEUE AND ANOTHER ["+queue.toBrickCount()+"] IN THE toBrick QUEUE");
					}

				} else if(elementsInErrorQueue.size() == 1 && elementInToBRICKQueue == null){
						// HAPPY - ONe EXPECTED
					int del = queue.deleteContainingCharSequenceToBrickErrorList(startsWith, correlID);
					loggerfile.info("DELETED ["+del+"] ELEMENTS FROM TOBRICK ERROR QUEUE WITH oF FILENAME START ["+startsWith+"] CORRELID ["+correlID+"] ");

				} else if(elementsInErrorQueue.size() == 1 && elementInToBRICKQueue != null){
					// SAME ELEMENT IN ERROR QUEUE AS WELL AS TO BRICK PENDING QUEUE 
					loggerfile.error("ERROR: ELEMENT STARTING WITH ["+startsWith+"] AND CORRELID ["+correlID+"] IS IN IN THE PENDING AND ERROR LISt QUEUE");
					throw new DataInconsistentRuntimeException("ERROR: ELEMENT STARTING WITH ["+startsWith+"] AND CORRELID ["+correlID+"] IS IN THE PENDING AND ONE 1 TIME IN THE ERROR LISt QUEUE");
				} else if (elementsInErrorQueue.size() > 1 && elementInToBRICKQueue != null){
					// MULTIPLE ELEMENTS IN PENDING QUEUE AND SAME ELEMENT IN ERROR QUEUE AS WELL AS TO BRICK PENDING QUEUE 
					loggerfile.error("ERROR: IN ERROR LIST QUEUE WE FOUND ["+elementsInErrorQueue.size()+"] COUNTS OF ELEMENT STARTING WITH ["+startsWith+"] AND CORRELID ["+correlID+"] IS IN IN THE PENDING AND ERROR LISt QUEUE");
					throw new DataInconsistentRuntimeException("ERROR: ELEMENT STARTING WITH ["+startsWith+"] AND CORRELID ["+correlID+"] IS IN IN THE PENDING AND MORe THAN 1 TIME IN THE ERROR LISt QUEUE");
					
				} else {
					throw new RuntimeException("ERROR: SHOULD NOT COMe IN HERE - CHECK FILE TO DELETE WITH QUEUE CONSINSTENCY TO BRICK PENDING AND EROR LIST QUEUE  - ELEMENT STARTING WITH ["+startsWith+"] AND CORRELID ["+correlID+"]");
				}
				
				
				
				
				
				
				
				
			}
			
			
			
		}
		catch(Exception ec){
            logger.error("ERROR: OCCURED WHILE PROCESSING DELETE FILE EVENT ON "+dir+" DIRECTORY" );
            loggerfile.error("ERROR: OCCURED WHILE PROCESSING DELETE FILE EVENT ON "+dir+" DIRECTORY" , ec);
            logger.error(ec);
		}
	}

	private void createFile(Path dir, WatchEvent<?> event) {
		@SuppressWarnings("unchecked")
		WatchEvent<Path> ev = (WatchEvent<Path>)event;
		Path filename = ev.context();

		// loggerfile.info("ITS AN ENTRY_CREATE EVENT FILE NAME = ["+ filename+"]");
		logger.info("*** ITS AN ENTRY_CREATE EVENT FILE NAME = ["+ LogHelper.writeShortFileName(filename.toString())+"]");

		// filename.getParent(); 
		Path child = dir.resolve(filename);
		java.io.File aFile = (child.toFile()); 
//		        		java.io.File aFile = new File(filename.getParent().toString(), filename.getFileName().toString()); 
		
		XBeeDataQueueSingleton queue = XBeeDataQueueSingleton.getInstance();
		queue.addToBrick(new ToBrickFileQueueElementDataObject(aFile));
		logger.info("*** NOW ["+queue.countToBrickDataList()+"] ELEMENTS IN TOBRICK QUEUE");
		
		loggerfile.info("FILE ADDED: " + LogHelper.writeShortFileName(filename.toString()) +"\tCOUNT ["+queue.toBrickCount()+"] IN toBRICK QUEUE AND ["+queue.errorCount()+"] IN error QUEUE");
		
		
		// logger.info(queue);

	}

	
	
	
}
