/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.config;

import java.util.StringTokenizer;

import org.apache.log4j.Logger;

import com.moneyhouse.sensors.config.CONTSTANTINTERFACE;

public class LogHelper {

	private final static Logger logger = Logger.getLogger(LogHelper.class);
	
	
	
	private static String separateFilenameFromPath(String filenmaeandpath, String fileseperator){
		String filename = "";
		
		StringTokenizer st = new StringTokenizer(filenmaeandpath, fileseperator);

		while (st.hasMoreElements()) {
			filename = (String)st.nextElement();
		}
		if(filename == ""){
			throw new RuntimeException("ERROR: SEPARATING FILE NAME FROM PATH ["+filenmaeandpath+"] DIDNT WORK OOUT. SEPAROTR USED ["+fileseperator+"]");
		}
		return filename;
	}
	
	
	
	
	/** TO GET A NICER LOOGER OUTPUT - FILENAMES ARE REALLy LONG
	 *  LIKE THIS: BATCH_PULL__BASESTATION__CORR_zUyu2Vdg__W6P9DSvBmbvX5G2BYu5x6w.txt
	 *  OR LIKE  : CALIB_PULL_TASKID_vAfJAyFPGwMTdur9QpDd6g__BRICKID_BASESTATION__CORR_CALIBRATION_3jsMZIO8__wY0OJc5jG6O01tJ8jdVBXA.txt
	 *  OR LIKE  : COMMAND__0x00,0x13,0xa2,0x00,0x40,0xa0,0x98,0xf3__CORR_oBFojA6n__1VUtHlZyfEOFYXr5IuqXtg.txt
	 * @param longfilename
	 * @return
	 */
	public static String writeShortFileName(String longfilename){
		if(longfilename.contains("\\")){
			// IT SEENS THE PATH IS WITH THE FILENAME - LETS GET THE FILNAME
			longfilename = separateFilenameFromPath(longfilename, "\\");
		} else if(longfilename.contains("/")){
			longfilename = separateFilenameFromPath(longfilename, "/");
			
		}
		
		String retval = "";
		String cleandup = longfilename.replace(",0x", "");
		if(cleandup.contains(CONTSTANTINTERFACE.FILENAME_PART_CORR) && cleandup.startsWith(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_BATCH_PULL)){
			int pos = cleandup.indexOf(CONTSTANTINTERFACE.FILENAME_PART_CORR);
			retval = "BATCH_PULL_";
			retval += cleandup.substring(pos-4, pos + 16).trim();
		} 
		else if(cleandup.contains(CONTSTANTINTERFACE.FILENAME_PART_CORR) && cleandup.startsWith(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_BATCH_PULL)){
			int pos = cleandup.indexOf("_CORR_CALIBRATION_");
			retval = "CALIB_PULL_\t";
			retval += cleandup.substring(pos+17, pos + 29).trim();
		}
		else if(cleandup.contains(CONTSTANTINTERFACE.FILENAME_PART_CORR) && cleandup.startsWith(CONTSTANTINTERFACE.FILENAME_PART_FOR_COMMAND)){
			int pos = cleandup.indexOf("_CORR_");
			retval = "COMMAND_\t";
			retval += cleandup.substring(pos-4, pos + 16).trim();
		}
		else if(cleandup.startsWith(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_HEARTBEAT)){
//			int pos = cleandup.indexOf("_CORR_");
//			retval = CONTSTANTINTERFACE.FILENAME_PART_START_FOR_HEARTBEAT;
			retval = cleandup.substring(0,24).trim();
		}
		else if(cleandup.startsWith(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_CONFIG_ASSEMBLER)){
			// BRICK_CONFIG_ASSEMBLER__0x00,0x13,0xa2,0x00,0x40,0xa0,0x98,0xf3__CORR_tWrj3oWl__twiycTFzVd4R1PoBmifCIQ.txt
			// contains sendPLIST
			int pos = cleandup.indexOf("_CORR_");
			retval = "_ASSEMBLER__";
			retval += cleandup.substring(pos-4, pos + 16).trim();
			
		}
		else if(cleandup.startsWith(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_RULEACTION)){
			// BRICK_CONFIG_ASSEMBLER__0x00,0x13,0xa2,0x00,0x40,0xa0,0x98,0xf3__CORR_tWrj3oWl__twiycTFzVd4R1PoBmifCIQ.txt
			// contains sendPLIST
			int pos = cleandup.indexOf("_CORR_");
			retval = "_RULEACTION__";
			retval += cleandup.substring(pos-4, pos + 16).trim();
			
		}
		else if(cleandup.startsWith(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_CALIBRATION)){
			int pos = cleandup.indexOf(CONTSTANTINTERFACE.FILENAME_PART_FOR_CALIBRATION);
			retval = "CALIB_PULL_";
			retval += cleandup.substring(pos+17, pos + 29).trim();

		}
		else {
			// no correlation ID retrun the long name 
			retval = longfilename;
			logger.error("ERROR: tHE FILE ["+longfilename+"] DoES NoT APPLY TO THE CONVENTIONS OF FILENAMES ");
		}
		return retval;
	}

}
