/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.filetrigger;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.apache.log4j.Logger;
/**
 * It writes a event file into the from brick folder
 * @since 2014.10.28 
 * @author Frederic
 *
 */
public class FileMonitorFromBrickSingleton {
	private static FileMonitorFromBrickSingleton instance = null;
	private static Logger logger = Logger.getLogger(FileMonitorFromBrickSingleton.class);	
	
	protected FileMonitorFromBrickSingleton(){}
	
	   public static FileMonitorFromBrickSingleton getInstance() {
		      if(instance == null) {
		    	  instance = new FileMonitorFromBrickSingleton();
		      }
		      return instance;
		   }	
	
	
	/**
	 * Create a trigger file in monitored folder
	 * @param filename
	 */
	public synchronized void createFileAsEvent(String pathfilename, String content){
		logger.info("-----> FileMonitorFromBrickSingleton.createFileAsEvent("+pathfilename+", content) INSTANCE = "+ this.hashCode());
		FileOutputStream fop = null;
		File file;
		try {
 
			file = new File(pathfilename);
			fop = new FileOutputStream(file);
 
			// if file doesnt exists, then create it
			if (!file.exists()) {
				file.createNewFile();
			}
 
			// get the content in bytes
			byte[] contentInBytes = content.getBytes();
 
			fop.write(contentInBytes);
			fop.flush();
			fop.close();
 
			logger.info("WRITING FILE ["+pathfilename+"] DONE");
 
		} catch (IOException e) {
			logger.error("ERROR: ERROR OCCURED WHILE WRITING FILE TO ["+pathfilename+"] ", e);
			e.printStackTrace();
		} finally {
			try {
				if (fop != null) {
					fop.close();
				}
			} catch (IOException e) {
				logger.error("ERROR: ERROR OCCURED WHILE CLOSING FILE TO ["+pathfilename+"] ", e);
				e.printStackTrace();
			}
		}
		logger.info("<----- FileMonitorFromBrickSingleton.createFileAsEvent("+pathfilename+", "+content+")");
		
	}
}
	

