/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.filetrigger;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.sql.Timestamp;
import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;

import com.moneyhouse.sensors.xbee.facade.XBeeCommandConstants;
import com.moneyhouse.tmor.serial.xbee.dto.XBeeComandObject;
import com.moneyhouse.tmor.serial.xbee.dto.XBeeDataTransportObject;

public class FileMonitorToBrickSingleton implements Serializable{
	private static final long serialVersionUID = -5333838836482531831L;
	private static FileMonitorToBrickSingleton instance = null;
	private static Logger logger = Logger.getLogger(FileMonitorToBrickSingleton.class);	
	private int filecounter = 0;
	
	
	protected FileMonitorToBrickSingleton(){}
	
	public static FileMonitorToBrickSingleton getInstance() {
		logger.debug("----> getInstance()");
		if(instance == null) {
			instance = new FileMonitorToBrickSingleton();
		}
		logger.debug("<---- getInstance() instance = "+instance);
		return instance;
	}	
	
	

	

	
	
	
   

	public synchronized XBeeDataTransportObject readFileAndReturnDataObject(String filename) {
		logger.info("----> readFileAndReturnDataObject("+filename+")");
		BufferedReader br = null;
		XBeeDataTransportObject retVal = null; 
		try {
 
			String sCurrentLine;
 
			br = new BufferedReader(new FileReader(filename));
 
			while ((sCurrentLine = br.readLine()) != null) {
				retVal = processLine(sCurrentLine);
			}
 
		} catch (IOException e) {
			e.printStackTrace(); logger.error("ERROR OCCURED: ",e);
		} finally {
			try {
				if (br != null)br.close();
			} catch (IOException ex) {
				ex.printStackTrace(); logger.error("ERROR OCCURED: ", ex);
			}
		}
		logger.info("<---- readFileAndReturnDataObject("+filename+")");
		
		return retVal;
	}
	
	public synchronized XBeeComandObject readCommandFileAndReturnDataObject(String filename) {
		logger.info("----> readCommandFileAndReturnDataObject("+filename+")");
		BufferedReader br = null;
		XBeeComandObject retVal = null; 
		try {
 
			String sCurrentLine;
 
			br = new BufferedReader(new FileReader(filename));
 
			while ((sCurrentLine = br.readLine()) != null) {
				retVal = processCommandLine(sCurrentLine);
			}
 
		} catch (IOException e) {
			e.printStackTrace(); logger.error("ERROR OCCURED: ",e);
		} finally {
			try {
				if (br != null)br.close();
			} catch (IOException ex) {
				ex.printStackTrace(); logger.error("ERROR OCCURED: ", ex);
			}
		}
		logger.info("<---- readCommandFileAndReturnDataObject("+filename+")");
		
		return retVal;
	}	
	/**
	 * Read file content of data request file)
	 * 
	 * @param currentLine
	 * @return
	 */
	private static XBeeDataTransportObject processLine(String currentLine) {
		logger.info("----> processLine("+currentLine+")");
		List<String> items = Arrays.asList(currentLine.split("\\s*\t\\s*"));
		XBeeDataTransportObject retVal = null;
		if (items.size() == 7) {
			// data request file
			retVal = new XBeeDataTransportObject(items.get(1).trim(), 
												items.get(3).trim(), 
												items.get(4).trim(), 
												items.get(5).trim());
			@SuppressWarnings("unused")
			String entrymode = items.get(6).trim();
			// input type = 0 brickdata ; 1 manual; -1 simulated

			for (int i = 0; i < items.size(); i++) {
				String token = items.get(i);
				System.out.println(token);
			}
		}
		else if (items.size() == 5) {
			// 31.10.2014 - NOW IT WORKS WITH 5 ELEMENTS - LIKE sendPING
			retVal = new XBeeDataTransportObject(items.get(0).trim(), 
												items.get(1).trim(), 
												items.get(3).trim(), 
												items.get(4).trim());

		}
		else if (items.size() == 12 && items.get(1).equals(XBeeCommandConstants.CMD_TO_BRICK_WRITE_DRULE)) {
			// 31.10.2014 - NOW IT WORKS WITH 5 ELEMENTS - LIKE sendPING
			retVal = new XBeeDataTransportObject(items.get(0).trim(), 
												items.get(1).trim(), 
												items.get(10).trim(), 
												items.get(11).trim());

			retVal.storeAdditionalParameter("EVENT", items.get(3));
			retVal.storeAdditionalParameter("EVENT_VALUE", items.get(4));
			
			retVal.storeAdditionalParameter("OW_COMPVALUE", items.get(5));
			retVal.storeAdditionalParameter("OW_OPERATION", items.get(6));
			
			// STATUS
			retVal.storeAdditionalParameter("STATUS", items.get(9));

			// ACTION
			retVal.storeAdditionalParameter("ACTION", items.get(3));
			retVal.storeAdditionalParameter("ACTION_VALUE", items.get(8));
		}

		
		else 
		{
			throw new RuntimeException("ERROR: DATA REQUEST FILE IS SUPPOSED TO HAVE 7 ELEMENTS - BUT THIS ONE INLY HAS ["+items.size()+"] - CHECK CONTENT: "+currentLine);
		}
		logger.info("<---- processLine("+currentLine+")");
		return retVal;
	}

	/**
	 * Read file content of data request file)
	 * 
	 * @param currentLine
	 * @return
	 */
	private static XBeeComandObject processCommandLine(String currentLine) {
		logger.info("----> processCommandLine("+currentLine+")");
		List<String> items = Arrays.asList(currentLine.split("\\s*\t\\s*"));
		logger.debug("     items count= " + items.size());

		XBeeComandObject retVal = null;
		if (items.size() == 4) {
			// data request file
			retVal = new XBeeComandObject();
			retVal.setBrickid(items.get(0).trim());
			retVal.setAtcommand(items.get(1).trim());
			retVal.setAtcommandparameter(items.get(2).trim());
			retVal.setCorrelID(items.get(3).trim());
		}
		else if (items.size() == 5) {
			// data request file
			retVal = new XBeeComandObject();
			retVal.setBrickid(items.get(0).trim());
			retVal.setAtcommand(items.get(1).trim());
			retVal.setAtcommandparameter(items.get(2).trim());
			retVal.setCorrelID(items.get(3).trim());
			String timestampstr = items.get(4).trim();
			Timestamp timestp = Timestamp.valueOf(timestampstr);
			retVal.setTimestamp(timestp);
		}
		else if (items.size() == 12 && items.get(1).equals(XBeeCommandConstants.CMD_TO_BRICK_WRITE_DRULE)) {
			// 31.10.2014 - NOW IT WORKS WITH 5 ELEMENTS - LIKE sendPING
			retVal = new XBeeComandObject();
			retVal.setBrickid(items.get(0).trim());
			retVal.setAtcommand(items.get(1).trim());
			retVal.setAtcommandparameter(items.get(2).trim());
			retVal.setCorrelID(items.get(10).trim());

			retVal.storeAdditionalParameter("EVENT", items.get(3));
			retVal.storeAdditionalParameter("EVENT_VALUE", items.get(4));
			
			retVal.storeAdditionalParameter("OW_COMPVALUE", items.get(5));
			retVal.storeAdditionalParameter("OW_OPERATION", items.get(6));
			
			// STATUS
			retVal.storeAdditionalParameter("STATUS", items.get(9));

			// ACTION
			retVal.storeAdditionalParameter("ACTION", items.get(7));
			retVal.storeAdditionalParameter("ACTION_VALUE", items.get(8));

			
			
		}	
		
		
		else 
		{
			throw new RuntimeException("ERROR: COMMAND FILE IS SUPPOSED TO HAVE 4 ELEMENTS - BUT THIS ONE INLY HAS ["+items.size()+"] - CHECK CONTENT: "+currentLine);
		}
		logger.info("<---- processCommandLine("+currentLine+")");
		
		return retVal;
	}	
	
	public boolean isCommandFile(String filename) {
		logger.debug("----> isCommandFile("+filename+")");
		boolean retval = false;	
		BufferedReader br = null;
		
		try {
			 
			String sCurrentLine;
 
			br = new BufferedReader(new FileReader(filename));
 
			while ((sCurrentLine = br.readLine()) != null) {
				List<String> items = Arrays.asList(sCurrentLine.split("\\s*\t\\s*"));
				if (items.size() == 4 || items.size() == 5 || items.size() == 12  ) {
					retval = true;
				}		
			}
 
		} catch (IOException e) {
			e.printStackTrace(); logger.error("ERROR OCCURED: ",e);
		} finally {
			try {
				if (br != null)br.close();
			} catch (IOException ex) {
				ex.printStackTrace(); logger.error("ERROR OCCURED: ", ex);
			}
		}
		logger.debug("<---- isCommandFile("+filename+") RETVAL= "+retval);
		
		return retval;
	}
	

	
	
	/**
	 * @see #isCommandFile(String)
	 * THIS ONE CHECKS WHETHER tHE typeofFile parameter can be found in the file
	 * @param filename
	 * @param typeofFile
	 * @return
	 */
	public boolean isFile(String filename, String typeofFile) {
		logger.debug("----> isFile("+filename+")");
		boolean retval = false;	
		BufferedReader br = null;
		
		try {
			String sCurrentLine;
 
			br = new BufferedReader(new FileReader(filename));
 
			while ((sCurrentLine = br.readLine()) != null) {
				List<String> items = Arrays.asList(sCurrentLine.split("\\s*\t\\s*"));
				for(int g= 0 ; g < items.size();g++){
					String ele =  items.get(g);
					if (typeofFile.equalsIgnoreCase(ele)) {
						retval = true;
						break;
					}		
				}
			}
 
		} catch (IOException e) {
			e.printStackTrace(); logger.error("ERROR OCCURED: ",e);
		} finally {
			try {
				if (br != null)br.close();
			} catch (IOException ex) {
				ex.printStackTrace(); logger.error("ERROR OCCURED: ", ex);
			}
		}
		logger.debug("<---- isFile("+filename+") RETVAL= "+ retval);
		
		return retval;
	}
	
	
	
	public boolean isFileExistant(String filename) {
		logger.info("----> isFileExistant(" + filename + ")");
		boolean retval = false;
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(filename));
			retval = true;
		} catch (java.io.FileNotFoundException fnfe) {
			retval = false;
		} catch (Exception e) {
			logger.error("SOME OTHER ERROR OCCURED WHILE TESTING FOR EXISTANCE OF FILE ", 	e);
		} finally {
			if(retval == true){
				try {
					if (br != null)
						br.close();
				} catch (IOException ex) {
					logger.error("ERROR OCCURED: ", ex);
				}
			}
		}
		logger.info("<---- isFileExistant(" + generateShortFileName(filename) + ") retval="+retval);
		return retval;
	}
	
	
	/** TO GET A NICER LOOGER OUTPUT - FILENAMES ARE REALLy LONG
	 * @param longfilename
	 * @return
	 */
	private String generateShortFileName(String longfilename){
		String retval = "";
		String cleandup = longfilename.replace(",0x", "");
		if(cleandup.contains("__CORR_")){
			int pos = cleandup.indexOf("__CORR_");
			retval = cleandup.substring(pos-4, pos + 16).trim();
		} else {
			// no correlation ID retrun the long name 
			retval = longfilename;
		}
		
		return retval;
	}
	
	
	public int decrementFileCounter(){
		filecounter--;
		return filecounter;
	}

	public int incrementFileCounter(){
		filecounter++;
		return filecounter;
	}

	public int getFilecounter() {
		return filecounter;
	}

	public void setFilecounter(int filecounter) {
		this.filecounter = filecounter;
	}
	
}
	

