/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.internal.idgenerator;

public class Base64Codec {
	private static final byte[] BASE64_ALPHABET_ARRAY = { 'A', 'B', 'C', 'D',
			'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q',
			'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd',
			'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q',
			'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3',
			'4', '5', '6', '7', '8', '9', '+', '/' };
	private static final byte[] BASE64_FOR_URL_ALPHABET_ARRAY = { 'A', 'B',
			'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
			'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b',
			'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
			'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1',
			'2', '3', '4', '5', '6', '7', '8', '9', '-', '_' };
	private static final byte PAD = '=';

	/**
	 * Encodes the specified <code>bytes</code> using the Base64 encoding. </p>
	 * 
	 * @param bytes
	 *            the bytes to be encoded.
	 * @return the Base64-encoded bytes.
	 * @see #encode(String)
	 */
	public static byte[] encode(byte[] bytes) {
		return encode(bytes, BASE64_ALPHABET_ARRAY, true);
	}

	/**
	 * Encodes the specified <code>string</code> using the Base64 encoding. </p>
	 * 
	 * @param string
	 *            the string to be encoded.
	 * @return the Base64-encoded string.
	 * @see #encode(byte[])
	 */
	public static String encode(String string) {
		if (string == null) {
			return null;
		}
		return new String(encode(string.getBytes()));
	}

	public static byte[] encodeForURL(byte[] bytes) {
		return encode(bytes, BASE64_FOR_URL_ALPHABET_ARRAY, false);
	}

	private static byte[] encode(byte[] bytes, byte[] alphabetArray,
			boolean usePadding) {

		if (bytes == null) {
			return null;
		} else if (bytes.length == 0) {
			return bytes;
		}
		int _length = bytes.length;
		int _remainder = _length % 3;
		byte[] _bytes;
		if (usePadding) {
			_bytes = new byte[(_length + 2) / 3 * 4];
		} else {
			_bytes = new byte[((_length + 2) / 3 * 4)
					- (_remainder != 0 ? 3 - _remainder : 0)];
		}
		_length -= _remainder;
		int _group;
		int _i;
		int _index = 0;
		for (_i = 0; _i < _length;) {
			_group = (bytes[_i++] & 0xFF) << 16 | (bytes[_i++] & 0xFF) << 8
					| bytes[_i++] & 0xFF;
			_bytes[_index++] = alphabetArray[_group >>> 18];
			_bytes[_index++] = alphabetArray[_group >>> 12 & 0x3F];
			_bytes[_index++] = alphabetArray[_group >>> 6 & 0x3F];
			_bytes[_index++] = alphabetArray[_group & 0x3F];
		}
		switch (_remainder) {
		case 0:
			break;
		case 1:
			_group = (bytes[_i] & 0xFF) << 4;
			_bytes[_index++] = alphabetArray[_group >>> 6];
			if (usePadding) {
				_bytes[_index++] = alphabetArray[_group & 0x3F];
				_bytes[_index++] = PAD;
				_bytes[_index] = PAD;
			} else {
				_bytes[_index] = alphabetArray[_group & 0x3F];
			}
			break;
		case 2:
			_group = ((bytes[_i++] & 0xFF) << 8 | (bytes[_i] & 0xFF)) << 2;
			_bytes[_index++] = alphabetArray[_group >>> 12];
			_bytes[_index++] = alphabetArray[_group >>> 6 & 0x3F];
			if (usePadding) {
				_bytes[_index++] = alphabetArray[_group & 0x3F];
				_bytes[_index] = PAD;
			} else {
				_bytes[_index] = alphabetArray[_group & 0x3F];
			}
			break;
		default:
			// this should never happen.
		}
		return _bytes;
	}
}
