/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.internal.idgenerator;

import java.security.MessageDigest;
import java.util.Random;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.NoSuchAlgorithmException;

import org.apache.log4j.Logger;

import com.moneyhouse.tmor.serial.internal.idgenerator.IDGenerator;
import com.moneyhouse.tmor.serial.internal.idgenerator.Base64Codec;

public class IDGenerator {
	private String seed;
	private long counter;
	private String ipAddress;
	private static MessageDigest md5;
	private static Logger logger = Logger.getLogger(IDGenerator.class);
	public IDGenerator() {
		this(String.valueOf(new Random().nextInt()));
	}

	public IDGenerator(String seed) {
		logger.debug("---> 2IDGenerator("+seed+")");
		this.seed = seed.trim();
		this.counter = 0;
		try {
			ipAddress = InetAddress.getLocalHost().getHostAddress();
			md5 = MessageDigest.getInstance("MD5");
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		} catch (UnknownHostException e) {
			logger.error("ERROR: 2IT SEEMS THERE IS nO INTERENT CONNECTION (NETWORK CABLE PLUGGED IN???) - CHECK "+e);
			logger.error("ERROR: 2IT SEEMS THERE IS nO INTERENT CONNECTION - WE USE 127.0.0.1");

			try {
				ipAddress = "127.0.0.1";
				md5 = MessageDigest.getInstance("MD5");
			} catch (NoSuchAlgorithmException exc) {
				throw new RuntimeException(exc);
			} 
		}
		logger.info(this);
		logger.debug("<--- 2IDGenerator("+seed+") ipAddress= "+this.ipAddress);
	}

	/**
	 * Creates a unique ID based on the specified <code>string</code>. </p>
	 * 
	 * @return a unique ID.
	 */
	public synchronized String getNewID() {
		StringBuffer buffer = new StringBuffer();
		buffer.append(++counter);
		buffer.append(System.currentTimeMillis());
		buffer.append(seed);
		buffer.append(ipAddress);
		buffer.append(Math.random());
		byte[] digest = md5.digest(buffer.toString().getBytes());
		byte[] encodedDigest = Base64Codec.encodeForURL(digest);
		String ret = new String(encodedDigest);
		String retVal = comformIDToJS(ret);
		return retVal;
	}

	/**
	 * Create short 8 charater chain
	 * @return
	 */
	public synchronized String createCorrelationID() {
		StringBuffer buffer = new StringBuffer();
		buffer.append(++counter);
		buffer.append(System.currentTimeMillis());
		buffer.append(seed);
		buffer.append(ipAddress);
		buffer.append(Math.random());
		byte[] digest = md5.digest(buffer.toString().getBytes());
		byte[] encodedDigest = Base64Codec.encodeForURL(digest);
		String ret = new String(encodedDigest);
		String retVal = ret.substring(8, 16);
		return retVal;
	}
	
	
	
	/**
	 * Javascript does not support _ and ither characters - so we need to get them out od the ID
	 * @param ret
	 * @return
	 */
	public synchronized String comformIDToJS(String ret){
		String retval = ret.replace("_", "X");
		
		return retval;
	}
}
