/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.json;

import java.util.ArrayList;

import com.moneyhouse.tmor.serial.xbee.dto.XBeeDataTransportObject;

//import com.moneyhouse.util.global.dto.BricksValueDataObject;
//import com.moneyhouse.util.global.dto.JSONBrickValueDataTimeSeries;

public class TransformDataToJasonFileFormat {

	
	
	
	/**
	 * Creates JSON For Messages coming from XBee
	 * @param List<BricksValueDataObject>
	 * @param BricksDataObject
	 * @return
	 */
	public synchronized static final ArrayList<JSONAbstract> transform(XBeeDataTransportObject data){
		ArrayList<JSONAbstract> retval = new ArrayList<JSONAbstract>();
		String brickid = data.getBrickstuniqueId().replace("0x", "").replace(",", "").trim();	
		
		if(data.getCommand() == null){
			throw new RuntimeException("ERROR: DATA RECEIVED HAS COMMAND == NULL - EXPECTED IS PING SLIST PLIST OR EVENT");
		} 
		else if(data.getCommand().equals("SLIST") || data.getCommand().equals("PLIST") || data.getCommand().equals("PING")){
			JSONPingAnswer ele = new JSONPingAnswer();
			ele.setCorrelid(data.getCorrelid());

			ele.setFromaddress(brickid);
			ele.setCommand(data.getCommand());
			
			// MIGHT need some transforming for better readability
			ele.setTimestamp(data.getCreatedtimestamp().toString());
			ele.setExecutiontime(""+data.getExecutiontime());
			retval.add(ele);
		}
		else if(data.getCommand().equals("EVENT")){
			JSONEvent ele = new JSONEvent();
			ele.setCorrelid(data.getCorrelid());

			ele.setFromaddress(brickid);
			ele.setCommand(data.getCommand());
			
			// MIGHT need some transforming for better readability
			ele.setTimestamp(data.getCreatedtimestamp().toString());
			ele.setPortname(data.getReadingport());
			ele.setPortvalue(data.getReadingvalue());
			ele.setValuetype(data.getReadingtype());
			retval.add(ele);
		}	
		else if(data.getCommand().equals("RDRL") || data.getCommand().equals("RADC")){
			JSONRDRL ele = new JSONRDRL();
			ele.setCorrelid(data.getCorrelid());

			ele.setFromaddress(brickid);
			ele.setCommand(data.getCommand());
			
			// MIGHT need some transforming for better readability
			ele.setTimestamp(data.getCreatedtimestamp().toString());
			ele.setPortname(data.getReadingport());
			ele.setPortvalue(data.getReadingvalue());
			ele.setValuetype(data.getReadingtype());
			retval.add(ele);
		}
		else if(data.getCommand().equals("RVERSI")){
			JSONRDRL ele = new JSONRDRL();
			ele.setCorrelid(data.getCorrelid());

			ele.setFromaddress(brickid);
			ele.setCommand(data.getCommand());
			
			// MIGHT need some transforming for better readability
			ele.setTimestamp(data.getCreatedtimestamp().toString());
			ele.setPortname("VERSION");
			ele.setPortvalue(data.getReadingvalue());
			ele.setValuetype(data.getReadingtype());
			retval.add(ele);
		}				
		else {
			throw new RuntimeException("ERROR: COMMAND ["+data.getCommand()+"] IS NOT KNOWN - EXPECTED IS EVENT, PLIST, SLIST OR PING");
			
		}
		return retval;
	}	
}
