/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.xbee;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.sql.Timestamp;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;

import com.moneyhouse.exceptions.BadDataInputRuntimeException;
import com.moneyhouse.sensors.config.CONTSTANTINTERFACE;
import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.util.TimeStampProducer;
import com.moneyhouse.tmor.serial.filetrigger.FileMonitorFromBrickSingleton;
import com.moneyhouse.tmor.serial.filetrigger.FileMonitorToBrickSingleton;
import com.moneyhouse.tmor.serial.internal.idgenerator.IDGenerator;
import com.moneyhouse.tmor.serial.json.CreateJsonFile;
import com.moneyhouse.tmor.serial.xbee.dto.XBeeDataTransportObject;
import com.rapplogic.xbee.api.XBeeAddress64;
import com.rapplogic.xbee.api.XBeeResponse;
import com.rapplogic.xbee.api.zigbee.ZNetRxResponse;
import com.rapplogic.xbee.util.ByteUtils;

@Deprecated
public class SerialReceiver6ComposeAnswer {
	private final static Logger logger = Logger.getLogger(SerialReceiver6ComposeAnswer.class);
	private final static Logger loggerfile = Logger.getLogger("requestLogger");
	
	
	
	
	/**
	 * Data Received from XBee Radio are processed here and written to File 
	 * @param answer --- serial input from the xbee radio
	 * @param list of files to brick - to get the match between send and receive correlId
	 */
	public synchronized static XBeeDataTransportObject prepareFileFROM_BRICK(XBeeResponse answer) {
		logger.info("----> prepareFileFROM_BRICK(XBeeResponse answer)");
		XBeeDataTransportObject retVal = null;
		ZNetRxResponse answerRxRes = (ZNetRxResponse)answer; 
		// its a return from the user
		// - process it
		FileMonitorFromBrickSingleton fm = FileMonitorFromBrickSingleton.getInstance();
		String filename = "FROM_BRICK__";			// FILE PREFIX
		String brickidfrom = "";					// 64 BIT ADDRESS OF RADIO MODULE
		String command = "";						// COMMAND RECEIVED PING, 
		String parameter = "";
		String correlId = "";						// CORRELATION ID RECEIVED FROM tHE BRICK
		Timestamp tsnow = currentTimestamp();		// THE TIMe NOW OF THE PROCESSING
		String execTime = "";						// TIME IN MS FOR THE EXECUTION OF A COMMAND
		
		String value = "";			// SENDING PORT OF THE EVENT
		String rule = "";			// SENDING TRIGGERED RULE ID 
		String eventValue = "";						// SENDING VALUE OF THE EVENT
		
		XBeeAddress64 address = answerRxRes.getRemoteAddress64();
		brickidfrom = address.toString();
		
		String commandAndCorrelIDAndVAL = ByteUtils.toString(answerRxRes.getData());		// Brick returns COMMAND[CORRELID] [VAL]
		logger.debug("RECEIVED DATA: " + commandAndCorrelIDAndVAL);
		
		
		loggerfile.info("RECEIVED DATA: " + commandAndCorrelIDAndVAL);
		
		StringTokenizer strTokens = new StringTokenizer(commandAndCorrelIDAndVAL, " ");
		
		int tokencounter = 0;
		while (strTokens.hasMoreElements()) {
			tokencounter++;
			String tok = strTokens.nextToken();
			if(tokencounter == 1){
				// get the command + correlId out
				StringTokenizer cmdTokens = new StringTokenizer(tok, "[");
				command = cmdTokens.nextToken();
				if(command != null && command.equals("EVENT")){
					// ITS AN EVENT
					String correlIdPre = cmdTokens.nextToken();
					if(correlIdPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						correlId = correlIdPre.substring(0, (correlIdPre.length()-1));
					} else {
						// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
					String eventPortNameNumberPre = cmdTokens.nextToken();
					if(eventPortNameNumberPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						value = eventPortNameNumberPre.substring(0, (eventPortNameNumberPre.length()-1));
					} else {
						// THE PORT IDENTIFIER TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
					String eventValuePre = cmdTokens.nextToken();
					if(eventValuePre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						eventValue = eventValuePre.substring(0, (eventValuePre.length()-1));
					} else {
						// THE PORT IDENTIFIER TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
				}
				
				else if(command != null && command.equals("RDOW")){
					// ITS AN READ ONE WIRE RETURN ANSWER
					String correlIdPre = cmdTokens.nextToken();
					if(correlIdPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						correlId = correlIdPre.substring(0, (correlIdPre.length()-1));
					} else {
						// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
					String valuePre = cmdTokens.nextToken();
					if(valuePre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						value = valuePre.substring(0, (valuePre.length()-1)).trim();
					} else {
						// THE PORT IDENTIFIER TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
				}
				else if(command != null && command.equals("RDRL")){
					// ITS AN READ RULE RETURN ANSWER
					String correlIdPre = cmdTokens.nextToken();
					if(correlIdPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						correlId = correlIdPre.substring(0, (correlIdPre.length()-1));
						retVal = parseRDRL(fm, filename, brickidfrom, tsnow, correlId, retVal, commandAndCorrelIDAndVAL, cmdTokens);
					} else {
						// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
				}
		
				
				else if(command != null && command.equals("RADC")){
					// ITS AN READ RULE RETURN ANSWER
					String correlIdPre = cmdTokens.nextToken();
					if(correlIdPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						correlId = correlIdPre.substring(0, (correlIdPre.length()-1));
						retVal = parseRADC(fm, filename, brickidfrom, tsnow, correlId, commandAndCorrelIDAndVAL, cmdTokens);
					} else {
						// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
				}				
				
				
				else if(command != null && command.equals("RULE")){
					// ITS AN READ ONE WIRE RETURN ANSWER
					String correlIdPre = cmdTokens.nextToken();
					if(correlIdPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						correlId = correlIdPre.substring(0, (correlIdPre.length()-1));
					} else {
						// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
					String ruleIdPre = cmdTokens.nextToken();
					if(ruleIdPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						rule = ruleIdPre.substring(0, (ruleIdPre.length()-1)).trim();
					} else {
						// THE PORT IDENTIFIER TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
					String valuePre = cmdTokens.nextToken();
					if(valuePre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						value = valuePre.substring(0, (valuePre.length()-1)).trim();
					} else {
						// THE PORT IDENTIFIER TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}

				}
				else if(command != null && command.equals("RDOP")){
					// ITS AN READ ONE WIRE RETURN ANSWER
					String correlIdPre = cmdTokens.nextToken();
					if(correlIdPre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						correlId = correlIdPre.substring(0, (correlIdPre.length()-1));
					} else {
						// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
					String valuePre = cmdTokens.nextToken();
					if(valuePre.endsWith("]")){
						// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
						value = valuePre.substring(0, (valuePre.length()-1)).trim();
					} else {
						// THE PORT IDENTIFIER TOKEN DOES NOT END WITH ] - ITS AN ERROR 
						// todo 
					}
				}
				else if(command != null && command.equals("SLIST")){
					// ITS IN THIS FORMAT
					// SLIST[B6uwm27A][A][01234567][O]
					retVal = parseSLIST(retVal, commandAndCorrelIDAndVAL, cmdTokens);
					
					// just read the rest of the tokes
					while(cmdTokens.hasMoreElements()){
						cmdTokens.nextToken();
					}
				}
				else if(command != null && command.equals("RVERSI")){
					// ITS IN THIS FORMAT
					// SLIST[B6uwm27A][A][01234567][O]
					retVal = parseREADVERSION(fm, filename, brickidfrom,  tsnow, retVal, commandAndCorrelIDAndVAL, cmdTokens); 
					// just read the rest of the tokes
					while(cmdTokens.hasMoreElements()){
						cmdTokens.nextToken();
					}
				}
				else if(command != null && command.equals("PLIST")){
					// ITS IN THIS FORMAT
					// SLIST[B6uwm27A][A][01234567][O]
					retVal = parsePLIST(retVal, commandAndCorrelIDAndVAL, cmdTokens);
					
					// just read the rest of the tokes
					while(cmdTokens.hasMoreElements()){
						cmdTokens.nextToken();
					}
				}
				
				//else if(command != null && command.equals("PING") || command.equals("PLIST"))
				else if(command != null && command.equals("PING"))	// 28.01.2015
				{
					// ITS A PING 
					
					retVal = parsePING( fm,retVal, command, cmdTokens, tsnow, execTime);
					
					

				}
				while (cmdTokens.hasMoreElements()) {
					// HERE COMES THE VALUE TREATMENT IN
					// disabled on the 20141209
					// command = cmdTokens.nextToken();
					cmdTokens.nextToken();
				}
			} else if(tokencounter >2){
				// there shouldn't be more than 2 Token... 
				throw new RuntimeException("ERROR: CHECK THE FORMAT OF XBEE RECEIVED DATA - IT HAS THE FORMAT COMMAND[CORRELID] [VALUE] --- BUT WE HAVE RECEIVED: "+commandAndCorrelIDAndVAL);
			}
		}
		if(command.equals("EVENT")){
			retVal = parseEVENT(fm, filename, brickidfrom, command, parameter, correlId, tsnow, value, eventValue);
		} else if(command.equals("RDOW")){
			retVal = parseRDOW(fm, filename, brickidfrom, command, correlId, tsnow, execTime, value);
		} else if(command.equals("RDOP")){
			String prevalue = commandAndCorrelIDAndVAL.substring(14, 26);
			if(!prevalue.startsWith("[")){
				throw new RuntimeException("ERROR: THIS PART SHOULD START WITH [");
			}
			if(!prevalue.endsWith("]")){
				throw new RuntimeException("ERROR: THIS PART SHOULD END WITH ]");
			}
			value = prevalue.replace("[", "").replace("]", "").replace(" ", "").trim();
			
			filename = filename + brickidfrom +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+".txt";
			String pathfilename = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_FROM_BRICK_DIR);;
			
			pathfilename = pathfilename + filename;
			logger.debug("send command by writing file into  "+pathfilename);
			
			String tsnowstr = tsnow.toString();
			tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);
			logger.debug("tsnowstr "+tsnowstr);
			String content =brickidfrom+"\t"+command+ "\t" + correlId+"\t"+value+"\t"+"\t"+tsnowstr+ "\tEXECUTIONTIME="+execTime;
			
			fm.createFileAsEvent(pathfilename, content);
			
	        XBeeDataTransportObject contentObj = new XBeeDataTransportObject(brickidfrom, "RDOP", "B", "",value, correlId,  tsnow); 
	        
	        // return value
	        retVal = contentObj;

			
		} 
		else if(command.equals("SLIST") || command.equals("PLIST") ){
			// WRITE THE FINDINGS INTO THE CONFIG FILE FOLDER	
			String correlIdd = retVal.getCorrelid();
			filename = filename + brickidfrom +"___CORR_"+correlIdd+"___" + new IDGenerator().getNewID()+".txt";
			String pathfilename = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_CONFIG_DIR);
			
			pathfilename = pathfilename + filename;
			logger.debug("send command by writing file into  "+pathfilename);
			parameter = retVal.getReadingport() +"\t"+retVal.getReadingvalue() + "\t"+retVal.getReadingtype();
			String tsnowstr = tsnow.toString();
			tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);
			logger.debug("tsnowstr "+tsnowstr);
			String content =brickidfrom+"\t"+command+ "\t" + correlIdd+"\t"+parameter+"\t"+tsnowstr+ "\tEXECUTIONTIME="+execTime;
			fm.createFileAsEvent(pathfilename, content);
			
		} else if(command.equals("RULE")){
			retVal = parseRULE(fm, filename, brickidfrom, command, correlId, rule, tsnow, execTime, value);
		} else if(command.equals("RDRL")){
			// EVERY THING IS DONE IN parseRDRL
			// THIS BLOCK IS TO MAKE SURE THAT NO ENTRY IS WRITTEN
			// INTO THE FROM BRICK FOLDER - 20170301
		} else if(command.equals("RADC")){
			// EVERY THING IS DONE IN parseADC
			// THIS BLOCK IS TO MAKE SURE THAT NO ENTRY IS WRITTEN
			// INTO THE FROM BRICK FOLDER - 20170323
		} else if(command.equals("PING")){
			// EVERY THING IS DONE IN parsePING
			// THIS BLOCK IS TO MAKE SURE THAT NO ENTRY IS WRITTEN
			// INTO THE FROM BRICK FOLDER - 20170323
		} else if(command.equals("RVERSI")){
			// EVERY THING IS DONE IN parseREADVERSION
			// THIS BLOCK IS TO MAKE SURE THAT NO ENTRY IS WRITTEN
			// INTO THE FROM BRICK FOLDER - 20170323
		}		else {
			for(int j = 0 ; j < 10; j++){
				logger.info("THE COMMAND ["+command+"] IS NOT SUPPORTED - CODE IT - 20150914");
			}
			filename = filename + brickidfrom +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+".txt";
			String pathfilename = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_FROM_BRICK_DIR);;
			
			pathfilename = pathfilename + filename;
			logger.debug("send command by writing file into  "+pathfilename);
			
			String tsnowstr = tsnow.toString();
			tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);

			logger.debug("tsnowstr "+tsnowstr);
			String content =brickidfrom+"\t"+command+ "\t" + correlId+"\t"+parameter+"\t"+"\t"+tsnowstr+ "\tEXECUTIONTIME="+execTime;
			
			fm.createFileAsEvent(pathfilename, content);
		}
		logger.info("<---- prepareFileFROM_BRICK(XBeeResponse answer) retVal="+retVal);

		return retVal;
	}


	private static XBeeDataTransportObject parseEVENT(FileMonitorFromBrickSingleton fm, String filename, String brickidfrom, String command, String parameter, String correlId, Timestamp tsnow, String value, String eventValue) {
		XBeeDataTransportObject retVal;
		filename = filename + brickidfrom +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+".txt";
		String pathfilename = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_FROM_BRICK_DIR);
		
		pathfilename = pathfilename + filename;
		logger.debug("send event by writing file into  "+pathfilename);
		
		String tsnowstr = tsnow.toString();
		// tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);
		String timeZone = PropertiesFileReader.getInstance().get3LetterTimeZoneCode();
		tsnowstr = TimeStampProducer.createTimestampStringForTimeZone(timeZone, "yyyy-MM-dd HH:mm:ss",tsnow);

		logger.info("COMMAND EVENT ["+correlId+"]\tPARAMETER ["+parameter+"]\tVALUE ["+value+"]");

		String content =brickidfrom+"\t"+command+ "\t" + correlId+"\t"+parameter+"\t"+value+"\t"+eventValue+"\t"+tsnowstr;
		
		fm.createFileAsEvent(pathfilename, content);
		
		
		// create JSON FILE FOR DISPLAY
		CreateJsonFile json = new CreateJsonFile();
		XBeeDataTransportObject contentObj = new XBeeDataTransportObject(brickidfrom, "EVENT", value, "",eventValue, correlId,  tsnow); 
		
		// return value
		retVal = contentObj;
		json.createXBeeResponseFile(contentObj, "msg_event.json");
		return retVal;
	}

	
	/**
	 * READS THE READ VERSION command ANSWER AND WRITES IT INTO THE BRICK CONFIG FOLDER
	 * @param fm
	 * @param filename
	 * @param brickidfrom
	 * @param tsnow
	 * @param retVal
	 * @param commandAndCorrelIDAndVAL
	 * @param cmdTokens
	 * @return
	 */
	private static XBeeDataTransportObject parseREADVERSION(FileMonitorFromBrickSingleton fm, String filename, String brickidfrom,  Timestamp tsnow, XBeeDataTransportObject retVal, String commandAndCorrelIDAndVAL, StringTokenizer cmdTokens) {
		String correlId = "";
		String versionValue = "";
		if(commandAndCorrelIDAndVAL.length() == 21){
			// YES RIGHT LENGTH 
			if(commandAndCorrelIDAndVAL.charAt(6) == '[' && commandAndCorrelIDAndVAL.charAt(15) == ']' && commandAndCorrelIDAndVAL.charAt(16) == '[' && commandAndCorrelIDAndVAL.charAt(20) == ']'){
				// FROMAT OF BRACKETS OK
				correlId = commandAndCorrelIDAndVAL.substring(7, 15);
				versionValue = commandAndCorrelIDAndVAL.substring(17,20);
			} else {
				logger.error("ERROR: WE EXPECTED AT POS 6 and 16 CHARCTER OPEN BRACKETS ([) BUT RECEIVED ["+commandAndCorrelIDAndVAL.charAt(6)+"] AND ["+commandAndCorrelIDAndVAL.charAt(16)+"] AND THE CLOSING BRACKES ON POS 15 and 20 ARE ["+commandAndCorrelIDAndVAL.charAt(15)+"] AND ["+commandAndCorrelIDAndVAL.charAt(20)+"]. THE DATAGRAM SHOULD LOOK LIKE RVERSI[RU7l7pWH][1.0] BUT WE RECIEVED ["+commandAndCorrelIDAndVAL+"]");
			}
		} else {
			logger.error("ERROR: CAN NOT PARSE COMMAND AND CORREL AND VALUES - EXPECTED IS RVERSI[RU7l7pWH][1.0] BUT WE RECIEVED ["+commandAndCorrelIDAndVAL+"]");
		}
		
		
		
		
		
		
		
		filename = filename + brickidfrom +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+".txt";
		String pathfilename = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_CONFIG_DIR);
		
		pathfilename = pathfilename + filename;
		logger.debug("send event by writing file into  "+pathfilename);
		
	
		
		String tsnowstr = tsnow.toString();
		// tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);
		String timeZone = PropertiesFileReader.getInstance().get3LetterTimeZoneCode();
		tsnowstr = TimeStampProducer.createTimestampStringForTimeZone(timeZone, "yyyy-MM-dd HH:mm:ss",tsnow);

		logger.info("COMMAND RVERSI ["+correlId+"]\tVALUE ["+versionValue+"]");

		String content =brickidfrom+"\tRVERSI\t" + correlId+"\t"+versionValue+"\t"+tsnowstr;
		
		fm.createFileAsEvent(pathfilename, content);
		
		
		// create JSON FILE FOR DISPLAY
		CreateJsonFile json = new CreateJsonFile();
		XBeeDataTransportObject contentObj = new XBeeDataTransportObject(brickidfrom, "RVERSI", "VERSION", "",versionValue, correlId,  tsnow); 
		
		// return value
		retVal = contentObj;
		json.createXBeeResponseFile(contentObj, "msg_event.json");
		return retVal;
	}	
	

	/**
	 * Extracted on the 14.09.2015
	 * @param fm
	 * @param filename
	 * @param brickidfrom
	 * @param command
	 * @param correlId
	 * @param tsnow
	 * @param execTime
	 * @param value
	 * @return
	 */
	private static XBeeDataTransportObject parseRDOW(FileMonitorFromBrickSingleton fm, String filename, String brickidfrom, String command, String correlId, Timestamp tsnow, String execTime, String value) {
		XBeeDataTransportObject retVal;
		logger.info("parseRDOW filename "+ filename);		
		logger.info("parseRDOW brickidfrom "+ brickidfrom + "\tbeautified "+ beautifyXBeeAddress(brickidfrom));		
		logger.info("parseRDOW command "+ command);		
		
		logger.info("parseRDOW correlId "+ correlId);		
		logger.info("parseRDOWTimestamp "+ tsnow);		
		logger.info("parseRDOW execTime "+ execTime);		
		logger.info("parseRDOW Value "+ value);		
		
		
		filename = filename + brickidfrom +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+".txt";
		String pathfilename = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_FROM_BRICK_DIR);;
		
		pathfilename = pathfilename + filename;
		logger.debug("send command by writing file into  "+pathfilename);
		
		String tsnowstr = tsnow.toString();
		tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);
		logger.info("COMMAND RDOW ["+correlId+"]\tVALUE ["+value+"]");
		String content =brickidfrom+"\t"+command+ "\t" + correlId+"\t"+value+"\t"+"\t"+tsnowstr+ "\tEXECUTIONTIME="+execTime;
		
		fm.createFileAsEvent(pathfilename, content);
		
		XBeeDataTransportObject contentObj = new XBeeDataTransportObject(brickidfrom, "RDOW", "OW", "",value, correlId,  tsnow); 
		
		// return value
		retVal = contentObj;
		
		// DELETE toBRICK file - check the correl ID
		// added on the 16.07.2015
//	        deleteFilesWithCorrelIDFromDir(correlId, PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_DIR));

		// REMOVE FROM tHE toBRickQueue
		XBeeDataQueueSingleton toBrick = XBeeDataQueueSingleton.getInstance();
		toBrick.deleteContainingCharSequenceToBrick(correlId);
		loggerfile.info("DELETE FROM toBRICK QUEUE: " + correlId);
		return retVal;
	}
	
	/**
	 * Copied from Rule on the 16.12.2016
	 * @param fm
	 * @param filename
	 * @param brickidfrom
	 * @param command
	 * @param correlId
	 * @param tsnow
	 * @param execTime
	 * @param value
	 * @return
	 */
	private static XBeeDataTransportObject parseRULE(FileMonitorFromBrickSingleton fm, String filename, String brickidfrom, String command, String correlId, String ruleId, Timestamp tsnow, String execTime, String value) {
		XBeeDataTransportObject retVal;
		logger.info("parseRULE filename "+ filename);		
		logger.info("parseRULE brickidfrom "+ brickidfrom + "\tbeautified "+ beautifyXBeeAddress(brickidfrom));		
		logger.info("parseRULE command "+ command);		
		
		logger.info("parseRULE correlId "+ correlId);		
		logger.info("parseRULETimestamp "+ tsnow);		
		logger.info("parseRULE execTime "+ execTime);		
		logger.info("parseRULE Value "+ value);		
		
		
		filename = filename + brickidfrom +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+"_RULE.txt";
		String pathfilename = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_FROM_BRICK_DIR);;
		
		pathfilename = pathfilename + filename;
		logger.debug("send command by writing file into  "+pathfilename);
		
		String tsnowstr = tsnow.toString();
		tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);
		logger.info("COMMAND RULE ["+correlId+"]\tID ["+ruleId+ "]\tVALUE ["+value+"]");
		String content =brickidfrom+"\t"+command+ "\t" + correlId+"\t"+ruleId+"\t"+value+"\t"+"\t"+tsnowstr+ "\tEXECUTIONTIME="+execTime;
		
		fm.createFileAsEvent(pathfilename, content);
		
		XBeeDataTransportObject contentObj = new XBeeDataTransportObject(brickidfrom, "RULE", "OW", "",value, correlId,  tsnow); 
		
		// return value
		retVal = contentObj;
		
		// DELETE toBRICK file - check the correl ID
		// added on the 16.07.2015
//	        deleteFilesWithCorrelIDFromDir(correlId, PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_DIR));

		// REMOVE FROM tHE toBRickQueue
		XBeeDataQueueSingleton toBrick = XBeeDataQueueSingleton.getInstance();
		toBrick.deleteContainingCharSequenceToBrick(correlId);
		loggerfile.info("DELETE FROM toBRICK QUEUE: " + correlId);
		return retVal;
	}
	
	private static XBeeDataTransportObject parsePLIST(
			XBeeDataTransportObject retVal, String commandAndCorrelIDAndVAL,
			StringTokenizer cmdTokens) {
			String correlIdPre = cmdTokens.nextToken();
			String correlIdstr = "";
			if(correlIdPre.endsWith("]")){
				// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
				correlIdstr = correlIdPre.substring(0, (correlIdPre.length()-1));
			} else {
				// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
				// todo 
			}
			String st = Character.toString(commandAndCorrelIDAndVAL.charAt(16));
			
			if(st.equals("A") || st.equals("B") || st.equals("C")){
				// ITS A IO PORT 
				// read out the pins
				retVal = new XBeeDataTransportObject(st);
				retVal.setReadingport(st); 
				retVal.setCorrelid(correlIdstr);
				retVal.setCommand("PLIST");
				String ports = "";
				for(int i = 19 ; i < 19+8; i++){
					st = Character.toString(commandAndCorrelIDAndVAL.charAt(i));
					if(st.equals("]")){
						// end of port number parameter
						break;
					} else {
						ports = ports + st;
					}
				}
				retVal.setReadingvalue(ports);
				retVal.setCreatedtimestamp(new Timestamp(System.currentTimeMillis()));
				String outOrIn =  st = Character.toString(commandAndCorrelIDAndVAL.charAt(29));
				if(outOrIn.equals("O") || outOrIn.equals("I") ){
					retVal.setReadingtype(outOrIn);
				} else {
					throw new RuntimeException("ERROR: EXPECTED WAS A O or I BUT RECEIVED ["+outOrIn+"] - CHEKC DATA GRAM SLIST FOR PORTS A, B, C ");
				}
			} else{
				throw new RuntimeException("ERROR: EXPECTED WAS A, B, C BUT RECEIVED ["+st+"] - CHEKC DATA GRAM SLIST FOR PORTS A, B, C ");
				
			}
			return retVal;
		}

	
	/**SLOW OR FAST VERSION 
	 * @param cmd
	 * @param result
	 * @param waittime in ms
	 * @return
	 */
	private static XBeeDataTransportObject parseSLIST(XBeeDataTransportObject retVal, String commandAndCorrelIDAndVAL, StringTokenizer cmdTokens) {
		String correlIdPre = cmdTokens.nextToken();
		String correlIdstr = "";
		if(correlIdPre.endsWith("]")){
			// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
			correlIdstr = correlIdPre.substring(0, (correlIdPre.length()-1));
		} else {
			// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
			// todo 
		}
		String st = Character.toString(commandAndCorrelIDAndVAL.charAt(16));
		
		if(st.equals("A") || st.equals("B") || st.equals("C") || st.equals("D") || st.equals("E") || st.equals("F")){
			// ITS A IO PORT 
			// read out the pins
			retVal = new XBeeDataTransportObject(st);
			retVal.setReadingport(st); 
			retVal.setCorrelid(correlIdstr);
			retVal.setCommand("SLIST");
			String ports = "";
			for(int i = 19 ; i < 19+8; i++){
				st = Character.toString(commandAndCorrelIDAndVAL.charAt(i));
				if(st.equals("]")){
					// end of port number parameter
					break;
				}	else if(st.equals(" ")) {
					// SKIPP EMPTY STRING
				}	else {
					ports = ports + st;
				}
			}
			retVal.setReadingvalue(ports);
			retVal.setCreatedtimestamp(new Timestamp(System.currentTimeMillis()));
			String outOrIn =  st = Character.toString(commandAndCorrelIDAndVAL.charAt(29));
			if(outOrIn.equals("O") || outOrIn.equals("I") ){
				retVal.setReadingtype(outOrIn);
			} else {
				throw new RuntimeException("ERROR: EXPECTED WAS A O or I BUT RECEIVED ["+st+"] - CHEKC DATA GRAM SLIST FOR PORTS A, B, C, D, E, F ");
			}
		} else if(st.equals("O")){
			// ITS A ONE WIRE PORT - GET THE ADDRESS
			String port = commandAndCorrelIDAndVAL.substring(16, 18);
			if(port.equals("OW")){
				// YES ITS THE ONE WIRE PORT - GET THE OW ADDRESS
				// THE FORMAT RECEIVED: SLIST[fCX-_ptM][OW][28040F0105000017]
				if(commandAndCorrelIDAndVAL.length() != "SLIST[fCX-_ptM][OW][28040F0105000017]".length()){
					throw new RuntimeException("ERROR: DATAGRAM LENGTH EXPEXTED IS 37 - BUT WE RECEIVED A DATAGRAM WITH LENGTH ["+commandAndCorrelIDAndVAL.length()+"] - CHECK FROMAT OF DATAGRAM ["+commandAndCorrelIDAndVAL+"]");
				} else {
					// right length
					// check the brackets on the right position 
					String brack1 = Character.toString(commandAndCorrelIDAndVAL.charAt(5));
					String brack2 = Character.toString(commandAndCorrelIDAndVAL.charAt(14));
					String brack3 = Character.toString(commandAndCorrelIDAndVAL.charAt(15));
					String brack4 = Character.toString(commandAndCorrelIDAndVAL.charAt(18));
					String brack5 = Character.toString(commandAndCorrelIDAndVAL.charAt(19));
					String brack6 = Character.toString(commandAndCorrelIDAndVAL.charAt(36));
					if(brack1.equals("[") && brack2.equals("]") && brack3.equals("[")&& brack4.equals("]")&& brack5.equals("[")&& brack6.equals("]")){
						// brackes ok la
						// pack the obj with data
						retVal = new XBeeDataTransportObject(st);
						retVal.setReadingport(port); 
						String correlIDfromOW = commandAndCorrelIDAndVAL.substring(6, 14);
						retVal.setCorrelid(correlIDfromOW);
						retVal.setCommand("SLIST");
						retVal.setCreatedtimestamp(new Timestamp(System.currentTimeMillis()));
						retVal.setReadingtype("I");
						String sensoraddress = commandAndCorrelIDAndVAL.substring(20, 36);
						retVal.setReadingvalue(sensoraddress);
						
						
					} else {
						// brackets wrong
						throw new RuntimeException("ERROR: DATAGRAM ["+commandAndCorrelIDAndVAL.length()+"] - HAS WRONG FORMATTING - CHECK THE BRACKETS AND THEIR POSITION");
						
					}
				}
			}
		
		
		}
		return retVal;
	}

	
	
	
	
	
	
	
	
	
	
	/**
	 * PARSES A READ ADC VALUE DATAGRAM LIKE THIS ONE
	 * RADC[ZUVY7BKR][09][003F][1]  -- THE RULE VERSION - SHOULD BE HANDLED VIA RULE CODE SEGMEnt
	 * RADC[vpTo9SKe][1134][1]
	 * @param fm
	 * @param filename
	 * @param brickidfrom
	 * @param tsnow
	 * @param correlId
	 * @param retVal
	 * @param commandAndCorrelIDAndVAL
	 * @param cmdTokens
	 * @return
	 */
	private static XBeeDataTransportObject parseRADC(FileMonitorFromBrickSingleton fm, String filename, String brickidfrom, Timestamp tsnow, String correlId, String commandAndCorrelIDAndVAL, StringTokenizer cmdTokens) {
		XBeeDataTransportObject retVal = new XBeeDataTransportObject();
		boolean isADC = commandAndCorrelIDAndVAL.contains("RADC");
		if(isADC){
			// YES ITS A ADC VALUE
			String readingValuePre = cmdTokens.nextToken();
			if(readingValuePre.length() == 5 && readingValuePre.contains("]")){
				// ITS 1234]
				readingValuePre = readingValuePre.substring(0,4);
				try{
					int readingValue = Integer.parseInt(readingValuePre, 10);
					retVal.setReadingvalue(""+readingValue);
				} catch(NumberFormatException ex){
					logger.error("ERROR: ADC READIN VALUe ["+readingValuePre+"] IS NOT A NUMBER ", ex);
				}
			} else {
				// NOt THE EXPECTED NUMBER FROMAT
				logger.error("ERROR: ADC READING VALUe IS EXPECTED TO BE A 4 DIGIT HEX NUMBER TRAILED BY A ] BUT IS ["+readingValuePre+"]");
			}
			// GET THE RULE NUMBER	
			String adcPortValuePre = cmdTokens.nextToken();
			if(adcPortValuePre.length() == 2 && adcPortValuePre.contains("]")){
				adcPortValuePre = adcPortValuePre.substring(0,1);
				try{
					int adcPortValue = Integer.parseInt(adcPortValuePre, 16);
					retVal.setReadingport("ADC"+adcPortValue);
				} catch(NumberFormatException ex){
					logger.error("ERROR: ADC PORT NUMBER VALUe ["+adcPortValuePre+"] IS NOT A NUMBER ", ex);
				}
			} else {
				// NOt THE EXPECTED NUMBER FROMAT
				logger.error("ERROR: ADC PORT NUMBER VALUe IS EXPECTED TO BE A 1 DIGIT DECIMAL NUMBER TRAILED BY A ] BUT IS ["+adcPortValuePre+"]");
			}
		} else {
			// NO ITS NOT A ADC VALUE - WHAT DO WE DO HERE?
		}
		
		retVal.setReadingtype("ADC");
		retVal.setCorrelid(correlId);
		retVal.setCommand("RADC");
		retVal.setBrickstuniqueId(beautifyXBeeAddress(brickidfrom));
		retVal.setCreatedtimestamp(new Timestamp(System.currentTimeMillis()));
		
		
		
		
		// DO THE FILE WRITING 
		
		writeFileRADCfromBrick(fm, filename, brickidfrom, tsnow, correlId,	retVal);
		
		
		// create JSON FILE FOR DISPLAY
		CreateJsonFile json = new CreateJsonFile();
		json.createXBeeResponseFile(retVal, "msg_event.json");
		
		return retVal;
	}


	private static void writeFileRADCfromBrick( FileMonitorFromBrickSingleton fm, String filename, String brickidfrom, Timestamp tsnow, String correlId, XBeeDataTransportObject retVal) {
		filename = filename + formatBricksUniqueID(brickidfrom) +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+"_RDRL.txt";
		String pathfilename = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_FROM_BRICK_DIR);;
		
		pathfilename = pathfilename + filename;
		logger.debug("send event by writing file into  "+pathfilename);
		
		String tsnowstr = tsnow.toString();
		// tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);
		String timeZone = PropertiesFileReader.getInstance().get3LetterTimeZoneCode();
		tsnowstr = TimeStampProducer.createTimestampStringForTimeZone(timeZone, "yyyy-MM-dd HH:mm:ss",tsnow);

		logger.info("COMMAND RADC  ["+correlId+"]["+retVal.getReadingport()+"]["+retVal.getReadingvalue()+"]");

		String content =brickidfrom+"\tRADC\t" + correlId+"\t"+retVal.getReadingport()+"\t"+retVal.getReadingvalue()+"\t"+tsnowstr;
		
		fm.createFileAsEvent(pathfilename, content);
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	



	private static Timestamp currentTimestamp() {
		Date d = new java.util.Date();
		Timestamp now = new Timestamp(d.getTime());
		return now;
	}	
	
	
	private static String beautifyXBeeAddress(String xbeeAddress){
		String retval = xbeeAddress.replace("0,0x", "").replace("0x0", "").trim();
		return retval;
	}

	// PARSE THE INCOMING DATA STREAM FROM SERIAL PORT
	// WRITES A JSON FILE FOR MESSAGING 
	private static XBeeDataTransportObject parsePING(FileMonitorFromBrickSingleton fm, XBeeDataTransportObject retVal, String command, StringTokenizer cmdTokens, Timestamp tsnow, String execTime) {
		String correlIdPre = cmdTokens.nextToken();

		String correlId = "";
		
		if(correlIdPre.endsWith("]")){
			// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
			correlId = correlIdPre.substring(0, (correlIdPre.length()-1));
			
			// lets look whether we have the originating trigger file here - like for ping
			int filecounterWithCorrelid = 0;
			
			String tobrickdir = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_DIR);
			File folder = new File(tobrickdir);
			File[] listOfFilesToBrick2 = folder.listFiles();
			
			for(int jd = 0 ; jd < listOfFilesToBrick2.length; jd++){
				File ff =  listOfFilesToBrick2[jd];
				String fname = ff.getName();
				if((ff.isFile() && fname.contains(correlId))|| ff.getPath().contains(correlId)){
					filecounterWithCorrelid++;
				}
			}
			if(filecounterWithCorrelid == 0){
				// no trigger file found - so its a push event comming from the brick
			} else if(filecounterWithCorrelid == 1){
				// one trigger file found - perfect
				File theonefile =  null;
				for(int jd = 0 ; jd < listOfFilesToBrick2.length; jd++){
					theonefile =  listOfFilesToBrick2[jd];
					if(theonefile.isFile() && theonefile.getName().contains(correlId)){
						break;
					}
				}
				String fn = theonefile.getAbsolutePath();
				FileMonitorToBrickSingleton fileaccess = FileMonitorToBrickSingleton.getInstance();
		        XBeeDataTransportObject content =  fileaccess.readFileAndReturnDataObject(fn);
		        
		        // return value
		        retVal = content;
		        long noww = tsnow.getTime();
				String timeZone = PropertiesFileReader.getInstance().get3LetterTimeZoneCode();
				// noww = TimeStampProducer.createTimestampStringForTimeZone(timeZone, "yyyy-MM-dd HH:mm:ss",tsnow);
				noww = TimeStampProducer.createTimestampForTimeZone(timeZone).getTime();
				String tsnowstr = TimeStampProducer.createTimestampWithTimeZoneUTC();
				Timestamp nowwTS = Timestamp.valueOf(TimeStampProducer.checkAndModifyTimeStamp(tsnowstr));
				
				long bfore = content.getCreatedtimestamp().getTime();

		        long difference = nowwTS.getTime() - bfore;		// get the roundtrip time in millis
		        
		        execTime = "" + difference; 		// store it into the execTime
		        content.setExecutiontime(difference);
		        content.setCommand(command);		// the Command like ping
		        // NOW DELETE THE TOBRICK FILE
		        theonefile.delete();
		        
		        
		        // create JSON FILE FOR DISPLAY
		        CreateJsonFile json = new CreateJsonFile();
		        json.createXBeeResponseFile(content, "msg_command.json");
		        
			} else if(filecounterWithCorrelid > 1){
				// more than one trigger file found - get the newest file - ignore the others
				File[] listOfFilesToBrick3 = folder.listFiles();
				
				// index to find the newest / last file 
				Timestamp last = null; 
				File theonefile =  null;
				
				for(int jd = 0 ; jd < listOfFilesToBrick3.length; jd++){
					File ff =  listOfFilesToBrick3[jd];
					String fname = ff.getName();
					if((ff.isFile() && fname.contains(correlId))|| ff.getPath().contains(correlId)){
						filecounterWithCorrelid++;
						
						String sCurrentLine = "";
						try 
						{
							BufferedReader br = new BufferedReader(new FileReader(ff.getAbsolutePath()));
							
							List<String> items = null;
							while ((sCurrentLine = br.readLine()) != null) {
								items = Arrays.asList(sCurrentLine.split("\\s*\t\\s*"));
							}

							String tsStr = items.get(4).trim();
							Timestamp ts = Timestamp.valueOf(tsStr);
							
							if(last == null || last.before(ts)){
								theonefile =  ff;
								last = ts;
							}
						}
						catch(Exception ex){
							throw new RuntimeException("ERROR: PROBLEMS READING toBrick FILES WITH Correlation ID ["+correlId+"]");
						}
					}
				}
				
				String fn = theonefile.getAbsolutePath();
				FileMonitorToBrickSingleton fileaccess = FileMonitorToBrickSingleton.getInstance();
		        XBeeDataTransportObject content =  fileaccess.readFileAndReturnDataObject(fn);
		        
		        // return value
		        retVal = content;
		        long noww = tsnow.getTime();
				String timeZone = PropertiesFileReader.getInstance().get3LetterTimeZoneCode();
				// noww = TimeStampProducer.createTimestampStringForTimeZone(timeZone, "yyyy-MM-dd HH:mm:ss",tsnow);
				noww = TimeStampProducer.createTimestampForTimeZone(timeZone).getTime();
				String tsnowstr = TimeStampProducer.createTimestampWithTimeZoneUTC();
				Timestamp nowwTS = Timestamp.valueOf(TimeStampProducer.checkAndModifyTimeStamp(tsnowstr));
				
				long bfore = content.getCreatedtimestamp().getTime();

		        long difference = nowwTS.getTime() - bfore;		// get the roundtrip time in millis
		        
		        execTime = "" + difference; 		// store it into the execTime
		        content.setExecutiontime(difference);
		        content.setCommand(command);		// the Command like ping
		        // NOW DELETE THE TOBRICK FILE
		        theonefile.delete();
		        
		        
		        // create JSON FILE FOR DISPLAY
		        CreateJsonFile json = new CreateJsonFile();
		        json.createXBeeResponseFile(content, "msg_command.json");
			}
	        writeFilePINGfromBrick(fm, retVal, command, tsnow, execTime, correlId);
		}
		return retVal;
	}


	private static void writeFilePINGfromBrick(FileMonitorFromBrickSingleton fm, XBeeDataTransportObject retVal, String command, Timestamp tsnow,
			String execTime, String correlId) {
		// CREATE THE PING FILE IN BRIKc FOLDER
		String filename = "FROM_BRICK__" + retVal.getBrickstuniqueId() +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+".txt";
		String pathfilename = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_FROM_BRICK_DIR);;
		
		pathfilename = pathfilename + filename;
		logger.debug("send command by writing file into  "+pathfilename);
		
		String tsnowstr = tsnow.toString();
		tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);

		logger.debug("tsnowstr "+tsnowstr);
		String fileContent =retVal.getBrickstuniqueId()+"\t"+command+ "\t" + correlId+"\t"+tsnowstr+ "\tEXECUTIONTIME="+execTime;
		
		fm.createFileAsEvent(pathfilename, fileContent);
	}

	// READ DIRECT RULE ANSWER
	// WRITES ANSER INTO filetrigger.dir.brickconfig 
	private static XBeeDataTransportObject parseRDRL(FileMonitorFromBrickSingleton fm, String filename, String brickidfrom, Timestamp tsnow, String correlId, XBeeDataTransportObject retVal, String commandAndCorrelIDAndVAL, StringTokenizer cmdTokens) {
		String ruleID = cmdTokens.nextToken();
		String eventPin = cmdTokens.nextToken();
		String eventValue = cmdTokens.nextToken();
		String owComapreValue = cmdTokens.nextToken();
		
		String owOperation = cmdTokens.nextToken();
		String ruleActionOutput = cmdTokens.nextToken();
		String ruleActionValue = cmdTokens.nextToken();
		String isRuleActive = cmdTokens.nextToken();
		
		while (cmdTokens.hasMoreElements()) {
			// HERE should not come any other token
			throw new BadDataInputRuntimeException("ERROR: RDRL PARAMETERS MORE THAN EXPECTED - CHECK RULES_DEFINITION.h - IT SHOULD LOOK LIKE THIS RDRL[RjwSDVzv][1][25][H][OFFOFF][5][11][H][1]");
		}
		
		String ruleIDstr = "";
		if(ruleID.endsWith("]")){
			// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
			ruleIDstr = ruleID.substring(0, (ruleID.length()-1));
		} else {
			// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
			throw new BadDataInputRuntimeException("ERROR: RDRL PARAMETER ruleID has no closing bracked!");
		}
		String eventPinstr = "";
		if(eventPin.endsWith("]")){
			// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
			eventPinstr = eventPin.substring(0, (eventPin.length()-1));
		} else {
			// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
			throw new BadDataInputRuntimeException("ERROR: RDRL PARAMETER eventPin has no closing bracked!");
		}
		String eventValuestr = "";
		if(eventValue.endsWith("]")){
			// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
			
			// THIS IS A HEX VALUE 
			eventValuestr = eventValue.substring(0, (eventValue.length()-1));
			if(eventValuestr.contains("L")){
				// DO NOTHING
				eventValuestr = "0";
			} else if (eventValuestr.contains("H")){
				// DO NOTHING
				eventValuestr = "1";
			} else {
				try{
					int decVal = Integer.parseInt(eventValuestr, 16);
					eventValuestr = "" + decVal;	
				} catch(NumberFormatException ex){
					logger.error("ERROR: RDRL PARAMETER eventValue EXPECTED A HEX VALUE BUT RECEIVED ["+eventValuestr+"] - CANT TRANSFORM TO DEC");
				}
				
			}
			
			
			
			
		} else {
			// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
			throw new BadDataInputRuntimeException("ERROR: RDRL PARAMETER ruleID has no closing bracked!");
		}
		String owComapreValuestr = "";
		if(ruleID.endsWith("]")){
			// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
			owComapreValuestr = owComapreValue.substring(0, (owComapreValue.length()-1));
		} else {
			// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
			throw new BadDataInputRuntimeException("ERROR: RDRL PARAMETER owComapreValueHi has no closing bracked!");
		}
		
		String owOperationstr = "";
		if(owOperation.endsWith("]")){
			// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
			owOperationstr = owOperation.substring(0, (owOperation.length()-1)).trim();
			try{
				// WE KEEP IT HEX VALUE IN FILE 
				// int decVal = Integer.parseInt(owOperationstr, 10);
				int decVal = Integer.parseInt(owOperationstr, 16);
				// owOperationstr = "" + decVal;	
			} catch(NumberFormatException ex){
				logger.error("ERROR: RDRL PARAMETER owOperation EXPECTED A HEX VALUE BUT RECEIVED ["+owOperationstr+"] - CANT TRANSFORM TO DECIMAL VALUE ");
			}
		} else {
			// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
			throw new BadDataInputRuntimeException("ERROR: RDRL PARAMETER owOperation has no closing bracked!");
		}
		String ruleActionOutputstr = "";
		if(ruleActionOutput.endsWith("]")){
			// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
			ruleActionOutputstr = ruleActionOutput.substring(0, (ruleActionOutput.length()-1));
		} else {
			// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
			throw new BadDataInputRuntimeException("ERROR: RDRL PARAMETER ruleActionOutput has no closing bracked!");
 
		}
		String ruleActionValuestr = "";
		if(ruleActionValue.endsWith("]")){
			// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
			ruleActionValuestr = ruleActionValue.substring(0, (ruleActionValue.length()-1));
		} else {
			// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
			throw new BadDataInputRuntimeException("ERROR: RDRL PARAMETER ruleActionValue has no closing bracked!");
 
		}
		String isRuleActivestr = "";
		if(isRuleActive.endsWith("]")){
			// YES THE FORMAT IS RIGHT - WE HAVE CLOSING BRACKET
			if(isRuleActive.substring(0, (isRuleActive.length()-1)).equalsIgnoreCase("I")){
				// INACTIVE
				isRuleActivestr = "0";
			} else if(isRuleActive.substring(0, (isRuleActive.length()-1)).equalsIgnoreCase("N")){
				// NOT DEFINED
				isRuleActivestr = "-1";
			} else {
				try{
					int intValForActive = new Integer(isRuleActive.substring(0, (isRuleActive.length()-1))).intValue();
					isRuleActivestr = "" + intValForActive;
				} catch (NumberFormatException nfe){
					throw new BadDataInputRuntimeException("ERROR: RDRL PARAMETER isRuleActive is not H or L and not a number - check the Value ["+isRuleActive.substring(0, (isRuleActive.length()-1))+"] AS IT SI WRONG!!");
					
				}
			}
		} else {
			// THE CORRELID TOKEN DOES NOT END WITH ] - ITS AN ERROR 
			throw new BadDataInputRuntimeException("ERROR: RDRL PARAMETER isRuleActive has no closing bracked!");
 
		}
		
		
		logger.info("PARSRED RDRL COMMAND WITH PARAMETERS ["+ruleIDstr+"]["+eventPinstr+"]["+eventValuestr+"]["+owComapreValuestr+"]["+owOperationstr+"]["+ruleActionOutputstr+"]["+ruleActionValuestr+"]["+isRuleActivestr+"]");
		
		
		

		filename = filename + formatBricksUniqueID(brickidfrom) +"___CORR_"+correlId+"___" + new IDGenerator().getNewID()+"_RDRL.txt";
		String pathfilename = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_CONFIG_DIR);
		
		pathfilename = pathfilename + filename;
		logger.debug("send event by writing file into  "+pathfilename);
		
		String tsnowstr = tsnow.toString();
		// tsnowstr = TimeStampProducer.createTimestampStringForTimeZone("UTC", "yyyy-MM-dd HH:mm:ss",tsnow);
		String timeZone = PropertiesFileReader.getInstance().get3LetterTimeZoneCode();
		tsnowstr = TimeStampProducer.createTimestampStringForTimeZone(timeZone, "yyyy-MM-dd HH:mm:ss",tsnow);

		logger.info("COMMAND RDRL  ["+correlId+"]["+ruleIDstr+"]["+eventPinstr+"]["+eventValuestr+"]["+owComapreValuestr+"]["+owOperationstr+"]["+ruleActionOutputstr+"]["+ruleActionValuestr+"]["+isRuleActivestr+"]");

		String content =brickidfrom+"\tRDRL\t" + correlId+"\t"+ruleIDstr+"\t"+eventPinstr+"\t"+eventValuestr+"\t"+owComapreValuestr+"\t"+owOperationstr+"\t"+ruleActionOutputstr+"\t"+ruleActionValuestr+"\t"+isRuleActivestr+"\t"+tsnowstr;
		
		fm.createFileAsEvent(pathfilename, content);
		
		
		// create JSON FILE FOR DISPLAY
		CreateJsonFile json = new CreateJsonFile();
		XBeeDataTransportObject contentObj = new XBeeDataTransportObject(brickidfrom, "RDRL", ruleIDstr, "",eventValue, correlId,  tsnow); 
		
		// return value
		retVal = contentObj;
		json.createXBeeResponseFile(contentObj, "msg_event.json");
	
		
		
		
		return retVal;
	}

	
	// BEAUTIFY THE XBEE ADDRESS
	private static String formatBricksUniqueID(String xbeeAddress){
		String retval = xbeeAddress.replace("0x", "").replace(",", "").trim();
		return retval;
	}
}
