/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.xbee;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.log4j.Logger;

import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.config.ServerEnvironmentSingleton;

/**
 * HOLDS ALL THE COMMON CODE FOR THE SERIAL RECIEVER IMPLEMENTATION
 * IMPLEMENTATION ARE RIGHT NOW FOR:
 * - RASPBERRY 2
 * - RASPBERRY 3 - SEARIAL DISABLED
 * @author Frederic
 *
 */
public abstract class SerialReceiverAbstract {
	private final static Logger logger = Logger.getLogger(SerialReceiverAbstract.class);
	private long startTimestamp = 0; 							// its the time when the SerialReciever Was Started
	
	/**
	 * DELETE THE FILES FROM BRICK TO BRICK OR BRICKCONFIG WHICH ARE OLDER THAN PARAMETER olderThank
	 * @param olderThan 1d, 2d, 1w, 10w etc
	 * @param directory
	 */
	protected static int deleteFilesOlderThanFromDir(String olderThan, String directory) {
		logger.info("---> deleteFilesOlderThanFromDir("+olderThan+", "+directory+")");  
		int retvalCountDel =0;
		try{
			
			int daysBack = 1;
			int multiplier = 1;
			int number = -1;
			long basehours = 24 * 60 * 60 * 1000;
			if(olderThan.contains("w") || olderThan.contains("W")){
				multiplier = 7;
			} else if(olderThan.contains("d") ||olderThan.contains("D")){
				multiplier = 1;
			} else if(olderThan.contains("h") ||olderThan.contains("H")){
				// change the base from days to hours
				basehours = 1 * 60 * 60 * 1000;		
			}  else {
				logger.error("ERROR: PARAMETER ["+olderThan+"] NOT SUPPORTED - EXPECTED IS 1d, 2D, 3w, 4W");
				throw new RuntimeException("PARAMETER ["+olderThan+"] h or H NOT SUPPORTED");
			}
			try{
				number = new Integer(olderThan.substring(0,  olderThan.length()-1)).intValue();
			} catch (NumberFormatException nfe){
				logger.error("ERROR: PARAMETER ["+olderThan+"] NOT SUPPORTED - EXPECTED IS 1h, 1d, 2D, 3w, 4W", nfe);
				throw new RuntimeException("ERROR: PARAMETER ["+olderThan+"] NOT SUPPORTED - EXPECTED IS 1d, 2D, 3w, 4W", nfe);
			}
	
			if(number < 0){
				logger.error("ERROR: PARAMETER ["+olderThan+"] CAN NOT BE NEGATIVE - EXPECTED IS 0h, 1h, 1d, 2D, 3w, 4W");
				throw new RuntimeException("ERROR: PARAMETER ["+olderThan+"] CAN NOT BE NEGATIVE - EXPECTED IS 1d, 2D, 3w, 4W");
			}
			daysBack = multiplier * number;
			
			long filecounter= 0, purgeTime= 0, delcounter = 0;
			
			File dir = new File(directory);
			if(dir.exists()){
	
			    File[] listFiles = dir.listFiles();    
			    filecounter = listFiles.length;
			    purgeTime = System.currentTimeMillis() - (daysBack * basehours);
			    for(File listFile : listFiles) {
			        if(listFile.lastModified() < purgeTime) {
			        	boolean isDelSuccess = listFile.delete();
			        	delcounter++;
			        	retvalCountDel++;
			            if(!isDelSuccess) {
			            	delcounter--;
			            	logger.error("Unable to delete file: " + listFile);
			            }
			         }
			      }
			   }
			// GETR THE TIMESTAMP
			DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			Date deldate = new Date();
			deldate.setTime(purgeTime);
			logger.info("DELETED ["+delcounter+"] FILES FROM TOTAL ["+filecounter+"] THAT WERE OLDER THAN ["+dateFormat.format(deldate)+"] ACCORDING TO PARAMETER ["+olderThan+"]");  
		}
		catch (Exception ex){
			logger.error("ERROR: ERROR WHILE DELETING FILES" , ex);
		}
		logger.info("<--- deleteFilesOlderThanFromDir("+olderThan+", "+directory+") COUNT DELETED FILES= "+ retvalCountDel);
		return retvalCountDel;
	}
	



	
	public void run(){}
	
	
	
	public long getStartTimestamp() {
		return startTimestamp;
	}

	
	
	public long getUptimeAsLong() {
		long now = System.currentTimeMillis();
		
		long durationL = now - this.getStartTimestamp();
		return durationL;
	}	
	
	/** 
	 * @return UPTIME: 151 days 21:11:49
	 */
	public String getUptimeAsString() {
		long durationL = getUptimeAsLong();
		System.out.println("durationL "+ durationL);
		
		// 1 day = 1000 ms * 60 * 60 * 24
		long days = durationL / (1000 * 60 * 60 * 24);
		long restofdayMS =  durationL - (days * (1000 * 60 * 60 * 24) );
		long hours = restofdayMS /(1000 * 60 * 60);
		long restofHourMS = restofdayMS -  hours  * (1000 * 60 * 60);
		long min = restofHourMS /(1000 * 60 );
		long restofMinuteMS = restofHourMS -  min  * (1000 * 60);
		long sec = restofMinuteMS / (1000 );
		
		String uptime = days +" days "; 
		
		if (hours < 10){
			uptime += "0"+hours+":";
		} else {
			uptime += +hours+":";
		}
		
		if(min < 10){
			uptime += "0"+min+":";
		} else {
			uptime += +min+":";
		}
		if(sec < 10){
			uptime += "0"+sec;
		} else {
			uptime += +sec;
		}
		System.out.println("days "+ days +"\thours "+hours+"\tmin "+min+"\tsec "+sec);
		System.out.println("UPTIME: "+uptime);
		return uptime;
	}
	
	public void setStartTimestamp(long startTimestamp) {
		this.startTimestamp = startTimestamp;
	}
	
	protected static void goWasteTime(long ms) {
		logger.debug("-----> goWasteTime("+ms+")");
		long start = System.currentTimeMillis();
		try{
			Thread.sleep(ms);
		} catch(Exception e){
			logger.error("ERROR WHILE SLEEPING!!!", e);
			e.printStackTrace();
		}
		long end = System.currentTimeMillis();

		BigDecimal res = new BigDecimal((end-start)).divide(new BigDecimal(1000), RoundingMode.FLOOR); 
		logger.debug("<----- goWasteTime("+ms+") \tWASTED = ["+res+"] s");
	}
	
	
	
	protected static void cleanFileFromDocRoot(String filename) throws FileNotFoundException {
		logger.debug("-----> cleanFileFromDocRoot("+filename+")");
		String docroot = ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION;
		logger.info("TMOR PRESENTATION DOC ROOT: "+docroot);

		if(isPresentationWarDeployedStatic()){
			File file =new File(docroot+filename);
			if(!file.exists()){
				try {
					file.createNewFile();
					PrintWriter writer = new PrintWriter(file);
					writer.print("");
					writer.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					logger.error("ERROR: WHILE CLEANING FILE FORM DOC ROOT ["+filename+"]");
					logger.error(e);
				}
			} else {
				PrintWriter writer = new PrintWriter(file);
				writer.print("");
				writer.close();
			}			
		}
		else{
			logger.debug("WAR IS NOT DEPLOYED - NO NEED TO CLEAN UP ["+filename+"]");
			
		}
		logger.debug("<----- cleanFileFromDocRoot("+filename+")");
	}

	protected static boolean isPresentationWarDeployedStatic(){
		boolean retval = false;
    	String docroot = null;
    	docroot = ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION;
		if(docroot == null ){
			logger.error("ERROR: docroot PROPERTY IS NOT CONFIGURED - IT IS NULL - CHECK FILE tmor.properties");
			throw new RuntimeException("ERROR: docroot PROPERTY IS NOT CONFIGURED - IT IS NULL - CHECK FILE tmor.properties");
		} else {
			try{
				File file = new File(docroot);
				if(file.isDirectory()){
					retval = true;
				}
			} catch (Exception ioe){
				logger.error("ERROR: APPLICATION NOT PROPERLY CONFIGURED - CHECK property docroot in FILE tmor.properties", ioe);
				throw new RuntimeException("ERROR: docroot PROPERTY IS NOT CONFIGURED - IT IS NULL - CHECK FILE tmor.properties");
			}
		}
		return retval;
	}
	

	protected void sleep(long ms){
		try{
			Thread.sleep(ms);
		} catch(InterruptedException ire){
			logger.error("SLEEP INTERRUPPTED");
			ire.printStackTrace();
		}
	}
}
