/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.xbee;

import java.io.File;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import com.moneyhouse.sensors.config.CONTSTANTINTERFACE;
import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.config.ServerEnvironmentSingleton;
import com.moneyhouse.tmor.serial.xbee.dto.FileDescDataObject;
import com.rapplogic.xbee.api.AtCommand;
import com.rapplogic.xbee.api.AtCommandResponse;
import com.rapplogic.xbee.api.XBee;
import com.rapplogic.xbee.api.XBeeException;
import com.rapplogic.xbee.util.ByteUtils;

/**
 * Wraps the logic of the serial port
 * Raspi V3 has Bluetooth connected to GPIO 14 and 15 - so Serial does not work anymore
 * This is the fix, so that TMOR still runs on Raspi2 with full serial support e.g. Xbee 
 * and also on Raspi V3 but then onlz with the BASESTATION sensors - Xbee will not be supported
 * 
 * THe two calsses wrapped are 
 * @see SerialReceiverNoSerialPort
 * @see SerialReceiverWithSerialPort
 * 
 * @author Frederic
 * @since 20160608
 * 
 */
public class SerialReceiverAdapter {
	private static XBee xbeeSingleton = null;	
	private final static Logger logger = Logger.getLogger(SerialReceiverAdapter.class);
	
	
	
	public static void main(String[] args) {
		logger.info("--------------------------------------------------------------------------- SLEEP SOME TIME");
		for(int g = 0; g < 10; g++){
			logger.info("[" + (g + 1) +"/ 10]");
			logger.info("DOC_ROOT_BATCH "+ServerEnvironmentSingleton.getInstance().DOC_ROOT_BATCH +"\t"+ServerEnvironmentSingleton.getInstance());
			logger.info("DOC_ROOT_PRESENTATION "+ServerEnvironmentSingleton.getInstance().DOC_ROOT_PRESENTATION +"\t"+ServerEnvironmentSingleton.getInstance());

			goWasteTime(700);
		}
		logger.info("--------------------------------------------------------------------------- SLEPT SOME TIME");

		
		init();
		boolean isSerialPortAccessable = isCOMPortAvailable();

		if(isSerialPortAccessable == true){
			// SERIAL PORT EXISTS AND IS ACCESSABLE 
			logger.info("SERIAL PORT IS ACCESSABLE --> LOAD THE RIGHT SERIAL ADAPTER SOFTWARE");
			SerialReceiverWithSerialPort.main(null); 
		} else{
			// SERIAL PORT DOES NOT EXIST AND/OR IS ACCESSABLE 
			logger.info("SERIAL PORT IS NOT ACCESSABLE --> HIDE THE SERIAL PORT FUNCTIONALITY");
			SerialReceiverNoSerialPort.main(null); 
		}
	}	
	
	
	
//	@Deprecated
//	private static void openCOMPort(XBee xbee) throws XBeeException, InterruptedException {
//		Thread.sleep(200);
//		String port = PropertiesFileReader.getInstance().getProperty("comport");
//		xbee.open(port, 9600);
//	}
	
	
	private static boolean isCOMPortAvailable(){
		logger.info("--->  isCOMPortAvailable()");
		boolean isAvailable = false;
		
		if(System.getProperty("file.separator").equals("\\")){
			isAvailable = isCOMPortAvailableWindows();
			
		} else if(System.getProperty("file.separator").equals("/")){
			isAvailable = isCOMPortAvailableLinux();
			
		} else {
			throw new RuntimeException("FATAL: CAN NOT DETERMINE WHETHER IT IS UNIX OR WINDOWS SYSTEM BY USING file.separator = ["+System.getProperty("file.separator")+"] ");
		}
		logger.info("<--- isCOMPortAvailable() retVal ="+isAvailable);

		return isAvailable;
	}



	private static boolean isCOMPortAvailableWindows() {
		logger.info("---> isCOMPortAvailableWindows()");
		boolean isAvailable = false;
		
		if(xbeeSingleton == null){
			logger.info("TMOR SERIAL RECEIVER ADAPTER TIMESTMAP: 20170510 16:00");
			
			logger.info("INSTANZIATING XBEE SINGLETON ");
			xbeeSingleton = getXbeeSingleton();
			logger.info("XBEE SINGLETON INSTANZIATED "+ getXbeeSingleton());
		}
		
		logger.info("\t 1a. Getting XBee Singleton");
		
		XBee xbee = XBeeSingleton.getInstance().getXbee();
		logger.info("\t 1b. XBee Singleton = "+ xbee);
		logger.info("\t 2a. Get COM port for XBee communication");
		String port = PropertiesFileReader.getInstance().getProperty("comport");
		logger.info("\t 2b. COM Port used: "+port);

		try 
		{
			logger.info("\t 3a. Prepare to open XBee communication");
			xbee.open(port, 9600);
			logger.info("\t 3b. XBee open");
			
//			boolean isXBEERadioMounted = true;
			logger.info("\t 3c. Prepare to check XBeE coordinator");
			String sh = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SH"))).getValue());
			String sl = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SL"))).getValue());
			logger.info("\t 3d. XBeE coordinator addess: " + sh +","+ sl);
			logger.info("\t 3e. Register XBeE coordinator exists");
			
			
			
			
			logger.info("\t 4a. Prepare to close XBee communication");
			xbee.close();
			logger.info("\t 4b. XBee closed");
			isAvailable = true;

			
		} 
		catch (XBeeException e) {
			// TODO Auto-generated catch block
			logger.info("\t 3b. XBee can not be opened or cant be closed");
		}
		logger.info("<--- isCOMPortAvailableWindows() retVal= "+ isAvailable);
		
		return isAvailable;
	}
	
	
	
	
	
	private static void init() {
		String log4jxbeeconfig = PropertiesFileReader.getInstance().getProperty("log4jxbeeconfig");
		System.out.println("log4jxbeeconfig property key to file log4jxbee.properties is located: "+ log4jxbeeconfig);
		PropertyConfigurator.configure(log4jxbeeconfig);
	
		deleteFilesOlderThanFromDir("0h", PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_DIR));
		deleteFilesOlderThanFromDir("0h", PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_FROM_BRICK_DIR));

		
		// ADDED ON tHE 7.09.2015
		deleteFilesOlderThanFromDir("0h", PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_PROCESSED_DIR));
		
		// ADDED ON tHE 10.11.2015
		deleteFilesOlderThanFromDir("0h", PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_CONFIG_DIR));
		
		
		logger.info("LISTENING TO DIRECTORY: "+PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_DIR));
		
		
		logger.info("Reading out this directory - to check permissions");
		ArrayList<FileDescDataObject> ls = createFileListOfFolder(PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_DIR));
		if(ls.size() > 0 )
		{
			for(int i = 0 ; i < ls.size(); i++){
				logger.info("|" + i+ "\t" + ls.get(i));
			}
		} 
		else 
		{
			logger.info("0 files found in directory");
		}
	}
	
	
	/**
	 * DELETE THE FILES FROM BRICK TO BRICK OR BRICKCONFIG WHICH ARE OLDER THAN PARAMETER olderThank
	 * @param olderThan 1d, 2d, 1w, 10w etc
	 * @param directory
	 */
	private static int deleteFilesOlderThanFromDir(String olderThan, String directory) {
		logger.info("---> deleteFilesOlderThanFromDir("+olderThan+", "+directory+")");  
		int retvalCountDel =0;
		try{
			
			int daysBack = 1;
			int multiplier = 1;
			int number = -1;
			long basehours = 24 * 60 * 60 * 1000;
			if(olderThan.contains("w") || olderThan.contains("W")){
				multiplier = 7;
			} else if(olderThan.contains("d") ||olderThan.contains("D")){
				multiplier = 1;
			} else if(olderThan.contains("h") ||olderThan.contains("H")){
				// change the base from days to hours
				basehours = 1 * 60 * 60 * 1000;		
			}  else {
				logger.error("ERROR: PARAMETER ["+olderThan+"] NOT SUPPORTED - EXPECTED IS 1d, 2D, 3w, 4W");
				throw new RuntimeException("PARAMETER ["+olderThan+"] h or H NOT SUPPORTED");
			}
			try{
				number = new Integer(olderThan.substring(0,  olderThan.length()-1)).intValue();
			} catch (NumberFormatException nfe){
				logger.error("ERROR: PARAMETER ["+olderThan+"] NOT SUPPORTED - EXPECTED IS 1h, 1d, 2D, 3w, 4W", nfe);
				throw new RuntimeException("ERROR: PARAMETER ["+olderThan+"] NOT SUPPORTED - EXPECTED IS 1d, 2D, 3w, 4W", nfe);
			}
	
			if(number < 0){
				logger.error("ERROR: PARAMETER ["+olderThan+"] CAN NOT BE NEGATIVE - EXPECTED IS 0h, 1h, 1d, 2D, 3w, 4W");
				throw new RuntimeException("ERROR: PARAMETER ["+olderThan+"] CAN NOT BE NEGATIVE - EXPECTED IS 1d, 2D, 3w, 4W");
			}
			daysBack = multiplier * number;
			
			long filecounter= 0, purgeTime= 0, delcounter = 0;
			
			File dir = new File(directory);
			if(dir.exists()){
	
			    File[] listFiles = dir.listFiles();    
			    filecounter = listFiles.length;
			    purgeTime = System.currentTimeMillis() - (daysBack * basehours);
			    for(File listFile : listFiles) {
			        if(listFile.lastModified() < purgeTime) {
			        	boolean isDelSuccess = listFile.delete();
			        	delcounter++;
			        	retvalCountDel++;
			            if(!isDelSuccess) {
			            	delcounter--;
			            	logger.error("Unable to delete file: " + listFile);
			            }
			         }
			      }
			   }
			// GETR THE TIMESTAMP
			DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			Date deldate = new Date();
			deldate.setTime(purgeTime);
			logger.info("DELETED ["+delcounter+"] FILES FROM TOTAL ["+filecounter+"] THAT WERE OLDER THAN ["+dateFormat.format(deldate)+"] ACCORDING TO PARAMETER ["+olderThan+"]");  
		}
		catch (Exception ex){
			logger.error("ERROR: ERROR WHILE DELETING FILES" , ex);
		}
		logger.info("<--- deleteFilesOlderThanFromDir("+olderThan+", "+directory+") COUNT DELETED FILES= "+ retvalCountDel);
		return retvalCountDel;
	}

	
	
	private static ArrayList<FileDescDataObject> createFileListOfFolder(String directory) {
		// PrepareDataDownload pdd = new PrepareDataDownload();
		File f = new File(directory);
		ArrayList<FileDescDataObject> fileinfolist = new ArrayList<FileDescDataObject>();

		ArrayList<File> files = new ArrayList<File>(Arrays.asList(f.listFiles()));
		Iterator<File> files_it = files.iterator();
		while(files_it.hasNext()){
			File fi = files_it.next();
			if(fi.isFile()){
				FileDescDataObject ne = new FileDescDataObject();
				ne.setName(fi.getName());
				ne.setSize(fi.length());
				ne.setCreatedtimestamp(fi.lastModified());
				fileinfolist.add(ne);
			}
		}
		return fileinfolist;
	}
	
	
	public static XBee getXbeeSingleton() {
		if(xbeeSingleton == null){
			xbeeSingleton = XBeeSingleton.getInstance().getXbee();
			try{
				Thread.sleep(10);
			}catch(Exception d){
				logger.error(d);
			}
		}
		return xbeeSingleton;
	}



	/**
	 * @return
	 */
	private static boolean isCOMPortAvailableLinux() {
		logger.info("---> isCOMPortAvailableLinux()");
		boolean isAvailable = false;
		logger.info("\t 1 Get COM port for XBee communication");
		String port = PropertiesFileReader.getInstance().getProperty("comport");
		logger.info("\t 2 COM Port used: "+port);
	
		try 
		{
			logger.info("\t 3 Check if port ["+port+"] can be accessed as file");
			
			File f = new File(port);
			if(f.exists() && !f.isDirectory()) { 
				// YES ITS A FILE 
				isAvailable = true;
				logger.info("\t 4 port ["+port+"] can be accessed");
			} else {
				logger.info("\t 4 port ["+port+"] can NOT be accessed");
				
			}
			
			
//			xbee.open(port, 9600);
//			logger.info("\t 3b. XBee open");
//			
//			boolean isXBEERadioMounted = true;
//			logger.info("\t 3c. Prepare to check XBeE coordinator");
//			String sh = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SH"))).getValue());
//			String sl = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SL"))).getValue());
//			logger.info("\t 3d. XBeE coordinator addess: " + sh +","+ sl);
//			logger.info("\t 3e. Register XBeE coordinator exists");
//			
//			
//			
//			
//			logger.info("\t 4a. Prepare to close XBee communication");
//			xbee.close();
//			logger.info("\t 4b. XBee closed");
//			isAvailable = true;
	
			
		} 
		catch (Exception e) {
			// TODO Auto-generated catch block
			logger.info("\t 4 Something went wrong while testing for COM port");
			isAvailable = false;
		}
		logger.info("<--- isCOMPortAvailableLinux() retVal= "+ isAvailable);
		
		return isAvailable;
	}
	
	
	
	protected static void goWasteTime(long ms) {
		logger.debug("-----> goWasteTime("+ms+")");
		long start = System.currentTimeMillis();
		try{
			Thread.sleep(ms);
		} catch(Exception e){
			logger.error("ERROR WHILE SLEEPING!!!", e);
			e.printStackTrace();
		}
		long end = System.currentTimeMillis();

		BigDecimal res = new BigDecimal((end-start)).divide(new BigDecimal(1000), RoundingMode.FLOOR); 
		logger.debug("<----- goWasteTime("+ms+") \tWASTED = ["+res+"] s");
	}
}
