/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.xbee;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.math.RoundingMode;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Iterator;

import org.apache.log4j.Logger;

import com.moneyhouse.exceptions.BadDirectRuleDataRuntimeException;
import com.moneyhouse.sensors.config.CONTSTANTINTERFACE;
import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.util.TimeStampProducer;
import com.moneyhouse.tmor.serial.filetrigger.FileMonitorFromBrickSingleton;
import com.moneyhouse.tmor.serial.internal.idgenerator.IDGenerator;
import com.moneyhouse.tmor.serial.pi.DSSensor;
import com.moneyhouse.tmor.serial.pi.dto.DSSensorResultDTO;
import com.moneyhouse.tmor.serial.xbee.dto.XBeeComandObject;
import com.moneyhouse.tmor.serial.xbee.dto.XBeeComandResultObject;
import com.rapplogic.xbee.api.ApiId;
import com.rapplogic.xbee.api.AtCommand;
import com.rapplogic.xbee.api.AtCommandResponse;
import com.rapplogic.xbee.api.PacketListener;
import com.rapplogic.xbee.api.XBee;
import com.rapplogic.xbee.api.XBeeAddress16;
import com.rapplogic.xbee.api.XBeeAddress64;
import com.rapplogic.xbee.api.XBeeException;
import com.rapplogic.xbee.api.XBeeFrameIdResponse;
import com.rapplogic.xbee.api.XBeeResponse;
import com.rapplogic.xbee.api.zigbee.NodeDiscover;
import com.rapplogic.xbee.api.zigbee.ZNetExplicitTxRequest;
import com.rapplogic.xbee.api.zigbee.ZNetTxRequest;
import com.rapplogic.xbee.api.zigbee.ZNetTxStatusResponse;
import com.rapplogic.xbee.util.ByteUtils;
import com.rapplogic.xbee.util.DoubleByte;

/**
 * CONTAINS ALL THE COMMANDS THAT ARE SENT TO THE XBEE RADIO IT WORKS IN
 * CONJUNCTION WITH SerialReceiver
 * 
 * @since 05.11.2014
 * @author Frederic
 * 
 */
public class SerialReceiverCommands {
	private final static Logger logger = Logger
			.getLogger(SerialReceiverCommands.class);

	/**
	 * SEND THE PORTLIST REQUEST COMMAND TO BRICK
	 * 
	 * @param cmd
	 * @param result
	 * @param waittime
	 *            in ms
	 * @return
	 */
	public static synchronized String sendPORTLIST(XBeeComandObject cmd,
			XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("-----> sendPORTLIST()");
		// XBee xbee = XBeeSingleton.getInstance().getXbee();
		String retval = "";
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);

			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x50, 0x4C, 0x49, 0x53, 0x54, 0x20, 0x20,	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {
				int correlIdLength = cmd.getCorrelID().length();

				payload = new int[] { 0x50, 0x4C, 0x49, 0x53, 0x54, 0x5B, 0x20,	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[6 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[6 + correlIdLength] = 0x5D; // close the parameter
													// bracket ]
			}

			XBeeAddress64 dest64 = new XBeeAddress64(
					XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			XBeeResponse response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);

			logger.info("RECEIVED RESPONSE " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<----- sendPORTLIST()");
		}
		return retval;
	}

	
	
	/**
	 * SEND THE SENSORLIST REQ COMMAND TO BASIS STATION - NOT THE BRICK
	 * WRITES THE RESULT IMMEDIATLY INTO FILE
	 * @param cmd
	 * @since 13.07.2015
	 */
	public static synchronized void sendSENSORLIST_BASIS(XBeeComandObject cmd) {
		// ON THE BASESTATION RASPBERRY PI
		DSSensor dss = new DSSensor();
		ArrayList<String> sensorlist = dss.readSensorList();
		
		String correlID = cmd.getCorrelID().substring(0, cmd.getCorrelID().length()-1);
		
		for(int i = 0; i < sensorlist.size(); i++){
			// write result
			String correl = correlID + i;
			String nomde = CONTSTANTINTERFACE.MODULE_ADDRESS_BASESTATION;
			String comanddo = "SLIST";
			String filename = "FROM_BRICK__"+nomde+"___CORR_"+correl+"__CMD_"+comanddo+"___" + new IDGenerator().getNewID()+".txt";
			String pathfilename = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_CONFIG_DIR);
			
			pathfilename = pathfilename + filename;
			logger.debug("send command by writing file into  "+pathfilename);
			Timestamp tsnow = TimeStampProducer.createTimestampForTimeZone("UTC");
			String tsnowstr = tsnow.toString();
			logger.debug("UTC tsnowstr "+tsnowstr);
			String content =nomde+"\t"+comanddo+ "\t" +correl+"\t"+"OW"+"\t"+sensorlist.get(i)+"\t"+tsnowstr+"\tEXECUTIONTIME=";
			
			FileMonitorFromBrickSingleton fm = FileMonitorFromBrickSingleton.getInstance();
			fm.createFileAsEvent(pathfilename, content);
		}
	}	
	
	
	
	
	/**
	 * SEND THE PORTLIST REQ COMMAND TO BRICK
	 * 
	 * @param cmd
	 * @param result
	 * @param waittime
	 *            in ms
	 * @return
	 */
	public static synchronized String sendSENSORLIST(XBeeComandObject cmd,
			XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("-----> sendSENSORLIST()");
		String retval = "";
		result.setCommand(cmd);
		try {
			logger.info("DEST BRICK ID: " + cmd.getBrickid());

			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x53, 0x4C, 0x49, 0x53, 0x54, 0x20, 0x20,	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
						0x20, 0x20, 0x20, 0x20 };
			} else {
				int correlIdLength = cmd.getCorrelID().length();

				payload = new int[] { 0x53, 0x4C, 0x49, 0x53, 0x54, 0x5B, 0x20,	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
						0x20, 0x20, 0x20, 0x20 };
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[6 + i] = digitnr; // char as int value into the payload
				}
				payload[6 + correlIdLength] = 0x5D; // close the parameter bracket ]
			}

			XBeeAddress64 dest64 = new XBeeAddress64(
					XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			XBeeResponse response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload) + "   RESPONSE:" + response.toString());
			result.setResponseOK(true);

			logger.info("RECEIVED RESPONSE " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			logger.info("<----- sendSENSORLIST()");
		}
		return retval;
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	/**SEND THE PINg COMMAND TO BRICK 
	 * @param cmd
	 * @param result
	 * @param waittime in ms
	 * @return
	 */
	public static synchronized String sendPING(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("-----> sendPING()");
		// XBee xbee = XBeeSingleton.getInstance().getXbee();
		String retval = "";
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);

			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
									//  P	  I     N     G	
				payload = new int[] { 0x50, 0x49, 0x4E, 0x47, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {
				int correlIdLength = cmd.getCorrelID().length();

				payload = new int[] { 0x50, 0x49, 0x4E, 0x47, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
													// bracket ]
			}

			XBeeAddress64 dest64 = new XBeeAddress64(
					XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("sending explicit " + request.toString());

			xbee.sendAsynchronous(req);
			// xbee.sendAsynchronous(request);
			XBeeResponse response = xbee.getResponse();
			// XBeeResponse response = xbee.sendSynchronous(request);
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)
					+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);

			logger.info("received response " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<----- sendPING()");
		}
		return retval;
	}
	
	
	
	/**SEND THE READ VERISON COMMAND TO BRICK 
	 * @param cmd
	 * @param result
	 * @param waittime in ms
	 * @return
	 */
	public static synchronized String sendREADVERSION(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("-----> sendREADVERSION()");
		// XBee xbee = XBeeSingleton.getInstance().getXbee();
		String retval = "";
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);

			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				//						R	  V     E     R     S     I	
				payload = new int[] { 0x52, 0x56, 0x45, 0x52, 0x53, 0x49, 0x20,	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {
				int correlIdLength = cmd.getCorrelID().length();
				//                      0     1     2     3     4     5     6    7     8     9     10    11    12    13    14    15   
				//						R	  V     E     R     S     I	    [    C0    C1    C2    C3    C4    C5    C6    C7     ] 
				payload = new int[] { 0x52, 0x56, 0x45, 0x52, 0x53, 0x49, 0x5B,	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x5D, 0x20, 0x20, 0x20, 0x20 };
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[7 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[7 + correlIdLength] = 0x5D; // close the parameter
													// bracket ]
			}

			XBeeAddress64 dest64 = new XBeeAddress64(XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("sending explicit " + request.toString());

			xbee.sendAsynchronous(req);
			// xbee.sendAsynchronous(request);
			XBeeResponse response = xbee.getResponse();
			// XBeeResponse response = xbee.sendSynchronous(request);
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)					+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);

			logger.info("received response " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<----- sendREADVERSION()");
		}
		return retval;
	}
	
	
	
	/** 
	 * READ THE COORDINATOR ADDRESS
	 * @since 20160129
	 * @author Frederic

	 */
	public static synchronized void cmdReadCoordinatorSHSL(XBee xbee,XBeeComandObject cmd,  XBeeComandResultObject result)	 {
		logger.info("-----> cmdReadCoordinatorSHSL()");
		String retval = "";
		
		result.setCommand(cmd);
		try {
			String sh = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SH"))).getValue());
			String sl = ByteUtils.toBase16(((AtCommandResponse)xbee.sendSynchronous(new AtCommand("SL"))).getValue());
			
			retval = sh +","+ sl;
			
			// get the 8 byte SH/SL address
			logger.info("SH is " + sh);
			logger.info("SL is " + sl);
			
			logger.info("RETURN VALUE IS: " + retval);
			result.setResponseValue("PAYLOAD: " + retval + "   RESPONSE:" +sh +sl);
			result.setResponseOK(true);
		} catch(Exception e){

			e.printStackTrace(); 
			logger.error("ERROR: WHILE READING COORDINATOR ADDRESS WE RECEIVED THE FOLLOWING ERROR",e);
		} 
		finally {
			logger.info("<----- cmdReadCoordinatorSHSL()");
		}
		//return retval;
	}	
	
	
	
	
	
	
	
	
	

	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	/**
	 * SEND THE VALUE TO PUT ON PORT B0
	 * 
	 * @since 20141127
	 * @param cmd
	 * @param result
	 * @param waittime
	 *            in ms
	 * @return
	 */
	public static synchronized XBeeResponse sendSETOUTPORTB0(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("------> sendSETOUTPORTB0()");
		// XBee xbee = XBeeSingleton.getInstance().getXbee();
		XBeeResponse retval = null;
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);
			int correlIdLength = cmd.getCorrelID().length();

			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x53, 0x50, 0x52, 0x54, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {				// S     P     R     T	

				payload = new int[] { 0x53, 0x50, 0x52, 0x54, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
			}
			// add the port B0 
			payload[6 + correlIdLength] = 0x5B; // open the parameter [
			payload[7 + correlIdLength] = 0x30; // port address 0
			payload[8 + correlIdLength] = 0x5D; // close the parameter ]
			payload[9 + correlIdLength] = 0x5B; // open the parameter [
			if(cmd.getAtcommandparameter().equals("0")){
				payload[10 + correlIdLength] = 0x30; // port value 0
			} else {
				payload[10 + correlIdLength] = 0x31; // port value 1
			}
			payload[11 + correlIdLength] = 0x5D; // close the parameter ]
			

			XBeeAddress64 dest64 = new XBeeAddress64(XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			XBeeResponse response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);
			retval = response;

			logger.info("RECEIVED RESPONSE " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<------ sendSETOUTPORTB0() retval= "+retval);
		}
		return retval;
	}
	
	
	
	
	/**
	 * SEND THE VALUE TO PUT ON PORT B1
	 * 
	 * @since 20141127
	 * @param cmd
	 * @param result
	 * @param waittime
	 *            in ms
	 * @return
	 */
	public static synchronized XBeeResponse sendSETOUTPORTB1(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("------> sendSETOUTPORTB1()");
		// XBee xbee = XBeeSingleton.getInstance().getXbee();
		XBeeResponse retval = null;
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);
			int correlIdLength = cmd.getCorrelID().length();

			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x53, 0x50, 0x52, 0x54, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {				// S     P     R     T	

				payload = new int[] { 0x53, 0x50, 0x52, 0x54, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
			}
			// add the port B0 
			payload[6 + correlIdLength] = 0x5B; // open the parameter [
			payload[7 + correlIdLength] = 0x31; // port address 0
			payload[8 + correlIdLength] = 0x5D; // close the parameter ]
			payload[9 + correlIdLength] = 0x5B; // open the parameter [
			if(cmd.getAtcommandparameter().equals("0")){
				payload[10 + correlIdLength] = 0x30; // port value 0
			} else {
				payload[10 + correlIdLength] = 0x31; // port value 1
			}
			payload[11 + correlIdLength] = 0x5D; // close the parameter ]
			

			XBeeAddress64 dest64 = new XBeeAddress64(
					XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			XBeeResponse response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);
			retval = response;

			logger.info("RECEIVED RESPONSE " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<------ sendSETOUTPORTB1()");
		}
		return retval;
	}

	
	/**
	 * ASSEBLE QUERY THE BASIS FILe DIRECTORY
	 * @param cmd
	 * @since 13.07.2015
	 */
	public static synchronized  XBeeComandResultObject  sendREADOW_BASIS(XBeeComandObject cmd) {
		logger.info("---> sendREADOW_BASIS("+cmd+")");
		// ITS ON THE PI BASIS-STATION
		XBeeComandResultObject retVal = new XBeeComandResultObject(cmd);
		DSSensor dss = new DSSensor();
		ArrayList<String> sensorlist = dss.readSensorList();
		logger.info("OW SENSORLIST BASESTATION HAS A SIZE Of ["+sensorlist.size()+"]");
		for(int i = 0 ; i < sensorlist.size(); i++){
			logger.info("["+i+"] SENSOR WITH ID ["+sensorlist.get(i)+"]");
		}
		
		//		sensorlist.add("28-000004475b0d");
		//		sensorlist.add("28-00000460d5b1");
		//	280D5B4704000049 SENSOR ID FORMAT ON XBEE MODULE
		
		String sensorID= cmd.getAtcommandparameter();
		if(sensorID.length() != "28-000004475b0d".length()){
			logger.error("ERROR: IT SEEMS THE OW SENSOR ADDRESS ["+sensorID+"] FOR BASESTATION IS NOT IN THE SAME LENGTH AS 28-000004475b0d");
			throw new RuntimeException("ERROR: IT SEEMS THE OW SENSOR ADDRESS ["+sensorID+"] FOR BASESTATION IS NOT IN THE SAME LENGTH AS 28-000004475b0d");
		}
		
		DSSensorResultDTO result = dss.readSensorValueWithID(sensorID);
		retVal.setResponseOK(true);
		retVal.setResponseValue("PAYLOAD: BASE STATION SENSOR "+result.getSensorID()+"\tRESPONSE: BASE STATION SENSOR READING: "+ result.getReadingValueStr());
		// write result
		String nomde = CONTSTANTINTERFACE.MODULE_ADDRESS_BASESTATION;
		String comanddo = "RDOW";
		String filename = "FROM_BRICK__"+nomde+"___CORR_"+cmd.getCorrelID()+"__CMD_"+comanddo+"___" + new IDGenerator().getNewID()+".txt";
		String pathfilename = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_FROM_BRICK_DIR);;
		
		pathfilename = pathfilename + filename;
		logger.debug("send command by writing file into  "+pathfilename);
//		Timestamp tsnow = new Timestamp(System.currentTimeMillis());
//		String tsnowstr = tsnow.toString();

		String tsnowstr = TimeStampProducer.createTimestampWithTimeZone("UTC");
		logger.debug("UTC tsnowstr "+tsnowstr);
		String content =nomde+"\t"+comanddo+ "\t" +cmd.getCorrelID()+"\t"+result.getReadingValueStr()+"\t"+"\t"+tsnowstr+"\tEXECUTIONTIME=";
		
		FileMonitorFromBrickSingleton fm = FileMonitorFromBrickSingleton.getInstance();
		fm.createFileAsEvent(pathfilename, content);	
		logger.info("<--- sendREADOW_BASIS("+cmd+")");
		return retVal;

	}	
	
	
	
	
	
	/**
	 * ASSEBLE THE XBEE COMMAND AND SEND IT OFF VIA THE COORDINATOR 
	 * @param cmd
	 * @param result
	 * @param waittime
	 * @param xbee
	 * @return
	 */
	public static synchronized String sendREADOW(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("-----> sendREADOW()");
		// XBee xbee = XBeeSingleton.getInstance().getXbee();
		String retval = "";
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);
			int correlIdLength = cmd.getCorrelID().length();

			String owaddress = cmd.getAtcommandparameter();
			if(owaddress ==null || owaddress.isEmpty() || owaddress.length() < 10){
				// TAKE THE STANDARD OW ADDRESS
				owaddress = "28040F0105000017";
				logger.warn("OW ADDRESS FROM tHE COMMAND PARAMETERS IS NULL OR EMPTY OR LENGTH < 10 - WE USE DEFAULT OW ADDRESS 28040F0105000017");

				
			}
			
			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x52, 0x44, 0x4F, 0x57, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {				// R     D     O     W	

				payload = new int[] { 0x52, 0x44, 0x4F, 0x57, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 , 0x20};
									// S     D     O     W     [     C     O     R      R     E    L	 I      D     ]     [   OW1   OW2   OW3   OW4   OW5   OW6   OW7   OW8   OW9   OW10  OW11  OW12  OW13  OW14  OW15  OW16   ]     CHCKS
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
			}
			// add the port B0 
			payload[6 + correlIdLength] = 0x5B; // open the parameter [
			
			int pos = 6 + correlIdLength;
			char[] oqaddressChar = owaddress.toCharArray();
			for(char b: oqaddressChar){
				pos++;
				payload[pos] = b; 				// 16 character OW address code
			}
			
			payload[pos + 1] = 0x5D; // close the parameter ]
			
			
			XBeeAddress64 dest64 = new XBeeAddress64(XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));
			logger.info("PAYLOAD LENGTH: " + payload.length);

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			Object response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);

			if(response instanceof ZNetTxStatusResponse){
				System.out.println("ZNetTxStatusResponse");
				ZNetTxStatusResponse zResponse = (ZNetTxStatusResponse)response;
				if(zResponse.getDeliveryStatus().equals(ZNetTxStatusResponse.DeliveryStatus.ADDRESS_NOT_FOUND)){
					// 
					XBeeDataQueueSingleton sing = XBeeDataQueueSingleton.getInstance();
					sing.moveFromToBrickToErrorListContainingCharSequence(cmd.getCorrelID(),CONTSTANTINTERFACE.ADDRESS_NOT_FOUND);
					logger.warn("XBEE DELIVERY STATUS: ADDRESS NOT FOUND");
					
				} else if(zResponse.getDeliveryStatus().equals(ZNetTxStatusResponse.DeliveryStatus.SUCCESS)){
					logger.info("XBEE DELIVERY STATUS: DELIVERY SUCCESSFUL");
				}
			} else if(response instanceof XBeeFrameIdResponse){
				logger.info("XBeeFrameIdResponse");
				
			} else if(response instanceof XBeeResponse){
				logger.info("XBeeResponse");
			
			} else {
				
				logger.info("???? ["+response.toString()+"]");
			}
			
			logger.info("OW RECEIVED RESPONSE " + response.toString());
			// response.getDeliveryStatus();
//			  ApiId ff = response.getApiId();
			  // response.
//			  System.out.println("FF = "+ff);
			  
			  
		} catch (Exception e) {
			logger.error("ERROR: SOME ERROR WIHLE RECEIVING",e);
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<----- sendREADOW()");
		}
		return retval;
	}

	
	
	/**
	 * ASSEBLE THE XBEE COMMAND RADC AND SEND IT OFF VIA THE COORDINATOR 
	 * @param cmd
	 * @param result
	 * @param waittime
	 * @param xbee
	 * @return
	 */
	public static synchronized String sendREADADC(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("-----> sendREADADC()");
		// XBee xbee = XBeeSingleton.getInstance().getXbee();
		String retval = "";
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);
			int correlIdLength = cmd.getCorrelID().length();
			
			String adcPortNumber = cmd.getAtcommandparameter();
			try{
				int adcPortNumberInt = new Integer(adcPortNumber).intValue();
				if(adcPortNumberInt < 0 || adcPortNumberInt > 15){
					logger.error("ERROR: PARAMETER ADC PORT WIth VALUE ["+cmd.getAtcommandparameter()+"] IS OUTSIDE THE VALUE FROM 0 To 15!");
				} else {
					adcPortNumber = Integer.toHexString(adcPortNumberInt);
				}
			}catch(NumberFormatException ex){
				logger.error("ERROR: PARAMETER ADC PORT WIth VALUE ["+cmd.getAtcommandparameter()+"] CANNOT BE CONVERTED INTO A HEX NUMBER!!!");
			}
			
			if(adcPortNumber ==null || adcPortNumber.isEmpty() || adcPortNumber.length() > 1){
				// TAKE THE STANDARD OW ADDRESS
				adcPortNumber = "1";
				logger.warn("ADC PORT FROM tHE COMMAND PARAMETERS ["+adcPortNumber+"] IS NOT VALID - WE USE DEFAULT ADC PORT NR. 1");

				
			}
			
			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x52, 0x41, 0x44, 0x43, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {				// R     A     D     C	

				payload = new int[] { 0x52, 0x41, 0x44, 0x43, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20};
									// R     A     D     C     [     C     O     R      R     E    L	 I      D     ]     [   PORT    ]   CHCKS
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
			}
			// add the port B0 
			payload[6 + correlIdLength] = 0x5B; // open the parameter [
			
			int pos = 6 + correlIdLength;
			char[] oqaddressChar = adcPortNumber.toCharArray();
			for(char b: oqaddressChar){
				pos++;
				payload[pos] = b; 				// 16 character OW address code
			}
			
			payload[pos + 1] = 0x5D; // close the parameter ]
			
			
			XBeeAddress64 dest64 = new XBeeAddress64(XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));
			logger.info("PAYLOAD LENGTH: " + payload.length);

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			Object response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);

			if(response instanceof ZNetTxStatusResponse){
				System.out.println("ZNetTxStatusResponse");
				ZNetTxStatusResponse zResponse = (ZNetTxStatusResponse)response;
				if(zResponse.getDeliveryStatus().equals(ZNetTxStatusResponse.DeliveryStatus.ADDRESS_NOT_FOUND)){
					// 
					XBeeDataQueueSingleton sing = XBeeDataQueueSingleton.getInstance();
					sing.moveFromToBrickToErrorListContainingCharSequence(cmd.getCorrelID(),CONTSTANTINTERFACE.ADDRESS_NOT_FOUND);
					logger.warn("XBEE DELIVERY STATUS: ADDRESS NOT FOUND");
					
				} else if(zResponse.getDeliveryStatus().equals(ZNetTxStatusResponse.DeliveryStatus.SUCCESS)){
					logger.info("XBEE DELIVERY STATUS: DELIVERY SUCCESSFUL");
				}
			} else if(response instanceof XBeeFrameIdResponse){
				logger.info("XBeeFrameIdResponse");
				
			} else if(response instanceof XBeeResponse){
				logger.info("XBeeResponse");
			
			} else {
				
				logger.info("???? ["+response.toString()+"]");
			}
			
			logger.info("OW RECEIVED RESPONSE " + response.toString());
			// response.getDeliveryStatus();
//			  ApiId ff = response.getApiId();
			  // response.
//			  System.out.println("FF = "+ff);
			  
			  
		} catch (Exception e) {
			logger.error("ERROR: SOME ERROR WIHLE RECEIVING",e);
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<----- sendREADADC()");
		}
		return retval;
	}


	
	
	
	
	
	
	
	
	
	/** SENDS A READ OUTPUT PORT VALUE - TO SEE WHETHER THE OUTPUT PORT IS O OR ONE
	 * @param cmd
	 * @param result
	 * @param waittime
	 * @param xbee
	 * @return
	 */
	public static synchronized XBeeResponse sendREADOP(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("------> sendREADOP()");
		XBeeResponse retval = null;
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);
			int correlIdLength = cmd.getCorrelID().length();

			String owaddress = cmd.getAtcommandparameter();
			if(owaddress ==null || owaddress.isEmpty() || owaddress.length() < 10){
				// TAKE THE STANDARD OW ADDRESS
				owaddress = "28040F0105000017";
			}
			
			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x52, 0x44, 0x4F, 0x50, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {				// R     D     O     P	

				payload = new int[] { 0x52, 0x44, 0x4F, 0x50, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 , 0x20};
									// S     D     O     P     [     C     O     R      R     E    L	 I      D     ]     [   OW1   OW2   OW3   OW4   OW5   OW6   OW7   OW8   OW9   OW10  OW11  OW12  OW13  OW14  OW15  OW16   ]     CHCKS
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
			}
			// add the port B0 
//			payload[6 + correlIdLength] = 0x5B; // open the parameter [
			
//			int pos = 6 + correlIdLength;
//			char[] oqaddressChar = owaddress.toCharArray();
//			for(char b: oqaddressChar){
//				pos++;
//				payload[pos] = b; 				// 16 character OW address code
//			}
			
//			payload[pos + 1] = 0x5D; // close the parameter ]
			
			
			XBeeAddress64 dest64 = new XBeeAddress64(
					XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));
			logger.info("PAYLOAD LENGTH: " + payload.length);

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA
					.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			XBeeResponse response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);
			retval = response;

			logger.info("RECEIVED RESPONSE " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<------ sendREADOP()");
		}
		return retval;
	}


	
	
	
	
	
	
	
	
	
	/** SENDS A READ DIRECT RULE TO BRICK
	 * @since 11.01.2017
	 * @param cmd
	 * @param result
	 * @param waittime
	 * @param xbee
	 * @return
	 */
	public static synchronized XBeeResponse sendREADDIRECTRULE(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("------> sendREADDIRECTRULE()");
		XBeeResponse retval = null;
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);
			int correlIdLength = cmd.getCorrelID().length();

			String ruleId = cmd.getAtcommandparameter();
			if(ruleId ==null || ruleId.isEmpty()){
				// THE RULE ID MUST bE BETWEEN 0 and 15 (F) 
				ruleId = "0";
			}
			int number = -99;
			try{
				number = new Integer(ruleId).intValue();
			} catch( NumberFormatException e){
				logger.error("ERROR: THE PARAMETER RULE ID ["+ruleId+"] IS NOT SUPPORTED - EXPECT A NuMBER bETWEEN 0 and 15 - WE SET PARAMETER To 0");
				number = 0;
			}
			if(number  <0 ){
				logger.error("ERROR: SOMETHING WENT WRONG WITH PARSING THE THE PARAMETER RULE ID ["+ruleId+"] - EXPECT A NuMBER bETWEEN 0 and 15 - WE SET PARAMETER To 0");
				number = 0;
				
			}
			
			
			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				payload = new int[] { 0x52, 0x44, 0x52, 0x4C, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 };
			} else {				// R     D     R     L	

				payload = new int[] { 0x52, 0x44, 0x52, 0x4C, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x5B, 0x20, 0x5D };
									// R     D     R     L     [     C     O     R      R     E    L	 I      D     ]     [   ID1    ]    
				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
				int ruleIdToInsertAsChar = 0;
				if(number < 10 && number > -1){
					ruleIdToInsertAsChar = 48 + number;
				} else if (number > 9 && number < 16){
					// 10 = A --> 55 + 10 ==> 65
					// 15 = F --> 55 + 15 ==> 70
					ruleIdToInsertAsChar = 55 + number;
				} else {
					logger.error("ERROR: CANNOT ENCODE THE RULE ID  ["+ruleId+"] PROPERLY - WE SET PARAMETER To 0");
					ruleIdToInsertAsChar = 40;
				}
				payload[15] = ruleIdToInsertAsChar;
			}
			
			
			// add the port B0 
//			payload[6 + correlIdLength] = 0x5B; // open the parameter [
			
//			int pos = 6 + correlIdLength;
//			char[] oqaddressChar = owaddress.toCharArray();
//			for(char b: oqaddressChar){
//				pos++;
//				payload[pos] = b; 				// 16 character OW address code
//			}
			
//			payload[pos + 1] = 0x5D; // close the parameter ]
			
			
			XBeeAddress64 dest64 = new XBeeAddress64(XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));
			logger.info("PAYLOAD LENGTH: " + payload.length);

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			XBeeResponse response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);
			retval = response;

			logger.info("RECEIVED RESPONSE " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<------ sendREADDIRECTRULE()");
		}
		return retval;
	}

	
	
	/** SENDS A WRITE DIRECT RULE TO BRICK
	 * XBEE.c Line 117 change the length of the fame
	 * BRICK.c WRITE DIRECT RULE START POSITION
	 * @since 07.03.2017
	 * @param cmd
	 * @param result
	 * @param waittime
	 * @param xbee
	 * @return
	 */
	public static synchronized XBeeResponse sendWRITEDIRECTRULE(XBeeComandObject cmd, XBeeComandResultObject result, long waittime, XBee xbee) {
		logger.info("------> sendWRITEDIRECTRULE()");
		XBeeResponse retval = null;
		result.setCommand(cmd);
		try {
			// openCOMPort(xbee);
			String destbrickID = cmd.getBrickid();
			logger.info("DEST BRICK ID: " + destbrickID);
			int correlIdLength = cmd.getCorrelID().length();

			String ruleId = cmd.getAtcommandparameter();
			if(ruleId ==null || ruleId.isEmpty()){
				// THE RULE ID MUST bE BETWEEN 0 and 15 (F) 
				ruleId = "0";
			}
			int number = -99;
			try{
				number = new Integer(ruleId).intValue();
			} catch( NumberFormatException e){
				logger.error("ERROR: THE PARAMETER RULE ID ["+ruleId+"] IS NOT SUPPORTED - EXPECT A NuMBER bETWEEN 0 and 15 - WE SET PARAMETER To 0");
				number = 0;
			}
			if(number  <0 ){
				logger.error("ERROR: SOMETHING WENT WRONG WITH PARSING THE THE PARAMETER RULE ID ["+ruleId+"] - EXPECT A NuMBER bETWEEN 0 and 15 - WE SET PARAMETER To 0");
				number = 0;
				
			}
			// 7E 00 27 90 00 13 A2 00 40 A0 99 32 00 00 01 57 44 52 4C 5B 73 49 4C 35 49 65 57 32 5D 5B 35 5D 5B 30 30 38 
			//                                              W  D  R  L  [  I                       ]  [     ]  [                
			
			int[] payload = null;
			if (cmd.getCorrelID() == null || cmd.getCorrelID().isEmpty()) {
				logger.error("ERROR: COMMAND wirteDIRECTRULE HAS NOT CORREL ID");
			} else {				// W     D     R     L	

//				payload = new int[] { 0x57, 0x44, 0x52, 0x4C, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x5D, 0x5B, 0x20, 0x5D, 0x5B, 0x20, 0x20, 0x20, 0x5D , 0x5B, 0x20, 0x20, 0x20, 0x5D, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x5D, 0x5B, 0x20, 0x5D, 0x5B, 0x20, 0x20, 0x20, 0x5D, 0x5B, 0x20, 0x20, 0x5D, 0x5B, 0x30, 0x5D};
									// W     D     R     L     [     C     O     R      R     E    L	 I      D     ]     [   ID1    ]      [   EVET2 EVET1 EVET0   ]      [   EVAL2 EVAL1 EVAL0   ]     [   OWCV3 OWV2 OWV1   OWv0    ]     [   OWOPRTN ]     [   ACTN2 ACTN1 ACTN0   ]     [   ACTv1 ACTv0   ]     [   STAT    ]                                                                   
									// 0	 1	   2     3     4     5     6     7      8     9    10    11     12    13    14   15    16     17   18    19    20     21    22     23   24    25    26    27    28    29    30    31    32    33    34    35    36    37    38    39    40    41    42    43    44    45    46    47                                                                                                                           
				payload = new int[] { 0x57, 0x44, 0x52, 0x4C, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x5D, 0x5B, 0x20, 0x5D, 0x5B, 0x20, 0x20, 0x20, 0x5D , 0x5B, 0x20, 0x20, 0x20, 0x5D, 0x5B, 0x20, 0x20, 0x20, 0x20, 0x5D, 0x5B, 0x20, 0x20, 0x5D, 0x5B, 0x20, 0x20, 0x20, 0x5D, 0x5B, 0x20, 0x20, 0x5D, 0x5B, 0x30, 0x5D};
									// 0	 1	   2     3     4     5     6     7      8     9    10    11     12    13    14   15    16     17   18    19    20     21    22     23   24    25    26    27    28    29    30    31    32    33    34    35    36    37    38    39    40    41    42    43    44    45    46    47    48                                                                                                                          
									// W     D     R     L     [     C     O     R      R     E    L	 I      D     ]     [   ID1    ]      [   EVET2 EVET1 EVET0   ]      [   EVAL2 EVAL1 EVAL0   ]     [   OWCV3 OWV2 OWV1   OWv0    ]     [OWOPRH  OWOPRL   ]     [   ACTN2 ACTN1 ACTN0   ]     [   ACTv1 ACTv0   ]     [   STAT    ]                                                                   

				for (int i = 0; i < correlIdLength; i++) {
					char digitstr = cmd.getCorrelID().charAt(i);

					int digitnr = (int) digitstr;
					payload[5 + i] = digitnr; // char as int value into the
												// payload
				}
				payload[5 + correlIdLength] = 0x5D; // close the parameter
				
				// RULE ID
				int ruleIdToInsertAsChar = 0;
				if(number < 10 && number > -1){
					ruleIdToInsertAsChar = 48 + number;
				} else if (number > 9 && number < 16){
					// 10 = A --> 55 + 10 ==> 65
					// 15 = F --> 55 + 15 ==> 70
					ruleIdToInsertAsChar = 55 + number;
				} else {
					logger.error("ERROR: CANNOT ENCODE THE RULE ID  ["+ruleId+"] PROPERLY - WE SET PARAMETER To 0");
					ruleIdToInsertAsChar = 40;
				}
				payload[15] = ruleIdToInsertAsChar;
				

				// EVENT
				String event = cmd.getAdditionalParameter("EVENT");
				int eventID = new Integer(event).intValue(); 
				
				if(eventID > 99 && eventID < 256){
					// 3 DIGITS
					int hundreds = eventID / 100;
					payload[18] = 0x30 + hundreds;		//0 + HUNDERTS
					int rest = 	eventID - hundreds * 100;
					int tens = rest / 10;
					payload[19] = 0x30 + tens; 			//0 + TENS
					int ones = 	eventID - hundreds*100 - 10*tens;
					payload[20] = 0x30 + ones;			// ONES + OFFSET
					
				} else if(eventID < 100 && eventID > 9 ){
					// 2 DIGITS
					int tens = eventID / 10;
					payload[18] = 0x30; 				//0
					payload[19] = 0x30 + tens; 			//0 + TENS
					int ones = 	eventID - 10*tens;
					payload[20] = 0x30 + ones;			// 0 + ONES 
				} else if(eventID < 10 && eventID > -1){
					// 1 DIGIT
					payload[18] = 0x30; 			//0
					payload[19] = 0x30; 			//0
					payload[20] = 0x30 + eventID; 	//0 + ONES
				} else {
					throw new BadDirectRuleDataRuntimeException("ERROR: DIRECT RULE EVENT MUST BE BETWEEN 0 and 255 but IS ["+event+"]");
				} 
				
				
				// EVENT VALUE
				String eventvalstr = cmd.getAdditionalParameter("EVENT_VALUE");
				int eventval = new Integer(eventvalstr).intValue(); 
				if(eventval > 99 && eventval < 256){
					// 3 DIGITS
					int hundreds = eventval / 100;
					payload[23] = 0x30 + hundreds;		//0 + HUNDERTS
					int rest = 	eventval - hundreds * 100;
					int tens = rest / 10;
					payload[24] = 0x30 + tens; 			//0 + TENS
					int ones = 	eventval -hundreds*100 - 10*tens;
					payload[25] = 0x30 + ones;			// ONES + OFFSET
					
				} else if(eventval < 100 && eventval > 9 ){
					// 2 DIGITS
					int tens = eventval / 10;
					payload[23] = 0x30; 				//0
					payload[24] = 0x30 + tens; 			//0 + TENS
					int ones = 	eventval - 10*tens;
					payload[25] = 0x30 + ones;			// 0 + ONES 
				} else if(eventval < 10 && eventval > -1){
					// 1 DIGIT
					payload[23] = 0x30; 			//0
					payload[24] = 0x30; 			//0
					payload[25] = 0x30 + eventval; 	//0 + ONES
				} else {
					throw new BadDirectRuleDataRuntimeException("ERROR: DIRECT RULE EVENT MUST BE BETWEEN 0 and 255 but IS ["+event+"]");
				} 
				
				

				
				
				// OW OPERATOR AS HEX
				String owoperatorStr = cmd.getAdditionalParameter("OW_OPERATION");
				int owoperator = 0;
				try{
					owoperator = new Integer(owoperatorStr).intValue();
					
					if(owoperator < 0 ){
						logger.error("ERROR: OW OPERATOR ["+owoperatorStr+"] (COMP VALUE B) NEEDS TO BE A POSITIVE NUMBER");
					}
				}catch(NumberFormatException ex){
					logger.error("ERROR: OW OPERATOR ["+owoperatorStr+"] (COMP VALUE B)NEEDS TO BE A VALUE");
				}
				if(owoperator >= 0 && owoperator< 10){
					// 0 - 9 
					payload[34] = 0x30 ;
					payload[35] = 0x30 + owoperator;
				} else if (owoperator >9 && owoperator< 16){
					// A - F 
					payload[34] = 0x30 ;
					payload[35] = 0x41 + owoperator - 10;
				} else if (owoperator > 15 && owoperator < 256){
					// DOUBLE DIGIT
					int owoperatorHigh = owoperator/16;
					int owoperatorLow = owoperator - 16*owoperatorHigh;
					// HIGH BYTE 
					if(owoperatorHigh >= 0 && owoperatorHigh< 10){
						// 0 - 9 
						payload[34] = 0x30 + owoperatorHigh;
						;
					} else if (owoperatorHigh >9 && owoperatorHigh< 16){
						// A - F 
						payload[34] = 0x41 + owoperatorHigh - 10;
					} else {
						logger.error("ERROR: CONVERTING OW OPERATOR ["+owoperatorStr+"] TO HEX VALUE HIGH BYTE ["+owoperatorHigh+"] IS NOT VALID ");
					}
					// LOW BYTE 
					if(owoperatorLow >= 0 && owoperatorLow< 10){
						// 0 - 9 
						payload[35] = 0x30 + owoperatorLow;
						;
					} else if (owoperatorLow >9 && owoperatorLow< 16){
						// A - F 
						payload[35] = 0x41 + owoperatorLow - 10;
					} else {
						logger.error("ERROR: CONVERTING OW OPERATOR ["+owoperatorStr+"] TO HEX VALUE LOW BYTE ["+owoperatorHigh+"] IS NOT VALID ");
					}

				}
				

				// COMP VALUE B - OW TEMPERATURE
				String owTempCompValBstr = cmd.getAdditionalParameter("OW_COMPVALUE");
				String owTempCompValBstrClean = "";
				try{
					BigDecimal owcompVal = new BigDecimal(owTempCompValBstr);
					owTempCompValBstrClean = owcompVal.toString();
					
				
				} catch (NumberFormatException ex){
					logger.error("ERROR: OW TEMP VALUE ["+owTempCompValBstr+"] (COMP VALUE B) IS NOT VALID");
				}
				String hex4Digits = "";
				if(owoperator > 9){
					// its a ADC COMP VALUE (2. CRITERIA)
					hex4Digits = calcADCValueToHexValue(ruleId, owTempCompValBstrClean);
				} else if(owoperator == 9){
					// (2. CRITERIA) is NOT ACTIVE 
					hex4Digits = "1000";	// WE WRITE INTO IT 4096
				} else {
					// ITS A OW COMP VALUE (2. CRITERIA)
					hex4Digits = calcTemperatureValueToHexValue(owTempCompValBstrClean);
				}
				payload[28] = hex4Digits.charAt(0);
				payload[29] = hex4Digits.charAt(1);
				payload[30] = hex4Digits.charAt(2);
				payload[31] = hex4Digits.charAt(3);
				
				
				
				
				// ACTION
				String actionStr = cmd.getAdditionalParameter("ACTION");
				int action = 0;
				try{
					action = new Integer(actionStr).intValue();
				}catch(NumberFormatException ed){
					logger.error("ERROR: ACTION  ["+actionStr+"] (e.g. OUTPUT PORT NUMBER) IS NOT VALID NUMBER");
				}
				
				if(action > 99 && action < 256){
					// 3 DIGITS
					int hundreds = action / 100;
					payload[38] = 0x30 + hundreds;		//0 + HUNDERTS
					int rest = 	action - hundreds * 100;
					int tens = rest / 10;
					payload[39] = 0x30 + tens; 			//0 + TENS
					int ones = 	action - 10*tens;
					payload[40] = 0x30 + ones;			// ONES + OFFSET
					
				} else if(action < 100 && action > 9 ){
					// 2 DIGITS
					int tens = action / 10;
					payload[38] = 0x30; 				//0
					payload[39] = 0x30 + tens; 			//0 + TENS
					int ones = 	action - 10*tens;
					payload[40] = 0x30 + ones;			// 0 + ONES 
				} else if(action < 10 && action > -1){
					// 1 DIGIT
					payload[38] = 0x30; 			//0
					payload[39] = 0x30; 			//0
					payload[40] = 0x30 + action; 	//0 + ONES
				} else {
					throw new BadDirectRuleDataRuntimeException("ERROR: ACTION MUST BE BETWEEN 0 and 255 but IS ["+actionStr+"]");
				} 				
				
				// ACTION VALUE AS HEX
				String actionValStr = cmd.getAdditionalParameter("ACTION_VALUE");
				int actionValue = 0;
				try{
					actionValue = new Integer(actionValStr).intValue();
				}catch(NumberFormatException ed){
					logger.error("ERROR: ACTION VALUE ["+actionStr+"] IS NOT VALID NUMBER");
				}
				
				if(actionValue > -1 && actionValue < 256){
					String actionValueHexStr = Integer.toHexString( actionValue);
					if(actionValueHexStr.length() > 2){
						logger.error("ERROR: ACTION VALUE ["+actionValStr+"] IS TOO BIG AS HEX VALUE ["+actionValueHexStr+"] - SHOULD NVER COM IN HERE!!");
					} 
					else if(actionValueHexStr.length() == 2){
						payload[43] = actionValueHexStr.charAt(0);
						payload[44] = actionValueHexStr.charAt(1);
					}
					else if(actionValueHexStr.length() == 1){
						payload[43] = 0x30; 							//0
						payload[44] = actionValueHexStr.charAt(0);
						
					}
					else if(actionValueHexStr.length() == 0){
						logger.error("ERROR: ACTION VALUE ["+actionValStr+"] HAS NO HEX VALUE!!! - SHOULD NVER COM IN HERE!!");
					}
				} else {
					throw new BadDirectRuleDataRuntimeException("ERROR: ACTION VALUE MUST BE BETWEEN 0 and 255 but IS ["+actionValStr+"]");
				} 				
				
				// STATUS
				String statusStr = cmd.getAdditionalParameter("STATUS");
				int status = 0;
				try{
					status = new Integer(statusStr).intValue();
				}catch(NumberFormatException ed){
					logger.error("ERROR: STATUS ["+statusStr+"] IS NOT VALID NUMBER");
				}

				payload[47] = status + 0x30;
				
				
				
				
			}
			

			
			
			
			
			
			
			
			
			
			
			
			
			
			XBeeAddress64 dest64 = new XBeeAddress64(XBeeDataTransmitter.parse64or16BitAddress(cmd.getBrickid()));

			ZNetTxRequest req = new ZNetTxRequest(dest64, payload);
			logger.info("PAYLOAD: " + ByteUtils.toString(payload));
			logger.info("PAYLOAD LENGTH: " + payload.length);

			// loopback test
			int sourceEndpoint = 0;
			int destinationEndpoint = ZNetExplicitTxRequest.Endpoint.DATA.getValue();

			logger.debug("       DESTINATION: " + cmd.getBrickid());
			logger.debug("       START SENDING");

			DoubleByte clusterId = new DoubleByte(0x0,
					ZNetExplicitTxRequest.ClusterId.SERIAL_LOOPBACK.getValue());
			// DoubleByte clusterId = new DoubleByte(0x0,
			// ZNetExplicitTxRequest.ClusterId.TRANSPARENT_SERIAL.getValue());

			// first request we just send 64-bit address. we get 16-bit network
			// address with status response
			ZNetExplicitTxRequest request = new ZNetExplicitTxRequest(0xff,
					dest64, XBeeAddress16.ZNET_BROADCAST,
					ZNetTxRequest.DEFAULT_BROADCAST_RADIUS,
					ZNetTxRequest.Option.UNICAST, payload, sourceEndpoint,
					destinationEndpoint, clusterId,
					ZNetExplicitTxRequest.znetProfileId);

			logger.info("SENDING EXPLICIT " + request.toString());

			xbee.sendAsynchronous(req);
			XBeeResponse response = xbee.getResponse();
			result.setResponseValue("PAYLOAD: " + ByteUtils.toString(payload)		+ "   RESPONSE:" + response.toString());
			result.setResponseOK(true);
			retval = response;

			logger.info("RECEIVED RESPONSE " + response.toString());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// xbee.close();
			logger.info("<------ sendWRITEDIRECTRULE()");
		}
		return retval;
	}



	/**
	 * FORMATS THE TCV (2. CRITERIA VALUE) FROM DECIMAL TO HEX IN CASE THE OPERATOR IS NOT OW READING BUT ADC  
	 * @param ruleId
	 * @param owTempCompValBstrClean
	 * @return String of Length 4
	 */
	protected static String calcADCValueToHexValue(String ruleId, String owTempCompValBstrClean) {
		BigDecimal bd = new BigDecimal(owTempCompValBstrClean);
		bd = bd.setScale(0, RoundingMode.UP);


		String preHex4Digits = Integer.toHexString(bd.intValue());
		if(preHex4Digits.length() > 4){
			logger.error("ERROR: 2. CRITERIA VALUE IS TOO BIG TO FIT IN 4 DIGIT HEX NUMBER ["+preHex4Digits+"] CHECK RULE ["+ruleId+"] PARAMETER TCV / COMP VALUE OW FOR ADC");
		} else if(preHex4Digits.length() == 4){
			// ITS THE SAME DO NOTHING 
		} else if(preHex4Digits.length() == 3){
			preHex4Digits = "0"+ preHex4Digits; 
		} else if(preHex4Digits.length() == 2){
			preHex4Digits = "00"+ preHex4Digits; 
		} else if(preHex4Digits.length() == 1){
			preHex4Digits = "000"+ preHex4Digits; 
		} else {
			preHex4Digits = "0000"; 
		}
		return preHex4Digits;
	}
	
	
	
	
	
	
	
	
	
	

	/** 
	 * FIND ALL XBEE RADIOS
	 * @param xbee
	 * @return ArrayList<NodeDiscover>
	 * @throws XBeeException
	 * @throws InterruptedException
	 */
	public static synchronized ArrayList<NodeDiscover> cmdFindNodes(XBee xbee)	throws XBeeException, InterruptedException {
		logger.info("-----> cmdFindNodes()");
		final ArrayList<NodeDiscover> retval = new ArrayList<NodeDiscover>();
		try {
			xbee.sendAsynchronous(new AtCommand("NT"));
			AtCommandResponse nodeTimeout = (AtCommandResponse) xbee.getResponse();
	
			// default is 6 seconds
			long nodeDiscoveryTimeout = ByteUtils.convertMultiByteToInt(nodeTimeout.getValue()) * 100;
	
			logger.info("Node discovery timeout is " + nodeDiscoveryTimeout + " milliseconds");
	
			xbee.addPacketListener(new PacketListener() {
				public void processResponse(XBeeResponse response) {
					if (response.getApiId() == ApiId.AT_RESPONSE) {
						NodeDiscover nd = NodeDiscover.parse((AtCommandResponse) response);
						logger.info("Node discover response is: " + nd);
						retval.add(nd);
					} else if(response.getApiId() == ApiId.ZNET_TX_STATUS_RESPONSE){
						logger.info("ITS OK I QUESS - ZNET TX STATUS RESPONSE: " + response);
					} 
					else {
						logger.info("Ignoring unexpected response: " + response);
					}
				}
			});
	
			logger.info("Sending node discover command");
			xbee.sendAsynchronous(new AtCommand("ND"));
	
			// wait for nodeDiscoveryTimeout milliseconds
			Thread.sleep(nodeDiscoveryTimeout);
	
			Iterator<NodeDiscover> doce_it = retval.iterator();
			while (doce_it.hasNext()) {
				NodeDiscover nod = doce_it.next();
				int count = 0;
				int marker = 0;
				for (int i = 0; i < retval.size(); i++) {
					NodeDiscover elem = retval.get(i);
					if (nod.equals(elem)) {
						count++;
						marker = i;
						if (count > 1) {
							// ups we have this element two times
							// remove one - marker points to it
							retval.remove(marker);
							i = 0;
							count--;
						}
					}
				}
				if (count == 1) {
					// this is expected
	
				} else {
					throw new RuntimeException("ERROR: THE NODE [" + nod + "] IS FOUND [" + count + "] TIME(S) - EXPECTED IS 1");
				}
	
			}
			logger.info("Time is up!  You should have heard back from all nodes by now.  If not make sure all nodes are associated and/or try increasing the node timeout (NT)");
		} finally {
			// xbee.close();
			logger.info("<----- cmdFindNodes()");
		}
		return retval;
	}	
	
	
	
	
	
	
	
	/**
	 * CONVERTS AN OW DS18B20 DECIMAL VALUE INTO HEX VALUE
	 * @param owTempCompValBstr as String "21.0" or "-0.5"
	 * @return
	 */
	private synchronized static String calcTemperatureValueToHexValue(String owTempCompValBstr) {
		String retval = "";
		try{
			BigDecimal owcompVal = new BigDecimal(owTempCompValBstr);
			int cmp = owcompVal.compareTo(new BigDecimal("125.0"));
			if(cmp == 1){
				if(owcompVal.compareTo(new BigDecimal("4096.0000")) == 0){
					// ITS OFF
					retval = "FF00";
				} else {
					// Bigger - Log error 
					logger.error("ERROR: VALUE ["+owTempCompValBstr+"] TOO BIG TO PROCESS");
				}
			} 
			cmp = owcompVal.compareTo(new BigDecimal("-50.0"));
			if(cmp == -1){
				// Bigger - Log error 
				logger.error("ERROR: VALUE ["+owTempCompValBstr+"] TOO SMALL TO PROCESS");
			} 

			if("FF00".compareTo(retval) == 0){
				// DO NOTHING - THE 2 RULE CRITERIA (OW COMP VALUE) Is OFF
				// return 4096
			} else {
				BigDecimal multiplier = owcompVal.divide(new BigDecimal("0.1")).setScale(0, RoundingMode.HALF_DOWN);
			
				int steps = multiplier.intValue();
				
				String hexVal = Integer.toHexString( steps);
				System.out.println(owTempCompValBstr + " --> " + steps + " --> HEX= " + hexVal);
				
				if(hexVal.length() > 4){
					// NEGATIVE VALUE
					retval = hexVal.substring(hexVal.length()-4, hexVal.length());
				} else if(hexVal.length() == 4){
					// Positive Value
					retval = hexVal;
				} else if(hexVal.length() == 3){
					// Positive Value
					retval = "0" + hexVal.substring(hexVal.length()-3, hexVal.length());
				} else if(hexVal.length() == 2){
					// Positive Value
					retval = "00" + hexVal.substring(hexVal.length()-2, hexVal.length());
				} else if(hexVal.length() == 1){
					// Positive Value
					retval = "000" + hexVal.substring(hexVal.length()-1, hexVal.length());
				}
			}
		} catch (NumberFormatException ex){
			System.out.println("ERROR: OW TEMP VALUE ["+owTempCompValBstr+"] (COMP VALUE B) IS NOT VALID");
		}
		
		return retval;
	}

	
	
	
	
	
	
}
