/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.xbee;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import com.moneyhouse.sensors.config.CONTSTANTINTERFACE;
import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.util.FileNameHelper;
import com.moneyhouse.tmor.serial.xbee.dto.ToBrickErrorFileQueueElementDataObject;
import com.moneyhouse.tmor.serial.xbee.dto.ToBrickFileQueueElementDataObject;

/**
 * QUEUE AS SINGLETON
 * @author Frederic
 *
 */
public class XBeeDataQueueSingleton implements Serializable{
	private static final long serialVersionUID = 216292712180630124L;
	private final static Logger logger = Logger.getLogger(XBeeDataQueueSingleton.class);
	private static XBeeDataQueueSingleton instance = null;
	private ArrayList<ToBrickFileQueueElementDataObject> toBrickDataList = new ArrayList<ToBrickFileQueueElementDataObject>();						// Files comming to the toBrick Dir are registered here
	private ArrayList<ToBrickErrorFileQueueElementDataObject> toBrickErrorDataList = new ArrayList<ToBrickErrorFileQueueElementDataObject>();		// Files with result in a XBee Error are copied into this queue

	/**
	 * @return ArrayList<ToBrickFileQueueElementDataObject> toBrickDataList
	 */
	public ArrayList<ToBrickFileQueueElementDataObject> obtainToBrickDataList(){
		return this.toBrickDataList;
	}

	/**
	 * @return ArrayList<ToBrickErrorFileQueueElementDataObject> toBrickErrorDataList
	 */
	public ArrayList<ToBrickErrorFileQueueElementDataObject> obtainToBrickErrorDataList(){
		return this.toBrickErrorDataList;
	}
	
	
	public static XBeeDataQueueSingleton getInstance() {
		if (instance == null) {
			instance = new XBeeDataQueueSingleton();
		}
//		try{
//			Thread.sleep(2000);
//		} catch(Exception c){
//			logger.error(c);
//		}
		return instance;
	}
	
	/** ADD FILE TO QUEUE 
	 * WE CHECK FOR DOUBLE NAME WHICH SHOULD NOT OCCUR
	 * @param in
	 * @return
	 */
	public synchronized int addToBrick(ToBrickFileQueueElementDataObject in){
		logger.debug("----> addToBrick("+in+")");
		boolean found = false;
		for(int i = 0 ; i < toBrickDataList.size(); i++){
			ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
			if(ele.getFilename().toString().equals(in.getFilename().toString())){
				found = true;
				break;
			}
		}
		if(found == true){
			logger.error("ERROR: THIS FILE WIth NAME ["+in.getFilename()+"] ALREADY EXISTS");
			throw new RuntimeException("ERROR: THIS FILE WIth NAME ["+in.getFilename()+"] ALREADY EXISTS");
		}
		
		toBrickDataList.add(in);
		logger.debug("<---- addToBrick("+in+") size ="+toBrickDataList.size());

		return toBrickDataList.size();
	}
	
	public int countToBrickDataList(){
		return  toBrickCount();
	}
	
	
	/** ADD FILE TO ERROR QUEUE 
	 * WE CHECK FOR DOUBLE NAME WHICH SHOULD NOT OCCUR
	 * @param in
	 * @return
	 */
	private synchronized int addToBrickError(ToBrickErrorFileQueueElementDataObject in){
		logger.debug("----> addToBrickError("+in+")");
		boolean found = false;
		for(int i = 0 ; i < toBrickErrorDataList.size(); i++){
			ToBrickErrorFileQueueElementDataObject ele = toBrickErrorDataList.get(i);
			if(ele.getFilename().toString().equals(in.getFilename().toString())){
				found = true;
				break;
			}
		}
		if(found == true){
			logger.error("ERROR: THIS FILE WIth NAME ["+in.getFilename()+"] ALREADY EXISTS");
			throw new RuntimeException("ERROR: THIS FILE WIth NAME ["+in.getFilename()+"] ALREADY EXISTS IN QUEUE ToBrickError");
		}
		
		toBrickErrorDataList.add(in);
		logger.debug("<---- addToBrickError("+in+") size ="+toBrickErrorDataList.size());

		return toBrickErrorDataList.size();
	}
	
	
	
	/**
	 * @return null when empty
	 */
	public synchronized ToBrickFileQueueElementDataObject  takeOldestToBrick(){
		logger.debug("----> takeOldestToBrick("+")");
		ToBrickFileQueueElementDataObject oldest = new ToBrickFileQueueElementDataObject();
		if(toBrickDataList.size() != 0){
			for(int i = 0 ; i < toBrickDataList.size(); i++){
				ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
				
				if(ele.getReceivedTimestamp().before(oldest.getReceivedTimestamp())){
					oldest = ele;
				}
			}
			if(oldest.getFilename() == null){
				logger.error("ERROR: THIS FILE WIth NAME [NULL] SHOULD NOT EXIST - ITS THE DUMMY CONSTRUCTOR - AND SHOULD HAVE BE REPLACED");
				throw new RuntimeException("ERROR: THIS FILE WIth NAME [NULL] SHOULD NOT EXIST - ITS THE DUMMY CONSTRUCTOR - AND SHOULD HAVE BE REPLACED");
			} else {
				// decent result - remove it form the queue
				boolean isSucess =toBrickDataList.remove(oldest);
				if(isSucess == false){
					// list inconsistent
					logger.error("ERROR: list inconsistent - WANTED TO REMOVE OBJ ["+oldest+"]");
					throw new RuntimeException("ERROR: list inconsistent - WANTED TO REMOVE OBJ ["+oldest+"]");
				}
			} 
		} else {
			// its empty
			// so return null
			oldest = null;
		}
		logger.debug("<---- takeOldestToBrick("+") oldest= "+oldest);
		return oldest;
	}

	
	
	/** READS OUT ALL THE ERROR ELEMENTS FROM THE LIST - UNTIL IT IS EMPTY
	 * @return
	 */
	public synchronized ArrayList<ToBrickErrorFileQueueElementDataObject>  flushToBrickError(){
		ArrayList<ToBrickErrorFileQueueElementDataObject> retVal = new ArrayList<ToBrickErrorFileQueueElementDataObject> ();
		for(;;){
			ToBrickErrorFileQueueElementDataObject element = this.takeOldestToBrickError();
			if(element != null){
				retVal.add(element);
			} else {
				// when empty - last element is null
				break;
			}
		}
		return retVal;
	}

	
	/**
	 * @return null when empty
	 */
	public synchronized ToBrickErrorFileQueueElementDataObject  takeOldestToBrickError(){
		logger.debug("----> takeOldestToBrickError("+")");
		ToBrickErrorFileQueueElementDataObject oldest = new ToBrickErrorFileQueueElementDataObject();
		if(toBrickDataList.size() != 0){
			for(int i = 0 ; i < toBrickErrorDataList.size(); i++){
				ToBrickErrorFileQueueElementDataObject ele = toBrickErrorDataList.get(i);
				
				if(ele.getReceivedTimestamp().before(oldest.getReceivedTimestamp())){
					oldest = ele;
				}
			}
			if(oldest.getFilename() == null){
				logger.error("ERROR: THIS FILE WIth NAME [NULL] SHOULD NOT EXIST - ITS THE DUMMY CONSTRUCTOR - AND SHOULD HAVE BE REPLACED");
				throw new RuntimeException("ERROR: THIS FILE WIth NAME [NULL] SHOULD NOT EXIST - ITS THE DUMMY CONSTRUCTOR - AND SHOULD HAVE BE REPLACED");
			} else {
				// decent result - remove it form the queue
				boolean isSucess =toBrickDataList.remove(oldest);
				if(isSucess == false){
					// list inconsistent
					logger.error("ERROR: list inconsistent - WANTED TO REMOVE OBJ ["+oldest+"]");
					throw new RuntimeException("ERROR: list inconsistent - WANTED TO REMOVE OBJ ["+oldest+"]");
				}
			} 
		} else {
			// its empty
			// so return null
			oldest = null;
		}
		logger.debug("<---- takeOldestToBrickError("+") oldest= "+oldest);
		return oldest;
	}	
	
	
	/** KEEPS THE ELEMENT IN - DOEs DELETE FROM LIST WHEN MORE READS THAN retries
	 * @param int retries - how often can we read it out of this queue before not doing anything anymore with it - leaving it there
	 * @param  retries how often can we hold it in our hands before we more it to the errorlist
	 * @param  waste some time before doing it 
	 * @return null when empty
	 */
	public synchronized ToBrickFileQueueElementDataObject  readOldestToBrick(int retries, long timeout){
		logger.debug("----> readOldestToBrick("+retries+", "+timeout+")");
		ToBrickFileQueueElementDataObject val = readOldestToBrick(retries);
		if(val != null){
			try{
				Thread.sleep(timeout);
			}catch (Exception ex){
				logger.error(ex);
			}
		}
		logger.debug("<---- readOldestToBrick("+retries+", "+timeout+")");
		
		return val; 
	}

	
	
	
	
	
	
	
	/** KEEPS THE ELEMENT IN - DOEs DELETE FROM LIST WHEN MORE READS THAN retries
	 * PRIORITIZES HEARTHBEAT 
	 * @param int retries - how often can we read it out of this queue before not doing anything anymore with it - leaving it there
	 * @param Type - such as HEARTHBEAT OR COMMAND OR BATCH_PULL
	 * @return null when empty
	 */
	public synchronized ToBrickFileQueueElementDataObject  readOldestToBrickWithType(int retries, String type){
		// logger.debug("----> readOldestToBrickWithType("+retries+", "+type+")");
		ToBrickFileQueueElementDataObject oldest = null;
		oldest = new ToBrickFileQueueElementDataObject();
		boolean found = false;
		if(toBrickDataList.size() != 0){
			for(int i = 0 ; i < toBrickDataList.size(); i++){
				ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
				
				if(ele.getReceivedTimestamp().before(oldest.getReceivedTimestamp())){
					String fname = FileNameHelper.extractFilenameFromFullpath(ele.getFilename().getName());
					if(fname.contains(type)){
						// YES IN THE FILE NAME WE HAVE THE TYPE
						oldest = ele;
						found = true;
					}
				}
			}
			if(found == true){
				if(oldest.getFilename() == null){
					logger.error("ERROR: THIS FILE WIth NAME [NULL] SHOULD NOT EXIST - ITS THE DUMMY CONSTRUCTOR - AND SHOULD HAVE BE REPLACED");
					throw new RuntimeException("ERROR: THIS FILE WIth NAME [NULL] SHOULD NOT EXIST - ITS THE DUMMY CONSTRUCTOR - AND SHOULD HAVE BE REPLACED");
				} else {
					oldest.incrementReadingCounter();
					if(retries < oldest.getReadingCounter()){
						// we got too many times this command in our processor
						logger.debug("["+retries+"]  < oldest.getReadingCounter() =[ "+oldest +"] MOVE TO ERROR QUEUE");
						
						moveFromToBrickToErrorList(oldest, CONTSTANTINTERFACE.TOO_MANY_TRIES);
					}
				}
			} 
			else 
			{
				// NO ENTRY WITH TYPE IN TOBRICK QUEUE	
				oldest = null;
			}
		} else {
			// its empty - so return null
			oldest = null;
		}
		// logger.debug("<---- readOldestToBrickWithType("+retries+", "+type+") oldest="+oldest);
	
		return oldest;
	}
	
	
	
	
	
	
	/** KEEPS THE ELEMENT IN - DOEs DELETE FROM LIST WHEN MORE READS THAN retries
	 * PRIORITIZES HEARTHBEAT 
	 * @param int retries - how often can we read it out of this queue before not doing anything anymore with it - leaving it there
	 * @param  retries how often can we hold it in our hands before we more it to the errorlist
	 * @return null when empty
	 */
	public synchronized ToBrickFileQueueElementDataObject  readOldestToBrick(int retries){
		// logger.debug("----> readOldestToBrick("+retries+")");
		
		// HIGHEST PRIO HAVE tHE HEARTBEAT ENTRIES
		ToBrickFileQueueElementDataObject oldest = readOldestToBrickWithType(retries, CONTSTANTINTERFACE.FILENAME_PART_START_FOR_HEARTBEAT);
		
		// SECON HIGHEST FOR COMMAND ENTRIES
		if(oldest == null){
			oldest = readOldestToBrickWithType(retries, CONTSTANTINTERFACE.FILENAME_PART_FOR_COMMAND);
		}
		
		
		if(oldest == null){
			oldest = new ToBrickFileQueueElementDataObject();
	
			if(toBrickDataList.size() != 0){
				for(int i = 0 ; i < toBrickDataList.size(); i++){
					ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
					
					if(ele.getReceivedTimestamp().before(oldest.getReceivedTimestamp())){
						oldest = ele;
					}
				}
				if(oldest.getFilename() == null){
					logger.error("ERROR: THIS FILE WIth NAME [NULL] SHOULD NOT EXIST - ITS THE DUMMY CONSTRUCTOR - AND SHOULD HAVE BE REPLACED");
					throw new RuntimeException("ERROR: THIS FILE WIth NAME [NULL] SHOULD NOT EXIST - ITS THE DUMMY CONSTRUCTOR - AND SHOULD HAVE BE REPLACED");
				} else {
					oldest.incrementReadingCounter();
					if(retries < oldest.getReadingCounter()){
						// we got too many times this command in our processor
						logger.debug("["+retries+"]  < oldest.getReadingCounter() =[ "+oldest +"] MOVE TO ERROR QUEUE");
						moveFromToBrickToErrorList(oldest, CONTSTANTINTERFACE.TOO_MANY_TRIES);
					}
				}
			} else {
				// its empty - so return null
				oldest = null;
			}
		}
		// logger.debug("<---- readOldestToBrick("+retries+") oldest="+oldest);
		return oldest;
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	/** KEEPS THE ELEMENT IN - DOEs NOT DELETE IT FROM LIST
	 * @return null when empty
	 */
	public synchronized ToBrickFileQueueElementDataObject  readOldestToBrick(){
		logger.debug("----> readOldestToBrick("+")");
		ToBrickFileQueueElementDataObject oldest = new ToBrickFileQueueElementDataObject();
		if(toBrickDataList.size() != 0){
			for(int i = 0 ; i < toBrickDataList.size(); i++){
				ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
				
				if(ele.getReceivedTimestamp().before(oldest.getReceivedTimestamp())){
					oldest = ele;
				}
			}
			if(oldest.getFilename() == null){
				logger.error("ERROR: THIS FILE WIth NAME [NULL] SHOULD NOT EXIST - ITS THE DUMMY CONSTRUCTOR - AND SHOULD HAVE BE REPLACED");
				throw new RuntimeException("ERROR: THIS FILE WIth NAME [NULL] SHOULD NOT EXIST - ITS THE DUMMY CONSTRUCTOR - AND SHOULD HAVE BE REPLACED");
			} 
		} else {
			// its empty - so return null
			oldest = null;
		}
		logger.debug("<---- readOldestToBrick("+") oldest = "+oldest);
		
		return oldest;
	}
	
	
	/** DELETE FILE FROM QUEUE 
	 * WE CHECK NOTHING
	 * @param in
	 * @return
	 */
	public synchronized boolean deleteToBrick(String filename){
		logger.debug("----> deleteToBrick("+filename+")");
		boolean found = false;
		for(int i = 0 ; i < toBrickDataList.size(); i++){
			ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
			if(ele.getFilename().toString().equals(filename)){
				toBrickDataList.remove(ele);
				found = true;
				break;
			}
		}
		logger.debug("<---- deleteToBrick("+filename+") found= "+found);
		
		return found;
	}	
	
	/** DELETE FILE FROM QUEUE 
	 * WE CHECK NOTHING
	 * @param in
	 * @return
	 */
	public synchronized boolean deleteToBrick(ToBrickFileQueueElementDataObject element){
		logger.debug("----> deleteToBrick("+element+")");
		boolean found = false;
		for(int i = 0 ; i < toBrickDataList.size(); i++){
			ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
			if(ele.getFilename().toString().equals(element.getFilename().toString())){
				toBrickDataList.remove(ele);
				found = true;
				break;
			}
		}
		logger.debug("<---- deleteToBrick("+element+") found="+found);
		
		return found;
	}	

	
	/** CREATES A NEW ERROR OBJ AND PUTS IT IN QUEUE toBrickErrorDataList
	 *  DELETES tHE FILE FROM toBrickDataList
	 * @param filenamesequence
	 * @param errorMsg
	 * @return
	 */
	public synchronized int moveFromToBrickToErrorListContainingCharSequence(String filenamesequence, String errorMsg){
		logger.debug("----> moveFromToBrickToErrorListContainingCharSequence("+filenamesequence+", "+errorMsg+")");
		int found = 0;
		ArrayList<ToBrickFileQueueElementDataObject> foundWithSequenceInName = findContainingCharSequenceToBrick(filenamesequence);
		for(int i = 0 ; i < foundWithSequenceInName.size(); i++){
			ToBrickFileQueueElementDataObject  ele = foundWithSequenceInName.get(i);
			ToBrickErrorFileQueueElementDataObject err = new ToBrickErrorFileQueueElementDataObject(ele, errorMsg)  ;
			addToBrickError(err);
			deleteContainingCharSequenceToBrick(filenamesequence);
			found ++;
		}
		logger.debug("<---- moveFromToBrickToErrorListContainingCharSequence("+filenamesequence+", "+errorMsg+") found ="+found );
		return found;
	}
	

	
	
	/** CREATES A NEW ERROR OBJ AND PUTS IT IN QUEUE toBrickErrorDataList
	 *  DELETES tHE FILE FROM toBrickDataList
	 * @param filenamesequence
	 * @param errorMsg
	 * @return
	 */
	private synchronized void moveFromToBrickToErrorList(ToBrickFileQueueElementDataObject element, String errorMsg){
		logger.debug("----> moveFromToBrickToErrorList("+element+", "+errorMsg+")");
		ToBrickErrorFileQueueElementDataObject err = new ToBrickErrorFileQueueElementDataObject(element, errorMsg)  ;
		addToBrickError(err);
		deleteToBrick(element);
		// return found;
		logger.debug("<---- moveFromToBrickToErrorList("+element+", "+errorMsg+")");

	}
	
	/** RETURN ELEMENT AND DELETE FROM QUEUE  - MIGHT GIVe YOU A NULLPOINTER EXPECTION 
	 * WE CHECK FOR DOUBLE NAME WHICH SHOULD NOT OCCUR
	 * @param in
	 * @see readContainingCharSequenceToBrick to avoid NullPointer
	 * @return
	 */
	public synchronized ToBrickFileQueueElementDataObject takeContainingCharSequenceToBrick(String filenamesequence){
		logger.debug("----> takeContainingCharSequenceToBrick("+filenamesequence+")");
		ToBrickFileQueueElementDataObject found = null;
		ArrayList<ToBrickFileQueueElementDataObject> foundWithSequenceInName = findContainingCharSequenceToBrick(filenamesequence);
		if(foundWithSequenceInName.size() > 1){
			logger.warn("WARNING: WE FOUND MORE THAN 1 FILE CONTAINING CHAR SEQUENCE ["+filenamesequence+"] IN NAME");
			
		} else if(foundWithSequenceInName.size() == 1){
			found = foundWithSequenceInName.get(0);
			for(int j = 0; j < foundWithSequenceInName.size();j++){
				ToBrickFileQueueElementDataObject mustBeDeleted = foundWithSequenceInName.get(j);
				
				for(int i = 0 ; i < toBrickDataList.size(); i++){
					ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
					if(mustBeDeleted.getFilename().toString().equals(ele.getFilename().toString())){
						toBrickDataList.remove(ele);
						i = 0;	// restart from the beginning and check for next one
					}
				}
			}
		} else {
			found = null;
		}
		logger.debug("<---- takeContainingCharSequenceToBrick("+filenamesequence+") found = "+found);
		
		return found;
	}	
	
	/** RETURN ELEMENT WITH THE CHAR SEQUENCE  
	 * WE CHECK FOR DOUBLE NAME WHICH SHOULD NOT OCCUR
	 * @param in
	 * @return
	 */
	public synchronized ToBrickFileQueueElementDataObject readContainingCharSequenceToBrick(String filenamesequence){
		logger.debug("----> readContainingCharSequenceToBrick("+filenamesequence+")");
		ToBrickFileQueueElementDataObject found = null;
		ArrayList<ToBrickFileQueueElementDataObject> foundWithSequenceInName = findContainingCharSequenceToBrick(filenamesequence);
		if(foundWithSequenceInName.size() > 1){
			logger.warn("WARNING: WE FOUND MORE THAN 1 FILE CONTAINING CHAR SEQUENCE ["+filenamesequence+"] IN NAME");
			
		} else if(foundWithSequenceInName.size() == 1){
			found = foundWithSequenceInName.get(0);
			
		} else {
			found = null;
		}
		logger.debug("<---- readContainingCharSequenceToBrick("+filenamesequence+")");
		
		return found;
	}		
	
	/** RETURN ELEMENT WITH THE CHAR SEQUENCE  
	 * WE CHECK FOR DOUBLE NAME WHICH SHOULD NOT OCCUR
	 * @param in
	 * @return
	 */
	public synchronized ToBrickFileQueueElementDataObject readContainingCharSequenceToBrick(String filenamestart, String correlid){
		logger.debug("----> readContainingCharSequenceToBrick("+filenamestart+", "+correlid+")");
		ToBrickFileQueueElementDataObject found = null;
		ArrayList<ToBrickFileQueueElementDataObject> foundWithSequenceInName = findInToBrickQueueFilename(filenamestart, correlid);
		if(foundWithSequenceInName.size() > 1){
			logger.warn("WARNING: WE FOUND IN toBrICK QUEUE MORE THAN  1 ENTRY REFERRING TO FILE STARTING WITH ["+filenamestart+"] AND CORREL ID ["+correlid+"]");
			
		} else if(foundWithSequenceInName.size() == 1){
			found = foundWithSequenceInName.get(0);
			
		} else {
			logger.warn("WARNING: WE FOUND IN toBrICK QUEUE NO ENTRY REFERRING TO FILE STARTING WITH ["+filenamestart+"] AND CORREL ID ["+correlid+"]");
			found = null;
		}
		logger.debug("<---- readContainingCharSequenceToBrick("+filenamestart+", "+correlid+")");
		
		return found;
	}		
	
	
	
	
	/** 
	 * SIMILAR TO #deleteContainingCharSequenceToBrick
	 * DELETE ENTRY WHICH HOLDS CORRELID AND FILENAME STARTS WITH STRING SEQUENCE
	 * WE CHECK FOR DOUBLE NAME WHICH SHOULD NOT OCCUR
	 * @param filenamestart
	 * @param correlid
	 * @return
	 */
	public synchronized boolean deleteContainingCharSequenceToBrick(String filenamestart, String correlid){
		logger.debug("----> deleteContainingCharSequenceToBrick("+filenamestart+", "+correlid+")");
		boolean found = false;
		ArrayList<ToBrickFileQueueElementDataObject> foundWithSequenceInName = findInToBrickQueueFilename(filenamestart, correlid);
		if(foundWithSequenceInName.size() > 1){
			logger.warn("WARNING: WE FOUND MORE THAN 1 toBRICK ENTRY HOLDING A FILENAME STARTING WITH ["+filenamestart+"] AND CORREL ID ["+correlid+"] IN NAME");
			found = true;
		} else if(foundWithSequenceInName.size() == 1){
			found = true;
			for(int j = 0; j < foundWithSequenceInName.size();j++){
				ToBrickFileQueueElementDataObject mustBeDeleted = foundWithSequenceInName.get(j);
				
				for(int i = 0 ; i < toBrickDataList.size(); i++){
					ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
					if(mustBeDeleted.getFilename().toString().equals(ele.getFilename().toString())){
						toBrickDataList.remove(ele);
						
						// we do not delete the file in file system
						i = 0;	// restart from the beginning and check for next one
					}
				}
			}
		} else {
			logger.warn("WARNING: WE FOUND 0 NO toBRICK ENTRY HOLDING A FILENAME STARTING WITH ["+filenamestart+"] AND CORREL ID ["+correlid+"] IN NAME");
		
			found = false;
		}
		logger.debug("<---- deleteContainingCharSequenceToBrick("+filenamestart+", "+correlid+") removed = "+found );
		return found;
	}

	
	
	
	
	/** 
	 * SIMILAR TO #deleteContainingCharSequenceToBrick
	 * DELETE ENTRY WHICH HOLDS CORRELID AND FILENAME STARTS WITH STRING SEQUENCE FROM THE ERROR QUEUE 
	 * WE CHECK FOR DOUBLE NAME WHICH SHOULD NOT OCCUR
	 * @param filenamestart
	 * @param correlid
	 * @return
	 */
	public synchronized int deleteContainingCharSequenceToBrickErrorList(String filenamestart, String correlid){
		logger.debug("----> deleteContainingCharSequenceToBrickErrorList("+filenamestart+", "+correlid+")");
		int delCounter = 0;
		ArrayList<ToBrickErrorFileQueueElementDataObject> foundWithSequenceInName = findInToBrickErrorListQueueFilename(filenamestart, correlid);
		if(foundWithSequenceInName.size() > 1){
			logger.warn("WARNING: WE FOUND MORE THAN 1 toBRICK ERROR LIST ENTRY HOLDING A FILENAME STARTING WITH ["+filenamestart+"] AND CORREL ID ["+correlid+"] IN NAME");
			delCounter = 0;
		} else if(foundWithSequenceInName.size() == 1){
			delCounter = 0;
			for(int j = 0; j < foundWithSequenceInName.size();j++){
				ToBrickErrorFileQueueElementDataObject mustBeDeleted = foundWithSequenceInName.get(j);
				
				for(int i = 0 ; i < toBrickErrorDataList.size(); i++){
					ToBrickErrorFileQueueElementDataObject ele = toBrickErrorDataList.get(i);
					if(mustBeDeleted.getFilename().toString().equals(ele.getFilename().toString())){
						toBrickErrorDataList.remove(ele);
						delCounter++;
						// we do not delete the file in file system
						i = 0;	// restart from the beginning and check for next one
					}
				}
			}
		} else {
			logger.warn("WARNING: WE FOUND 0 NO toBRICK ERROR LIST  ENTRY HOLDING A FILENAME STARTING WITH ["+filenamestart+"] AND CORREL ID ["+correlid+"] IN NAME");
		
			delCounter = 0;
		}
		logger.debug("<---- deleteContainingCharSequenceToBrickErrorList("+filenamestart+", "+correlid+") removed = "+delCounter );
		return delCounter;
	}

	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	/** DELETE ENTRY WHICH CONTAINS THE STRING SEQUENCE IN FILE FROM QUEUE 
	 * WE CHECK FOR DOUBLE NAME WHICH SHOULD NOT OCCUR
	 * @param in
	 * @return
	 */
	public synchronized boolean deleteContainingCharSequenceToBrick(String filenamesequence){
		logger.debug("----> deleteContainingCharSequenceToBrick("+filenamesequence+")");
		boolean found = false;
		ArrayList<ToBrickFileQueueElementDataObject> foundWithSequenceInName = findContainingCharSequenceToBrick(filenamesequence);
		if(foundWithSequenceInName.size() > 1){
			logger.warn("WARNING: WE FOUND MORE THAN 1 FILE CONTAINING CHAR SEQUENCE ["+filenamesequence+"] IN NAME");
			found = true;
		} else if(foundWithSequenceInName.size() == 1){
			found = true;
			for(int j = 0; j < foundWithSequenceInName.size();j++){
				ToBrickFileQueueElementDataObject mustBeDeleted = foundWithSequenceInName.get(j);
				
				for(int i = 0 ; i < toBrickDataList.size(); i++){
					ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
					if(mustBeDeleted.getFilename().toString().equals(ele.getFilename().toString())){
						toBrickDataList.remove(ele);
						
						// we do not delete the file in file system
						i = 0;	// restart from the beginning and check for next one
					}
				}
			}
		} else {
			found = false;
		}
		logger.debug("<---- deleteContainingCharSequenceToBrick("+filenamesequence+") found = "+found );
		return found;
	}


	
	/** DELETE FILE WHICH CONTAINS THE STRING SEQUENCE IN FILE FROM QUEUE 
	 * WE CHECK FOR DOUBLE NAME WHICH SHOULD NOT OCCUR
	 * 
	 * @see #deleteContainingCharSequenceToBrick BUT THIS ONE DOES NOT DELETE THE FILES SOMe FILES NEED TO STAY - SPECIALLY FOR THE rwadOW to get the MAPPING VIA CORRELATION ID
	 * @param String filenamesequence
	 * @return true when file was deleted
	 */
	public synchronized boolean deleteContainingCharSequenceToBrickAndFile(String filenamesequence){
		logger.debug("----> deleteContainingCharSequenceToBrick("+filenamesequence+")");
		boolean found = false;
		ArrayList<ToBrickFileQueueElementDataObject> foundWithSequenceInName = findContainingCharSequenceToBrick(filenamesequence);
		if(foundWithSequenceInName.size() > 1){
			logger.warn("WARNING: WE FOUND MORE THAN 1 FILE CONTAINING CHAR SEQUENCE ["+filenamesequence+"] IN NAME");
			found = true;
		} else if(foundWithSequenceInName.size() == 1){
			found = true;
			for(int j = 0; j < foundWithSequenceInName.size();j++){
				ToBrickFileQueueElementDataObject mustBeDeleted = foundWithSequenceInName.get(j);
				
				for(int i = 0 ; i < toBrickDataList.size(); i++){
					ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
					if(mustBeDeleted.getFilename().toString().equals(ele.getFilename().toString())){
						toBrickDataList.remove(ele);
						
						// also delete the file in file system
						String listentodir = PropertiesFileReader.getInstance().getProperty(CONTSTANTINTERFACE.PROP_KEY_TO_BRICK_DIR);
						ArrayList<File> withName = findFilesWithCharSequence(filenamesequence, listentodir);
						
						if(withName.size() > 1){
							// THIS IS AMBIGUOUS 
						} else if(withName.size() == 1){
							// SUNSHINE CASE
							withName.get(0).delete();
						} else if(withName.size() == 0) {
							// THIS CAN NOT BE - WE NEED TO HAVE AN ETRY IN THe FILE SYSTEM FOR tHE QUEUE ENTRY
							throw new RuntimeException("ERROR: THIS CAN NOT BE - NAME CONTAINING ["+filenamesequence+"] MST BE PRESENT ON FILE DIRECTORY ["+listentodir+"] BUT IS NOT - EG> QUEUE AND FILESYSTEM ARE NOT CONSISTENT!!");
						} else {
							throw new RuntimeException("ERROR: THIS CAN NOT BE - FILE WITH NAME CONTAINING ["+filenamesequence+"] IN FILE DIRECTORY ["+listentodir+"] CAUSES A STATE WHICH IS NOT POSSIBLE!!");
							
						}
						i = 0;	// restart from the beginning and check for next one
					}
				}
			}
		} else {
			found = false;
		}
		logger.debug("<---- deleteContainingCharSequenceToBrick("+filenamesequence+") found = "+found );
		return found;
	}
	
	
	
	
	/** READS CONTENT OF FILE DIRECOTRY AND RETURN 
	 * @param sequenceWeLookFor
	 * @param fileDirectory
	 * @return
	 */
	private ArrayList<File> findFilesWithCharSequence(String sequenceWeLookFor, String fileDirectory){
		ArrayList<File> filesWithSeqInName = new ArrayList<File>();
		File folder = new File(fileDirectory);
		for (final File fileEntry : folder.listFiles()) {
	        if (fileEntry.isDirectory()) {
	            // NO NEED TO DEAL WITH DIR
	        } else {
	            String nameOfFIle = folder.getPath() + System.getProperty("file.separator") + fileEntry.getName();
	            if(nameOfFIle.contains(sequenceWeLookFor)){
	            	filesWithSeqInName.add(new File(nameOfFIle));
	            }
	        }
	    }
		return filesWithSeqInName;
	}
	

	
	private ArrayList<ToBrickFileQueueElementDataObject> findInToBrickQueueFilename(String filenamebeginning, String correlId) {
		logger.info("----> findInToBrickQueueFilename("+filenamebeginning+"," +correlId+")");
		ArrayList<ToBrickFileQueueElementDataObject> foundWithSequenceInName = new ArrayList<ToBrickFileQueueElementDataObject>();
		
		logger.debug("["+correlId+"] toBrickDataList.size() = "+ toBrickDataList.size());
		for(int i = 0 ; i < toBrickDataList.size(); i++){
			ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
			logger.debug("["+correlId+"] ele from to  bq: " + ele);
			String filename = FileNameHelper.extractFilenameFromFullpath(ele.getFilename().toString()).trim();
			logger.debug("["+correlId+"] ele filena   bq: " + filename);
			boolean filestart = filename.startsWith(filenamebeginning);
			boolean contains = filename.contains(correlId);
			logger.debug("["+correlId+"] ele startwit bq: " + filestart + "\tcontains: " + contains);

			if(filename.startsWith(filenamebeginning) && ele.getFilename().toString().contains(correlId)){
				foundWithSequenceInName.add(ele);
				logger.debug("["+correlId+"] we add ["+correlId+"] it to the tobrick returnlist");
			}
		}
		logger.info("<---- findInToBrickQueueFilename("+filenamebeginning+"," +correlId+") retval size ="+foundWithSequenceInName.size());
		return foundWithSequenceInName;
	}		
	
	
	public synchronized ArrayList<ToBrickErrorFileQueueElementDataObject> findInToBrickErrorListQueueFilename(String filenamebeginning, String correlId) {
		logger.info("----> findInToBrickErrorListQueueFilename("+filenamebeginning+"," +correlId+")");
		ArrayList<ToBrickErrorFileQueueElementDataObject> foundWithSequenceInName = new ArrayList<ToBrickErrorFileQueueElementDataObject>();
		logger.debug("["+correlId+"] toBrickErrorDataList.size() = "+ toBrickErrorDataList.size());
	
		for(int i = 0 ; i < toBrickErrorDataList.size(); i++){
			ToBrickErrorFileQueueElementDataObject ele = toBrickErrorDataList.get(i);
			logger.debug("["+correlId+"] ele from err q: " + ele);
			String filename = FileNameHelper.extractFilenameFromFullpath(ele.getFilename().toString()).trim();
			logger.debug("["+correlId+"] ele filename q: " + filename);
			boolean filestart = filename.startsWith(filenamebeginning.trim());
			boolean contains = filename.contains(correlId.trim());
			logger.debug("["+correlId+"] ele startwit q: " + filestart + "\tcontains: " + contains);
			if(filename.startsWith(filenamebeginning.trim()) && filename.contains(correlId.trim())){
				foundWithSequenceInName.add(ele);
				logger.debug("["+correlId+"] we add ["+correlId+"] it to the error list returnlist");
			}
		}
		logger.info("<---- findInToBrickErrorListQueueFilename("+filenamebeginning+"," +correlId+") retval size ="+foundWithSequenceInName.size());
		return foundWithSequenceInName;
	}		
	
	
	
	private ArrayList<ToBrickFileQueueElementDataObject> findContainingCharSequenceToBrick(String filenamesequence) {
		logger.debug("----> findContainingCharSequenceToBrick("+filenamesequence+")");
		ArrayList<ToBrickFileQueueElementDataObject> foundWithSequenceInName = new ArrayList<ToBrickFileQueueElementDataObject>();
		
		for(int i = 0 ; i < toBrickDataList.size(); i++){
			ToBrickFileQueueElementDataObject ele = toBrickDataList.get(i);
			if(ele.getFilename().toString().contains(filenamesequence)){
				foundWithSequenceInName.add(ele);
			}
		}
		logger.debug("<---- findContainingCharSequenceToBrick("+filenamesequence+") retval size ="+foundWithSequenceInName.size());
		return foundWithSequenceInName;
	}		
	
	public String toString(){
		String toBrick = "toBrick QUEUE SIZE ["+this.toBrickDataList.size()+"]\tCONTENT OF: "+ this.hashCode()+"\n";
		toBrick += "-S-T-A-R-T-TO-B-R-I-C-K-["+this.toBrickDataList.size()+"]------------- toBrick QUEUE\n";
		for(int i = 0 ; i < this.toBrickDataList.size(); i++){
			toBrick += "POS["+i+"]\t" + this.toBrickDataList.get(i) +"\n"; 
		}
		toBrick += "-E-N-D-----TO-B-R-I-C-K------------ \n";
		String toBrickError = "toBrickError QUEUE SIZE ["+this.toBrickErrorDataList.size()+"]\tCONTENT OF: "+ this.hashCode()+"\n";
		toBrickError += "-S-T-A-R-T--E-R-R-O-R-["+this.toBrickErrorDataList.size()+"]----------- toBrickError QUEUE\n";
		for(int i = 0 ; i < this.toBrickErrorDataList.size(); i++){
			toBrickError += "["+i+"]\t" + this.toBrickErrorDataList.get(i) +"\n"; 
		}
		toBrickError += "-E-N-D------E-R-R-O-R------------ \n";

		
		return toBrick + toBrickError;
	}

	public int errorCount(){
		return this.toBrickErrorDataList.size();
	}

	public int toBrickCount(){
		return this.toBrickDataList.size();
	}

}
