/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.xbee;

import java.util.StringTokenizer;

import org.apache.log4j.Logger;

import com.rapplogic.xbee.api.Checksum;
import com.rapplogic.xbee.util.ByteUtils;
import com.rapplogic.xbee.util.IntArrayOutputStream;

/**
 * Class that helps you form and assemble data packages
 * @author Frederic
 *
 */
public class XBeeDataTransmitter {
	private final static Logger logger = Logger.getLogger(XBeeDataTransmitter.class);

	
	
	
	
	
	public static synchronized int[] assembleTXRequest(String longbitaddr, String shortbitaddr, String data){
		int [] longbitaddrarray = parse64or16BitAddress(longbitaddr);
		int [] shortbitaddrarray = parse64or16BitAddress(shortbitaddr);
		int [] dataarray =parseData(data);
		return assembleTXRequest(longbitaddrarray, shortbitaddrarray, dataarray);
		
	}

	
	/** transforms an address of 64  bit or 16 bit in String format to int[]
	 * @param String addressStruntrimmed
	 * @return int[]
	 */
	public static int [] parse64or16BitAddress(String addressStruntrimmed){
		boolean patternfound = false;
		int octetsmax = 0;
		String addressStrraw = addressStruntrimmed.trim();
		String addressStr = addressStrraw.replace(",", " ").replace("   ", " ").replace("  ", " ").replace("  ", " ").replace("0x", " ").replace("  ", " ").trim();
		
		if(addressStr.length() == 0 || addressStr.length() < 5){
			throw new RuntimeException("ERROR: ADDRESS LENGTH WITH ["+addressStr.length()+"] NOT CORRECT - IT NEEDS TO BE 5 CHARACTERS OR MORE... ");
			
		} else if(addressStr.length() < 11){
			// its a 16 Bit Address
			// POS  1234567890
			// ADDR 0x00 , 0x00
			octetsmax = 2;
		} else if(addressStr.length() > 10 && addressStr.length() < 47){
			// its a 64 Bit Address
			// POS  1234567890123456789012345678901234567890123456
			// ADDR 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
			
			// POS  12345678901234567890123
			// ADDR 00 00 00 00 00 00 00 00
			
			octetsmax = 8;
		}
		int [] address = new int[octetsmax];


		// is it the format 0x00 0x00 0x00 0x00 ?
		StringTokenizer st = new StringTokenizer(addressStr, " ");
		if(st.countTokens() == octetsmax){
			for (int i = 0; i < address.length; i++) {
				String byteStr = st.nextToken();
				address[i] = Integer.parseInt(byteStr, 16);
			}
			patternfound = true;
		} 
		
		if(patternfound == false){
			throw new RuntimeException("ERROR: NON OF THE DATA PATTERN MATCHES - CHECK SEPARATION OF ADDRESS OCTETS ");
		}
		return address;
	}
	
	/** transforms a string into int []
	 * @param string datauntrimmed
	 * @return int []
	 */
	private static int [] parseData(String datauntrimmed){
		boolean patternfound = false;
		String datastrStrraw = datauntrimmed.trim();
		String datastr = datastrStrraw.replace(",", " ").replace("   ", " ").replace("  ", " ").replace("  ", " ").replace("0x", " ").replace("  ", " ").trim();
		

		IntArrayOutputStream outputStrem = new IntArrayOutputStream();


		// is it the format 0x00 0x00 0x00 0x00 ?
		StringTokenizer st = new StringTokenizer(datastr, " ");
		if(st.countTokens() > 0){
			while (st.hasMoreElements()) {
				String byteStr = st.nextToken();
				outputStrem.write(Integer.parseInt(byteStr, 16));
			}
			patternfound = true;
		} 
		if(patternfound == false){
			throw new RuntimeException("ERROR: NON OF THE DATA PATTERN MATCHES - CHECK SEPARATION OF DATA OCTETS - DATA RECEIVED ["+datauntrimmed+"]");
		}
		return outputStrem.getIntArray();
	}
	
	
	
	/**
	 * Generate Frame Type 10 
	 * ZigBee TX request as described in P50 / chapter 7 / api operations
	 * @param int[] longbitaddr - fixed Xbee address
	 * @param int[] shortbitaddr - dynamic address 
	 * @param int[] data payloa
	 * @return
	 */
	public static synchronized int[] assembleTXRequest(int[] longbitaddr, int[]shortbitaddr, int[]data){
		logger.debug("---> assembleTXRequest("+ByteUtils.toBase16(longbitaddr)+", "+ByteUtils.toBase16(shortbitaddr)+", "+ByteUtils.toBase16(data)+")");
		int[] frameheader = {0x7E};
		int[] length = {0x00, 0x00};
		int[] apiframetype = {0x10};
		int[] frameid = {0x01};
		int[] bradcast = {0x00};
		int[] option = {0x00};
		
		if(longbitaddr.length != 8){
			throw new RuntimeException("ERROR: XBEE 64 BIT ADDRESS MusT BE 8 Bytes LONg - BUT IS ["+longbitaddr+"]");
		}
		if(shortbitaddr.length != 2){
			throw new RuntimeException("ERROR: XBEE 16 BIT ADDRESS MusT BE 2 Bytes LONg - BUT IS ["+shortbitaddr+"]");
		}
		int len = apiframetype.length + frameid.length +longbitaddr.length + shortbitaddr.length + bradcast.length + option.length + data.length;

		if(len > 255){
			throw new RuntimeException("ERROR: LENGTH ["+len+"] IS > 255 -- NOT POSSILBE - MAKE DATA SMALLER OR ADJUST ALGORITHM FOR MSB ");
		}
		
		int[] calclength = {0, len};
//		int[] calclength = ByteUtils.stringToIntArray(ut);
		
		
		
		if(calclength.length == 1){
			length[1] = calclength[0];
		} else if(calclength.length == 2){
			length[0] = calclength[0];
			length[1] = calclength[1];
		} else {
			throw new RuntimeException("ERROR: THE CALCULATED LENGTH Of THE TRANSMIT PACKAGE IS NOT CORRECT - CHECK RESULT ["+ByteUtils.toString(calclength)+"] - EXPECTED IS 1 or 2 BYTE LENGTH BUT NOT LENGTH= ["+calclength.length+"]");
			}
		
		
		// assemble the whole frame
		IntArrayOutputStream outputStrem = new IntArrayOutputStream();
		outputStrem.write(frameheader );
		outputStrem.write( length );
		outputStrem.write( apiframetype );
		outputStrem.write( frameid );
		outputStrem.write( longbitaddr );
		outputStrem.write( shortbitaddr );
		outputStrem.write( bradcast );
		outputStrem.write( option );
		outputStrem.write( data );

		int [] halfcompleteframe = outputStrem.getIntArray();	// no checksum yet
		
		
		Checksum ck = new Checksum();
		for(int i = 3; i < halfcompleteframe.length; i++){
			ck.addByte(halfcompleteframe[i]);
		}
		ck.compute();
		ck.verify();

		outputStrem.write( ck.getChecksum() );
		int [] completeframe = outputStrem.getIntArray();		// with the checksum
		int [] retval = completeframe;
		
		
		logger.debug("PACKAGE TO SEND: "+ByteUtils.toBase16(completeframe));
		logger.debug("<--- assembleTXRequest("+ByteUtils.toBase16(longbitaddr)+", "+ByteUtils.toBase16(shortbitaddr)+", "+ByteUtils.toBase16(data)+")");
		return retval; 
	}
}
