/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.xbee.dto;

import java.io.Serializable;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import com.moneyhouse.tmor.serial.xbee.XBeeDataQueueSingleton;

/**
 * CLASS CONTAINING ALL THE INFORMATION OF thE SERIAL QUEUES FOR DISPLAY TO THE USER
 * @author Frederic
 * @since 5.11.2015
 */

public class JSONQueueContainer implements Serializable{
	private static final long serialVersionUID = 6980055689014747696L;

	private ToBrickFileQueueElementDataObject[] toBrickDataList = null;						// Files comming to the toBrick Dir are registered here
	private ToBrickErrorFileQueueElementDataObject[] toBrickErrorDataList = null;				// Files with result in a XBee Error are copied into this queue
	private int countToBrick = -1;						// zaehlt die Elemente in der toBrickDataList 
	private int countErrors  = -1;						// zaehlt die Elemente in der toBrickErrorDataList 
	private String timeStampOfSendingStr  = "";			// wann wurde the heartbeat abgeschickt als STR
	private Timestamp timeStampOfSending  = null;			// wann wurde the heartbeat abgeschickt
	private long timeStampOfSendingLong  = -1;				// wann wurde the heartbeat abgeschickt als long
	
	
    private ToBrickFileQueueElementDataObject[] createArrayToBrick(int increaseBy)  
    {  
    	ToBrickFileQueueElementDataObject[] newArray = new ToBrickFileQueueElementDataObject[increaseBy];  
        return newArray;  
    }

    private ToBrickErrorFileQueueElementDataObject[] createArrayToErrorDataList(int increaseBy)  
    {  
    	ToBrickErrorFileQueueElementDataObject[] newArray = new ToBrickErrorFileQueueElementDataObject[increaseBy];  
        return newArray;  
    }

    
	
	public JSONQueueContainer(){}
	
	/**
	 * MAPS THE WHOLE QUEUE STRUCTURE INTO A JSON COMPATIBLE DATA STRUCUTRE 
	 * @param toBrickQueue
	 */
	public JSONQueueContainer(XBeeDataQueueSingleton toBrickQueue){

		int length = toBrickQueue.obtainToBrickDataList().size();
		toBrickDataList = createArrayToBrick(length);

		for(int i = 0; i < length; i++){
			toBrickDataList[i] = toBrickQueue.obtainToBrickDataList().get(i);
		}
		
		length = toBrickQueue.obtainToBrickErrorDataList().size();
		toBrickErrorDataList = createArrayToErrorDataList(length);

		for(int i = 0; i < length; i++){
			toBrickErrorDataList[i] = toBrickQueue.obtainToBrickErrorDataList().get(i);
		}
		
	
		
		
		this.countToBrick = toBrickQueue.countToBrickDataList();
		this.countErrors = toBrickQueue.obtainToBrickErrorDataList().size();
		
		
	}
	
	
	
	
	
	
	
	
	

	public ToBrickFileQueueElementDataObject[] getToBrickDataList() {
		return toBrickDataList;
	}

	public void setToBrickDataList(
			ToBrickFileQueueElementDataObject[] toBrickDataList) {
		this.toBrickDataList = toBrickDataList;
	}

	public ToBrickErrorFileQueueElementDataObject[] getToBrickErrorDataList() {
		return toBrickErrorDataList;
	}

	public void setToBrickErrorDataList(
			ToBrickErrorFileQueueElementDataObject[] toBrickErrorDataList) {
		this.toBrickErrorDataList = toBrickErrorDataList;
	}

	@Deprecated
	public void setTimeStampOfSendingStr(String timeStampOfSendingStr) {
		this.timeStampOfSendingStr = timeStampOfSendingStr;
	}

	@Deprecated
	public void setTimeStampOfSendingLong(long timeStampOfSendingLong) {
		this.timeStampOfSendingLong = timeStampOfSendingLong;
	}

	public int getCountToBrick() {
		return countToBrick;
	}
	public void setCountToBrick(int countToBrick) {
		this.countToBrick = countToBrick;
	}
	public int getCountErrors() {
		return countErrors;
	}
	public void setCountErrors(int countErrors) {
		this.countErrors = countErrors;
	}

	

	
	
	public String getTimeStampOfSendingStr() {
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.mmm");
		this.timeStampOfSendingStr = dateFormat.format(getTimeStampOfSending()) ;
		return timeStampOfSendingStr;
	}
	public void setTimeStampOfSendingStr() {
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.mmm");
		this.timeStampOfSendingStr = dateFormat.format(getTimeStampOfSending()) ;
	}

	
	
	
	
	
	
	public Timestamp getTimeStampOfSending() {
		return timeStampOfSending;
	}
	public void setTimeStampOfSending(Timestamp timeStampOfSending) {
		this.timeStampOfSending = timeStampOfSending;
		this.timeStampOfSendingLong = timeStampOfSending.getTime();
		setTimeStampOfSendingStr();

	}
	public long getTimeStampOfSendingLong() {
		this.timeStampOfSendingLong = timeStampOfSending.getTime();
		return   this.timeStampOfSendingLong;
	}
	public void setTimeStampOfSendingLong() {
		this.timeStampOfSendingLong = this.timeStampOfSending.getTime();
	}
	
	
	
}
