/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.xbee.dto;

import java.sql.Timestamp;


/**
 * @author Frederic
 * @since 5.11.2015
 */
public class JSONXBeeCommandHeartBeatObject extends XBeeComandResultObject{
	private static final long serialVersionUID = 708306931039418282L;
	// XBeeComandObject FIELDS
	private String brickid = "_";					// BrickUniqueID or Wildcard such as *
	private String atcommand = "_";					// XBee command such as ATID
	private String atcommandparameter = "_";		// Parameter that goes with the command
	private String correlID = "";					// correlation Id to match request with response
	private Timestamp timestamp = null;				// Timestamp of command call
	private String timestampLongStr = "";			// Timestamp as number L:ong in String form

	private String serialUptimeLongString = "";				// starttime of the SerialReceiver
	private String serialUptimeString = "";					// uptime in this fomr 151 days 21:11:49
	private String serialStartTimeLongString = "";			// Start time in ms

	
	// THE FIELDS FROM XBeeComandResultObject
	private String commandCorrelID = "";					// correlation Id to match request with response
	private Timestamp commandTimestamp = null;				// Timestamp of command execution
	private XBeeComandObject command = null;
	private boolean isResponseOK = false;			// from Xbee Response
	private boolean isResponseError = false;		// from Xbee Response
	private String responseValue = "";				// from Xbee Response
	private long executiontime = -1;				// from Xbee Response
	private String executiontimeStr = "-1";				// from Xbee Response
	private long executiontimesec = -1;				// from Xbee Response
	private String executiontimesecStr = "";
	private String xbeeResultAsString = "";			// its the xbeeresult obj as string added on the 20150914


	// THE FIELDS FROM JSONQueueContainer
	private String errorQueueCountStr = "";
	private String toBrickCountStr = "";			// its the xbeeresult obj as string added on the 20150914

	
	private JSONQueueContainer jsonQueueContainer = null;
	
	public JSONXBeeCommandHeartBeatObject(XBeeComandObject command){
		brickid = command.getBrickid();											// BrickUniqueID or Wildcard such as *
		atcommand = command.getAtcommand();										// XBee command such as ATID
		atcommandparameter = command.getAtcommandparameter();					// Parameter that goes with the command
		correlID = command.getCorrelID();										// correlation Id to match request with response
		timestamp = command.getTimestamp();										// Timestamp of command call
		timestampLongStr = "" + command.getTimestamp().getTime();				// Timestamp as number L:ong in String form

	}
	
	public JSONXBeeCommandHeartBeatObject(XBeeComandResultObject res, JSONQueueContainer queuese){
		this(res.getCommand());
		
		
		commandCorrelID = res.getCorrelID();					// correlation Id to match request with response
		commandTimestamp = res.getTimestamp();				// Timestamp of command execution
		command = res.getCommand();
		isResponseOK = res.isResponseOK();					// from Xbee Response
		isResponseError = res.isResponseError();				// from Xbee Response
		responseValue = res.getResponseValue();						// from Xbee Response
		executiontime = res.getExecutiontime();						// from Xbee Response
		executiontimeStr = ""+ res.getExecutiontime();				// from Xbee Response
		executiontimesec = res.getExecutiontimesec();						// from Xbee Response
		executiontimesecStr = ""+ res.getExecutiontimesec();
		xbeeResultAsString = res.getXbeeResultAsString();
		
		
		this.jsonQueueContainer = queuese;
		errorQueueCountStr = ""+ queuese.getCountErrors();
		toBrickCountStr = "" + queuese.getCountToBrick();			// its the xbeeresult obj as string added on the 20150914
		
	}

	
	
	
	
	public String getSerialUptimeLongString() {
		return serialUptimeLongString;
	}

	public void setSerialUptimeLongString(String serialUptimeLongString) {
		this.serialUptimeLongString = serialUptimeLongString;
	}

	public String getSerialUptimeString() {
		return serialUptimeString;
	}

	public void setSerialUptimeString(String serialUptimeString) {
		this.serialUptimeString = serialUptimeString;
	}

	public JSONQueueContainer getJsonQueueContainer() {
		return jsonQueueContainer;
	}

	public void setJsonQueueContainer(JSONQueueContainer jsonQueueContainer) {
		this.jsonQueueContainer = jsonQueueContainer;
	}

	public String getBrickid() {
		return brickid;
	}

	public void setBrickid(String brickid) {
		this.brickid = brickid;
	}

	public String getAtcommand() {
		return atcommand;
	}

	public void setAtcommand(String atcommand) {
		this.atcommand = atcommand;
	}

	public String getAtcommandparameter() {
		return atcommandparameter;
	}

	public void setAtcommandparameter(String atcommandparameter) {
		this.atcommandparameter = atcommandparameter;
	}

	public String getCorrelID() {
		return correlID;
	}

	public void setCorrelID(String correlID) {
		this.correlID = correlID;
	}

	public Timestamp getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(Timestamp timestamp) {
		this.timestamp = timestamp;
	}

	public String getTimestampLongStr() {
		return timestampLongStr;
	}

	public void setTimestampLongStr(String timestampLongStr) {
		this.timestampLongStr = timestampLongStr;
	}

	public String getCommandCorrelID() {
		return commandCorrelID;
	}

	public void setCommandCorrelID(String commandCorrelID) {
		this.commandCorrelID = commandCorrelID;
	}

	public Timestamp getCommandTimestamp() {
		return commandTimestamp;
	}

	public void setCommandTimestamp(Timestamp commandTimestamp) {
		this.commandTimestamp = commandTimestamp;
	}

	public XBeeComandObject getCommand() {
		return command;
	}

	public void setCommand(XBeeComandObject command) {
		this.command = command;
	}

	public boolean isResponseOK() {
		return isResponseOK;
	}

	public void setResponseOK(boolean isResponseOK) {
		this.isResponseOK = isResponseOK;
	}

	public boolean isResponseError() {
		return isResponseError;
	}

	public void setResponseError(boolean isResponseError) {
		this.isResponseError = isResponseError;
	}

	public String getResponseValue() {
		return responseValue;
	}

	public void setResponseValue(String responseValue) {
		this.responseValue = responseValue;
	}

	public long getExecutiontime() {
		return executiontime;
	}

	public void setExecutiontime(long executiontime) {
		this.executiontime = executiontime;
	}

	public String getExecutiontimeStr() {
		return executiontimeStr;
	}

	public void setExecutiontimeStr(String executiontimeStr) {
		this.executiontimeStr = executiontimeStr;
	}

	public long getExecutiontimesec() {
		return executiontimesec;
	}

	public void setExecutiontimesec(long executiontimesec) {
		this.executiontimesec = executiontimesec;
	}

	public String getExecutiontimesecStr() {
		return executiontimesecStr;
	}

	public void setExecutiontimesecStr(String executiontimesecStr) {
		this.executiontimesecStr = executiontimesecStr;
	}

	public String getXbeeResultAsString() {
		return xbeeResultAsString;
	}

	public void setXbeeResultAsString(String xbeeResultAsString) {
		this.xbeeResultAsString = xbeeResultAsString;
	}

	public String getErrorQueueCountStr() {
		return errorQueueCountStr;
	}

	public void setErrorQueueCountStr(String errorQueueCountStr) {
		this.errorQueueCountStr = errorQueueCountStr;
	}

	public String getToBrickCountStr() {
		return toBrickCountStr;
	}

	public void setToBrickCountStr(String toBrickCountStr) {
		this.toBrickCountStr = toBrickCountStr;
	}

	public String getSerialStartTimeLongString() {
		return serialStartTimeLongString;
	}

	public void setSerialStartTimeLongString(String serialStartTimeLongString) {
		this.serialStartTimeLongString = serialStartTimeLongString;
	}


	
	
	
}
