/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.xbee.dto;

import java.io.File;
import java.io.Serializable;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import com.moneyhouse.sensors.config.CONTSTANTINTERFACE;

/** ELEMENTS FOR XBEE RECEIVING QUEUE - WITH ERROR MESSAGE 
 * HOLD THE FILENAME AND CREATION DATE AND ERROR CODE 
 * @author Frederic
 *
 */
public class ToBrickErrorFileQueueElementDataObject implements Serializable {
	private static final long serialVersionUID = 2630967300847831082L;

	
	private File filename = null;
	private Timestamp receivedTimestamp = null;
	private String receivedTimestampStr = "";
	private String errorMsg = "";
	private int readingCounter = 0;
	private String displayShortFrom = "";		// ERROR MSG +READING COUNTER + TIMESTAMP + CORRELID + TYPE BATCH_PULL/HEARTBEAT + ADDRESS 
	
	
	public ToBrickErrorFileQueueElementDataObject(){
		this.filename = null;
		receivedTimestamp = new Timestamp(System.currentTimeMillis());
		
	}

	/** COPY CONSTRUCTOR
	 * @param ToBrickFileQueueElementDataObject tbfqedo
	 * @param error message
	 */
	public ToBrickErrorFileQueueElementDataObject(ToBrickFileQueueElementDataObject tbfqedo, String errormsg){
		this.filename = tbfqedo.getFilename();
		receivedTimestamp = tbfqedo.getReceivedTimestamp();
		this.errorMsg = errormsg;
		this.readingCounter = tbfqedo.getReadingCounter();
	}	
	
	
	public File getFilename() {
		return filename;
	}

	public void setFilename(File filename) {
		this.filename = filename;
	}

	public Timestamp getReceivedTimestamp() {
		return receivedTimestamp;
	}

	public String getReceivedTimestampFormated() {
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.mmm");
		String formatted = dateFormat.format(getReceivedTimestamp()) ;
		return formatted; 
	}

	
	public void setReceivedTimestamp(Timestamp receivedTimestamp) {
		this.receivedTimestamp = receivedTimestamp;
	}
	
	public String toString(){
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.mmm");
		String formatted = dateFormat.format(getReceivedTimestamp()) ;
		String errormsg = "";
		if(this.errorMsg.equals(CONTSTANTINTERFACE.ADDRESS_NOT_FOUND)){
			errormsg = this.getErrorMsg();
		} 
		else if (this.errorMsg.equals(CONTSTANTINTERFACE.TOO_MANY_TRIES)){
			errormsg = this.getErrorMsg()+"\t";
		} else {
			// NOT SPECIALLY TREATED ERROR MSG
			errormsg = this.getErrorMsg();
		}
		return formatted + "\t"+ errormsg + "\tCOUNTER= "+ this.getReadingCounter()+"\t"   + this.getFilename().toString();
	}

	public String getErrorMsg() {
		return errorMsg;
	}

	public void setErrorMsg(String errorMsg) {
		this.errorMsg = errorMsg;
	}

	public int getReadingCounter() {
		return readingCounter;
	}

	public void setReadingCounter(int readingCounter) {
		this.readingCounter = readingCounter;
	}

	public String getReceivedTimestampStr() {
		DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.mmm");
		this.receivedTimestampStr = dateFormat.format(getReceivedTimestamp()) ;
		return receivedTimestampStr;
	}

	public void setReceivedTimestampStr(String receivedTimestampStr) {
		DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.mmm");
		this.receivedTimestampStr = dateFormat.format(getReceivedTimestamp()) ;
	}

	// ERROR MSG +READING COUNTER + TIMESTAMP + TYPE BATCH_PULL/HEARTBEAT + ADDRESS 
	public String getDisplayShortFrom() {
		String filmename = this.getFilename().getName();
		String correlId = "";
		String type = "UNKNOWN";
		if(filmename.contains(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_CALIBRATION)){
			type = "CALIBRATION";
			correlId = filmename.substring(filmename.indexOf("_CORR_") +6, filmename.indexOf("_CORR_") +14);
		} else if(filmename.contains(CONTSTANTINTERFACE.FILENAME_PART_FOR_COMMAND)){
			type = "COMMAND";
			correlId = filmename.substring(filmename.indexOf("_CORR_") +6, filmename.indexOf("_CORR_") +14);
			
		} else if(filmename.contains(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_HEARTBEAT)){
			type = "HEARTBEAT";
			correlId = filmename.substring(filmename.indexOf("_UID_") +6, filmename.indexOf("_UID_") +14);
			
		} else if(filmename.contains(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_BATCH_PULL)){
			type = "PULL";
			correlId = filmename.substring(filmename.indexOf("_CORR_") +6, filmename.indexOf("_CORR_") +14);
			
			
		}  else if(filmename.contains(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_CONFIG_ASSEMBLER)){
			type = "ASSEMBELR";
			correlId = filmename.substring(filmename.indexOf("_CORR_") +6, filmename.indexOf("_CORR_") +14);
			
			
		}  else {
			// THIS IS NOT KNOWN 
			throw new RuntimeException("ERROR: TYPE OF THE FILENAME ["+filmename+"] NOt KNOWN!!!");
		}
		String address = "";
		if(filmename.contains("0x00")){
			// YES IT GOES TO A BRICK
			address = filmename.substring(filmename.indexOf("0x00"), filmename.indexOf("0x00")+40);
			address = address.replace("0x00", "").replace(",0x", "").trim();
		} else if(filmename.contains("BASE")){
			// YES IT GOES TO THE BASESTATION
			address = CONTSTANTINTERFACE.MODULE_ADDRESS_BASESTATION;
		} else if(filmename.contains(CONTSTANTINTERFACE.FILENAME_PART_START_FOR_HEARTBEAT) && !filmename.contains("0x00")){
			address = "";
		}
		
		
		
		String strVal = "REASON: "+this.getErrorMsg() + "\tCOUNTER: " + this.getReadingCounter() + "\t" +this.getReceivedTimestampStr() +"\tCORRELID: " +correlId+"\tTYPE: " + type+"\tADDRESS: "+address;
		displayShortFrom = strVal;
		return displayShortFrom;
	}

	public void setDisplayShortFrom(String displayShortFrom) {
		this.displayShortFrom = displayShortFrom;
	}
	
	
}
