/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.xbee.dto;

import java.io.File;
import java.io.Serializable;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

/** ELEMENTS FOR XBEE RECEIVING QUEUE 
 * HOLD THE FILENAME AND CREATION DATE
 * @author Frederic
 *
 */
public class ToBrickFileQueueElementDataObject implements Serializable {
	private static final long serialVersionUID = 2630967300847831082L;

	
	private File filename = null;
	private Timestamp receivedTimestamp = null;
	private String receivedTimestampStr = "";			// STRING reprensentation of receivedTimestamp
	private int readingCounter = 0;

	
	
	public ToBrickFileQueueElementDataObject(){
		this.filename = null;
		receivedTimestamp = new Timestamp(System.currentTimeMillis());
		
	}

	/** TAKES NOW AS TIMESTAMP
	 * @param filen
	 */
	public ToBrickFileQueueElementDataObject(File filen){
		this.filename = filen;
		receivedTimestamp = new Timestamp(System.currentTimeMillis());
	}	
	
	
	public File getFilename() {
		return filename;
	}

	public void setFilename(File filename) {
		this.filename = filename;
	}

	public Timestamp getReceivedTimestamp() {
		return receivedTimestamp;
	}

	public String getReceivedTimestampFormated() {
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.mmm");
		String formatted = dateFormat.format(getReceivedTimestamp()) ;
		return formatted; 
	}

	
	public void setReceivedTimestamp(Timestamp receivedTimestamp) {
		this.receivedTimestamp = receivedTimestamp;
	}
	
	public String toString(){
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.mmm");
		String formatted = dateFormat.format(getReceivedTimestamp()) ;
		return formatted +"\tREADING COUNTER: " + getReadingCounter() +"\t" + this.getFilename().toString();
	}

	public int getReadingCounter() {
		return readingCounter;
	}

	public void setReadingCounter(int readingCounter) {
		this.readingCounter = readingCounter;
	}
	public void incrementReadingCounter() {
		this.readingCounter++;
	}
	
	public String getReceivedTimestampStr() {
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.mmm");
		this.receivedTimestampStr = dateFormat.format(getReceivedTimestamp()) ;
		return receivedTimestampStr;
	}

	public void setReceivedTimestampStr(String receivedTimestampStr) {
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.mmm");
		this.receivedTimestampStr = dateFormat.format(getReceivedTimestamp()) ;
	}

	
}
