/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.xbee.dto;

import java.io.Serializable;
import java.sql.Timestamp;

import com.moneyhouse.sensors.util.TimeStampProducer;

/**
 * HOLD RESULT AND ITS ORIGIN COMMAND
 * such as ATDT 1F
 * 
 * The _ character means NULL
 * 
 * @author QQ
 *
 */
/**
 * @author QQ
 *
 */
/**
 * @author Frederic
 *
 */
public class XBeeComandResultObject implements Serializable{
	private static final long serialVersionUID = 2191663022615962087L;
	private String correlID = "";					// correlation Id to match request with response
	private Timestamp timestamp = null;				// Timestamp of command execution
	private XBeeComandObject command = null;
	private boolean isResponseOK = false;			// from Xbee Response
	private boolean isResponseError = false;		// from Xbee Response
	private String responseValue = "";				// from Xbee Response
	private long executiontime = -1;				// from Xbee Response
	private long executiontimesec = -1;				// from Xbee Response
	private String xbeeResultAsString = "";			// its the xbeeresult obj as string added on the 20150914

	
	
	public XBeeComandResultObject(){}
	
	public XBeeComandResultObject(XBeeComandObject command) {
		this.command = command;
		this.correlID = command.getCorrelID();
		
	}
	
	
	public XBeeComandResultObject(XBeeComandResultObject copy) {
		this.command = copy.getCommand();
		this.correlID = copy.getCorrelID();
		this.executiontime = copy.getExecutiontime();
		this.executiontimesec = copy.getExecutiontimesec();
		this.isResponseError = copy.isResponseError();
		this.isResponseOK = copy.isResponseOK();
		this.responseValue = copy.getResponseValue();
		this.timestamp = copy.getTimestamp();
		this.xbeeResultAsString = copy.getXbeeResultAsString();
		
		

	}

	
	
	
	
	public String getCorrelID() {
		return correlID;
	}
	public void setCorrelID(String correlID) {
		this.correlID = correlID;
	}

	public long calcExecutionTimeInSeconds(){
		long retval = 0;
		long duration = calcExecutionTimeInMillis();
		if(duration == -1){
			retval = -1;
		} else {
			retval = duration/1000;
		}
		return retval;
	}
	
	
	
	public long calcExecutionTimeInMillis(){
		long retval = 0;
		Timestamp start = command.getTimestamp();
		Timestamp end = getTimestamp();
		
		if(start == null || end == null){
			retval = -1;
		}
		else {
			long startlong = start.getTime();
			long endlong = end.getTime();
			retval = endlong - startlong;
		}
		this.executiontime = retval;
		return retval;
	}
	
	
	public Timestamp getTimestamp() {
		return timestamp;
	}
	
	public void setTimestampNow() {
		this.timestamp = TimeStampProducer.createTimestampForTimeZone("UTC");
	}
	
	public void setTimestamp(Timestamp timestamp) {
		this.timestamp = timestamp;
	}
	public XBeeComandObject getCommand() {
		return command;
	}
	public void setCommand(XBeeComandObject command) {
		this.command = command;
	}
	public boolean isResponseOK() {
		return isResponseOK;
	}
	public void setResponseOK(boolean isResponseOK) {
		if(isResponseOK == true){
			this.isResponseOK = true;
			this.isResponseError = false;
		} else {
			this.isResponseOK = false;
			this.isResponseError = true;
			
		}
	}
	public boolean isResponseError() {
		return isResponseError;
	}
	public void setResponseError(boolean isResponseError) {
		if(isResponseError == true){
			this.isResponseOK = false;
			this.isResponseError = true;
		} else {
			this.isResponseOK = true;
			this.isResponseError = false;
		}
	}
	public String getResponseValue() {
		return responseValue;
	}
	public void setResponseValue(String responseValue) {
		this.responseValue = responseValue;
	}


	public long getExecutiontime() {
		executiontime = calcExecutionTimeInMillis();
		return executiontime;
	}


	public void setExecutiontime(long executiontime) {
		this.executiontime = executiontime;
	}


	public long getExecutiontimesec() {
		executiontimesec = executiontime / 1000;
		return executiontimesec;
	}


	public void setExecutiontimesec(long executiontimesec) {
		this.executiontimesec = executiontimesec;
	}


	public String getXbeeResultAsString() {
		return xbeeResultAsString;
	}


	public void setXbeeResultAsString(String xbeeResultAsString) {
		if(xbeeResultAsString == null){
			xbeeResultAsString = "";
		}
		this.xbeeResultAsString = xbeeResultAsString;
	}
	
	public String toString(){
		String retVal = "XBeeComandResultObject";
		retVal += "\tCORREL_ID: "+ this.correlID;
		retVal += "\tXBEE_RESULT: "+ this.xbeeResultAsString;
		
		return retVal;
	}

	
	
	
}
