/**
 * Copyright (c) 2016 Money House Limited Qian Zhuang 88 All rights reserved.
 *  
 * This file is part of TMOR system, is free software and comes with no WARRANTY
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM, 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
 * THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY 
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK 
 * AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
 * DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 
 * 
 * This program is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software Foundation; 
 * version 3 of the License, or (at your option) any later version. 
 * 
 * TMOR is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License along with TMOR.  
 * If not, see <http://www.gnu.org/licenses/>.
 */
package com.moneyhouse.tmor.serial.xbee.dto;

import java.sql.Timestamp;
import java.util.HashMap;

import com.moneyhouse.sensors.util.TimeStampProducer;

/**
 * HOLDS A FILE CONTENT FORM THE FOLDERS TOBRICK - FROMBRICK AS AN OBJECT
 * IT MIGHT BE, THAT tHE CHECKSUM IS NOT NEEDED AS XBEE PROTOCOLL CARES FOR IT
 * @since 31.10.2014
 * @author Frederic
 *
 */
public class XBeeDataTransportObject {
	private String uniqueId;
	private String brickstuniqueId;
	private String command;					// like sendPING
	private String readingtype;				// TEMP, SWITCH
	private String readingvalue;			// 23,-10, ON, OFF etc.
	private String readingport;				// A1, A2... , O-WIRE ADDRESS, OR WHATEVER

	private String correlid;				// CORRELATION ID - TO MATCH REQ WITH RESPONSE
	private Timestamp createdtimestamp;
	private String checksum; 			// 0 brickdata ; 1 manual; -1 simulated
	private long executiontime = 0 ;
	private HashMap<String, String> additionalParamters = new HashMap<String, String>();	// ADDED FOR ADDITIONAL PARAMTERS SUCH AS WRITE RULE
	
	
	
	public void storeAdditionalParameter(String key, String value){
		additionalParamters.put(key, value);
	}
	
	public String getAdditionalParameter(String key){
		return additionalParamters.get(key);
	}
	
	public XBeeDataTransportObject(){
		
	}
	
	
	public XBeeDataTransportObject(String readingport){
		this.setReadingport(readingport);
	}
	
	/**
	 * FOR COMPLEX FILE CONTENT - SUCH AS requestDATA - or EVENT
	 * @param brickstuniqueId
	 * @param command
	 * @param readingport
	 * @param readingtype
	 * @param readingtvalue
	 * @param correlid
	 * @param createdtimestamp
	 */
	public XBeeDataTransportObject(String brickstuniqueId, String command, String readingport, String readingtype , String readingtval, String correlid, Timestamp createdtimestamp) {
		this.setBrickstuniqueId(brickstuniqueId);
		this.setCreatedtimestamp(createdtimestamp);
		this.setReadingtype(readingtype);
		this.setReadingvalue(readingtval);
		this.setCorrelid(correlid);
		this.setCommand(command);
		this.setReadingport(readingport);
	}	
	
	
	
	
	
	
	
	
	
	
	
	
	/**
	 * FOR COMPLEX FILE CONTENT - SUCH AS requestDATA
	 * @param brickstuniqueId
	 * @param readingtype
	 * @param readingtvalue
	 * @param correlid
	 * @param createdtimestamp
	 */
	public XBeeDataTransportObject(String brickstuniqueId, String readingtype , String readingtvalue, String correlid, String createdtimestamp) {
		Timestamp ts = Timestamp.valueOf(TimeStampProducer.checkAndModifyTimeStamp(createdtimestamp));
		long fractions = getFractionalSeconds(createdtimestamp);
		long tstime = ts.getTime();
		tstime += fractions;
		this.setBrickstuniqueId(brickstuniqueId);
		this.setCreatedtimestamp(new Timestamp(tstime));
		this.setReadingtype(readingtype);
		this.setReadingvalue(readingvalue);
		this.setCorrelid(correlid);
	}

	/**
	 * FOR SIMPLE FILE CONTENT - SUCH AS sendPING
	 * @param brickstuniqueId
	 * @param command
	 * @param correlid
	 * @param createdtimestamp
	 */
	public XBeeDataTransportObject(String brickstuniqueId, String command, String correlid, String createdtimestamp) {
		Timestamp ts = Timestamp.valueOf(TimeStampProducer.checkAndModifyTimeStamp(createdtimestamp));
		long fractions = getFractionalSeconds(createdtimestamp);
		long tstime = ts.getTime();
		tstime += fractions;

		this.setBrickstuniqueId(brickstuniqueId);
		this.setCreatedtimestamp(new Timestamp(tstime));
		this.setCommand(command);
		this.setCorrelid(correlid);
	}
	

	
	
	/** Gets the ms from the String timestamp
	 * @param ts timestamp
	 * @return
	 */
	private long getFractionalSeconds(String ts){
		long retVal = 0;
		if(ts.contains(".")){
			// YES - WE HAVE ms - GET RID OF THEM
			if(ts.charAt(ts.length()-4) == '.' && ts.length() == 23){
				String ms = ts.substring(ts.length()-3, ts.length());
				retVal = new Long(ms).longValue();
			} 
			else if(ts.charAt(ts.length()-3) == '.' && ts.length() == 22){
				String ms = ts.substring(ts.length()-2, ts.length());
				retVal = new Long(ms).longValue();
			} 
			else if(ts.charAt(ts.length()-2) == '.' && ts.length() == 21){
				String ms = ts.substring(ts.length()-1, ts.length());
				retVal = new Long(ms).longValue();
			} 
			
			else{
				// UNEXPECTED timestamp format THROW EXPECTION 
				throw new RuntimeException("ERROR: TIMESTAMP ["+ts+"] EXPECTED TO BE IN FORMAT [2014-10-31 08:29:25.814] BUT IS NOT");
			}
			
		} else {
			// NO - WE DO NOT HAVE ms
			retVal = 0;
		}
		
		return retVal;
	}
	
	
	
	/**
	 * FROMAT OF DATAGRAM:
	 * # START
	 * ^ END
	 * @ DELEMETERS
	 * CORREL 	= 8 charaters
	 * BRICKID	= 17 charaters
	 * TYPE		
	 * CHECKSUM
	 * 
	 * #@CORREL@BRICKID@CHECKSUM@^
	 * #@
	 * 
	 * @return String to pass to brick via Xbee
	 */
	public String generateDataGram(){
		String retval = "#";
		retval = retval + "@" + this.getCorrelid();
		retval = retval + "@" + this.getBrickstuniqueId();
		retval = retval + "@" + this.getReadingtype();
		

		long checksum = 10000019;
		for(int index = 0; index < retval.length(); index++){
//			checksum += 100003 * (int)retval.charAt(index);
			checksum += multiply(retval.charAt(index));
		}			 // 7457
		checksum += (int) '@';
		checksum += (int) '^';
		retval = retval + "@" + checksum + "@^";
		
		this.setChecksum(""+checksum);
		
		return retval;
	}
	
	private long multiply(char a){
		long retval = 0;
		if((int)a < 70){
			retval = (int)a * 139;
		} else if((int)a < 75){
			retval = (int)a * 4517;
			
		} else if((int)a < 85){
			retval = (int)a * 7001;
			
		} else {
			retval = 100003 * a;
		}
		
		return retval;
	}
	
	public String getUniqueId() {
		return uniqueId;
	}

	public void setUniqueId(String uniqueId) {
		this.uniqueId = uniqueId;
	}

	public String getBrickstuniqueId() {
		return brickstuniqueId;
	}

	public void setBrickstuniqueId(String brickstuniqueId) {
		this.brickstuniqueId = brickstuniqueId;
	}

	public String getReadingtype() {
		return readingtype;
	}

	public void setReadingtype(String readingtype) {
		this.readingtype = readingtype;
	}


	public Timestamp getCreatedtimestamp() {
		return createdtimestamp;
	}

	public void setCreatedtimestamp(Timestamp createdtimestamp) {
		this.createdtimestamp = createdtimestamp;
	}


	public String getCorrelid() {
		return correlid;
	}

	public void setCorrelid(String correlid) {
		this.correlid = correlid;
	}

	public String getChecksum() {
		return checksum;
	}

	public void setChecksum(String checksum) {
		this.checksum = checksum;
	}
	
	public String getReadingvalue() {
		return readingvalue;
	}

	public void setReadingvalue(String readingvalue) {
		this.readingvalue = readingvalue;
	}

	public String getCommand() {
		return command;
	}

	public void setCommand(String command) {
		this.command = command;
	}

	public long getExecutiontime() {
		return executiontime;
	}

	public void setExecutiontime(long executiontime) {
		this.executiontime = executiontime;
	}












	public String getReadingport() {
		return readingport;
	}












	public void setReadingport(String readingport) {
		this.readingport = readingport;
	}

	
	
}
