package com.moneyhouse.tmor.serial.xbee.flag;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.sql.Timestamp;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.util.TimeStampProducer;

/**
 * READS THE FILE xbee_status.log LOCATED IN PROPERTY xbeestatusfiles.dir defined in tmor.propoerties
 * IF FILE DOES NOT EXIST IT IS CREATED - WITH THE 4 LINES OF HEADER
 * @author QQ
 * @since 2017.06.29
 *
 */
public class XBeeStatusLogReader {
	private final static Logger logger = Logger.getLogger(XBeeStatusLogReader.class);
	private final static int TIME_WINDOW_FOR_COUNT = 25;		// HOW LONG TO GO BACK IN TIME AND COUNT OCCURENCES
	public static void main(String[] args) {
		
		
		readAndCountXBeeStatusLog();
		
	}

	/**
	 * READ LOG FILE AND COUNT THE RETRIES (FAILURES TO CONNECT TO RADIO MODULE)
	 * WITHIN A TIME DURATION OF 30 min OR SINCE THE LAST SUCCESSFUL START
	 */
	public static int readAndCountXBeeStatusLog() {
		logger.info("---> readAndCountXBeeStatusLog()");
		ArrayList<XBeeStatusLineDTO> rebootTimeStampsAsOBJ = readXBeeStatusLogFile();
		
		
		// GET THE CURRENt TIMESTMAP WITH TIMEZONE FROM TOMR.PROPERTIES
		
		// DOES NOT WORK ON LINUX
		// String timeZone = PropertiesFileReader.getInstance().getProperty("timezone");
		// Timestamp  now = currentTime(timeZone);
		
		// TIME ZONE FROM tmor.properties
		String timezone = PropertiesFileReader.getInstance().getProperty("timezone");
		Timestamp  now = TimeStampProducer.createTimestampForTimeZone(timezone);
		
		Timestamp  utcnow = new Timestamp(System.currentTimeMillis());
		logger.warn(now + "\tTIMESTAMP FOR TIMEZONE ["+timezone+"]");
		logger.warn(utcnow + "\tTIMESTAMP UTC TIME ");
		

		
		
		Timestamp  start = calculateStartTime(now);
		int counts = countRetriesWithinStartToNow(start, now, rebootTimeStampsAsOBJ);
		logger.info("<--- readAndCountXBeeStatusLog() counts= " + counts);
		return counts;
	}

	/**
	 * READS THE LOG FILE xbee_status.log ANd RETURNS A LIST OF OBJs
	 * IF FILE DOES NOT EXIST - IT WILL BE CREATED
	 * @return
	 */
	public static ArrayList<XBeeStatusLineDTO> readXBeeStatusLogFile() {
		ArrayList<String> rebootTimeStamps = new ArrayList<String>();
		File file = null;
		String filePlusPath = null;
		try{
			// FILE TO WRITE TO
			String dir = PropertiesFileReader.getInstance().getProperty("xbeestatusfiles.dir");
			String filename = "xbee_status.log";
			filePlusPath = dir + System.getProperty("file.separator")+filename;
			file = new File(filePlusPath);
			
			BufferedReader br = new BufferedReader(new FileReader(file));
		    String line;
		    while ((line = br.readLine()) != null) {
		       // process the line.
		    	rebootTimeStamps.add(line);
		    }
		    br.close();
		} catch(FileNotFoundException ex){
			logger.warn("WARN: THE LOG FILE ["+filePlusPath+"] CAN NOT BE FOUND - IT MAy NOT EXIST - WE CREATE IT NOW.");

			try {
				file.createNewFile();
				logger.info("CREATE NEW LOG FILE ["+filePlusPath+"]");
	        	// HERE WE WIRTE THE TIMESTAMP TO THE END OF THE FILE
	            Files.write(Paths.get(filePlusPath), ("THIS IT THE XBEE RADIO MODULE LOAD LOG FILE - CHECK xbee_serial.log FOR ERRORS\n").getBytes(), StandardOpenOption.APPEND);
	            Files.write(Paths.get(filePlusPath), ("THIS FILE RECORDS THE XBEE CONNECTION TRYS - PROPERTY xbeeconnectionattemps DETERMINES HOW\n").getBytes(), StandardOpenOption.APPEND);
	            Files.write(Paths.get(filePlusPath), ("MANY TIMES THE SYSTEM REBOOTS UNTIL SERIAL CONNECTION IS ESTABLISHED\n").getBytes(), StandardOpenOption.APPEND);
	            Files.write(Paths.get(filePlusPath), ("EXISTANCE OF FILE xbee_status_connected PROHIBITS SYSTEM FROM REBOOT\n").getBytes(), StandardOpenOption.APPEND);
	            
			}catch (IOException e) {
	            //exception handling left as an exercise for the reader
    			logger.error("ERROR: CANT CREATE NEW LOG FILE ["+filePlusPath+"]", e);
	        	e.printStackTrace();
	        }
		} catch(Exception ex){
			logger.error("ERROR: SOME EXCEPTION OCCURED WHILE READING LOG FILE ["+filePlusPath+"]", ex);
			ex.printStackTrace();
		}
		
		
		// FILL FILE CONTENT INTO OBJs
		ArrayList<XBeeStatusLineDTO> rebootTimeStampsAsOBJ = new ArrayList<XBeeStatusLineDTO>();
		for(int i= 0; i < rebootTimeStamps.size(); i++){
			System.out.println("PLAIN " + rebootTimeStamps.get(i));
			if(i > 3){
				rebootTimeStampsAsOBJ.add(new XBeeStatusLineDTO(rebootTimeStamps.get(i)));
			}
		}
		
		for(int i= 0; i < rebootTimeStampsAsOBJ.size(); i++){
			System.out.println("OBJ " + rebootTimeStampsAsOBJ.get(i));
			
		}
		return rebootTimeStampsAsOBJ;
	}
	
	/**
	 * STARTTIME IS 30 min BEFORE CURRENT TIME
	 * @param now
	 * @return
	 */
	private static Timestamp  calculateStartTime(Timestamp now){
		long nowMs = now.getTime();
		long thirtyMin = 1000 * 60 * TIME_WINDOW_FOR_COUNT;
		long startTimeMs = nowMs - thirtyMin;
		Timestamp startTs = new Timestamp(startTimeMs);
		return startTs;
		
	}
	
	
	/**
	 * COUNTS FAILED RECORDS BETWEEN START AND NOW
	 * @param now
	 * @return
	 */
	private static int countRetriesWithinStartToNow(Timestamp start, Timestamp now, ArrayList<XBeeStatusLineDTO> records){
		logger.debug("----> countRetriesWithinStartToNow("+start+", "+now+", "+records.size()+")");
		
		logger.debug("----------------------------------------------------------------------------------------------------------");
		for(int i = 0; i < records.size(); i++){
			logger.debug("["+i+"]\t" + records.get(i));
		}
		logger.debug("----------------------------------------------------------------------------------------------------------");
		
		
		
		int counter = 0;
		for(int i = records.size() -1; i > -1; i--){
			XBeeStatusLineDTO ele = records.get(i);
//			logger.debug("LOOP COUNT ["+i+"]\t" + ele +"\t\tSTART TIME ["+start.getTime()+"]\t\t ELEMENT ["+ele.getTime().getTime()+"]\t\t NOW["+now.getTime()+"]");

			if(ele.getTime().getTime() > start.getTime() && ele.getTime().getTime() <= now.getTime()){
//				logger.debug("["+ele+"] IS IN TIME");
				if(ele.isSuccessConnect()){
					break;
				} else {
					// COUNT THE UNSUCCESSFUL ONES	
					counter++;
				}
			} 
			else {
//				logger.debug("["+ele+"] IS NOT IN THE TIME FRAME");
				
			}
		}
		logger.debug("<---- countRetriesWithinStartToNow("+start+", "+now+", "+records.size()+") count= "+ counter);
		
		return counter;
	}
}