package com.moneyhouse.tmor.serial.xbee.flag;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

import org.apache.log4j.Logger;

import com.moneyhouse.sensors.config.PropertiesFileReader;
import com.moneyhouse.sensors.util.TimeStampProducer;

/**
 * WRTIES THE SUCCESSFUL CONNECT TP XBEE COORDINATOR INTO LOG FILE
 * WHEN SUCCESSFUL - WRITES FLAG FILE TO DIR 
 * WHEN NOT SUCCESSFUL NO FILe IS WRITTEN
 * USED IN CONJUNCTION WITH startup script
 * 
 * USES THE FOLLOWING CONFIG PARAMETERS FROM TMOR.PROPERTIES
 * 
 *  #XBEE STATUS LOG FILE
 *  xbeestatusfiles.dir.win=c\:\\temp
 *  xbeestatusfiles.dir.unix=/usr/share/xbee
 *  xbeeconnectionattempts=10
 *  
 *  
 * @since 20170623
 * @author QQ
 *
 */
public class XBeeStatusLogWriter {
	public final static String XBEE_CONNECT_STATUS_SUCCESS = "XBEE_CONNECT_STATUS_SUCCESS";
	public final static String XBEE_CONNECT_STATUS_FAIL = "XBEE_CONNECT_STATUS_FAIL";
	public final static String XBEE_CONNECT_STATUS_FAIL_FLAG_EXISTS = "XBEE_CONNECT_STATUS_FAIL_FF_EXISTS";
	public final static String XBEE_CONNECT_STATUS_IGNORE_FAIL = "XBEE_CONNECT_STATUS_IGNORE";
	private final static Logger logger = Logger.getLogger(XBeeStatusLogWriter.class);
	
	/**WRITES OR DELETES THE FILE xbee_status_connected AND
	 * LOGGS THE TRIES TO ThE xbee_status_log FILE
	 * @param status
	 */
	public static void rebootMechanism(String status) {
		// FILE TO WRITE TO
		String dir = PropertiesFileReader.getInstance().getProperty("xbeestatusfiles.dir");
		String filename = "xbee_status.log";
		String filePlusPath = dir + System.getProperty("file.separator")+filename;
		File file =new File(filePlusPath);
		
		// TIME ZONE FROM tmor.properties
		String timezone = PropertiesFileReader.getInstance().getProperty("timezone");
		
		//if file doesnt exists, then create it
		if(!file.exists()){
			try {
				file.createNewFile();
				logger.info("CREATE NEW LOG FILE ["+filePlusPath+"]");
	        	// HERE WE WIRTE THE TIMESTAMP TO THE END OF THE FILE
	            Files.write(Paths.get(filePlusPath), ("THIS IT THE XBEE RADIO MODULE LOAD LOG FILE - CHECK xbee_serial.log FOR ERRORS\n").getBytes(), StandardOpenOption.APPEND);
	            Files.write(Paths.get(filePlusPath), ("THIS FILE RECORDS THE XBEE CONNECTION TRYS - PROPERTY xbeeconnectionattemps DETERMINES HOW\n").getBytes(), StandardOpenOption.APPEND);
	            Files.write(Paths.get(filePlusPath), ("MANY TIMES THE SYSTEM REBOOTS UNTIL SERIAL CONNECTION IS ESTABLISHED\n").getBytes(), StandardOpenOption.APPEND);
	            Files.write(Paths.get(filePlusPath), ("EXISTANCE OF FILE xbee_status_connected PROHIBITS SYSTEM FROM REBOOT\n").getBytes(), StandardOpenOption.APPEND);
	        }catch (IOException e) {
	            //exception handling left as an exercise for the reader
    			logger.error("ERROR: CANT CREATE NEW LOG FILE ["+filePlusPath+"]", e);
	        	e.printStackTrace();
	        }
		}	
			
		
		
        String timeStamp = TimeStampProducer.createTimestampWithTimeZone(timezone,"yyyy-MM-dd HH:mm:ss.SSS");
	    
        try {
        	// HERE WE WIRTE THE TIMESTAMP TO THE END OF THE FILE
    		if(XBEE_CONNECT_STATUS_SUCCESS.equals(status)){
	            Files.write(Paths.get(filePlusPath), (timeStamp+"\tCONNECTED\n").getBytes(), StandardOpenOption.APPEND);
	            writeXBeeSuccessFlagFile(timeStamp);
    		} else if(XBEE_CONNECT_STATUS_FAIL.equals(status)){
    			Files.write(Paths.get(filePlusPath), (timeStamp+"\tFAILED TO CONNECT TO XBEE COORDINATOR -- RETRY BY REBOOT: SCRIPT /etc/init.d/xbee REBOOTS SYSTEM\n").getBytes(), StandardOpenOption.APPEND);
    			// COMMENTED OUT ON THE 02180312 22:15
    			// clearXBeeSuccessFlagFile();
    		} else if(XBEE_CONNECT_STATUS_IGNORE_FAIL.equals(status)){
    			Files.write(Paths.get(filePlusPath), (timeStamp+"\tFAILED TO CONNECT - MORE THAN ["+PropertiesFileReader.getInstance().getPropertyAsInt("xbeeconnectionattempts")+"] RETIRES - IGNORE AND CARRY ON WITHOUT BASE STATION XBEE RADIO MODULE\n").getBytes(), StandardOpenOption.APPEND);
    			writeXBeeSuccessFlagFile(timeStamp);
    		} else if(XBEE_CONNECT_STATUS_FAIL_FLAG_EXISTS.equals(status)){
    			Files.write(Paths.get(filePlusPath), (timeStamp+"\tFAILED TO CONNECT TO XBEE COORDINATOR -- BUT FLAGFILE xbee_status_connected EXISTS - WE CARRY ON WITHOUT BASE STATION XBEE RADIO MODULE\n").getBytes(), StandardOpenOption.APPEND);
    			writeXBeeSuccessFlagFile(timeStamp);
    			
    		} else{
    			logger.error("ERROR: VALUE ["+status+"] FOR STATUS PARAPMETER IS NOT SUPPORTED - CHECK CODE!!!");
    		}	        
        }catch (IOException e) {
            //exception handling left as an exercise for the reader
			logger.error("ERROR: CANT WRITE ENTRY INTO LOG FILE ["+filePlusPath+"]", e);
        	e.printStackTrace();
        }
 	}

	
	/**
	 * WRITES THE FILE xbee_status_connected INTO THE DIR xbeestatusfiles.dir
	 * @param timestamp
	 */
	public static void writeXBeeSuccessFlagFile(String timestamp) {
		// FILE TO WRITE TO
		String dir = PropertiesFileReader.getInstance().getProperty("xbeestatusfiles.dir");
		String filename = "xbee_status_connected";
		String filePlusPath = dir + System.getProperty("file.separator")+filename;
		File file =new File(filePlusPath);

		//if file doesnt exists, then create it
		if(!file.exists()){
			try {
				// CREATE FLAG FILE 
				file.createNewFile();
			} catch (IOException e) {
				logger.error("ERROR: CANT CREATE FLAG FILE ["+filePlusPath+"]", e);
	        	e.printStackTrace();			}
		}	
		try {
        	// HERE WE WIRTE THE TIMESTAMP TO THE END OF THE FILE
            Files.write(Paths.get(filePlusPath), ("SUCCESSFULLY CONNECTED TO XBEE COORDINATOR AT ["+timestamp+"]\n").getBytes(), StandardOpenOption.APPEND);
        }catch (IOException e) {
			logger.error("ERROR: CANT WRITE INTO FLAG FILE ["+filePlusPath+"]", e);
        	e.printStackTrace();			
        }
	}
	

	/**
	 * DELETES THE FILE xbee_status_connected IN DIR xbeestatusfiles.dir
	 * @param timestamp
	 */
	public static void clearXBeeSuccessFlagFile() {
		String filePlusPath = "";
		try {
			// FILE TO WRITE TO
			String dir = PropertiesFileReader.getInstance().getProperty("xbeestatusfiles.dir");
			String filename = "xbee_status_connected";
			filePlusPath = dir + System.getProperty("file.separator")+filename;
			File file =new File(filePlusPath);
	
			//if file doesnt exists, then create it
			if(!file.exists()){
				// IT FLAG FILE DOES NOT EXIST - SO NOTHING TO DO	
			} else {
				// DELETE FLAG FILE 
				file.delete();
			}	
        }catch (Exception e) {
			logger.error("ERROR: THERE WAS A PROBLEM DELETING FLAG FILE ["+filePlusPath+"]", e);
        	e.printStackTrace();			
        }
	}
	
	
	
	/**
	 * CHECKS IF THE FILE xbee_status_connected IS IN DIR xbeestatusfiles.dir
	 * @param timestamp
	 */
	public static boolean isXBeeSuccessFlagFileHere() {
		boolean retval = false;
		String filePlusPath = "";
		try {
			// FILE TO WRITE TO
			String dir = PropertiesFileReader.getInstance().getProperty("xbeestatusfiles.dir");
			String filename = "xbee_status_connected";
			filePlusPath = dir + System.getProperty("file.separator")+filename;
			File file =new File(filePlusPath);
	
			//if file doesnt exists, then create it
			if(file.exists() && !file.isDirectory()){
				// IT FLAG FILE EXISTS 
				retval = true;
			} else {
				// FLAG FILE DOES NOT EXIST
				retval = false;
			}	
        }catch (Exception e) {
			logger.error("ERROR: THERE WAS A PROBLEM CHECKING FOR FLAG FILE ["+filePlusPath+"]", e);
        	e.printStackTrace();			
        }
		return retval;
	}
}
