package com.moneyhouse.util.network.serverping;



import java.net.InetAddress;
import java.util.regex.Pattern;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import com.moneyhouse.psvn.util.timestamp.TimeStampProducer;

/**
 * PING A IP ADDRESS WHICH IS PASSED AS PPINGServerIP PARAMETER. NO PARAMETER - THE PROGRAM PINGS 192.168.1.1 BY DEFAULT.
 * 
 * IF THE PINGING OF PPINGServerIP PARAMETER (OR 192.168.1.1) IS NOT SUCCESSFULL, THEN WE PING 8.8.8.8 TO RULE OUT THAT THE LOCAL CONFIG GATEWAY IS NOT 192.168.1.1
 * 
 * THE TIME BETWEEN PINGS IS PASSED AS PPINGsleepS PARAMETER IN SECONDS. DEFAULT IS 30 SECONDS
 * 
 * SHORT DESCRIPTION HOWTO ADD IT TO RASPI/LINUX - THE JAR NEEDS TO BE IN THE home/pi FOLDER STRUCTURE - DOEs NOT WORK IN /usr/share...
 * 
 * 1a) nano /etc/rsyslog.conf
 * 1b)  Uncomment   
 * 1c) cron.*                          /var/log/cron.log
 * 
 * 2a) nano /etc/crontab
 * 2b) add line
 * 2c) 10 *    * * *   pi    /home/pi/pping.sh
 * 
 * 3) service rsyslog restart
 * 4) service cron restart
 * 5) tail -f /var/log/cron.log
 * 
 * 
 * 
 * 
 * THE SCRIPT pping.sh LOOKS LIKE THIS:
 * 
 * 
 * #!/bin/sh
 * java -DPPINGServerIP=192.168.1.12 -DPPINGsleepS=10 -classpath "./lib/log4j-core-2.17.1.jar:./lib/log4j-api-2.17.1.jar:./lib/TMOR_NETWORK.jar" com.moneyhouse.util.network.serverping.PPing
 * STATUS="${?}"
 *
 * case $STATUS in 
 * 0) echo "OK";;
 * 1) echo "NOT OK - REBOOT"  | sudo -S reboot;;
 * *) echo "NEITER";;
esac

:

 * 
 * 
 * 
 * @author Frederic
 *
 */
public class PPing {
	private static Logger log = LogManager.getLogger("ppinglogger");  
	
//	private static Logger log = LogManager.getLogger("ppinglogger");  
	private static final Pattern PATTERN = Pattern.compile("^(([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\.){3}([01]?\\d\\d?|2[0-4]\\d|25[0-5])$");
	public static void main(String[] args) {
		System.out.println("PPing Logging to ./logs/pping.log");
		String timeStp = TimeStampProducer.createTimestampWithTimeZoneUTC();
		
		String ip = System.getProperty("PPINGServerIP");
		String ipTarget = "";
		if(ip == null || ip.isEmpty()) {
//			System.out.println("PPING SERVER STARTED AT ["+timeStp+"] - PROPERTY PPINGServerIP NOT SET - WE USER IP ADDRESS 192.168.1.1");
			log.warn("PPING SERVER STARTED AT ["+timeStp+"] - PROPERTY PPINGServerIP NOT SET - WE USER IP ADDRESS 192.168.1.1");
			ipTarget = "192.168.1.1";
		} else {
			if(PATTERN.matcher(ip).matches()) {
//				System.out.println("PPING SERVER STARTED AT ["+timeStp+"] - PROPERTY PPINGServerIP SET TO ["+ip+"]");
				log.info("PPING SERVER STARTED AT ["+timeStp+"] - PROPERTY PPINGServerIP SET TO ["+ip+"]");
				ipTarget = ip;
				
			} else {
//				System.out.println("PPING SERVER STARTED AT ["+timeStp+"] - NO VALID PROPERTY FOR PPINGServerIP SET ["+ip+"] - WE USE 192.168.1.1");
				log.info("PPING SERVER STARTED AT ["+timeStp+"] - NO VALID PROPERTY FOR PPINGServerIP SET ["+ip+"] - WE USE 192.168.1.1");
				ipTarget = "192.168.1.1";
			}
		}
		
		
		int basicSleepTime = 30;
		String sleepInS = System.getProperty("PPINGsleepS");
		if(sleepInS == null || sleepInS.isEmpty()) {
//			System.out.println("PPING SLEEPING CONSTANT PPINGsleepS IS NOT SET - WE WILL USE 30 SECONDS. 1 PING 30 SEC WAIT 2. PING 1 MIN WAIT 3. PING 90 SEC WAIT 4. PING 2 MIN WAIT 5 PING");
		} else {
			try {
				basicSleepTime = new Integer(sleepInS).intValue();
				if(basicSleepTime < 1) {
//					System.out.println("PPING SLEEPING CONSTANT PPINGsleepS ["+sleepInS+"] NOT VALID - WE EXPECTED A NUMBER FROM 1 To 600  - WE WILL USE 30 SECONDS. 1 PING 30 SEC WAIT 2. PING 1 MIN WAIT 3. PING 90 SEC WAIT 4. PING 2 MIN WAIT 5 PING");
					log.info("PPING SLEEPING CONSTANT PPINGsleepS ["+sleepInS+"] NOT VALID - WE EXPECTED A NUMBER FROM 1 To 600  - WE WILL USE 30 SECONDS. 1 PING 30 SEC WAIT 2. PING 1 MIN WAIT 3. PING 90 SEC WAIT 4. PING 2 MIN WAIT 5 PING");
					basicSleepTime = 30;
				} else if(basicSleepTime > 600) {
//					System.out.println("PPING SLEEPING CONSTANT PPINGsleepS ["+sleepInS+"] NOT VALID - WE EXPECTED A NUMBER FROM 1 To 600  - WE WILL USE 30 SECONDS. 1 PING 30 SEC WAIT 2. PING 1 MIN WAIT 3. PING 90 SEC WAIT 4. PING 2 MIN WAIT 5 PING");
					log.info("PPING SLEEPING CONSTANT PPINGsleepS ["+sleepInS+"] NOT VALID - WE EXPECTED A NUMBER FROM 1 To 600  - WE WILL USE 30 SECONDS. 1 PING 30 SEC WAIT 2. PING 1 MIN WAIT 3. PING 90 SEC WAIT 4. PING 2 MIN WAIT 5 PING");
					basicSleepTime = 30;
				} else {
//					System.out.println("PPING SLEEPING CONSTANT PPINGsleepS ["+sleepInS+"] VALID");
					log.info("PPING SLEEPING CONSTANT PPINGsleepS ["+sleepInS+"] VALID");
				}
			} catch (NumberFormatException nfe) {
				basicSleepTime = 30;
//				System.out.println("PPING SLEEPING CONSTANT PPINGsleepS ["+sleepInS+"] NOT VALID - WE EXPECTED A NUMBER FROM 1 To 600  - WE WILL USE 30 SECONDS. 1 PING 30 SEC WAIT 2. PING 1 MIN WAIT 3. PING 90 SEC WAIT 4. PING 2 MIN WAIT 5 PING");
				log.error("PPING SLEEPING CONSTANT PPINGsleepS ["+sleepInS+"] NOT VALID - WE EXPECTED A NUMBER FROM 1 To 600  - WE WILL USE 30 SECONDS. 1 PING 30 SEC WAIT 2. PING 1 MIN WAIT 3. PING 90 SEC WAIT 4. PING 2 MIN WAIT 5 PING");
			}
		} 
		
		log.info("PPING SLEEPS 90 SECONDS BEFORE FIRST PING");
		sleep(90);
		
		boolean isSuccessful = false;
		for(int i = 0; i < 5; i++) {
			isSuccessful = pinging(ipTarget, (i+1));
			if(isSuccessful) {
				break;
			} else {
				isSuccessful = pinging("8.8.8.8", (i+1));
				if(isSuccessful) {
					break;
				}				
			}
			if(i +1 < 5) {
				// NO NEED TO SLEEP IN THE LAST ROUND
				sleep((i+1)*basicSleepTime);
			}
		}

		if(isSuccessful) {
			System.exit(0);
		} else {
			log.info("PPING RETURNS NON 0 VALUE TO SYSTEM - THIS MIGHT TRIGGER A REBOOT");
			System.exit(1);
		}
	}
	/**
	 * 
	 */
	private static void sleep(long seconds) {
		try {
			Thread.sleep(seconds * 1000);
		} catch(InterruptedException ie) {
			ie.printStackTrace();
		}
	}
	
	
	
	/**
	 * @param ipTarget
	 */
	private static boolean pinging(String ipTarget, int numberOfAttempt) {
		boolean retVal = false;
		try{
            InetAddress address = InetAddress.getByName(ipTarget);
            boolean reachable = address.isReachable(10000);
            retVal = reachable;
//            System.out.println("ATTEMPT NO. ["+numberOfAttempt+"]\tIS PPING IUP ["+ipTarget+"] REACHABLE? [" + reachable+"]");
            log.info("ATTEMPT NO. ["+numberOfAttempt+"]\tIS PPING IUP ["+ipTarget+"] REACHABLE? [" + reachable+"]");
        } catch (Exception e){
            e.printStackTrace();
        }
		return retVal;
	}

}



